@@ -40,7 +40,7 @@
  * Desc: Simulation device proxy
  * Author: Douglas S. Blank
  * Date: 13 May 2002
- * CVS: $Id: dev_simulation.c 7865 2009-06-20 13:03:14Z robotos $
+ * CVS: $Id: dev_simulation.c 8799 2010-06-28 04:12:42Z jpgr87 $
  **************************************************************************/
 
 #include <math.h>
@@ -224,8 +224,8 @@ int playerc_simulation_get_property(playerc_simulation_t *device,
   req.prop = property;
   req.prop_count = strlen(req.prop) + 1;
 
-  req.value = NULL;
-  req.value_count = 0;
+  req.value = value;
+  req.value_count = value_len;
   
   if( playerc_client_request(device->info.client, &device->info, 
                                 PLAYER_SIMULATION_REQ_GET_PROPERTY,
