@@ -96,7 +96,7 @@ int main(int argc, char** argv)
     // is called from the callback
     client.Run();
   }
-  catch (PlayerCc::PlayerError e)
+  catch (PlayerCc::PlayerError & e)
   {
     std::cerr << e << std::endl;
     return -1;
