@@ -265,7 +265,7 @@ main(int argc, char **argv)
       nanosleep(&sleep, NULL);
     }
   }
-  catch (PlayerCc::PlayerError e)
+  catch (PlayerCc::PlayerError & e)
   {
     std::cerr << e << std::endl;
     return -1;
