@@ -157,7 +157,7 @@ int main(int argc, char** argv)
       pp.SetSpeed(newspeed, newturnrate);
     }
   }
-  catch (PlayerCc::PlayerError e)
+  catch (PlayerCc::PlayerError & e)
   {
     std::cerr << e << std::endl;
     return -1;
