@@ -113,7 +113,7 @@ int main(int argc, char **argv)
       pp.SetSpeed(2 * newspeed, newturnrate);
     }
   }
-  catch (PlayerCc::PlayerError e)
+  catch (PlayerCc::PlayerError & e)
   {
     std::cerr << e << std::endl;
     return -1;
