@@ -1,8 +1,8 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2000  
+ *  Copyright (C) 2000
  *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
- *                      
+ *
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,9 +19,26 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
+/********************************************************************
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ ********************************************************************/
 
 /*
- * $Id: configfile.cc,v 1.13.2.1 2006/06/09 01:17:52 gerkey Exp $
+ * $Id: configfile.cc 8004 2009-07-13 14:03:44Z thjc $
  */
 
 #include <assert.h>
@@ -30,27 +47,38 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#if !defined WIN32
+  #include <unistd.h>
+#endif
 #include <math.h>
 #include <locale.h>
 
 #include <replace/replace.h>
 
-#include <libplayercore/player.h>
-#include <libplayercore/playercommon.h>
-#include <libplayercore/error.h>
+#include <libplayerinterface/player.h>
+#include <libplayercommon/playercommon.h>
+#include <libplayercommon/playercommon.h>
+#include <libplayerinterface/interface_util.h>
+#include <libplayerinterface/addr_util.h>
+
 #include <libplayercore/configfile.h>
-#include <libplayercore/interface_util.h>
 #include <libplayercore/driver.h>
 #include <libplayercore/drivertable.h>
 #include <libplayercore/devicetable.h>
 #include <libplayercore/globals.h>
 #include <libplayercore/plugins.h>
-#include <libplayercore/addr_util.h>
+#include <libplayerinterface/functiontable.h>
+
+#if defined WIN32
+  #define snprintf _snprintf
+  #define strdup _strdup
+  #define getcwd _getcwd
+  #include <direct.h>   // For getcwd()
+#endif
 
 //extern int global_playerport;
 
-#define COLOR_DATABASE "/usr/X11R6/lib/X11/rgb.txt"
+const char * COLOR_DATABASE[5] = {"/usr/X11R6/lib/X11/rgb.txt","/usr/share/X11/rgb.txt","/etc/X11/rgb.txt","/usr/lib/X11/rgb.txt", NULL};
 
 ///////////////////////////////////////////////////////////////////////////
 // the isblank() macro is not standard - it's a GNU extension
@@ -79,7 +107,7 @@
 
 ///////////////////////////////////////////////////////////////////////////
 // Default constructor
-ConfigFile::ConfigFile(uint32_t _default_host, uint32_t _default_robot) 
+ConfigFile::ConfigFile(uint32_t _default_host, uint32_t _default_robot)
 {
   this->default_host = _default_host;
   this->default_robot = _default_robot;
@@ -98,6 +126,15 @@ ConfigFile::ConfigFile(const char* _default_host, uint32_t _default_robot)
   this->InitFields();
 }
 
+/// Alternate constructor, used when not loading from a file
+ConfigFile::ConfigFile()
+{
+  this->default_host = 0;
+  this->default_robot = 0;
+  this->filename = strdup("");
+  this->InitFields();
+}
+
 void
 ConfigFile::InitFields()
 {
@@ -162,7 +199,7 @@ bool ConfigFile::Load(const char *filename)
   }
 
   ClearTokens();
-  
+
   // Read tokens from the file
   if (!LoadTokens(file, 0))
   {
@@ -190,7 +227,7 @@ bool ConfigFile::Load(const char *filename)
     fclose(file);
     return false;
   }
-  
+
   // Work out what the length units are
   const char *unit = ReadString(0, "unit_length", "m");
   if (strcmp(unit, "m") == 0)
@@ -206,12 +243,48 @@ bool ConfigFile::Load(const char *filename)
     this->unit_angle = M_PI / 180;
   else if (strcmp(unit, "radians") == 0)
     this->unit_angle = 1;
-  
+
   //DumpTokens();
-  fclose(file);  
+  fclose(file);
   return true;
 }
 
+///////////////////////////////////////////////////////////////////////////
+/// Add a (name,value) pair directly into the database, without
+/// reading from a file.  The (name,value) goes into the "global" section.
+void ConfigFile::InsertFieldValue(int index,
+                                  const char* name,
+                                  const char* value)
+{
+  // Work out what the length units are
+  if(!strcmp(name, "unit_length"))
+  {
+    if (strcmp(value, "m") == 0)
+      this->unit_length = 1.0;
+    else if (strcmp(value, "cm") == 0)
+      this->unit_length = 0.01;
+    else if (strcmp(value, "mm") == 0)
+      this->unit_length = 0.001;
+    return;
+  }
+
+  // Work out what the angle units are
+  if(!strcmp(name, "unit_angle"))
+  {
+    if (strcmp(value, "degrees") == 0)
+      this->unit_angle = M_PI / 180;
+    else if (strcmp(value, "radians") == 0)
+      this->unit_angle = 1;
+    return;
+  }
+
+  // AddField checks whether the field already exists
+  int field = this->AddField(-1,name,0);
+  this->AddToken(ConfigFile::TokenWord, value, 0);
+  this->AddFieldValue(field, index, this->token_count-1);
+
+}
+
 
 ///////////////////////////////////////////////////////////////////////////
 // Save world to file
@@ -219,7 +292,7 @@ bool ConfigFile::Save(const char *filename)
 {
   // Debugging
   //DumpFields();
-  
+
   // If no filename is supplied, use default
   if (!filename)
     filename = this->filename;
@@ -287,7 +360,7 @@ bool ConfigFile::LoadTokens(FILE *file, int include)
   int ch;
   int line;
   char token[256];
-  
+
   line = 1;
 
   while (true)
@@ -295,7 +368,7 @@ bool ConfigFile::LoadTokens(FILE *file, int include)
     ch = fgetc(file);
     if (ch == EOF)
       break;
-    
+
     if ((char) ch == '#')
     {
       ungetc(ch, file);
@@ -353,7 +426,7 @@ bool ConfigFile::LoadTokens(FILE *file, int include)
     else if ( 0x0d == ch )
     {
       ch = fgetc(file);
-      if ( 0x0a != ch ) 
+      if ( 0x0a != ch )
         ungetc(ch, file);
       line++;
       AddToken(TokenEOL, "\n", include);
@@ -361,7 +434,7 @@ bool ConfigFile::LoadTokens(FILE *file, int include)
     else if ( 0x0a == ch )
     {
       ch = fgetc(file);
-      if ( 0x0d != ch ) 
+      if ( 0x0d != ch )
         ungetc(ch, file);
       line++;
       AddToken(TokenEOL, "\n", include);
@@ -387,7 +460,7 @@ bool ConfigFile::LoadTokenComment(FILE *file, int *line, int include)
 
   len = 0;
   memset(token, 0, sizeof(token));
-  
+
   while (true)
   {
     ch = fgetc(file);
@@ -417,10 +490,10 @@ bool ConfigFile::LoadTokenWord(FILE *file, int *line, int include)
   char token[1024];
   int len;
   int ch;
-  
+
   len = 0;
   memset(token, 0, sizeof(token));
-  
+
   while (true)
   {
     ch = fgetc(file);
@@ -443,8 +516,8 @@ bool ConfigFile::LoadTokenWord(FILE *file, int *line, int include)
         if (!LoadTokenInclude(file, line, include))
           return false;
       }
-      else if(strcmp(token, "true") == 0 || strcmp(token, "false") == 0 || 
-              strcmp(token, "yes") == 0  || strcmp(token, "no") == 0) 
+      else if(strcmp(token, "true") == 0 || strcmp(token, "false") == 0 ||
+              strcmp(token, "yes") == 0  || strcmp(token, "no") == 0)
       {
         ungetc(ch, file);
         AddToken(TokenBool, token, include);
@@ -469,7 +542,7 @@ bool ConfigFile::LoadTokenInclude(FILE *file, int *line, int include)
   int ch;
   const char *filename;
   char *fullpath;
-  
+
   ch = fgetc(file);
 
   if (ch == EOF)
@@ -488,7 +561,7 @@ bool ConfigFile::LoadTokenInclude(FILE *file, int *line, int include)
     return false;
 
   ch = fgetc(file);
-  
+
   if (ch == EOF)
   {
     TOKEN_ERR("incomplete include statement", *line);
@@ -522,7 +595,7 @@ bool ConfigFile::LoadTokenInclude(FILE *file, int *line, int include)
     fullpath = (char*) malloc(PATH_MAX);
     memset(fullpath, 0, PATH_MAX);
     strcat( fullpath, dirname(tmp));
-    strcat( fullpath, "/" ); 
+    strcat( fullpath, "/" );
     strcat( fullpath, filename );
     assert(strlen(fullpath) + 1 < PATH_MAX);
     free(tmp);
@@ -534,10 +607,16 @@ bool ConfigFile::LoadTokenInclude(FILE *file, int *line, int include)
     // There's no bounds-checking, but what the heck.
     char *tmp = strdup(this->filename);
     fullpath = (char*) malloc(PATH_MAX);
-    getcwd(fullpath, PATH_MAX);
-    strcat( fullpath, "/" ); 
+    char* ret = getcwd(fullpath, PATH_MAX);
+    if (ret == NULL)
+    {
+    	PLAYER_ERROR("Failed to get working directory");
+    	assert(false);
+    	// TODO: handle this error properly
+    }
+    strcat( fullpath, "/" );
     strcat( fullpath, dirname(tmp));
-    strcat( fullpath, "/" ); 
+    strcat( fullpath, "/" );
     strcat( fullpath, filename );
     assert(strlen(fullpath) + 1 < PATH_MAX);
     free(tmp);
@@ -576,10 +655,10 @@ bool ConfigFile::LoadTokenNum(FILE *file, int *line, int include)
   char token[1024];
   int len;
   int ch;
-  
+
   len = 0;
   memset(token, 0, sizeof(token));
-  
+
   while (true)
   {
     ch = fgetc(file);
@@ -612,12 +691,12 @@ bool ConfigFile::LoadTokenString(FILE *file, int *line, int include)
   int ch;
   int len;
   char token[1024];
-  
+
   len = 0;
   memset(token, 0, sizeof(token));
 
   ch = fgetc(file);
-      
+
   while (true)
   {
     ch = fgetc(file);
@@ -649,10 +728,10 @@ bool ConfigFile::LoadTokenSpace(FILE *file, int *line, int include)
   int ch;
   int len;
   char token[1024];
-  
+
   len = 0;
   memset(token, 0, sizeof(token));
-  
+
   while (true)
   {
     ch = fgetc(file);
@@ -684,7 +763,7 @@ bool ConfigFile::SaveTokens(FILE *file)
 {
   int i;
   Token *token;
-  
+
   for (i = 0; i < this->token_count; i++)
   {
     token = this->tokens + i;
@@ -692,7 +771,7 @@ bool ConfigFile::SaveTokens(FILE *file)
     if (token->include > 0)
       continue;
     if (token->type == TokenString)
-      fprintf(file, "\"%s\"", token->value);  
+      fprintf(file, "\"%s\"", token->value);
     else
       fprintf(file, "%s", token->value);
   }
@@ -729,11 +808,11 @@ bool ConfigFile::AddToken(int type, const char *value, int include)
     this->tokens = (Token*) realloc(this->tokens, this->token_size * sizeof(this->tokens[0]));
   }
 
-  this->tokens[this->token_count].include = include;  
+  this->tokens[this->token_count].include = include;
   this->tokens[this->token_count].type = type;
   this->tokens[this->token_count].value = strdup(value);
   this->token_count++;
-  
+
   return true;
 }
 
@@ -746,7 +825,7 @@ bool ConfigFile::SetTokenValue(int index, const char *value)
 
   free(this->tokens[index].value);
   this->tokens[index].value = strdup(value);
-  
+
   return true;
 }
 
@@ -793,11 +872,11 @@ bool ConfigFile::ParseTokens()
 
   ClearSections();
   ClearFields();
-  
+
   // Add in the "global" section.
   section = AddSection(-1, "");
   line = 1;
-  
+
   for (i = 0; i < this->token_count; i++)
   {
     token = this->tokens + i;
@@ -815,6 +894,11 @@ bool ConfigFile::ParseTokens()
           if (!ParseTokenDefine(&i, &line))
             return false;
         }
+        else if (strcmp(token->value, "plugin") == 0)
+        {
+          if (!ParseTokenPlugin(&i, &line))
+            return false;
+        }
         else
         {
           if (!ParseTokenWord(section, &i, &line))
@@ -866,6 +950,43 @@ bool ConfigFile::ParseTokenInclude(int *index, int *line)
   return false;
 }
 
+///////////////////////////////////////////////////////////////////////////
+// Parse a plugin statement
+bool ConfigFile::ParseTokenPlugin(int *index, int *line)
+{
+  int i;
+  Token *token;
+
+  lt_dlhandle handle;
+
+  for (i = *index + 1; i < this->token_count; i++)
+  {
+    token = this->tokens + i;
+
+    switch (token->type)
+    {
+      case TokenString:
+        // Load the plugin
+        if((handle = LoadPlugin(token->value,this->filename)) == NULL)
+        {
+          PLAYER_ERROR1("failed to load plugin: %s", token->value);
+          return false;
+        }
+        InitDriverPlugin(handle);
+        break;
+      case TokenSpace:
+        break;
+      case TokenEOL:
+        *index = i;
+        (*line)++;
+        return true;
+      default:
+        PARSE_ERR("syntax error in plugin statement", *line);
+    }
+  }
+  PARSE_ERR("incomplete include statement", *line);
+  return false;
+}
 
 ///////////////////////////////////////////////////////////////////////////
 // Parse a macro definition
@@ -976,7 +1097,7 @@ bool ConfigFile::ParseTokenWord(int section, int *index, int *line)
         return false;
     }
   }
-  
+
   return false;
 }
 
@@ -992,7 +1113,7 @@ bool ConfigFile::ParseTokenSection(int section, int *index, int *line)
 
   name = *index;
   macro = LookupMacro(GetTokenValue(name));
-  
+
   // If the section name is a macro...
   if (macro >= 0)
   {
@@ -1081,7 +1202,7 @@ bool ConfigFile::ParseTokenField(int section, int *index, int *line)
   name = *index;
   value = -1;
   count = 0;
-  
+
   for (i = *index + 1; i < this->token_count; i++)
   {
     token = this->tokens + i;
@@ -1128,7 +1249,7 @@ bool ConfigFile::ParseTokenTuple(int section, int field, int *index, int *line)
   Token *token;
 
   count = 0;
-  
+
   for (i = *index + 1; i < this->token_count; i++)
   {
     token = this->tokens + i;
@@ -1188,7 +1309,7 @@ int ConfigFile::AddMacro(const char *macroname, const char *sectionname,
   this->macros[macro].starttoken = starttoken;
   this->macros[macro].endtoken = endtoken;
   this->macro_count++;
-  
+
   return macro;
 }
 
@@ -1200,7 +1321,7 @@ int ConfigFile::LookupMacro(const char *macroname)
 {
   int i;
   CMacro *macro;
-  
+
   for (i = 0; i < this->macro_count; i++)
   {
     macro = this->macros + i;
@@ -1260,7 +1381,7 @@ int ConfigFile::AddSection(int parent, const char *type)
   this->sections[section].parent = parent;
   this->sections[section].type = type;
   this->section_count++;
-  
+
   return section;
 }
 
@@ -1350,7 +1471,7 @@ int ConfigFile::AddField(int section, const char *name, int line)
 {
   int i;
   Field *field;
-  
+
   // See if this field already exists; if it does, we dont need to
   // add it again.
   for (i = 0; i < this->field_count; i++)
@@ -1408,7 +1529,7 @@ void ConfigFile::AddFieldValue(int field, int index, int value_token)
 
 
 ///////////////////////////////////////////////////////////////////////////
-// Get a field 
+// Get a field
 int ConfigFile::GetField(int section, const char *name)
 {
   // Find first instance of field
@@ -1447,18 +1568,18 @@ void ConfigFile::SetFieldValue(int field, int index, const char *value)
 
 
 ///////////////////////////////////////////////////////////////////////////
-// Get the value of an field 
+// Get the value of an field
 const char *ConfigFile::GetFieldValue(int field, int index, bool flag_used)
 {
   assert(field >= 0);
   Field *pfield = this->fields + field;
-  
+
   if(index >= pfield->value_count)
     return NULL;
 
   if (flag_used)
     pfield->useds[index] = true;
-  
+
   return GetTokenValue(pfield->values[index]);
 }
 
@@ -1472,7 +1593,7 @@ void ConfigFile::DumpFields()
   {
     Field *field = this->fields + i;
     Section *section = this->sections + field->section;
-    
+
     printf("## [%d]", field->section);
     printf("[%s]", section->type);
     printf("[%s]", field->name);
@@ -1502,7 +1623,7 @@ void ConfigFile::WriteString(int section, const char *name, const char *value)
   int field = GetField(section, name);
   if (field < 0)
     return;
-  SetFieldValue(field, 0, value);  
+  SetFieldValue(field, 0, value);
 }
 
 
@@ -1612,7 +1733,7 @@ uint32_t ConfigFile::ReadColor(int section, const char *name, uint32_t value)
 {
   int field;
   const char *color;
-  
+
   field = GetField(section, name);
   if (field < 0)
     return value;
@@ -1634,7 +1755,7 @@ const char *ConfigFile::ReadFilename(int section, const char *name, const char *
     return value;
 
   const char *filename = GetFieldValue(field, 0);
-  
+
   if( filename[0] == '/' || filename[0] == '~' )
     return filename;
 
@@ -1647,12 +1768,12 @@ const char *ConfigFile::ReadFilename(int section, const char *name, const char *
     char *fullpath = (char*) malloc(MAX_FILENAME_SIZE);
     memset(fullpath, 0, MAX_FILENAME_SIZE);
     strcat( fullpath, dirname(tmp));
-    strcat( fullpath, "/" ); 
+    strcat( fullpath, "/" );
     strcat( fullpath, filename );
     assert(strlen(fullpath) + 1 < MAX_FILENAME_SIZE);
 
     SetFieldValue(field, 0, fullpath);
-    
+
     free(fullpath);
     free(tmp);
   }
@@ -1664,15 +1785,21 @@ const char *ConfigFile::ReadFilename(int section, const char *name, const char *
     // There's no bounds-checking, but what the heck.
     char *tmp = strdup(this->filename);
     char *fullpath = (char*) malloc(MAX_FILENAME_SIZE);
-    getcwd(fullpath, MAX_FILENAME_SIZE);
-    strcat( fullpath, "/" ); 
+    char * ret = getcwd(fullpath, MAX_FILENAME_SIZE);
+    if (ret == NULL)
+    {
+    	PLAYER_ERROR("Failed to get working directory");
+    	assert(false);
+    	// TODO: handle this error properly
+    }
+    strcat( fullpath, "/" );
     strcat( fullpath, dirname(tmp));
-    strcat( fullpath, "/" ); 
+    strcat( fullpath, "/" );
     strcat( fullpath, filename );
     assert(strlen(fullpath) + 1 < MAX_FILENAME_SIZE);
 
     SetFieldValue(field, 0, fullpath);
-    
+
     free(fullpath);
     free(tmp);
   }
@@ -1720,7 +1847,7 @@ void ConfigFile::WriteTupleString(int section, const char *name,
   if (field < 0)
     field = InsertField(section, name);
   */
-  SetFieldValue(field, index, value);  
+  SetFieldValue(field, index, value);
 }
 
 
@@ -1852,19 +1979,26 @@ uint32_t ConfigFile::ReadTupleColor(int section, const char *name, int index, ui
 // Look up the color in a data based (transform color name -> color value).
 uint32_t ConfigFile::LookupColor(const char *name)
 {
-  FILE *file;
-  const char *filename;
+  FILE *file = NULL;
 
-  filename = COLOR_DATABASE;
-  file = fopen(filename, "r");
+  for (int i=0; COLOR_DATABASE[i] != NULL; ++i)
+  {
+    file = fopen(COLOR_DATABASE[i], "r");
+    if (file)
+      break;
+  }
   if (!file)
   {
-    PLAYER_ERROR2("unable to open color database %s : %s",
-                  filename, strerror(errno));
+    PLAYER_ERROR("unable to open color database: tried the following files");
+    for (int i=0; COLOR_DATABASE[i] != NULL; ++i)
+    {
+      PLAYER_ERROR1("\t: %s", COLOR_DATABASE[i]);
+    }
     fclose(file);
     return 0xFFFFFF;
   }
-  
+
+
   while (true)
   {
     char line[1024];
@@ -1872,7 +2006,7 @@ uint32_t ConfigFile::LookupColor(const char *name)
       break;
 
     // it's a macro or comment line - ignore the line
-    if (line[0] == '!' || line[0] == '#' || line[0] == '%') 
+    if (line[0] == '!' || line[0] == '#' || line[0] == '%')
       continue;
 
     // Trim the trailing space
@@ -1883,7 +2017,7 @@ uint32_t ConfigFile::LookupColor(const char *name)
     int r, g, b;
     int chars_matched = 0;
     sscanf( line, "%d %d %d %n", &r, &g, &b, &chars_matched );
-      
+
     // Read the name
     char* nname = line + chars_matched;
 
@@ -1919,18 +2053,18 @@ int ConfigFile::ReadDeviceAddr(player_devaddr_t *addr, int section,
   // Get the field index
   if ((prop = GetField(section, name)) < 0)
   {
-    CONFIG_ERR1("missing field [%s]", this->fields[prop].line, name);
+    CONFIG_ERR1("missing field [%s]", 0, name);
     return -1;
   }
 
   // Find the number of values in the field
   count = GetFieldValueCount(prop);
-  
+
   // Consider all the values, looking for a match
   for (i = 0; i < count; i++)
   {
     assert(sizeof(str) > strlen(GetFieldValue(prop, i, false)));
-    strcpy(str, GetFieldValue(prop, i, false));
+    strncpy(str, GetFieldValue(prop, i, false),sizeof(str));
 
     memset(tokens, 0, sizeof(tokens));
     token_count = 5;
@@ -1975,7 +2109,7 @@ int ConfigFile::ReadDeviceAddr(player_devaddr_t *addr, int section,
       else
       {
         // No; assume it's a string containing a hostname or IP address
-        if(hostname_to_packedaddr(&host, tokens[1]) < 0)
+        if(hostname_to_packedaddr((uint32_t *) &host, tokens[1]) < 0)
         {
           PLAYER_ERROR1("name lookup failed for host \"%s\"", tokens[1]);
           return -1;
@@ -2012,7 +2146,7 @@ int ConfigFile::ReadDeviceAddr(player_devaddr_t *addr, int section,
     // is no match.
     if (key && k == NULL)
       continue;
-    
+
     // If the key is expected and present in the file, it must match
     if (key && k && strcmp(key, k) != 0)
       continue;
@@ -2025,14 +2159,14 @@ int ConfigFile::ReadDeviceAddr(player_devaddr_t *addr, int section,
     addr->interf = interf.interf;
     addr->index = ind;
     return 0;
- 
+
   }
 
   return -1;
 }
 
 // Parse all driver blocks
-bool 
+bool
 ConfigFile::ParseAllDrivers()
 {
   for(int i = 1; i < this->GetSectionCount(); i++)
@@ -2048,28 +2182,34 @@ ConfigFile::ParseAllDrivers()
 }
 
 // Parse a driver block, and update the deviceTable accordingly
-bool 
+bool
 ConfigFile::ParseDriver(int section)
 {
-  const char *pluginname;  
+  const char *pluginname;
   const char *drivername;
   DriverEntry *entry;
   Driver *driver;
   Device *device;
   int count;
+  lt_dlhandle handle;
 
   entry = NULL;
   driver = NULL;
-  
+
   // Load any required plugins
   pluginname = this->ReadString(section, "plugin", NULL);
   if (pluginname != NULL)
   {
-    if(!LoadPlugin(pluginname,this->filename))
+    if((handle = LoadPlugin(pluginname,this->filename)) == NULL)
     {
       PLAYER_ERROR1("failed to load plugin: %s", pluginname);
       return (false);
     }
+    if(!InitDriverPlugin(handle))
+    {
+      PLAYER_ERROR1("failed to initialise plugin: %s", pluginname);
+      return (false);
+    }
   }
 
   // Get the driver name
@@ -2079,7 +2219,7 @@ ConfigFile::ParseDriver(int section)
     PLAYER_ERROR1("No driver name specified in section %d", section);
     return (false);
   }
-  
+
   // Look for the driver
   entry = driverTable->GetDriverEntry(drivername);
   if (entry == NULL)
@@ -2123,12 +2263,95 @@ ConfigFile::ParseDriver(int section)
     PLAYER_ERROR1("Driver has no (usable) interfaces \"%s\"", drivername);
     return false;
   }
-  
-  // Should this device be "always on"?  
+
+  // Should this device be "always on"?
   if (driver)
-    driver->alwayson = this->ReadInt(section, "alwayson", driver->alwayson);
+    driver->alwayson = this->ReadInt(section, "alwayson", driver->alwayson) ? true : false;
 
   return true;
 }
 
+// Parse all interface blocks
+bool
+ConfigFile::ParseAllInterfaces()
+{
+  for(int i = 1; i < this->GetSectionCount(); i++)
+  {
+    // Check for new-style device block
+    if(strcmp(this->GetSectionType(i), "interface") == 0)
+    {
+      if(!this->ParseInterface(i))
+        return false;
+    }
+  }
+  return true;
+}
 
+// Parse an interface block, and update the interface systems accordingly
+bool
+ConfigFile::ParseInterface(int section)
+{
+  const char *pluginname;
+  const char *interfacename;
+  int interfacecode;
+  int replace;
+  lt_dlhandle handle;
+  playerxdr_function_t *flist;
+
+  // Check if this interface is replacing an existing one's messages
+  replace = this->ReadInt(section, "replace", 0);
+
+  // Get the interface name
+  interfacename = this->ReadString(section, "name", NULL);
+  if (interfacename == NULL)
+  {
+    PLAYER_ERROR1("No interface name specified in section %d", section);
+    return false;
+  }
+
+  // Get the interface code
+  interfacecode = this->ReadInt(section, "code", -1);
+  if (interfacecode == -1)
+  {
+    PLAYER_ERROR1("No interface code specified in section %d", section);
+    return false;
+  }
+
+  // Find the plugin name
+  pluginname = this->ReadString(section, "plugin", NULL);
+  if (pluginname == NULL)
+  {
+    PLAYER_ERROR1("No plugin name specified for plugin interface in section %d", section);
+    return false;
+  }
+
+  // Load the plugin
+  if((handle = LoadPlugin(pluginname,this->filename)) == NULL)
+  {
+    PLAYER_ERROR1("failed to load plugin: %s", pluginname);
+    return false;
+  }
+
+  // Get the function table list
+  if((flist = InitInterfacePlugin(handle)) == NULL)
+  {
+    PLAYER_ERROR1("failed to initialise plugin: %s", pluginname);
+    return false;
+  }
+
+  // Add the function table list
+  if(playerxdr_ftable_add_multi(flist, replace) < 0)
+  {
+    PLAYER_ERROR1("Failed to add interface functions for plugin interface %s", interfacename);
+    return false;
+  }
+
+  // Add the interface name to the interface names table
+  if(itable_add (interfacename, interfacecode, replace) < 0)
+  {
+    PLAYER_ERROR2("Failed to add interface name/code: %s/%d", interfacename, interfacecode);
+    return false;
+  }
+
+  return true;
+}
