@@ -19,16 +19,45 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
+/********************************************************************
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ ********************************************************************/
 
 /*
- * $Id: configfile.h,v 1.8.2.1 2006/06/09 01:17:52 gerkey Exp $
+ * $Id: configfile.h 8004 2009-07-13 14:03:44Z thjc $
  */
 #ifndef CONFFILE_H
 #define CONFFILE_H
 
+#if defined (WIN32)
+  #if defined (PLAYER_STATIC)
+    #define PLAYERCORE_EXPORT
+  #elif defined (playercore_EXPORTS)
+    #define PLAYERCORE_EXPORT    __declspec (dllexport)
+  #else
+    #define PLAYERCORE_EXPORT    __declspec (dllimport)
+  #endif
+#else
+  #define PLAYERCORE_EXPORT
+#endif
+
 #include <stdio.h>
 
-#include <libplayercore/player.h>
+#include <libplayerinterface/player.h>
 
 /** @brief Class for loading configuration file information.
 
@@ -164,7 +193,7 @@ to retrieve information from the "requires" line of the configuration file.
 
 */
 
-class ConfigFile
+class PLAYERCORE_EXPORT ConfigFile
 {
   /// @brief Standard constructor
   public: ConfigFile(uint32_t _default_host, uint32_t _default_robot);
@@ -172,6 +201,9 @@ class ConfigFile
   /// @brief Alternate constructor, to specify the host as a string
   public: ConfigFile(const char* _default_host, uint32_t _default_robot);
 
+  /// Alternate constructor, used when not loading from a file
+  public: ConfigFile();
+
   /// @brief Standard destructor
   public: ~ConfigFile();
 
@@ -183,6 +215,18 @@ class ConfigFile
   /// @returns Returns true on success.
   public: bool Load(const char *filename);
 
+  /// @brief Add a (name,value) pair directly into the database, without
+  /// reading from a file.  The (name,value) goes into the "global" section.
+  /// Can be called multiple times with different index to create a tuple
+  /// field.
+  /// @param index Index of the value within the field (0 unless the field
+  ///              is a tuple)
+  /// @param name Name of the field
+  /// @param value Value to be assigned
+  public: void InsertFieldValue(int index,
+                                const char* name, 
+                                const char* value);
+
   // Save config back into file
   // Set filename to NULL to save back into the original file
   private: bool Save(const char *filename);
@@ -429,9 +473,15 @@ class ConfigFile
   // Parse a driver block, and update the deviceTable accordingly
   public: bool ParseDriver(int section);
 
+  // Parse an interface block, and update the interface systems accordingly
+  public: bool ParseInterface(int section);
+
   // Parse all driver blocks
   public: bool ParseAllDrivers();
 
+  // Parse all interface blocks
+  public: bool ParseAllInterfaces();
+
   /// @brief Get the number of sections.
   public: int GetSectionCount();
 
@@ -498,6 +548,9 @@ class ConfigFile
   // Parse a macro definition
   private: bool ParseTokenDefine(int *index, int *line);
 
+  // Parse a macro definition
+  private: bool ParseTokenPlugin(int *index, int *line);
+
   // Parse an word (could be a section or an field) from the token list.
   private: bool ParseTokenWord(int section, int *index, int *line);
 
