@@ -22,7 +22,7 @@
  * Desc: Rtk fig functions
  * Author: Andrew Howard
  * Contributors: Richard Vaughan
- * CVS: $Id: rtk_fig.c 7942 2009-07-02 23:50:06Z gbiggs $
+ * CVS: $Id: rtk_fig.c 8799 2010-06-28 04:12:42Z jpgr87 $
  *
  * Notes:
  *   Some of this is a horrible hack, particular the xfig stuff.
@@ -38,6 +38,10 @@
 #include "rtk.h"
 #include "rtkprivate.h"
 
+#ifndef M_PI
+	#define M_PI 3.14159265358979323846
+#endif
+
 // Draw the selection
 void rtk_fig_render_selection(rtk_fig_t *fig);
 
