@@ -1,8 +1,8 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2000  
+ *  Copyright (C) 2000
  *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
- *                      
+ *
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 /*
- * $Id: acts.cc,v 1.3.2.1 2006/06/07 16:12:39 gerkey Exp $
+ * $Id: acts.cc 7208 2008-12-19 02:10:56Z gbiggs $
  *
  *   the P2 vision device.  it takes pan tilt zoom commands for the
  *   sony PTZ camera (if equipped), and returns color blob data gathered
@@ -139,7 +139,10 @@ driver
 
 /** @} */
 
+#include "config.h"
+
 #include <assert.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <unistd.h> /* close(2),fcntl(2),getpid(2),usleep(3),execvp(3),fork(2)*/
 #include <netdb.h> /* for gethostbyname(3) */
@@ -156,8 +159,8 @@ driver
 #include <libplayercore/playercore.h>
 
 #define ACTS_NUM_CHANNELS 32
-#define ACTS_HEADER_SIZE_1_0 2*ACTS_NUM_CHANNELS  
-#define ACTS_HEADER_SIZE_1_2 4*ACTS_NUM_CHANNELS  
+#define ACTS_HEADER_SIZE_1_0 2*ACTS_NUM_CHANNELS
+#define ACTS_HEADER_SIZE_1_2 4*ACTS_NUM_CHANNELS
 #define ACTS_BLOB_SIZE_1_0 10
 #define ACTS_BLOB_SIZE_1_2 16
 #define ACTS_MAX_BLOBS_PER_CHANNEL 10
@@ -179,7 +182,7 @@ typedef enum
 /* default is to use older ACTS (until we change our robots) */
 #define DEFAULT_ACTS_VERSION ACTS_VERSION_1_0
 #define DEFAULT_ACTS_CONFIGFILE "/usr/local/acts/actsconfig"
-/* default is to give no path for the binary; in this case, use execvp() 
+/* default is to give no path for the binary; in this case, use execvp()
  * and user's PATH */
 #define DEFAULT_ACTS_PATH ""
 #define DEFAULT_ACTS_WIDTH 160
@@ -204,7 +207,7 @@ typedef struct acts_header_elt
 
   /** Number of blobs for this channel. */
   uint16_t num;
-  
+
 } acts_header_elt_t;
 
 
@@ -223,7 +226,7 @@ typedef struct acts_blob_elt
 
   /** Bounding box for the blob (image coords). */
   uint16_t left, right, top, bottom;
-  
+
 } acts_blob_elt_t;
 
 
@@ -238,27 +241,27 @@ typedef struct acts_data
 
   /** The list of blobs. */
   acts_blob_elt_t blobs[ACTS_MAX_BLOBS];
-  
+
 } acts_data_t;
 
 
 
-class Acts:public Driver 
+class Acts:public ThreadedDriver
 {
   private:
     int debuglevel;             // debuglevel 0=none, 1=basic, 2=everything
     int pid;      // ACTS's pid so we can kill it later
 
-    
+
     // returns the enum representation of the given version string, or
     // ACTS_VERSION_UNKNOWN on failure to match.
-    acts_version_t version_string_to_enum(char* versionstr);
-    
+    acts_version_t version_string_to_enum(const char* versionstr);
+
     // writes the string representation of the given version number into
     // versionstr, up to len.
     // returns  0 on success
     //         -1 on failure to match.
-    int version_enum_to_string(acts_version_t versionnum, char* versionstr, 
+    int version_enum_to_string(acts_version_t versionnum, char* versionstr,
                                int len);
 
     // stuff that will be used on the cmdline to start ACTS
@@ -291,16 +294,16 @@ class Acts:public Driver
   public:
     int sock;               // socket to ACTS
 
-    // constructor 
+    // constructor
     //
     Acts( ConfigFile* cf, int section);
 
     virtual void Main();
-
+    virtual void MainQuit();
+    
     void KillACTS();
 
-    int Setup();
-    int Shutdown();
+    int MainSetup();
 };
 
 // a factory creation function
@@ -310,8 +313,8 @@ Driver* Acts_Init( ConfigFile* cf, int section)
 }
 
 // a driver registration function
-void 
-Acts_Register(DriverTable* table)
+void
+acts_Register(DriverTable* table)
 {
   table->AddDriver("acts", Acts_Init);
 }
@@ -330,7 +333,7 @@ void QuitACTS(void* visiondevice);
 
 
 Acts::Acts( ConfigFile* cf, int section)
-  : Driver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_BLOBFINDER_CODE)
+  : ThreadedDriver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_BLOBFINDER_CODE)
 {
   char tmpstr[MAX_FILENAME_SIZE];
   int tmpint;
@@ -344,7 +347,7 @@ Acts::Acts( ConfigFile* cf, int section)
   strncpy(binarypath,
           cf->ReadFilename(section, "path", DEFAULT_ACTS_PATH),
           sizeof(binarypath));
-  strncpy(configfilepath, 
+  strncpy(configfilepath,
           cf->ReadFilename(section, "configfile", DEFAULT_ACTS_CONFIGFILE),
           sizeof(configfilepath));
   strncpy(tmpstr,
@@ -358,7 +361,7 @@ Acts::Acts( ConfigFile* cf, int section)
   }
   width = cf->ReadInt(section, "width", DEFAULT_ACTS_WIDTH);
   height = cf->ReadInt(section, "height", DEFAULT_ACTS_HEIGHT);
-  
+
   // now, get the optionals
   memset(minarea,0,sizeof(minarea));
   if((tmpint = cf->ReadInt(section, "pixels", -1)) >= 0)
@@ -369,19 +372,19 @@ Acts::Acts( ConfigFile* cf, int section)
     snprintf(fps,sizeof(fps),"%d",tmpint);
   memset(drivertype,0,sizeof(drivertype));
   if(cf->ReadString(section, "drivertype", NULL))
-    strncpy(drivertype, cf->ReadString(section, "drivertype", NULL), 
+    strncpy(drivertype, cf->ReadString(section, "drivertype", NULL),
             sizeof(drivertype)-1);
   invertp = cf->ReadInt(section, "invert", -1);
   memset(devicepath,0,sizeof(devicepath));
   if(cf->ReadString(section, "devicepath", NULL))
-    strncpy(devicepath, cf->ReadString(section, "devicepath", NULL), 
+    strncpy(devicepath, cf->ReadString(section, "devicepath", NULL),
             sizeof(devicepath)-1);
   memset(channel,0,sizeof(channel));
   if((tmpint = cf->ReadInt(section, "channel", -1)) >= 0)
     snprintf(channel,sizeof(channel),"%d",tmpint);
   memset(norm,0,sizeof(norm));
   if(cf->ReadString(section, "norm", NULL))
-    strncpy(norm, cf->ReadString(section, "norm", NULL), 
+    strncpy(norm, cf->ReadString(section, "norm", NULL),
             sizeof(norm)-1);
   pxc200p = cf->ReadInt(section, "pxc200", -1);
   memset(brightness,0,sizeof(brightness));
@@ -410,7 +413,7 @@ Acts::Acts( ConfigFile* cf, int section)
   header_elt_len = header_len / ACTS_MAX_CHANNELS;
   memset(portnumstring, 0, sizeof(portnumstring));
   snprintf(portnumstring,sizeof(portnumstring),"%d",portnum);
-  
+
   memset(widthstring, 0, sizeof(widthstring));
   snprintf(widthstring,sizeof(widthstring),"%d",width);
 
@@ -427,10 +430,10 @@ Acts::Acts( ConfigFile* cf, int section)
     this->colors[ch] = color;
   }
 }
-    
+
 // returns the enum representation of the given version string, or
 // -1 on failure to match.
-acts_version_t Acts::version_string_to_enum(char* versionstr)
+acts_version_t Acts::version_string_to_enum(const char* versionstr)
 {
   if(!strcmp(versionstr,ACTS_VERSION_1_0_STRING))
     return(ACTS_VERSION_1_0);
@@ -446,7 +449,7 @@ acts_version_t Acts::version_string_to_enum(char* versionstr)
 // versionstr, up to len.
 // returns  0 on success
 //         -1 on failure to match.
-int Acts::version_enum_to_string(acts_version_t versionnum, 
+int Acts::version_enum_to_string(acts_version_t versionnum,
                                           char* versionstr, int len)
 {
   switch(versionnum)
@@ -466,7 +469,7 @@ int Acts::version_enum_to_string(acts_version_t versionnum,
 }
 
 int
-Acts::Setup()
+Acts::MainSetup()
 {
   int i;
   int j;
@@ -479,7 +482,6 @@ Acts::Setup()
 
   static struct sockaddr_in server;
   char host[] = "localhost";
-  struct hostent* entp;
 
   printf("ACTS vision server connection initializing...");
   fflush(stdout);
@@ -495,105 +497,122 @@ Acts::Setup()
   i = 0;
   acts_args[i++] = acts_bin_name;
   // build the argument list, based on version
+  // these are needed as execv expects a const array of char *'s not an array of const char *'s
+  static char dash_d[] = "-d";
+  static char dash_i[] = "-i";
+  static char dash_n[] = "-n";
+  static char dash_p[] = "-p";
+  static char dash_s[] = "-s";
+  static char dash_t[] = "-t";
+  static char dash_w[] = "-w";
+  static char dash_x[] = "-x";
+
+  static char dash_B[] = "-B";
+  static char dash_C[] = "-C";
+  static char dash_G[] = "-G";
+  static char dash_H[] = "-H";
+  static char dash_R[] = "-R";
+  static char dash_V[] = "-V";
+  static char dash_W[] = "-W";
   switch(acts_version)
   {
     case ACTS_VERSION_1_0:
-      acts_args[i++] = "-t";
+      acts_args[i++] = dash_t;
       acts_args[i++] = configfilepath;
       if(strlen(portnumstring))
       {
-        acts_args[i++] = "-s";
+        acts_args[i++] = dash_s;
         acts_args[i++] = portnumstring;
       }
       if(strlen(devicepath))
       {
-        acts_args[i++] = "-d";
+        acts_args[i++] = dash_d;
         acts_args[i++] = devicepath;
       }
       break;
     case ACTS_VERSION_1_2:
-      acts_args[i++] = "-t";
+      acts_args[i++] = dash_t;
       acts_args[i++] = configfilepath;
       if(strlen(portnumstring))
       {
-        acts_args[i++] = "-p";
+        acts_args[i++] = dash_p;
         acts_args[i++] = portnumstring;
       }
       if(strlen(devicepath))
       {
-        acts_args[i++] = "-d";
+        acts_args[i++] = dash_d;
         acts_args[i++] = devicepath;
       }
       if(strlen(contrast))
       {
-        acts_args[i++] = "-C";
+        acts_args[i++] = dash_C;
         acts_args[i++] = contrast;
       }
       if(strlen(brightness))
       {
-        acts_args[i++] = "-B";
+        acts_args[i++] = dash_B;
         acts_args[i++] = brightness;
       }
-      acts_args[i++] = "-W";
+      acts_args[i++] = dash_W;
       acts_args[i++] = widthstring;
-      acts_args[i++] = "-H";
+      acts_args[i++] = dash_H;
       acts_args[i++] = heightstring;
       break;
     case ACTS_VERSION_2_0:
-      acts_args[i++] = "-t";
+      acts_args[i++] = dash_t;
       acts_args[i++] = configfilepath;
       if(strlen(minarea))
       {
-        acts_args[i++] = "-w";
+        acts_args[i++] = dash_w;
         acts_args[i++] = minarea;
       }
       if(strlen(portnumstring))
       {
-        acts_args[i++] = "-p";
+        acts_args[i++] = dash_p;
         acts_args[i++] = portnumstring;
       }
       if(strlen(fps))
       {
-        acts_args[i++] = "-R";
+        acts_args[i++] = dash_R;
         acts_args[i++] = fps;
       }
       if(strlen(drivertype))
       {
-        acts_args[i++] = "-G";
+        acts_args[i++] = dash_G;
         acts_args[i++] = drivertype;
       }
       if(invertp > 0)
-        acts_args[i++] = "-i";
+        acts_args[i++] = dash_i;
       if(strlen(devicepath))
       {
-        acts_args[i++] = "-d";
+        acts_args[i++] = dash_d;
         acts_args[i++] = devicepath;
       }
       if(strlen(channel))
       {
-        acts_args[i++] = "-n";
+        acts_args[i++] = dash_n;
         acts_args[i++] = channel;
       }
       if(strlen(norm))
       {
-        acts_args[i++] = "-V";
+        acts_args[i++] = dash_V;
         acts_args[i++] = norm;
       }
       if(pxc200p > 0)
-        acts_args[i++] = "-x";
+        acts_args[i++] = dash_x;
       if(strlen(brightness))
       {
-        acts_args[i++] = "-B";
+        acts_args[i++] = dash_B;
         acts_args[i++] = brightness;
       }
       if(strlen(contrast))
       {
-        acts_args[i++] = "-C";
+        acts_args[i++] = dash_C;
         acts_args[i++] = contrast;
       }
-      acts_args[i++] = "-W";
+      acts_args[i++] = dash_W;
       acts_args[i++] = widthstring;
-      acts_args[i++] = "-H";
+      acts_args[i++] = dash_H;
       acts_args[i++] = heightstring;
       break;
     case ACTS_VERSION_UNKNOWN:
@@ -610,7 +629,7 @@ Acts::Setup()
   for(int j=0;acts_args[j];j++)
     printf("%s ", acts_args[j]);
   puts("\n");
-  
+
   if(!(pid = fork()))
   {
     // make sure we don't get that "ACTS: Packet" bullshit on the console
@@ -631,7 +650,7 @@ Acts::Setup()
     {
       if(execvp(acts_bin_name,acts_args) == -1)
       {
-        /* 
+        /*
         * some error.  print it here.  it will really be detected
         * later when the parent tries to connect(2) to it
          */
@@ -643,7 +662,7 @@ Acts::Setup()
     {
       if(execv(binarypath,acts_args) == -1)
       {
-        /* 
+        /*
         * some error.  print it here.  it will really be detected
         * later when the parent tries to connect(2) to it
          */
@@ -654,13 +673,33 @@ Acts::Setup()
   }
   else
   {
-    /* in parent */
-    /* fill in addr structure */
-    server.sin_family = PF_INET;
-    /* 
-     * this is okay to do, because gethostbyname(3) does no lookup if the 
+    memset(&server, 0, sizeof server);
+    server.sin_port = htons(portnum);
+    server.sin_family = AF_INET;
+#if HAVE_GETADDRINFO
+    struct addrinfo* addr_ptr = NULL;
+    if (getaddrinfo(host, NULL, NULL, &addr_ptr))
+    {
+      PLAYER_ERROR("getaddrinfo() failed with error");
+      KillACTS();
+      return(1);
+    }
+    assert(addr_ptr);
+    assert(addr_ptr->ai_addr);
+    if ((addr_ptr->ai_addr->sa_family) != AF_INET)
+    {
+      PLAYER_ERROR("unsupported internet address family");
+      KillACTS();
+     return(1);
+    }
+    server.sin_addr.s_addr = (reinterpret_cast<struct sockaddr_in *>(addr_ptr->ai_addr))->sin_addr.s_addr;
+    freeaddrinfo(addr_ptr); addr_ptr = NULL;
+#else
+    /*
+     * this is okay to do, because gethostbyname(3) does no lookup if the
      * 'host' * arg is already an IP addr
      */
+    struct hostent* entp;
     if((entp = gethostbyname(host)) == NULL)
     {
       fprintf(stderr, "Acts::Setup(): \"%s\" is unknown host; "
@@ -671,9 +710,7 @@ Acts::Setup()
     }
 
     memcpy(&server.sin_addr, entp->h_addr_list[0], entp->h_length);
-
-    server.sin_port = htons(portnum);
-
+#endif
     /* ok, we'll make this a bit smarter.  first, we wait a baseline amount
      * of time, then try to connect periodically for some predefined number
      * of times
@@ -682,10 +719,10 @@ Acts::Setup()
 
     for(j = 0;j<ACTS_STARTUP_CONN_LIMIT;j++)
     {
-      /* 
+      /*
        * hook it up
        */
-      
+
       // make a new socket, because connect() screws with the old one somehow
       if((sock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
       {
@@ -693,7 +730,7 @@ Acts::Setup()
         KillACTS();
         return(1);
       }
-      if(connect(sock,(struct sockaddr*)&server, sizeof(server)) == 0)
+      if(connect(sock,reinterpret_cast<struct sockaddr*> (&server), sizeof(server)) == 0)
         break;
       usleep(ACTS_STARTUP_INTERVAL_USEC);
     }
@@ -705,31 +742,18 @@ Acts::Setup()
     }
     puts("Done.");
 
-    /* now spawn reading thread */
-    StartThread();
-
     return(0);
   }
-
-  // shut up compiler!
   return(0);
 }
 
-int
-Acts::Shutdown()
+void Acts::MainQuit()
 {
-  /* if Setup() was never called, don't do anything */
-  if(sock == -1)
-    return(0);
-
-  StopThread();
-
-  sock = -1;
-  puts("ACTS vision server has been shutdown");
-  return(0);
+	sock = -1;
+	puts("ACTS vision server has been shutdown");
 }
 
-void 
+void
 Acts::KillACTS()
 {
   if(kill(pid,SIGKILL) == -1)
@@ -742,10 +766,10 @@ Acts::Main()
   int numread;
   int num_blobs;
   int i;
-  
+
   // we'll convert the data into this structured buffer
   acts_data_t acts_data;
-  
+
   // we'll write the data from this buffer
   player_blobfinder_data_t local_data;
 
@@ -768,11 +792,11 @@ Acts::Main()
     // clean our buffers
     memset(&acts_data,0,sizeof(acts_data));
     memset(&local_data,0,sizeof(local_data));
-    
+
     // put in some stuff that doesnt change
     acts_data.width = htons(this->width);
     acts_data.height = htons(this->height);
-    
+
     /* test if we are supposed to cancel */
     pthread_testcancel();
 
@@ -793,7 +817,7 @@ Acts::Main()
     else if(numread != header_len)
     {
       fprintf(stderr,"RunVisionThread: something went wrong\n"
-              "              expected %d bytes of header, but only got %d\n", 
+              "              expected %d bytes of header, but only got %d\n",
               header_len,numread);
       break;
     }
@@ -805,9 +829,9 @@ Acts::Main()
       {
         // convert 2-byte ACTS 1.0 encoded entries to byte-swapped integers
         // in a structured array
-        acts_data.header[i].index = 
+        acts_data.header[i].index =
           htons(acts_hdr_buf[header_elt_len*i]-1);
-        acts_data.header[i].num = 
+        acts_data.header[i].num =
           htons(acts_hdr_buf[header_elt_len*i+1]-1);
       }
     }
@@ -818,19 +842,19 @@ Acts::Main()
         // convert 4-byte ACTS 1.2/2.0 encoded entries to byte-swapped integers
         // in a structured array
         acts_data.header[i].index = acts_hdr_buf[header_elt_len*i]-1;
-        acts_data.header[i].index = 
+        acts_data.header[i].index =
           acts_data.header[i].index << 6;
-        acts_data.header[i].index |= 
+        acts_data.header[i].index |=
           acts_hdr_buf[header_elt_len*i+1]-1;
-        acts_data.header[i].index = 
+        acts_data.header[i].index =
           htons(acts_data.header[i].index);
 
         acts_data.header[i].num = acts_hdr_buf[header_elt_len*i+2]-1;
-        acts_data.header[i].num = 
+        acts_data.header[i].num =
           acts_data.header[i].num << 6;
-        acts_data.header[i].num |= 
+        acts_data.header[i].num |=
           acts_hdr_buf[header_elt_len*i+3]-1;
-        acts_data.header[i].num = 
+        acts_data.header[i].num =
           htons(acts_data.header[i].num);
       }
     }
@@ -849,11 +873,12 @@ Acts::Main()
     else if(numread != num_blobs*blob_size)
     {
       fprintf(stderr,"RunVisionThread: something went wrong\n"
-              "              expected %d bytes of blob data, but only got %d\n", 
+              "              expected %d bytes of blob data, but only got %d\n",
               num_blobs*blob_size,numread);
       break;
     }
 
+
     if(acts_version == ACTS_VERSION_1_0)
     {
       // convert 10-byte ACTS 1.0 blobs to new byte-swapped structured array
@@ -864,7 +889,7 @@ Acts::Main()
         // TODO: put a descriptive color in here (I'm not sure where
         // to get it from).
         acts_data.blobs[i].color = 0xFF0000;
-        
+
         // get the 4-byte area first
         acts_data.blobs[i].area = 0;
         for(int j=0;j<4;j++)
@@ -903,10 +928,10 @@ Acts::Main()
 
         // Put in a descriptive color.
         if (ch < (int) (sizeof(colors) / sizeof(colors[0])))
-          acts_data.blobs[i].color = colors[ch];            
+          acts_data.blobs[i].color = colors[ch];
         else
           acts_data.blobs[i].color = 0xFF0000;
-        
+
         // get the 4-byte area first
         acts_data.blobs[i].area = 0;
         for(int j=0;j<4;j++)
@@ -914,7 +939,7 @@ Acts::Main()
           acts_data.blobs[i].area = acts_data.blobs[i].area << 6;
           acts_data.blobs[i].area |= acts_blob_buf[tmpptr++] - 1;
         }
-        
+
         // Get the other 2 byte values
         acts_data.blobs[i].x = acts_blob_buf[tmpptr++] - 1;
         acts_data.blobs[i].x = acts_data.blobs[i].x << 6;
@@ -941,12 +966,13 @@ Acts::Main()
         acts_data.blobs[i].bottom |= acts_blob_buf[tmpptr++] - 1;
       }
     }
-    
+
     // Convert data to interface format
     local_data.width = acts_data.width;
     local_data.height = acts_data.height;
     local_data.blobs_count = num_blobs;
-      
+    local_data.blobs = (player_blobfinder_blob_t *)calloc(num_blobs, sizeof(local_data.blobs[0]));
+
     for (i = 0; i < num_blobs; i++)
     {
       src = acts_data.blobs + i;
@@ -963,15 +989,15 @@ Acts::Main()
     }
 
     /* got the data. now fill it in */
-    Publish(device_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_BLOBFINDER_DATA_BLOBS, &local_data, sizeof(local_data) - sizeof(local_data.blobs) +
-            (local_data.blobs_count) * sizeof(local_data.blobs[0]), NULL);
+    Publish(device_addr, PLAYER_MSGTYPE_DATA, PLAYER_BLOBFINDER_DATA_BLOBS, &local_data);
+    free(local_data.blobs);
   }
 
   pthread_cleanup_pop(1);
   pthread_exit(NULL);
 }
 
-void 
+void
 QuitACTS(void* visiondevice)
 {
   char acts_request_quit = ACTS_REQUEST_QUIT;
