@@ -1,7 +1,7 @@
 /*
  *  Player - One Hell of a Robot Server
  *  Copyright (C) 2000  Brian Gerkey et al
- *                      gerkey@usc.edu    
+ *                      gerkey@usc.edu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 // Desc: Driver for detecting simple shapes in a camera image
 // Author: Andrew Howard
 // Date: 15 Feb 2004
-// CVS: $Id: simpleshape.cc,v 1.4.2.1 2006/06/07 16:12:39 gerkey Exp $
+// CVS: $Id: simpleshape.cc 8230 2009-08-26 08:36:23Z gbiggs $
 //
 ///////////////////////////////////////////////////////////////////////////
 
@@ -76,11 +76,11 @@ form of a binary image (such as the one shown below).
   - Filename of the model image file.  This should by a binary,
     grayscale image.
 
-- canny_thresh (float tuple) 
+- canny_thresh (float tuple)
   - Default: [40 20]
   - Thresholds for the Canny edge detector.
 
-- match_thresh (float tuple) 
+- match_thresh (float tuple)
   - Default: [0.50 20.0 0.20]
   - Match thresholds (?)
 
@@ -100,10 +100,14 @@ driver
 */
 /** @} */
 
+#include <stddef.h>
+#include <stdlib.h>
+#include <assert.h>
+#include <string.h>
 #include "../../base/imagebase.h"
 
-#include <opencv/cv.h>
-#include <opencv/highgui.h>
+#include <cv.h>
+#include <highgui.h>
 
 // Invariant feature set for a contour
 class FeatureSet
@@ -124,7 +128,7 @@ class FeatureSet
 
 
 // Info on potential shapes.
-class Shape 
+class Shape
 {
   // Id (-1) if undetermined.
   public: int id;
@@ -137,13 +141,13 @@ class Shape
 // Driver for detecting laser retro-reflectors.
 class SimpleShape : public ImageBase
 {
-  public: 
+  public:
     // Constructor
     SimpleShape( ConfigFile* cf, int section);
 
     // Setup/shutdown routines.
-    virtual int Setup();
-    virtual int Shutdown();
+    virtual int MainSetup();
+    virtual void MainQuit();
 
   // Load a shape model
   private: int LoadModel();
@@ -163,6 +167,8 @@ class SimpleShape : public ImageBase
   // Write the outgoing blobfinder data
   private: void WriteBlobfinderData();
 
+  private: void WriteCameraData();
+
   // Model data (this is the shape to search for)
   private: const char *modelFilename;
   private: CvMemStorage *modelStorage;
@@ -182,6 +188,11 @@ class SimpleShape : public ImageBase
   // List of potential shapes
   private: Shape shapes[256];
   private: unsigned int shapeCount;
+
+  private: player_devaddr_t blobfinder_addr;
+  private: player_devaddr_t debugcam_addr;
+
+  private: bool debugcam;
 };
 
 
@@ -193,7 +204,7 @@ Driver* SimpleShape_Init( ConfigFile* cf, int section)
 
 
 // a driver registration function
-void SimpleShape_Register(DriverTable* table)
+void simpleshape_Register(DriverTable* table)
 {
   table->AddDriver("simpleshape", SimpleShape_Init);
 }
@@ -202,11 +213,36 @@ void SimpleShape_Register(DriverTable* table)
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 SimpleShape::SimpleShape( ConfigFile* cf, int section)
-	: ImageBase(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_BLOBFINDER_CODE)
+	: ImageBase(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN)
 {
   this->inpImage = NULL;
   this->outImage = NULL;
   this->workImage = NULL;
+  memset(&(this->blobfinder_addr), 0, sizeof blobfinder_addr);
+  memset(&(this->debugcam_addr), 0, sizeof debugcam_addr);
+  this->debugcam = true;
+
+  if (cf->ReadDeviceAddr(&(this->blobfinder_addr), section, "provides",
+                           PLAYER_BLOBFINDER_CODE, -1, NULL))
+  {
+    this->SetError(-1);
+    return;
+  }
+  if (this->AddInterface(this->blobfinder_addr))
+  {
+    this->SetError(-1);
+    return;
+  }
+  if (cf->ReadDeviceAddr(&(this->debugcam_addr), section, "provides",
+                           PLAYER_CAMERA_CODE, -1, NULL))
+  {
+    PLAYER_WARN("debug preview will not be available");
+    this->debugcam = false;
+  } else if (this->AddInterface(this->debugcam_addr))
+  {
+    this->SetError(-1);
+    return;
+  }
 
   // Filename for the target shape image
   this->modelFilename = cf->ReadFilename(section, "model", NULL);
@@ -217,22 +253,21 @@ SimpleShape::SimpleShape( ConfigFile* cf, int section)
 
   this->matchThresh[0] = cf->ReadTupleFloat(section, "match_thresh", 0, 0.50);
   this->matchThresh[1] = cf->ReadTupleFloat(section, "match_thresh", 1, 20.0);
-  this->matchThresh[2] = cf->ReadTupleFloat(section, "match_thresh", 2, 0.20);  
+  this->matchThresh[2] = cf->ReadTupleFloat(section, "match_thresh", 2, 0.20);
 
   this->shapeCount = 0;
-  return;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device (called by server thread).
-int SimpleShape::Setup()
+int SimpleShape::MainSetup()
 {
   // Load the shape model
   if (this->LoadModel() != 0)
     return -1;
 
-  return ImageBase::Setup();
+  return ImageBase::MainSetup();
 
   return 0;
 }
@@ -240,17 +275,15 @@ int SimpleShape::Setup()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the device (called by server thread).
-int SimpleShape::Shutdown()
+void SimpleShape::MainQuit()
 {
-  ImageBase::Shutdown();
-  
+  ImageBase::MainQuit();
+
   // Free images
   if (this->inpImage)
     cvReleaseImage(&(this->inpImage));
   if (this->outImage)
     cvReleaseImage(&(this->outImage));
-
-  return 0;
 }
 
 
@@ -280,9 +313,9 @@ int SimpleShape::LoadModel()
 
   // Extract contours
   this->modelStorage = cvCreateMemStorage(0);
-  cvFindContours(work, this->modelStorage, &contour, sizeof(CvContour), 
+  cvFindContours(work, this->modelStorage, &contour, sizeof(CvContour),
                  CV_RETR_LIST, CV_CHAIN_APPROX_NONE);
-  
+
   // Find the contour with the largest area (we will use the outer
   // contour only)
   maxArea = 0;
@@ -320,18 +353,20 @@ int SimpleShape::LoadModel()
 int SimpleShape::ProcessFrame()
 {
   CvSize size;
-  int width, height;
-  
+  int i, j, width, height, depth;
+  unsigned char * row1;
+  unsigned char * row2;
+
   width = this->stored_data.width;
   height = this->stored_data.height;
   assert(width > 0 && height > 0);
-    
+
   // Create input and output image if it doesnt exist
   size = cvSize(width, height);
   if (this->inpImage == NULL)
     this->inpImage = cvCreateImage(size, IPL_DEPTH_8U, 1);
   size = cvSize(2 * width, 2 * height);
-  if (this->outImage == NULL)
+  if ((this->outImage == NULL) && (this->debugcam))
   {
     this->outImage = cvCreateImage(size, IPL_DEPTH_8U, 1);
     cvGetSubRect(this->outImage, this->outSubImages + 0, cvRect(0, 0, width, height));
@@ -343,26 +378,38 @@ int SimpleShape::ProcessFrame()
   // Create a main image and copy in the pixels
   switch (this->stored_data.format)
   {
-    case PLAYER_CAMERA_FORMAT_MONO8:
+  case PLAYER_CAMERA_FORMAT_MONO8:
+    // Copy pixels to input image (grayscale)
+    assert(this->inpImage->imageSize >= static_cast<int>(this->stored_data.image_count));
+    memcpy(this->inpImage->imageData, this->stored_data.image, this->inpImage->imageSize);
+    break;
+  case PLAYER_CAMERA_FORMAT_RGB888:
+    depth = this->stored_data.bpp / 8;
+    assert((depth == 4) || (depth == 3));
+    assert(this->inpImage->imageSize >= static_cast<int>((this->stored_data.image_count) / depth));
+    for (i = 0; i < height; i++)
     {
-      // Copy pixels to input image (grayscale)
-      assert(this->inpImage->imageSize >= (int) this->stored_data.image_count);
-      memcpy(this->inpImage->imageData, this->stored_data.image, this->inpImage->imageSize);
-      break;
-    }
-    default:
-    {
-      PLAYER_WARN1("image format [%d] is not supported", this->stored_data.format);
-      return -1;
+      row1 = (reinterpret_cast<unsigned char *>(this->stored_data.image)) + (i * width * depth);
+      row2 = (reinterpret_cast<unsigned char *>(this->inpImage->imageData)) + (i * width);
+      for (j = 0; j < width; j++)
+      {
+        row2[0] = static_cast<unsigned char>((static_cast<double>(row1[0]) * 0.3) + (static_cast<double>(row1[1]) * 0.59) + (static_cast<double>(row1[2]) * 0.11));
+        row1 += depth;
+        row2++;
+      }
     }
+    break;
+  default:
+    PLAYER_WARN1("image format [%d] is not supported", this->stored_data.format);
+    return -1;
   }
 
   // Copy original image to output
-/*  if (this->out_camera_id.port)
+  if (this->debugcam)
   {
     cvSetZero(this->outImage);
     cvCopy(this->inpImage, this->outSubImages + 0);
-  }*/
+  }
 
   // Clone the input image to our workspace
   this->workImage = cvCloneImage(this->inpImage);
@@ -374,8 +421,9 @@ int SimpleShape::ProcessFrame()
   cvReleaseImage(&this->workImage);
   this->workImage = NULL;
 
-  WriteBlobfinderData();
-    
+  this->WriteBlobfinderData();
+  if (this->debugcam) this->WriteCameraData();
+
   return 0;
 }
 
@@ -399,14 +447,14 @@ void SimpleShape::FindShapes()
   cvCanny(this->workImage, this->workImage, this->cannyThresh1, this->cannyThresh2);
 
   // Copy edges to output image
-/*  if (this->out_camera_id.port)
-    cvCopy(this->workImage, this->outSubImages + 1);*/
+  if (this->debugcam)
+    cvCopy(this->workImage, this->outSubImages + 1);
 
   // Find contours on a binary image
   storage = cvCreateMemStorage(0);
-  cvFindContours(this->workImage, storage, &contour, sizeof(CvContour), 
+  cvFindContours(this->workImage, storage, &contour, sizeof(CvContour),
                  CV_RETR_LIST, CV_CHAIN_APPROX_NONE);
-    
+
   for(; contour != NULL; contour = contour->h_next)
   {
     rect = cvBoundingRect(contour);
@@ -424,11 +472,11 @@ void SimpleShape::FindShapes()
     if (rect.y + rect.height >= this->workImage->height - 5)
       continue;
 
-    
+
     // Draw eligable contour on the output image; useful for debugging
-/*    if (this->out_camera_id.port)
+    if (this->debugcam)
       cvDrawContours(this->outSubImages + 2, contour, CV_RGB(255, 255, 255),
-                     CV_RGB(255, 255, 255), 0, 1, 8);*/
+                     CV_RGB(255, 255, 255), 0, 1, 8);
 
     // Compute the contour features
     this->ExtractFeatureSet((CvContour*) contour, &featureSet);
@@ -439,21 +487,21 @@ void SimpleShape::FindShapes()
       continue;
 
     // Draw contour on the main image; useful for debugging
-/*    if (this->out_camera_id.port)
+    if (this->debugcam)
     {
       cvDrawContours(this->outSubImages + 3, contour, CV_RGB(128, 128, 128),
                      CV_RGB(128, 128, 128), 0, 1, 8);
       cvRectangle(this->outSubImages + 3, cvPoint(rect.x, rect.y),
                   cvPoint(rect.x + rect.width, rect.y + rect.height), CV_RGB(255, 255, 255), 1);
-    }*/
+    }
 
     // Check for overrun
-    if (this->shapeCount >= sizeof(this->shapes) / sizeof(this->shapes[0]))
+    if (this->shapeCount >= (sizeof(this->shapes) / sizeof(class Shape)))
     {
       PLAYER_WARN("image contains too many shapes");
       break;
     }
-    
+
     // Add the shape to our internal list
     shape = this->shapes + this->shapeCount++;
     shape->id = -1;
@@ -480,20 +528,18 @@ void SimpleShape::ExtractFeatureSet(CvContour *contour, FeatureSet *feature)
   double aa, bb;
   double dx, dy;
   double var;
-  
+
   // Get the moments (we will use the Hu invariants)
   cvMoments(contour, &feature->moments);
 
   // Compute the compactness measure: perimeter squared divided by
-  // area.  
+  // area.
   rect = cvBoundingRect(contour);
   feature->compact = (contour->total * contour->total) / fabs(cvContourArea(contour));
 
   // Compute elliptical variance
   box = cvFitEllipse2(contour);
 
-  //printf("%f %f %f %f\n", box.center.x, box.center.y, box.size.width / 2, box.size.height / 2);
-
   aa = box.size.width * box.size.width / 4;
   bb = box.size.height * box.size.height / 4;
 
@@ -517,7 +563,7 @@ void SimpleShape::ExtractFeatureSet(CvContour *contour, FeatureSet *feature)
   CvPoint *a, *b, *c;
   double ax, ay, bx, by, cx, cy;
   double d, n, m;
-  
+
   // Construct a string describing the polygon (used for syntactic
   // matching)
   for (i = 0; i < poly->total; i++)
@@ -535,7 +581,7 @@ void SimpleShape::ExtractFeatureSet(CvContour *contour, FeatureSet *feature)
 
     bx = -ay;
     by = +ax;
-    
+
     cx = c->x - b->x;
     cy = c->y - b->y;
     d = sqrt(cx * cx + cy * cy);
@@ -563,18 +609,18 @@ int SimpleShape::MatchFeatureSet(FeatureSet *a, FeatureSet *b)
   int i, j;
   int sim, minSim;
   int na, nb;
-  
+
   if (a->vertexCount != b->vertexCount)
     return INT_MAX;
 
   minSim = INT_MAX;
-  
+
   // Look for the lowest dissimalarity by trying all possible
   // string shifts
   for (i = 0; i < a->vertexCount; i++)
   {
     sim = 0;
-    
+
     for (j = 0; j < a->vertexCount; j++)
     {
       na = a->vertexString[j];
@@ -604,7 +650,7 @@ void SimpleShape::WriteBlobfinderData()
   data.height = (this->stored_data.height);
 
   data.blobs_count = (this->shapeCount);
-    
+  data.blobs = (player_blobfinder_blob_t*)calloc(shapeCount,sizeof(data.blobs[0]));
   for (i = 0; i < this->shapeCount; i++)
   {
     shape = this->shapes + i;
@@ -623,11 +669,11 @@ void SimpleShape::WriteBlobfinderData()
   }
 
   // Copy data to server.
-  Publish(device_addr,NULL,PLAYER_MSGTYPE_DATA,PLAYER_BLOBFINDER_DATA_BLOBS,&data,sizeof(data));
-  // Copy data to server
-/*  size = sizeof(data) - sizeof(data.blobs) + this->shapeCount * sizeof(data.blobs[0]);
-  this->PutMsg(this->blobfinder_id, NULL, PLAYER_MSGTYPE_DATA, 0, &data, size, &this->cameraTime);*/
-  
+  Publish(blobfinder_addr,PLAYER_MSGTYPE_DATA,PLAYER_BLOBFINDER_DATA_BLOBS,&data);
+  if (data.blobs) free(data.blobs);
+  data.blobs = NULL;
+  data.blobs_count = 0;
+
   return;
 }
 
@@ -635,34 +681,32 @@ void SimpleShape::WriteBlobfinderData()
 ////////////////////////////////////////////////////////////////////////////////
 // Write camera data; this is a little bit naughty: we re-use the
 // input camera data, but modify the pixels
-/*void SimpleShape::WriteCameraData()
+void SimpleShape::WriteCameraData()
 {
-  size_t size;
-  
-  if (this->camera_id.port == 0)
-    return;
+  player_camera_data_t * outCameraData;
+
   if (this->outImage == NULL)
     return;
 
+  outCameraData = reinterpret_cast<player_camera_data_t *>(malloc(sizeof(player_camera_data_t)));
+  assert(outCameraData);
   // Do some byte swapping
-  this->outCameraData.width = htons(this->outImage->width);
-  this->outCameraData.height = htons(this->outImage->height);
-  this->outCameraData.bpp = 8;
-  this->outCameraData.format = PLAYER_CAMERA_FORMAT_MONO8;
-  this->outCameraData.compression = PLAYER_CAMERA_COMPRESS_RAW;
-  this->outCameraData.image_size = htonl(this->outImage->imageSize);
-
-  // Copy in the pixels
-  memcpy(this->outCameraData.image, this->outImage->imageData, this->outImage->imageSize);
-
-  // Compute message size
-  size = sizeof(this->outCameraData) - sizeof(this->outCameraData.image)
-    + this->outImage->imageSize;
-
-  // Copy data to server
-  this->PutMsg(this->out_camera_id, NULL, PLAYER_MSGTYPE_DATA, 0, &this->outCameraData, size, &this->cameraTime);
-  
-  return;
-}*/
-
-
+  outCameraData->width = this->outImage->width;
+  outCameraData->height = this->outImage->height;
+  outCameraData->bpp = 8;
+  outCameraData->format = PLAYER_CAMERA_FORMAT_MONO8;
+  outCameraData->compression = PLAYER_CAMERA_COMPRESS_RAW;
+  outCameraData->fdiv = 0;
+  outCameraData->image_count = this->outImage->imageSize;
+  outCameraData->image = NULL;
+  if (outCameraData->image_count)
+  {
+    outCameraData->image = reinterpret_cast<uint8_t *>(malloc(outCameraData->image_count));
+    // Copy in the pixels
+    memcpy(outCameraData->image, this->outImage->imageData, outCameraData->image_count);
+  }
+  Publish(this->device_addr,
+          PLAYER_MSGTYPE_DATA, PLAYER_CAMERA_DATA_STATE,
+          reinterpret_cast<void *>(outCameraData),
+          0, NULL, false); // this call should also free outCameraData and outCameraData->image
+}
