@@ -147,9 +147,7 @@ driver
 // ONLY if you need something that was #define'd as a result of configure
 // (e.g., HAVE_CFMAKERAW), then #include <config.h>, like so:
 /*
-#if HAVE_CONFIG_H
-  #include <config.h>
-#endif
+#include <config.h>
 */
 #include "sphere_mixed.h"
 
@@ -191,7 +189,7 @@ SphereDriver_Init(ConfigFile* cf, int section)
 // that it can be invoked without object context.  In this function, we add
 // the driver into the given driver table, indicating which interface the
 // driver can support and how to create a driver instance.
-void SphereDriver_Register(DriverTable* table)
+void sphere_Register(DriverTable* table)
 {
   table->AddDriver("sphere", SphereDriver_Init);
 }
@@ -200,7 +198,7 @@ void SphereDriver_Register(DriverTable* table)
 // Constructor.  Retrieve options from the configuration file and do any
 // pre-Setup() setup.
 SphereDriver::SphereDriver(ConfigFile* cf, int section)
-    : Driver(cf, section),
+    : ThreadedDriver(cf, section),
     mPanMin(0),
     mPanMax(1),
     mTiltMin(0),
@@ -275,7 +273,7 @@ SphereDriver::SphereDriver(ConfigFile* cf, int section)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device.  Return 0 if things go well, and -1 otherwise.
-int SphereDriver::Setup()
+int SphereDriver::MainSetup()
 {
   puts("Setting up sphere driver");
   mFg = fg_open(mDevice);
@@ -355,26 +353,18 @@ int SphereDriver::Setup()
 
   puts("Sphere driver ready");
 
-  // Start the device thread; spawns a new thread and executes
-  // SphereDriver::Main(), which contains the main loop for the driver.
-  StartThread();
-
   return(0);
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the device
-int SphereDriver::Shutdown()
+void SphereDriver::MainQuit()
 {
   puts("Shutting Sphere driver down");
 
-  // Stop and join the driver thread
-  StopThread();
-
   // Set to 0,0
-  set_pan_or_tilt(mFg->fd, SET_PAN, 0);
-  set_pan_or_tilt(mFg->fd, SET_TILT, 0);
+  set_pan_and_tilt(mFg->fd, 0, 0);
 
   // Free resources
   if (mFrame != NULL)
@@ -384,12 +374,7 @@ int SphereDriver::Shutdown()
   if (mFg != NULL)
     fg_close(mFg);
 
-  // Here you would shut the device down by, for example, closing a
-  // serial port.
-
   puts("Sphere driver has been shutdown");
-
-  return(0);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -417,13 +402,11 @@ void SphereDriver::Main()
 }
 
 // Process an incoming message
-int SphereDriver::ProcessMessage(MessageQueue* resp_queue,
+int SphereDriver::ProcessMessage(QueuePointer & resp_queue,
                                  player_msghdr* hdr,
                                  void* data)
 {
-  assert(resp_queue);
   assert(hdr);
-  assert(data);
 
   if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
                            PLAYER_PTZ_CMD_STATE, mPtzAddr))
@@ -432,6 +415,22 @@ int SphereDriver::ProcessMessage(MessageQueue* resp_queue,
     ProcessCommand(hdr, *reinterpret_cast<player_ptz_cmd_t *>(data));
     return(0);
   }
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ,
+                           PLAYER_PTZ_REQ_STATUS, mPtzAddr))
+  {
+    player_ptz_req_status_t ptz_status;
+    int status;
+
+    query_pan_tilt_status(mFg->fd, &status);
+    ptz_status.status = (uint32_t) status;
+
+    Publish(device_addr, resp_queue,
+      PLAYER_MSGTYPE_RESP_ACK,
+      PLAYER_PTZ_REQ_STATUS,
+      (void*)&status
+    );
+    return(0);
+  }
   else
   {
     PLAYER_ERROR1("SphereDriver received unknown message: %s", hdr->type);
@@ -460,15 +459,14 @@ void SphereDriver::ProcessCommand(player_msghdr_t* hdr, player_ptz_cmd_t &data)
   mPtzData.tiltspeed = -1;
 
   // pan is inverted
-  set_pan_or_tilt(mFg->fd, SET_PAN,  -pan);
-  set_pan_or_tilt(mFg->fd, SET_TILT, tilt);
+  set_pan_and_tilt(mFg->fd, -pan, tilt);
 }
 
 void SphereDriver::RefreshData()
 {
   // Grab the next frame (blocking)
   fg_grab_frame(mFg, mFrame);
-  size_t image_size, size;
+  size_t image_size;
 
   // Compute size of image
   image_size = mWidth * mHeight * mDepth / 8;
@@ -478,13 +476,12 @@ void SphereDriver::RefreshData()
   mCameraData.bpp    = mDepth;
   mCameraData.compression = PLAYER_CAMERA_COMPRESS_RAW;
   mCameraData.image_count = image_size;
+  mCameraData.image = new unsigned char [image_size];
 
-  assert(image_size <= sizeof(mCameraData.image));
 
   // Copy the image pixels
   if (0 == strcasecmp(mPalette, "GREY"))
   {
-    assert(image_size <= (size_t) mFrame->size);
     memcpy(mCameraData.image, mFrame->data, image_size);
   }
   else if (0 == strcasecmp(mPalette, "YUV420P"))
@@ -502,16 +499,13 @@ void SphereDriver::RefreshData()
     memcpy(mCameraData.image, mRGBFrame->data, image_size);
   }
 
-  // Copy data to server
-  size = sizeof(mCameraData) - sizeof(mCameraData.image) + image_size;
-
-  Publish(mCameraAddr, NULL,
+  Publish(mCameraAddr,
           PLAYER_MSGTYPE_DATA, PLAYER_CAMERA_DATA_STATE,
-          reinterpret_cast<void*>(&mCameraData), size, NULL);
+          reinterpret_cast<void*>(&mCameraData));
 
-  Publish(mPtzAddr, NULL,
+  Publish(mPtzAddr,
           PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE,
-          reinterpret_cast<void*>(&mPtzData), sizeof(mPtzData), NULL);
+          reinterpret_cast<void*>(&mPtzData));
 
   // Save frames
   if (FALSE!=mSave)
@@ -520,6 +514,7 @@ void SphereDriver::RefreshData()
     snprintf(filename, sizeof(filename), "click-%04d.ppm", mFrameNumber++);
     frame_save(mFrame, filename);
   }
+  delete [] mCameraData.image;
 
 }
 
