@@ -50,7 +50,7 @@
 //  Requires:       1.  The Video4Linux subsystem to be properly initialised
 //                  2.  Either a valid device name (such as "/dev/bttv") or
 //                      NULL to use the default.
-//                  
+//
 //  Promises:       1.  To open the video device
 //                  2.  Initialise the video device to reasonable defaults
 //                  3.  To not leak any memory by freeing upon any error
@@ -161,7 +161,7 @@ FRAMEGRABBER* fg_open( const char* dev )
     free( fg );
     return NULL;
   }
-  
+
   // set the default max_buffer
   fg->max_buffer = fg->mbuf.frames;
 
@@ -189,7 +189,7 @@ FRAMEGRABBER* fg_open( const char* dev )
 
 void fg_close(FRAMEGRABBER* fg)
 {
-    munmap( fg->mb_map, fg->mbuf.size);     
+    munmap( fg->mb_map, fg->mbuf.size);
     close( fg->fd );
 
     // Make sure we free all memory (backwards!)
@@ -380,6 +380,9 @@ FRAME* fg_grab( FRAMEGRABBER* fg )
 FRAME* fg_grab_frame( FRAMEGRABBER* fg, FRAME* fr )
 {
     int capture_frame = fg->cur_frame + 1;
+    int i;
+    int insize, count;
+    unsigned char * buf;
 
     //----------------------
     // Very first time only
@@ -412,7 +415,7 @@ FRAME* fg_grab_frame( FRAMEGRABBER* fg, FRAME* fr )
     {
         capture_frame = 0;
     }
-    
+
     // Set up capture parameters
     fg->mmap.format = fg->picture.palette;
     fg->mmap.frame  = capture_frame;
@@ -437,10 +440,28 @@ FRAME* fg_grab_frame( FRAMEGRABBER* fg, FRAME* fr )
         return NULL;
     }
 
-    // Save video buffer into our own memory
-    memcpy( fr->data,
-            fg->mb_map + fg->mbuf.offsets[fg->cur_frame],
-            frame_get_size( fr ) );
+    if ((fg->picture.palette) == VIDEO_PALETTE_JPEG)
+    {
+        buf = ((unsigned char *)(fg->mb_map)) + fg->mbuf.offsets[fg->cur_frame];
+	insize = frame_get_size(fr) - sizeof(int);
+	count = insize - 1;
+	for (i = 1024; i < count; i++)
+	{
+	    if (buf[i] == 0xff) if (buf[i + 1] == 0xd9)
+	    {
+		insize = i + 10;
+		break;
+	    }
+	}
+	memcpy(fr->data, &insize, sizeof(int));
+	memcpy(((unsigned char *)(fr->data)) + sizeof(int), buf, insize);
+    } else
+    {
+	// Save video buffer into our own memory
+	memcpy( fr->data,
+        	((unsigned char *)(fg->mb_map)) + fg->mbuf.offsets[fg->cur_frame],
+        	frame_get_size( fr ) );
+    }
 
     // Move along to the next one
     fg->cur_frame = capture_frame;
@@ -456,18 +477,42 @@ FRAME* fg_grab_read( FRAMEGRABBER* fg )
                            fg->window.height,
                            fg->picture.palette );
 
-    read( fg->fd,
+    int ret = read( fg->fd,
           fr->data,
           frame_get_size( fr ) );
 
+    if (ret < 0)
+    {
+    	fprintf(stderr,"Failed to read from frame grabber");
+    	return NULL;
+    }
+
     return fr;
 }
 
+int fg_read(FRAMEGRABBER * fg, FRAME * fr)
+{
+    int count;
+
+    if ((fg->picture.palette) == VIDEO_PALETTE_JPEG)
+    {
+	count = read(fg->fd, ((unsigned char *)(fr->data)) + sizeof(int), frame_get_size(fr) - sizeof(int));
+	memcpy(fr->data, &count, sizeof(int));
+    } else count = read(fg->fd, fr->data, frame_get_size(fr));
+    return count;
+}
+
 //--------------------------------------------------------------------------
 
 int fg_set_brightness( FRAMEGRABBER* fg, int br )
 {
-    fg->picture.brightness = FG_PERCENT( br );
+    if ( ioctl( fg->fd, VIDIOCGPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_brightness(): get attribute failed" );
+        return -1;
+    }
+
+    fg->picture.brightness = br;
 
     if ( ioctl( fg->fd, VIDIOCSPICT, &(fg->picture) ) < 0 )
     {
@@ -480,9 +525,57 @@ int fg_set_brightness( FRAMEGRABBER* fg, int br )
 
 //--------------------------------------------------------------------------
 
+int fg_set_hue( FRAMEGRABBER* fg, int hue )
+{
+    if ( ioctl( fg->fd, VIDIOCGPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_hue(): get attribute failed" );
+        return -1;
+    }
+
+    fg->picture.hue = hue;
+
+    if ( ioctl( fg->fd, VIDIOCSPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_hue(): set attribute failed" );
+        return -1;
+    }
+
+    return 0;
+}
+
+//--------------------------------------------------------------------------
+
+int fg_set_colour( FRAMEGRABBER* fg, int clr )
+{
+    if ( ioctl( fg->fd, VIDIOCGPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_colour(): get attribute failed" );
+        return -1;
+    }
+
+    fg->picture.colour = clr;
+
+    if ( ioctl( fg->fd, VIDIOCSPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_colour(): set attribute failed" );
+        return -1;
+    }
+
+    return 0;
+}
+
+//--------------------------------------------------------------------------
+
 int fg_set_contrast( FRAMEGRABBER* fg, int ct )
 {
-    fg->picture.contrast = FG_PERCENT( ct );
+    if ( ioctl( fg->fd, VIDIOCGPICT, &(fg->picture) ) < 0 )
+    {
+        perror( "fg_set_contrast(): get attribute failed" );
+        return -1;
+    }
+
+    fg->picture.contrast = ct;
 
     if ( ioctl( fg->fd, VIDIOCSPICT, &(fg->picture) ) < 0 )
     {
@@ -598,7 +691,7 @@ void fg_dump_info(FRAMEGRABBER* fg)
   // mmap
 
   // Dump out frame buffer setup
-	printf( "  fbuffer.base         = 0x%08x\n", (int)fg->fbuffer.base );
+	printf( "  fbuffer.base         = 0x%p\n", fg->fbuffer.base );
   printf( "  fbuffer.width        = %u\n", fg->fbuffer.width );
 	printf( "  fbuffer.height       = %u\n", fg->fbuffer.height );
 	printf( "  fbuffer.depth        = %u\n", fg->fbuffer.depth );
@@ -612,7 +705,7 @@ void fg_dump_info(FRAMEGRABBER* fg)
     printf( "  mbuf.offsets[%u] = %u\n", i, fg->mbuf.offsets[i] );
   }
 
-  printf( "  mb_map      = 0x%08x\n", (int)(fg->mb_map) );
+  printf( "  mb_map      = 0x%p\n", (fg->mb_map) );
 }
 
 //==========================================================================
