@@ -23,7 +23,7 @@
 // Desc: Driver for detecting combined laser/visual barcodes.
 // Author: Andrew Howard
 // Date: 17 Aug 2002
-// CVS: $Id: laservisualbarcode.cc,v 1.11.2.2 2007/03/01 21:33:16 gerkey Exp $
+// CVS: $Id: laservisualbarcode.cc 7305 2009-01-27 01:18:55Z gbiggs $
 //
 // Theory of operation:
 //   Parses a laser scan to find the retro-reflective patches (lines or
@@ -87,7 +87,7 @@ driver_laserbarcode drivers.
 
 @par Configuration requests
 
-- PLAYER_FIDUCIAL_GET_GEOM
+- PLAYER_FIDUCIAL_REQ_GET_GEOM
 
 @par Configuration file options
 
@@ -129,8 +129,6 @@ driver
 #include <string.h>
 #include <math.h>
 #include <stdlib.h>       // for atoi(3)
-#include <netinet/in.h>   // for htons(3)
-#include <unistd.h>
 
 #include <libplayercore/playercore.h>
 
@@ -144,8 +142,8 @@ class LaserVisualBarcode : public Driver
   public: virtual int Setup();
   public: virtual int Shutdown();
 
-  // Process incoming messages from clients 
-  int ProcessMessage (MessageQueue * resp_queue, player_msghdr * hdr, void * data);
+  // Process incoming messages from clients
+  int ProcessMessage (QueuePointer &resp_queue, player_msghdr * hdr, void * data);
 
   // Info on potential fiducials.
   private: struct fiducial_t
@@ -176,7 +174,7 @@ class LaserVisualBarcode : public Driver
   {
     // Blob channel.
     int ch;
-  
+
     // Blob position in image.
     int x, y;
   };
@@ -187,7 +185,7 @@ class LaserVisualBarcode : public Driver
 
   // Analyze the laser data to find fidicuials (reflectors).
   private: void FindLaserFiducials(double time, player_laser_data_t *data);
-    
+
   // Find the line of best fit for the given segment of the laser
   // scan.  Fills in the pose of the reflector relative to the laser.
   private: void FitLaserFiducial(player_laser_data_t *data, int first, int last, double pose[3]);
@@ -217,10 +215,10 @@ class LaserVisualBarcode : public Driver
   // Search the list of valid blobs to locate the visual fiducials.
   private: int FindVisualFiducials(double time, player_blobfinder_data_t *data,
                                    int depth, blob_t *prevblob);
-    
+
   // Update the device data (the data going back to the client).
   private: void UpdateData();
-  
+
   // Fiducial properties.
   private: int barcount;
   private: double barwidth, barheight;
@@ -263,9 +261,10 @@ class LaserVisualBarcode : public Driver
   // List of current valid blobs.
   private: int blob_count;
   private: blob_t blobs[256];
-  
+
   // Local copy of the current fiducial data.
   private: player_fiducial_data_t fdata;
+  int fdata_allocated;
 };
 
 
@@ -277,7 +276,7 @@ Driver* LaserVisualBarcode_Init( ConfigFile* cf, int section)
 
 
 // a driver registration function
-void LaserVisualBarcode_Register(DriverTable* table)
+void laservisualbarcode_Register(DriverTable* table)
 {
   table->AddDriver("laservisualbarcode", LaserVisualBarcode_Init);
 }
@@ -292,7 +291,7 @@ LaserVisualBarcode::LaserVisualBarcode( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->laser_id, section, "requires",
                        PLAYER_LASER_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->laser = NULL;
@@ -302,7 +301,7 @@ LaserVisualBarcode::LaserVisualBarcode( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->ptz_id, section, "requires",
                        PLAYER_PTZ_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->ptz = NULL;
@@ -312,7 +311,7 @@ LaserVisualBarcode::LaserVisualBarcode( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->blobfinder_id, section, "requires",
                        PLAYER_BLOBFINDER_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->blobfinder = NULL;
@@ -321,7 +320,7 @@ LaserVisualBarcode::LaserVisualBarcode( ConfigFile* cf, int section)
   this->max_ptz_attention = cf->ReadFloat(section, "max_ptz_attention", 2.0);
   this->retire_time = cf->ReadFloat(section, "retire_time", 1.0);
   this->max_dist = cf->ReadFloat(section, "max_dist", 0.2);
-  
+
   // Default fiducial properties.
   this->barcount = cf->ReadInt(section, "bit_count", 3);
   this->barwidth = cf->ReadLength(section, "bit_width", 0.08);
@@ -344,7 +343,9 @@ LaserVisualBarcode::LaserVisualBarcode( ConfigFile* cf, int section)
 // Set up the device (called by server thread).
 int LaserVisualBarcode::Setup()
 {
-  
+  fdata_allocated = 0;
+  fdata.fiducials = NULL;
+
   // Subscribe to the laser.
   if (!(laser = deviceTable->GetDevice (laser_id)))
   {
@@ -397,6 +398,8 @@ int LaserVisualBarcode::Shutdown()
   ptz->Unsubscribe(InQueue);
   blobfinder->Unsubscribe(InQueue);
 
+  free(fdata.fiducials);
+
   return 0;
 }
 
@@ -404,14 +407,12 @@ int LaserVisualBarcode::Shutdown()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Process an incoming message
-int LaserVisualBarcode::ProcessMessage (MessageQueue * resp_queue, player_msghdr * hdr, void * data)
+int LaserVisualBarcode::ProcessMessage (QueuePointer &resp_queue, player_msghdr * hdr, void * data)
 {
   assert(hdr);
-  assert(data);
-  
+
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_LASER_DATA_SCAN, laser_id))
   {
-    assert(hdr->size == sizeof(player_laser_data_t));
     player_laser_data_t * l_data = reinterpret_cast<player_laser_data_t * > (data);
 
     UpdateLaser(l_data, hdr->timestamp);
@@ -421,40 +422,17 @@ int LaserVisualBarcode::ProcessMessage (MessageQueue * resp_queue, player_msghdr
 
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, ptz_id))
   {
-    assert(hdr->size == sizeof(player_ptz_data_t));
     UpdatePtz(reinterpret_cast<player_ptz_data_t * > (data), hdr->timestamp);
     return 0;
   }
 
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_BLOBFINDER_DATA_BLOBS, blobfinder_id))
   {
-    assert(hdr->size == sizeof(player_blobfinder_data_t));
     UpdateBlobfinder(reinterpret_cast<player_blobfinder_data_t * > (data), hdr->timestamp);
     return 0;
   }
 
-/* 
-  if (MatchMessage(hdr, PLAYER_MSGTYPE_REQ, PLAYER_FIDUCIAL_GET_GEOM, device_id))
-  {
-    int ret = laser->ProcessMessage(PLAYER_MSGTYPE_REQ, PLAYER_LASER_GET_GEOM, laser_id, 0, resp_data, resp_data, resp_len);
-    if (ret != PLAYER_MSGTYPE_RESP_ACK)
-    	return ret;
-  	assert(*resp_len == sizeof(player_laser_geom_t));
-  	player_laser_geom_t lgeom = * reinterpret_cast<player_laser_geom_t * > (resp_data);
-  	player_fiducial_geom_t * fgeom = reinterpret_cast<player_fiducial_geom_t * > (resp_data);
 
-    fgeom->pose[0] = lgeom.pose[0];
-    fgeom->pose[1] = lgeom.pose[1];
-    fgeom->pose[2] = lgeom.pose[2];
-    fgeom->size[0] = lgeom.size[0];
-    fgeom->size[1] = lgeom.size[1];
-    fgeom->fiducial_size[0] = ntohs((int) (this->barwidth * 1000));
-    fgeom->fiducial_size[1] = ntohs((int) (this->barwidth * 1000));
-  
-  	*resp_len=sizeof(player_fiducial_geom_t);
-  
-    return ret;
-  }*/
   return -1;
 }
 
@@ -464,7 +442,7 @@ int LaserVisualBarcode::ProcessMessage (MessageQueue * resp_queue, player_msghdr
 int LaserVisualBarcode::UpdateLaser(player_laser_data_t * data, double timestamp)
 {
   this->laser_time = timestamp;
-  
+
   // Find possible fiducials in this scan.
   this->FindLaserFiducials(timestamp, data);
 
@@ -489,14 +467,14 @@ void LaserVisualBarcode::FindLaserFiducials(double time, player_laser_data_t *da
 
   // Empty the fiducial list.
   this->fdata.fiducials_count = 0;
-  
+
   // Initialise patch statistics.
   mn = 0.0;
   mr = 0.0;
   mb = 0.0;
   mrr = 0.0;
   mbb = 0.0;
-    
+
   // Look for a candidate patch in scan.
   for (i = 0; i < data->ranges_count; i++)
   {
@@ -530,7 +508,7 @@ void LaserVisualBarcode::FindLaserFiducials(double time, player_laser_data_t *da
       db = atan2(this->barwidth / 2, mr);
       valid &= (mrr < (dr * dr));
       valid &= (mbb < (db * db));
-      
+
       if (valid)
       {
         // Do a best fit to determine the pose of the reflector.
@@ -539,7 +517,7 @@ void LaserVisualBarcode::FindLaserFiducials(double time, player_laser_data_t *da
         // Match this fiducial against the ones we are already tracking.
         this->MatchLaserFiducial(time, pose);
       }
-      
+
       mn = 0.0;
       mr = 0.0;
       mb = 0.0;
@@ -597,7 +575,7 @@ void LaserVisualBarcode::MatchLaserFiducial(double time, double pose[3])
   double mindr;
   fiducial_t *fiducial;
   fiducial_t *minfiducial;
-  
+
   // Observations must be at least this close to the existing
   // fiducial.
   mindr = this->max_dist;
@@ -622,18 +600,21 @@ void LaserVisualBarcode::MatchLaserFiducial(double time, double pose[3])
   // If we didnt find a matching fiducial, add a new one.
   if (minfiducial == NULL)
   {
-    if (this->fiducial_count < ARRAYSIZE(this->fiducials))
+    this->fiducial_count++;
+    if (this->fiducial_count+1 > this->fdata_allocated)
     {
-      minfiducial = this->fiducials + this->fiducial_count++;
-      minfiducial->id = -1;
-      minfiducial->pose[0] = pose[0];
-      minfiducial->pose[1] = pose[1];
-      minfiducial->pose[2] = pose[2];
-      minfiducial->laser_time = time;
-      minfiducial->ptz_select_time = -1;
-      minfiducial->ptz_lockon_time = -1;
-      minfiducial->id_time = -1;
+      this->fdata_allocated = this->fiducial_count+1;
+      this->fdata.fiducials = (player_fiducial_item_t*)realloc(this->fdata.fiducials, sizeof(this->fdata.fiducials[0])*this->fdata_allocated);
     }
+    minfiducial = &this->fiducials[this->fiducial_count-1];
+    minfiducial->id = -1;
+    minfiducial->pose[0] = pose[0];
+    minfiducial->pose[1] = pose[1];
+    minfiducial->pose[2] = pose[2];
+    minfiducial->laser_time = time;
+    minfiducial->ptz_select_time = -1;
+    minfiducial->ptz_lockon_time = -1;
+    minfiducial->id_time = -1;
   }
 
   // Otherwise, update the existing fiducial.
@@ -644,7 +625,7 @@ void LaserVisualBarcode::MatchLaserFiducial(double time, double pose[3])
     minfiducial->pose[2] = pose[2];
     minfiducial->laser_time = time;
   }
-  
+
   return;
 }
 
@@ -655,7 +636,7 @@ void LaserVisualBarcode::RetireLaserFiducials(double time, player_laser_data_t *
 {
   int i;
   fiducial_t *fiducial;
-  
+
   // Remove any old fiducials.
   for (i = 0; i < this->fiducial_count; i++)
   {
@@ -679,7 +660,7 @@ void LaserVisualBarcode::RetireLaserFiducials(double time, player_laser_data_t *
 int LaserVisualBarcode::UpdatePtz(player_ptz_data_t * data, double timestamp)
 {
   this->ptz_time = timestamp;
-  
+
   // Pick a fiducial to look at.
   this->SelectPtzTarget(timestamp, data);
 
@@ -710,7 +691,7 @@ void LaserVisualBarcode::SelectPtzTarget(double time, player_ptz_data_t *data)
   // Find one we havent looked at for while.
   this->ptz_fiducial = NULL;
   maxt = -1;
-  
+
   for (i = 0; i < this->fiducial_count; i++)
   {
     fiducial = this->fiducials + i;
@@ -728,7 +709,7 @@ void LaserVisualBarcode::SelectPtzTarget(double time, player_ptz_data_t *data)
     this->ptz_fiducial->ptz_select_time = time;
     this->ptz_fiducial->ptz_lockon_time = -1;
   }
-  
+
   return;
 }
 
@@ -749,7 +730,7 @@ void LaserVisualBarcode::ServoPtz(double time, player_ptz_data_t *data)
   // Deadband values.
   deadpan = 2;
   deadzoom = 2;
-  
+
   fiducial = this->ptz_fiducial;
   if (fiducial == NULL)
   {
@@ -781,12 +762,12 @@ void LaserVisualBarcode::ServoPtz(double time, player_ptz_data_t *data)
       tilt = maxtilt * sin((time - fiducial->ptz_lockon_time) /
                            this->max_ptz_attention * 2 * M_PI);
   }
-  
+
   // Compose the command packet to send to the PTZ device.
-  cmd.pan = pan;
-  cmd.tilt = tilt;
-  cmd.zoom = zoom;
-  
+  cmd.pan = static_cast<float> (pan);
+  cmd.tilt = static_cast<float> (tilt);
+  cmd.zoom = static_cast<float> (zoom);
+
   this->ptz->PutMsg(InQueue, PLAYER_MSGTYPE_CMD, PLAYER_PTZ_CMD_STATE, &cmd, sizeof(cmd), NULL);
 
   // Compute the dimensions of the image at the range of the target fiducial.
@@ -818,7 +799,7 @@ int LaserVisualBarcode::UpdateBlobfinder(player_blobfinder_data_t * data, double
       this->ptz_fiducial->id_time = timestamp;
     }
   }
-  
+
   return 1;
 }
 
@@ -842,7 +823,7 @@ void LaserVisualBarcode::FindBlobs(double time, player_blobfinder_data_t *data)
   // Compute expect width and height at the current range and zoom.
   width = (int) (this->barwidth / this->zoomwidth * data->width);
   height = (int) (this->barheight / this->zoomheight * data->height);
-      
+
   // Set limits
   minx = (int) ((1 - tol) * data->width / 2);
   maxx = (int) ((1 + tol) * data->width / 2);
@@ -860,7 +841,7 @@ void LaserVisualBarcode::FindBlobs(double time, player_blobfinder_data_t *data)
     printf("image %d %d\n", data->width, data->height);
     printf("w, h %d %d\n", width, height);
   */
-  
+
   this->blob_count = 0;
   for (i = 0; i < data->blobs_count; i++)
   {
@@ -892,7 +873,7 @@ void LaserVisualBarcode::FindBlobs(double time, player_blobfinder_data_t *data)
       nblob->y = blob->y;
     }
   }
-  
+
   return;
 }
 
@@ -937,7 +918,7 @@ int LaserVisualBarcode::FindVisualFiducials(double time, player_blobfinder_data_
       id = blob->ch;
       return id;
     }
-    
+
     id = this->FindVisualFiducials(time, data, depth + 1, blob);
     if (id >= 0)
     {
@@ -969,7 +950,7 @@ void LaserVisualBarcode::UpdateData()
     // scan.
     if (fiducial->laser_time != this->laser_time)
       continue;
-    
+
     r = sqrt(fiducial->pose[0] * fiducial->pose[0] +
              fiducial->pose[1] * fiducial->pose[1]);
     b = atan2(fiducial->pose[1], fiducial->pose[0]);
@@ -981,12 +962,12 @@ void LaserVisualBarcode::UpdateData()
     data.fiducials[data.fiducials_count].pose.pyaw = o;
     data.fiducials_count++;
   }
-  
+
   // Compute the data timestamp (from laser).
   timestamp = this->laser_time;
-  
+
   // Copy data to server.
-  Publish(device_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_FIDUCIAL_DATA_SCAN, (void*) &data, sizeof(data), &timestamp);
+  Publish(device_addr, PLAYER_MSGTYPE_DATA, PLAYER_FIDUCIAL_DATA_SCAN, (void*) &data, 0, &timestamp);
 }
 
 
