@@ -1,9 +1,9 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2003  
+ *  Copyright (C) 2003
  *     Brian Gerkey, Andrew Howard
- *                      
- * 
+ *
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -24,7 +24,7 @@
  * Desc: Read data from a standard linux joystick
  * Author: Andrew Howard
  * Date: 25 July 2004
- * CVS: $Id: linuxjoy.cc,v 1.30 2006/02/27 18:19:03 gerkey Exp $
+ * CVS: $Id: linuxjoy.cc 8062 2009-07-21 03:47:16Z gbiggs $
  *
  */
 
@@ -45,15 +45,18 @@ converting joystick positions to velocity commands.
 @par Provides
 
 - @ref interface_joystick : joystick data
-- @ref interface_position2d : joystick data represented as 2-D 
-  position data.  Raw X- and Y-axis values are reported as xpos and ypos in the 
-  position packet (all other fields are zero).
+- @ref interface_position2d : joystick data represented as 2-D
+  position data.  Raw X-axis, Y-axis and Yaw-axis values are reported
+  as pos[0], pos[1] and pos[3] in the position packet (all other
+  fields are zero).
 
 @par Requires
 
 - @ref interface_position2d : if present, joystick positions will be
   interpreted as velocities and sent as commands to this position2d device.
-  See also max_xspeed, max_yawspeed, and deadman_button options below.
+  See also max_speed, and deadman_button options below.
+- @ref interface_gripper : if present, joystick buttons will be used to
+  control given gripper (open, close, store, retrieve, stop).
 
 @par Configuration requests
 
@@ -65,35 +68,45 @@ converting joystick positions to velocity commands.
   - Default: "/dev/js0"
   - The joystick to be used.
 - axes (integer tuple)
-  - Default: [0 1]
-  - Which joystick axes to call the "X" and "Y" axes, respectively.
-- axis_maxima (integer tuple)
-  - Default: [32767 32767]
-  - Maximum absolute values attainable on the X and Y axes, respectively.
-- axis_minima (integer tuple)
-  - Default: [0 0]
-  - Minimum values on the X and Y axes, respectively.  Anything smaller
+  - Default: [1 2 0]
+  - Which joystick axes to call the "X" , "Y" and "Yaw" axes, respectively.
+- axes_maxima (integer tuple)
+  - Default: [32767 32767 32767]
+  - Maximum absolute values attainable on the X, Y and Yaw axes, respectively.
+- axes_minima (integer tuple)
+  - Default: [0 0 0]
+  - Minimum values on the X and Yaw axes, respectively.  Anything smaller
     in absolute value than this limit will be reported as zero.
     Useful for implementing a dead zone on a touchy joystick.
 - deadman_button (integer)
   - Default: -1
-  - When controlling a @ref interface_position2d device, if deadman_button is 
-    >= 0, this joystick button must be depressed for commands to be 
+  - When controlling a @ref interface_position2d device, if deadman_button is
+    >= 0, this joystick button must be depressed for commands to be
     sent to that device.
-- max_xspeed (length / sec)
-  - Default: 0.5 m/sec
-  - The maximum absolute translational velocity to be used when commanding a
-    position device.
-- max_yawspeed (angle / sec)
-  - Default: 30 deg/sec
-  - The maximum absolute rotational velocity to be used when commanding a
-    position device.
+- max_speed (float tupple, m / sec or angle / sec)
+  - Default: [0.5 0.5 30]
+  - The maximum absolute X and Y translational and rotational velocities to be
+    used when commanding a position device. (Y is only revelant for
+    holonomous robot)
+- scale_pos (float tuple)
+  - Default: [1.0 1.0 1.0]
+  - Position2d readings scale
 - timeout (float)
   - Default: 5.0
   - Time (in seconds) since receiving a new joystick event after which
     the underlying position device will be stopped, for safety.  Set to
     0.0 for no timeout.
 
+- Notes:
+  - Joysticks use X for side-to-side and Y for forward-back, also
+    their axes are backwards with respect to intuitive driving
+    controls.
+  - This driver does not swap any axis, you have to handle this in the
+    configuration file options via "axes". However the defaults values
+    should suit your needs.
+  - This driver reverse the axes so that the joystick respect the
+    intuitive driving controls.
+  - The Y axis is only revelant for holonomous platform (like the WizBot).
 @par Examples
 
 Basic configuration
@@ -107,14 +120,15 @@ driver
 )
 @endverbatim
 
-Provide a position interface, instead of a joystick interface.
+Provide a position2d interface, instead of a joystick interface.
 
 @verbatim
 driver
 (
   name "linuxjoystick"
-  provides ["position:0"]
+  provides ["position2d:0"]
   port "/dev/js0"
+  scale_pos [ 0.0001 0.0001 0.0001 ]
 )
 @endverbatim
 
@@ -129,22 +143,61 @@ driver
   port "/dev/usb/tts/0"
 )
 
+# 1 m/sec max linear velocity
+# 30 deg/sec max angular velocity
+# Axis 4 is X
+# Axis 3 is Yaw
+# Y is not used here
 driver
 (
   name "linuxjoystick"
   provides ["joystick:0"]
   requires ["odometry::position:0"]
-  max_yawspeed 50
-  max_xspeed 0.5
-  axes [3 4]
-  axis_minima [5000 5000]
+  max_speed [1 0 30]
+  axes [4 -1 3]
+  axes_minima [5000 0 5000]
+  port "/dev/js0"
+  alwayson 1
+)
+@endverbatim
+
+Controlling a WizBot in Gazebo, plus remapping joystick axes.
+
+@verbatim
+driver
+(
+  name "gazebo"
+  provides ["simulation:0"]
+  plugin "libgazeboplugin"
+  server_id "default"
+)
+
+driver
+(
+  name "gazebo"
+  provides ["position2d:0"]
+  gz_id "position_iface_0"
+)
+
+# 0.5 m/sec max linear velocity
+# 120 deg/sec max angular velocity
+# Axis 1 is X
+# Axis 2 is Y
+# Axis 0 is Yaw
+driver
+(
+  name "linuxjoystick"
+  provides ["joystick:0"]
+  requires ["position2d:0"]
+  max_speed [0.5 0.5 120]
+  axes [1 2 0]
   port "/dev/js0"
   alwayson 1
 )
 @endverbatim
 
 @todo
-Add support for continuously sending commands, which might be needed for 
+Add support for continuously sending commands, which might be needed for
 position devices that use watchdog timers.
 
 @author Andrew Howard, Brian Gerkey, Paul Osmialowski
@@ -154,6 +207,7 @@ position devices that use watchdog timers.
 
 #include <netinet/in.h>
 #include <string.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -168,34 +222,41 @@ position devices that use watchdog timers.
 
 #include <replace/replace.h> // for poll(2)
 #include <libplayercore/playercore.h>
-#include <libplayercore/error.h>
 
-extern PlayerTime *GlobalTime;
-
-#define XAXIS 0
-#define YAXIS 1
+#define XAXIS 1
+#define YAXIS 2
+#define YAWAXIS 0
+#define SLIDER 3
 
 #define MAX_XSPEED 0.5
-#define MAX_YAWSPEED DTOR(30.0)
-#define AXIS_MAX ((int16_t) 32767)
+#define MAX_YSPEED 0.5
+#define MAX_YAWSPEED 30.0
+
+#define XAXIS_MAX ((int16_t) 32767)
+#define YAXIS_MAX ((int16_t) 32767)
+#define YAWAXIS_MAX ((int16_t) 32767)
+
+#define XAXIS_MIN ((int16_t) 0)
+#define YAXIS_MIN ((int16_t) 0)
+#define YAWAXIS_MIN ((int16_t) 0)
 
 ////////////////////////////////////////////////////////////////////////////////
 // The class for the driver
-class LinuxJoystick : public Driver
+class LinuxJoystick : public ThreadedDriver
 {
   // Constructor; need that
   public: LinuxJoystick(ConfigFile* cf, int section);
 
   // Must implement the following methods.
-  public: int Setup();
-  public: int Shutdown();
+  public: int MainSetup();
+  public: void MainQuit();
 
   // Main function for device thread.
   private: virtual void Main();
 
-  public: virtual int ProcessMessage(MessageQueue* resp_queue, 
-                                     player_msghdr * hdr, 
-                                     void * data) {return -1;}
+  public: virtual int ProcessMessage(QueuePointer & resp_queue,
+                                     player_msghdr * hdr,
+                                     void * data) {return 0;}
 
   // Read the joystick
   private: void ReadJoy();
@@ -213,10 +274,12 @@ class LinuxJoystick : public Driver
   private: player_devaddr_t joystick_addr;
   private: const char *dev;
   private: int fd;
-  private: int16_t xpos, ypos;
-  private: uint16_t buttons;
-  private: int xaxis_max, yaxis_max;
-  private: int xaxis_min, yaxis_min;
+  private: int axes_count;
+  private: int32_t pos[4];
+  private: uint32_t buttons;
+  private: int axes_max[4];
+  private: int axes_min[4];
+  private: double scale_pos[4];
   private: double timeout;
   private: struct timeval lastread;
 
@@ -225,11 +288,13 @@ class LinuxJoystick : public Driver
   private: player_position2d_data_t pos_data;
 
   // These are used when we send commands to a position device
-  private: double max_xspeed, max_yawspeed;
-  private: int xaxis, yaxis;
+  private: double max_speed[4];
+  private: int axes[4];
   private: int deadman_button;
   private: player_devaddr_t cmd_position_addr;
-  private: Device* position;
+  private: Device * position;
+  private: player_devaddr_t cmd_gripper_addr;
+  private: Device * gripper;
 
   // Joystick
   private: player_joystick_data_t joy_data;
@@ -253,7 +318,7 @@ Driver* LinuxJoystick_Init(ConfigFile* cf, int section)
 // that it can be invoked without object context.  In this function, we add
 // the driver into the given driver table, indicating which interface the
 // driver can support and how to create a driver instance.
-void LinuxJoystick_Register(DriverTable* table)
+void linuxjoystick_Register(DriverTable* table)
 {
   table->AddDriver("linuxjoystick", LinuxJoystick_Init);
 }
@@ -262,12 +327,15 @@ void LinuxJoystick_Register(DriverTable* table)
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor.  Retrieve options from the configuration file and do any
 // pre-Setup() setup.
-LinuxJoystick::LinuxJoystick(ConfigFile* cf, int section) : Driver(cf, section)
+LinuxJoystick::LinuxJoystick(ConfigFile* cf, int section) : ThreadedDriver(cf, section)
 {
   // zero ids, so that we'll know later which interfaces were requested
-  memset(&this->cmd_position_addr, 0, sizeof(player_devaddr_t));
-  memset(&this->position_addr, 0, sizeof(player_devaddr_t));
-  memset(&this->joystick_addr, 0, sizeof(player_devaddr_t));
+  memset(&(this->cmd_position_addr), 0, sizeof(player_devaddr_t));
+  memset(&(this->cmd_gripper_addr), 0, sizeof(player_devaddr_t));
+  memset(&(this->position_addr), 0, sizeof(player_devaddr_t));
+  memset(&(this->joystick_addr), 0, sizeof(player_devaddr_t));
+  this->position = NULL;
+  this->gripper = NULL;
 
   // Do we create a position interface?
   if(cf->ReadDeviceAddr(&(this->position_addr), section, "provides",
@@ -275,7 +343,7 @@ LinuxJoystick::LinuxJoystick(ConfigFile* cf, int section) : Driver(cf, section)
   {
     if(this->AddInterface(this->position_addr))
     {
-      this->SetError(-1);    
+      this->SetError(-1);
       return;
     }
   }
@@ -285,30 +353,46 @@ LinuxJoystick::LinuxJoystick(ConfigFile* cf, int section) : Driver(cf, section)
   {
     if(this->AddInterface(this->joystick_addr))
     {
-      this->SetError(-1);    
+      this->SetError(-1);
       return;
     }
   }
 
   this->dev = cf->ReadString(section, "port", "/dev/js0");
-  this->xaxis = cf->ReadTupleInt(section,"axes", 0, XAXIS);
-  this->yaxis = cf->ReadTupleInt(section,"axes", 1, YAXIS);
-  this->deadman_button = cf->ReadInt(section,"deadman", -1);
-  this->xaxis_max = cf->ReadTupleInt(section, "axis_maxima", 0, AXIS_MAX);
-  this->yaxis_max = cf->ReadTupleInt(section, "axis_maxima", 1, AXIS_MAX);
-  this->xaxis_min = cf->ReadTupleInt(section, "axis_minima", 0, 0);
-  this->yaxis_min = cf->ReadTupleInt(section, "axis_minima", 1, 0);
+// partially implementing axes_count
+  this->axes_count=4;
+  this->axes[0] = cf->ReadTupleInt(section,"axes", 0, XAXIS);
+  this->axes[1] = cf->ReadTupleInt(section,"axes", 1, YAXIS);
+  this->axes[2] = cf->ReadTupleInt(section,"axes", 2, YAWAXIS);
+  this->axes[3] = cf->ReadTupleInt(section,"axes", 3, SLIDER);
 
-  // Do we talk to a position device?
+
+  this->deadman_button = cf->ReadInt(section,"deadman", -1);
+  this->axes_max[0] = cf->ReadTupleInt(section, "axes_maxima", 0, XAXIS_MAX);
+  this->axes_max[1] = cf->ReadTupleInt(section, "axes_maxima", 1, YAXIS_MAX);
+  this->axes_max[2] = cf->ReadTupleInt(section, "axes_maxima", 2, YAWAXIS_MAX);
+  this->axes_min[0] = cf->ReadTupleInt(section, "axes_minima", 0, XAXIS_MIN);
+  this->axes_min[1] = cf->ReadTupleInt(section, "axes_minima", 1, YAXIS_MIN);
+  this->axes_min[2] = cf->ReadTupleInt(section, "axes_minima", 2, YAWAXIS_MIN);
+  this->scale_pos[0] = cf->ReadTupleFloat(section, "scale_pos", 0, 1.0);
+  this->scale_pos[1] = cf->ReadTupleFloat(section, "scale_pos", 1, 1.0);
+  this->scale_pos[2] = cf->ReadTupleFloat(section, "scale_pos", 2, 1.0);
+
+  // Do we talk to any device?
   if(cf->GetTupleCount(section, "requires"))
   {
-    if(cf->ReadDeviceAddr(&(this->cmd_position_addr), section, "requires", 
+    if(cf->ReadDeviceAddr(&(this->cmd_position_addr), section, "requires",
                           PLAYER_POSITION2D_CODE, -1, NULL) == 0)
     {
-      this->max_xspeed = cf->ReadLength(section, "max_xspeed", MAX_XSPEED);
-      this->max_yawspeed = cf->ReadAngle(section, "max_yawspeed", MAX_YAWSPEED);
+      this->max_speed[0] = cf->ReadTupleFloat(section, "max_speed", 0, MAX_XSPEED);
+      this->max_speed[1] = cf->ReadTupleFloat(section, "max_speed", 1, MAX_YSPEED);
+      this->max_speed[2] = DTOR(cf->ReadTupleFloat(section, "max_speed", 2, MAX_YAWSPEED));
       this->timeout = cf->ReadFloat(section, "timeout", 5.0);
-    }
+    } else memset(&(this->cmd_position_addr), 0, sizeof(player_devaddr_t));
+    if (cf->ReadDeviceAddr(&(this->cmd_gripper_addr), section, "requires",
+                          PLAYER_GRIPPER_CODE, -1, NULL) == 0)
+    {
+    } else memset(&(this->cmd_gripper_addr), 0, sizeof(player_devaddr_t));
   }
 
   return;
@@ -316,7 +400,7 @@ LinuxJoystick::LinuxJoystick(ConfigFile* cf, int section) : Driver(cf, section)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device.  Return 0 if things go well, and -1 otherwise.
-int LinuxJoystick::Setup()
+int LinuxJoystick::MainSetup()
 {
   // Open the joystick device
   this->fd = open(this->dev, O_RDONLY);
@@ -329,29 +413,78 @@ int LinuxJoystick::Setup()
 
   this->lastread.tv_sec = this->lastread.tv_usec = 0;
 
-  // If we're asked, open the position device
-  if(this->cmd_position_addr.interf)
+  this->position = NULL;
+  // If we're asked, open the position2d device
+  if (this->cmd_position_addr.interf)
   {
-    if(!(this->position = deviceTable->GetDevice(this->cmd_position_addr)))
+    if (!(this->position = deviceTable->GetDevice(this->cmd_position_addr)))
     {
-      PLAYER_ERROR("unable to locate suitable position device");
-      return(-1);
+      PLAYER_ERROR("unable to locate suitable position2d device");
+      return -1;
     }
-    if(this->position->Subscribe(this->InQueue) != 0)
+    if (this->position->Subscribe(this->InQueue) != 0)
     {
-      PLAYER_ERROR("unable to subscribe to position device");
-      return(-1);
+      PLAYER_ERROR("unable to subscribe to position2d device");
+      return -1;
     }
+  }
+  this->gripper = NULL;
+  // If we're asked, open the gripper device
+  if (this->cmd_gripper_addr.interf)
+  {
+    this->gripper = deviceTable->GetDevice(this->cmd_gripper_addr);
+    if (!(this->gripper))
+    {
+      PLAYER_ERROR("unable to locate suitable gripper device");
+      if ((this->cmd_position_addr.interf) && (this->position))
+        this->position->Unsubscribe(this->InQueue);
+      return -1;
+    }
+    if (this->gripper->Subscribe(this->InQueue) != 0)
+    {
+      PLAYER_ERROR("unable to subscribe to gripper device");
+      if ((this->cmd_position_addr.interf) && (this->position))
+        this->position->Unsubscribe(this->InQueue);
+      return -1;
+    }
+  }
 
+  this->pos[0] = this->pos[1] = this->pos[2] = this->pos[3] = 0;
+
+  return 0;
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+// Shutdown the device
+void LinuxJoystick::MainQuit()
+{
+  if ((this->cmd_position_addr.interf) && (this->position))
+    this->position->Unsubscribe(this->InQueue);
+  if ((this->cmd_gripper_addr.interf) && (this->gripper))
+    this->gripper->Unsubscribe(this->InQueue);
+
+  // Close the joystick
+  close(this->fd);
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+// Main function for device thread
+void LinuxJoystick::Main()
+{
+  if (this->cmd_position_addr.interf)
+  {
     // Enable the motors
     player_position2d_power_config_t motorconfig;
+    memset(&motorconfig, 0, sizeof motorconfig);
     motorconfig.state = 1;
     Message* msg;
-    if(!(msg = this->position->Request(this->InQueue,
-                                       PLAYER_MSGTYPE_REQ, 
-                                       PLAYER_POSITION2D_REQ_MOTOR_POWER,
-                                       (void*)&motorconfig,
-                                       sizeof(motorconfig),NULL,false)))
+    if (!(msg = this->position->Request(this->InQueue,
+                                        PLAYER_MSGTYPE_REQ,
+                                        PLAYER_POSITION2D_REQ_MOTOR_POWER,
+                                        reinterpret_cast<void *>(&motorconfig),
+                                        sizeof motorconfig, NULL)))
     {
       PLAYER_WARN("failed to enable motors");
     }
@@ -360,54 +493,25 @@ int LinuxJoystick::Setup()
 
     // Stop the robot
     player_position2d_cmd_vel_t cmd;
-    memset(&cmd,0,sizeof(cmd));
+    memset(&cmd, 0, sizeof cmd);
     this->position->PutMsg(this->InQueue,
                            PLAYER_MSGTYPE_CMD,
                            PLAYER_POSITION2D_CMD_VEL,
-                           (void*)&cmd, sizeof(player_position2d_cmd_vel_t),
+                           reinterpret_cast<void *>(&cmd), sizeof cmd,
                            NULL);
   }
-  
-  this->xpos = this->ypos = 0;
-  
-  // Start the device thread; spawns a new thread and executes
-  // LinuxJoystick::Main(), which contains the main loop for the driver.
-  this->StartThread();
-
-  return(0);
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-// Shutdown the device
-int LinuxJoystick::Shutdown()
-{
-  // Stop and join the driver thread
-  this->StopThread();
-
-  if(this->cmd_position_addr.interf)
-    this->position->Unsubscribe(this->InQueue);
-
-  // Close the joystick
-  close(this->fd);
-
-  return(0);
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-// Main function for device thread
-void LinuxJoystick::Main() 
-{
   // The main loop; interact with the device here
   while (true)
   {
     // test if we are supposed to cancel
     pthread_testcancel();
 
+    // Process incoming messages
+    this->ProcessMessages();
+
     // Run and process output
     this->ReadJoy();
-    
+
     // Write outgoing data
     this->RefreshData();
 
@@ -419,7 +523,6 @@ void LinuxJoystick::Main()
       {
         this->PutPositionCommand();
       }
-#if 0
       else
       {
         player_position2d_cmd_vel_t cmd;
@@ -430,10 +533,28 @@ void LinuxJoystick::Main()
                                (void*)&cmd, sizeof(player_position2d_cmd_vel_t),
                                NULL);
       }
-#endif
+    }
+    // Send new commands to gripper device
+    if (this->cmd_gripper_addr.interf)
+    {
+      if ((this->buttons) & 0x01)
+      {
+        this->gripper->PutMsg(this->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_GRIPPER_CMD_CLOSE, NULL, 0, NULL);
+      } else if ((this->buttons) & 0x02)
+      {
+        this->gripper->PutMsg(this->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_GRIPPER_CMD_OPEN, NULL, 0, NULL);
+      } else if ((this->buttons) & 0x04)
+      {
+        this->gripper->PutMsg(this->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_GRIPPER_CMD_STORE, NULL, 0, NULL);
+      } else if ((this->buttons) & 0x08)
+      {
+        this->gripper->PutMsg(this->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_GRIPPER_CMD_RETRIEVE, NULL, 0, NULL);
+      } else if ((this->buttons) & 0x10)
+      {
+        this->gripper->PutMsg(this->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_GRIPPER_CMD_STOP, NULL, 0, NULL);
+      }
     }
   }
-  return;
 }
 
 
@@ -444,29 +565,37 @@ void LinuxJoystick::ReadJoy()
   struct pollfd fd;
   struct js_event event;
   int count;
-  
+
   fd.fd = this->fd;
   fd.events = POLLIN | POLLHUP;
   fd.revents = 0;
 
+
   count = poll(&fd, 1, 10);
   if (count < 0)
     PLAYER_ERROR1("poll returned error [%s]", strerror(errno));
   else if(count > 0)
   {
     // get the next event from the joystick
-    read(this->fd, &event, sizeof(struct js_event));
+    int ret = read(this->fd, &event, sizeof(struct js_event));
+    if (ret < 0)
+    {
+    	PLAYER_ERROR("Failed to read from the joystick");
+    	return;
+    }
 
-    //printf( "value % d type %u  number %u state %X \n", 
+    //printf( "value % d type %u  number %u state %X \n",
     //        event.value, event.type, event.number, this->joy_data.buttons );
 
     // Update buttons
     if ((event.type & ~JS_EVENT_INIT) == JS_EVENT_BUTTON)
     {
+
       if (event.value)
         this->buttons |= (1 << event.number);
       else
         this->buttons &= ~(1 << event.number);
+
     }
 
     // ignore the startup events
@@ -477,25 +606,39 @@ void LinuxJoystick::ReadJoy()
     {
       case JS_EVENT_AXIS:
         {
-          if(event.number == this->xaxis)
+          if(event.number == this->axes[0])
           {
-            this->xpos = event.value;
-            if(abs(this->xpos) < this->xaxis_min)
-              this->xpos = 0;
+            this->pos[0] = event.value;
+            if(abs(this->pos[0]) < this->axes_min[0])
+              this->pos[0] = 0;
             GlobalTime->GetTime(&this->lastread);
           }
-          else if(event.number == this->yaxis)
+          else if(event.number == this->axes[1])
           {
-            this->ypos = event.value;
-            if(abs(this->ypos) < this->yaxis_min)
-              this->ypos = 0;
+            this->pos[1] = event.value;
+            if(abs(this->pos[1]) < this->axes_min[1])
+              this->pos[1] = 0;
             GlobalTime->GetTime(&this->lastread);
           }
-        }	  
+          else if(event.number == this->axes[2])
+          {
+            this->pos[2] = event.value;
+            if(abs(this->pos[2]) < this->axes_min[2])
+              this->pos[2] = 0;
+            GlobalTime->GetTime(&this->lastread);
+          }
+          else if(event.number == this->axes[3])
+          {
+            this->pos[3] = event.value;
+            if(abs(this->pos[3]) < this->axes_min[3])
+              this->pos[3] = 0;
+            GlobalTime->GetTime(&this->lastread);
+          }
+        }
         break;
     }
   }
-      
+
   return;
 }
 
@@ -504,15 +647,23 @@ void LinuxJoystick::ReadJoy()
 // Send new data out
 void LinuxJoystick::RefreshData()
 {
+  unsigned i;
   if(this->joystick_addr.interf)
   {
     memset(&(this->joy_data),0,sizeof(player_joystick_data_t));
-    this->joy_data.xpos = this->xpos;
-    this->joy_data.ypos = this->ypos;
-    this->joy_data.xscale = this->xaxis_max;
-    this->joy_data.yscale = this->yaxis_max;
+
     this->joy_data.buttons = this->buttons;
-    this->Publish(this->joystick_addr, NULL,
+    this->joy_data.axes_count = this->axes_count;
+    for (i = 0; i < joy_data.axes_count ; i++)
+        this->joy_data.pos[i] = this->pos[i];
+
+
+    this->joy_data.scale[0] = this->axes_max[0];
+    this->joy_data.scale[1] = this->axes_max[1];
+    this->joy_data.scale[2] = this->axes_max[2];
+    this->joy_data.scale[3] = this->axes_max[3];
+
+    this->Publish(this->joystick_addr,
                   PLAYER_MSGTYPE_DATA, PLAYER_JOYSTICK_DATA_STATE,
                   (void*)&this->joy_data, sizeof(this->joy_data), NULL);
   }
@@ -520,9 +671,10 @@ void LinuxJoystick::RefreshData()
   if(this->position_addr.interf)
   {
     memset(&(this->pos_data),0,sizeof(player_position2d_data_t));
-    this->pos_data.pos.px = this->xpos;
-    this->pos_data.pos.py = -this->ypos;
-    this->Publish(this->position_addr, NULL,
+    this->pos_data.pos.px = this->pos[0] * this->scale_pos[0];
+    this->pos_data.pos.py = this->pos[1] * this->scale_pos[1];
+    this->pos_data.pos.pa = this->pos[2] * this->scale_pos[2];
+    this->Publish(this->position_addr,
                   PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE,
                   (void*)&this->pos_data, sizeof(this->pos_data), NULL);
   }
@@ -531,50 +683,59 @@ void LinuxJoystick::RefreshData()
 // command the robot
 void LinuxJoystick::PutPositionCommand()
 {
-  double scaled_x, scaled_y;
-  double xspeed, yawspeed;
+  double scaled[3];
+  double speed[3];
   player_position2d_cmd_vel_t cmd;
   struct timeval curr;
   double diff;
 
-  scaled_x = this->xpos / (double) this->xaxis_max;
-  scaled_y = this->ypos / (double) this->yaxis_max;
+  scaled[0] = this->pos[0] / (double) this->axes_max[0];
+  scaled[1] = this->pos[1] / (double) this->axes_max[1];
+  scaled[2] = this->pos[2] / (double) this->axes_max[2];
 
   // sanity check
-  if((scaled_x > 1.0) || (scaled_x < -1.0))
+  if((scaled[0] > 1.0) || (scaled[0] < -1.0))
   {
-    PLAYER_ERROR2("X position (%d) outside of axis max (+-%d); ignoring", 
-                  this->xpos, this->xaxis_max);
+    PLAYER_ERROR2("X position (%d) outside of axis max (+-%d); ignoring",
+                  this->pos[0], this->axes_max[0]);
     return;
   }
-  if((scaled_y > 1.0) || (scaled_y < -1.0))
+  if((scaled[1] > 1.0) || (scaled[1] < -1.0))
   {
-    PLAYER_ERROR2("Y position (%d) outside of axis max (+-%d); ignoring", 
-                  this->ypos, this->yaxis_max);
+    PLAYER_ERROR2("Y position (%d) outside of axis max (+-%d); ignoring",
+                  this->pos[1], this->axes_max[1]);
+    return;
+
+  }
+  if((scaled[2] > 1.0) || (scaled[2] < -1.0))
+  {
+    PLAYER_ERROR2("Yaw position (%d) outside of axis max (+-%d); ignoring",
+                  this->pos[2], this->axes_max[2]);
     return;
   }
 
-  // Note that joysticks use X for side-to-side and Y for forward-back, and
-  // also that their axes are backwards with respect to intuitive driving
-  // controls.
-  xspeed = -scaled_y * this->max_xspeed;
-  yawspeed = -scaled_x * this->max_yawspeed;
+  // As joysticks axes are backwards with respect to intuitive driving
+  // controls, we invert the values here.
+  speed[0] = -scaled[0] * this->max_speed[0];
+  speed[1] = -scaled[1] * this->max_speed[1];
+  speed[2] = -scaled[2] * this->max_speed[2];
 
   // Make sure we've gotten a joystick fairly recently.
   GlobalTime->GetTime(&curr);
   diff = (curr.tv_sec - curr.tv_usec/1e6) -
           (this->lastread.tv_sec - this->lastread.tv_usec/1e6);
-  if(this->timeout && (diff > this->timeout) && (xspeed || yawspeed))
+  if(this->timeout && (diff > this->timeout) && (speed[0] || speed[1] || speed[2]))
   {
     PLAYER_WARN("Timeout on joystick; stopping robot");
-    xspeed = yawspeed = 0.0;
+    speed[0] = speed[1] = speed[2] = 0.0;
   }
 
-  PLAYER_MSG2(2,"sending speeds: (%f,%f)", xspeed, yawspeed);
+  PLAYER_MSG3(2,"sending speeds: (%f,%f,%f)", speed[0], speed[1], speed[2]);
 
   memset(&cmd,0,sizeof(cmd));
-  cmd.vel.px = xspeed;
-  cmd.vel.pa = yawspeed;
+  cmd.vel.px = speed[0];
+  cmd.vel.py = speed[1];
+  cmd.vel.pa = speed[2];
   //cmd.type=0;
   cmd.state=1;
   this->position->PutMsg(this->InQueue,
