@@ -1,9 +1,9 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2000  
+ *  Copyright (C) 2000
  *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
- *                      
- * 
+ *
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -23,7 +23,7 @@
  Desc: Driver for the SICK S3000 laser
  Author: Toby Collett (based on lms200 by Andrew Howard)
  Date: 7 Nov 2000
- CVS: $Id: sicks3000.cc,v 1.1.2.1 2006/09/22 23:58:35 gerkey Exp $
+ CVS: $Id: sicks3000.cc 7305 2009-01-27 01:18:55Z gbiggs $
 */
 
 /** @ingroup drivers Drivers */
@@ -36,7 +36,7 @@ The sicks3000 driver controls the SICK S 3000 safety laser scanner interpreting
 The driver is very basic and assumes the S3000 has already been configured to continuously output
 its measured data on the RS422 data lines.
 
-It is also assumed that the laser is outputing its full 190 degree scan in a single scanning block.
+It is also assumed that the laser is outputing its full scan in a single scanning block.
 
 @par Compile-time dependencies
 
@@ -48,25 +48,15 @@ It is also assumed that the laser is outputing its full 190 degree scan in a sin
 
 @par Requires
 
-- none
+- opaque
 
 @par Configuration requests
 
 - PLAYER_LASER_REQ_GET_GEOM
 - PLAYER_LASER_REQ_GET_CONFIG
-  
+
 @par Configuration file options
 
-- port (string)
-  - Default: "/dev/ttyS0"
-  - Serial port to which laser is attached.  If you are using a
-    USB/232 or USB/422 converter, this will be "/dev/ttyUSBx".
-
-- transfer_rate (integer)
-  - Rate desired for data transfers, negotiated after connection
-  - Default: 38400
-  - Baud rate.  Valid values are 9600, 19200, 38400, 125k, 250k, 500k
-
 - pose (length tuple)
   - Default: [0.0 0.0 0.0]
   - Pose (x,y,theta) of the laser, relative to its parent object (e.g.,
@@ -75,28 +65,34 @@ It is also assumed that the laser is outputing its full 190 degree scan in a sin
 - size (length tuple)
   - Default: [0.15 0.15]
   - Footprint (x,y) of the laser.
-      
-@par Example 
+
+@par Example
 
 @verbatim
 driver
 (
   name "sicks3000"
   provides ["laser:0"]
+  requires ["opaque:0"]
+)
+
+driver
+(
+  name "serialstream"
+  provides ["opaque:0]
   port "/dev/ttyS0"
 )
+
 @endverbatim
 
 @author Toby Collett
 
 */
 /** @} */
-  
 
-  
-#if HAVE_CONFIG_H
-  #include <config.h>
-#endif
+
+
+#include <config.h>
 
 #include <assert.h>
 #include <math.h>
@@ -107,91 +103,59 @@ driver
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <termios.h>
-#include <unistd.h>
-#include <sys/ioctl.h>
-#include <arpa/inet.h> // for htons etc
-
-#undef HAVE_HI_SPEED_SERIAL
-#ifdef HAVE_LINUX_SERIAL_H
-  #ifndef DISABLE_HIGHSPEEDSICK
-    #include <linux/serial.h>
-    #define HAVE_HI_SPEED_SERIAL
-  #endif
+#if !defined (WIN32)
+  #include <unistd.h>
+  #include <arpa/inet.h> // for htons etc
 #endif
 
 #include <libplayercore/playercore.h>
-#include <replace/replace.h>
-extern PlayerTime* GlobalTime;
+//#include <replace/replace.h>
 
-#define DEFAULT_LASER_PORT "/dev/ttyS0"
-#define DEFAULT_LASER_TRANSFER_RATE 38400
+// 1 second of data at 500kbaud
+#define DEFAULT_RX_BUFFER_SIZE 500*1024/8
 
-#define RX_BUFFER_SIZE 4096
+#if defined (WIN32)
+  typedef unsigned int ssize_t;
+#endif
 
 // The laser device class.
-class SickS3000 : public Driver
+class SickS3000 : public ThreadedDriver
 {
   public:
-    
+
     // Constructor
     SickS3000(ConfigFile* cf, int section);
     ~SickS3000();
 
-    int Setup();
-    int Shutdown();
+    int MainSetup();
+    void MainQuit();
 
     // MessageHandler
-    int ProcessMessage(MessageQueue * resp_queue, 
-		       player_msghdr * hdr, 
+    int ProcessMessage(QueuePointer &resp_queue,
+		       player_msghdr * hdr,
 		       void * data);
   private:
 
     // Main function for device thread.
     virtual void Main();
 
-    // Open the terminal
-    // Returns 0 on success
-    int OpenTerm();
-
-    // Close the terminal
-    // Returns 0 on success
-    int CloseTerm();
-    
-    // Set the terminal speed
-	// can be any value valid for the s3000
-    // Returns 0 on success
-    int ChangeTermSpeed(int speed);
-
-    // Set the laser data rate
-    // Valid values are 9600 and 38400
-    // Returns 0 on success
-    int SetLaserSpeed(int speed);
-
-    // Read range data from laser
-    int ReadLaserData();
-
     // Process range data from laser
     int ProcessLaserData();
 
-    
+
     // Calculates CRC for a telegram
     unsigned short CreateCRC(uint8_t *data, ssize_t len);
 
     // Get the time (in ms)
     int64_t GetTime();
 
+    void SetScannerParams(int data_count);
+
   protected:
 
     // Laser pose in robot cs.
     double pose[3];
     double size[2];
-    
-    // Name of device used to communicate with the laser
-    const char *device_name;
-    
-    // laser device file descriptor
-    int laser_fd;           
 
     // Scan width and resolution.
     int scan_width, scan_res;
@@ -199,28 +163,19 @@ class SickS3000 : public Driver
     // Start and end scan angles (for restricted scan).  These are in
     // units of 0.01 degrees.
     int min_angle, max_angle;
-    
+
     // Start and end scan segments (for restricted scan).  These are
     // the values used by the laser.
     int scan_min_segment, scan_max_segment;
 
-    // Range resolution (1 = 1mm, 10 = 1cm, 100 = 10cm).
-    int range_res;
+    IntProperty mirror;
 
-    // Turn intensity data on/off
-    bool intensity;
+    bool recognisedScanner;
 
-    // Is the laser upside-down? (if so, we'll reverse the ordering of the
-    // readings)
-    int invert;
+    // Opaque Driver info
+    Device *opaque;
+    player_devaddr_t opaque_id;
 
-    bool can_do_hi_speed;
-    int connect_rate;  // Desired rate for first connection
-    int transfer_rate; // Desired rate for operation
-    int current_rate;  // Current rate
-
-    int scan_id;
-    
     // rx buffer
     uint8_t * rx_buffer;
     unsigned int rx_buffer_size;
@@ -229,11 +184,6 @@ class SickS3000 : public Driver
     // storage for outgoing data
     player_laser_data_t data_packet;
     player_laser_config_t config_packet;
-    
-
-#ifdef HAVE_HI_SPEED_SERIAL
-  struct serial_struct old_serial;
-#endif
 };
 
 // a factory creation function
@@ -243,7 +193,7 @@ Driver* SickS3000_Init(ConfigFile* cf, int section)
 }
 
 // a driver registration function
-void SickS3000_Register(DriverTable* table)
+void sicks3000_Register(DriverTable* table)
 {
   table->AddDriver("sicks3000", SickS3000_Init);
 }
@@ -260,29 +210,35 @@ void SickS3000_Register(DriverTable* table)
 ////////////////////////////////////////////////////////////////////////////////
 // Error macros
 #define RETURN_ERROR(erc, m) {PLAYER_ERROR(m); return erc;}
- 
+
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 SickS3000::SickS3000(ConfigFile* cf, int section)
-    : Driver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_LASER_CODE)
+    : ThreadedDriver(cf, section, false, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_LASER_CODE), mirror("mirror", 0, 0)
 {
+
+  rx_count = 0;
   // allocate our recieve buffer
-  rx_buffer_size = RX_BUFFER_SIZE;
+  rx_buffer_size = cf->ReadInt(section, "buffer_size", DEFAULT_RX_BUFFER_SIZE);
   rx_buffer = new uint8_t[rx_buffer_size];
   assert(rx_buffer);
-  
+
+  this->RegisterProperty ("mirror", &this->mirror, cf, section);
+
+  recognisedScanner = false;
+
   memset(&data_packet,0,sizeof(data_packet));
-  data_packet.min_angle = DTOR(-95);
-  data_packet.max_angle = DTOR(95);
-  data_packet.resolution = DTOR(0.25);
+  data_packet.min_angle = static_cast<float> (DTOR(-135));
+  data_packet.max_angle = static_cast<float> (DTOR(135));
+  data_packet.resolution = static_cast<float> (DTOR(0.5));
   data_packet.max_range = 49;
 
   memset(&config_packet,0,sizeof(config_packet));
-  config_packet.min_angle = DTOR(-95);
-  config_packet.max_angle = DTOR(95);
-  config_packet.resolution = DTOR(0.25);
+  config_packet.min_angle = static_cast<float> (DTOR(-135));
+  config_packet.max_angle = static_cast<float> (DTOR(135));
+  config_packet.resolution = static_cast<float> (DTOR(0.5));
   config_packet.max_range = 49;
-  
+
   // Laser geometry.
   this->pose[0] = cf->ReadTupleLength(section, "pose", 0, 0.0);
   this->pose[1] = cf->ReadTupleLength(section, "pose", 1, 0.0);;
@@ -290,89 +246,113 @@ SickS3000::SickS3000(ConfigFile* cf, int section)
   this->size[0] = 0.15;
   this->size[1] = 0.15;
 
-  // Serial port
-  this->device_name = cf->ReadString(section, "port", DEFAULT_LASER_PORT);
-
-  // Serial rate
-  this->transfer_rate = cf->ReadInt(section, "transfer_rate", DEFAULT_LASER_TRANSFER_RATE);
-  this->current_rate = 0;
-
-#ifdef HAVE_HI_SPEED_SERIAL
-  this->can_do_hi_speed = true;
-#else
-  this->can_do_hi_speed = false;
-#endif
-
-  if (!this->can_do_hi_speed && this->transfer_rate > 38400)
+  this->opaque = NULL;
+  // Must have an opaque device
+  if (cf->ReadDeviceAddr(&this->opaque_id, section, "requires",
+                       PLAYER_OPAQUE_CODE, -1, NULL) != 0)
   {
-    PLAYER_ERROR1("sicklms200: requested hi speed serial, but no support compiled in. Defaulting to %d bps.",
-		    DEFAULT_LASER_TRANSFER_RATE);
-    this->connect_rate = DEFAULT_LASER_TRANSFER_RATE;
+	puts ("No Opaque driver specified");
+    this->SetError(-1);
+    return;
   }
 
+
   return;
 }
 
 SickS3000::~SickS3000()
 {
-  delete [] rx_buffer; 
+  delete [] rx_buffer;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Set up scanner parameters based on number of results per scan
+void SickS3000::SetScannerParams(int data_count)
+{
+	if (data_count == 761) // sicks3000
+	{
+		data_packet.min_angle = static_cast<float> (DTOR(-95));
+		data_packet.max_angle = static_cast<float> (DTOR(95));
+		data_packet.resolution = static_cast<float> (DTOR(0.25));
+		data_packet.max_range = 49;
+
+		config_packet.min_angle = static_cast<float> (DTOR(-95));
+		config_packet.max_angle = static_cast<float> (DTOR(95));
+		config_packet.resolution = static_cast<float> (DTOR(0.25));
+		config_packet.max_range = 49;
+
+		recognisedScanner = true;
+	}
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device
-int SickS3000::Setup()
+int SickS3000::MainSetup()
 {
-  PLAYER_MSG1(2, "Laser initialising (%s)", this->device_name);
-    
-  // Open the terminal
-  if (OpenTerm())
-    return 1;
 
-  if (ChangeTermSpeed(this->transfer_rate))
+  PLAYER_MSG0(2, "Laser initialising");
+  // Subscribe to the opaque device.
+  if(Device::MatchDeviceAddress(this->opaque_id, this->device_addr))
   {
-    return 1;
+    PLAYER_ERROR("attempt to subscribe to self");
+    return(-1);
+  }
+
+  if(!(this->opaque = deviceTable->GetDevice(this->opaque_id)))
+  {
+    PLAYER_ERROR("unable to locate suitable opaque device");
+    return(-1);
+  }
+
+  if(this->opaque->Subscribe(this->InQueue) != 0)
+  {
+    PLAYER_ERROR("unable to subscribe to opaque device");
+    return(-1);
   }
 
   PLAYER_MSG0(2, "laser ready");
 
-  // Start the device thread
-  StartThread();
-
   return 0;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the device
-int SickS3000::Shutdown()
+void SickS3000::MainQuit()
 {
-  // shutdown laser device
-  StopThread();
-
-  CloseTerm();
-
+  opaque->Unsubscribe(InQueue);
   PLAYER_MSG0(2, "laser shutdown");
-  
-  return(0);
 }
 
 
-int 
-SickS3000::ProcessMessage(MessageQueue * resp_queue, 
+int
+SickS3000::ProcessMessage(QueuePointer &resp_queue,
                            player_msghdr * hdr,
                            void * data)
 {
+
+  if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, PLAYER_OPAQUE_DATA_STATE, opaque_id))
+  {
+    player_opaque_data_t * recv = reinterpret_cast<player_opaque_data_t * > (data);
+    unsigned int messageOffset = rx_count;
+    rx_count += recv->data_count;
+    if (rx_count > rx_buffer_size)
+    {
+      PLAYER_WARN("S3000 Buffer Full");
+      rx_count = 0;
+    }
+    else
+    {
+      memcpy(&rx_buffer[messageOffset], recv->data, recv->data_count);
+      ProcessLaserData();
+    }
+    return 0;
+  }
+
   if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ,
                                  PLAYER_LASER_REQ_GET_CONFIG,
                                  this->device_addr))
   {
-    if(hdr->size != 0)
-    {
-      PLAYER_ERROR2("request is wrong length (%d != %d); ignoring",
-                    hdr->size, 0);
-      return(-1);
-    }
     this->Publish(this->device_addr,
                   resp_queue,
                   PLAYER_MSGTYPE_RESP_ACK,
@@ -384,18 +364,13 @@ SickS3000::ProcessMessage(MessageQueue * resp_queue,
                                  PLAYER_LASER_REQ_GET_GEOM,
                                  this->device_addr))
   {
-    if(hdr->size != 0)
-    {
-      PLAYER_ERROR2("request is wrong length (%d != %d); ignoring",
-                    hdr->size, 0);
-      return(PLAYER_MSGTYPE_RESP_NACK);
-    }
     player_laser_geom_t geom;
-    geom.pose.px = this->pose[0];
-    geom.pose.py = this->pose[1];
-    geom.pose.pa = this->pose[2];
-    geom.size.sl = this->size[0];
-    geom.size.sw = this->size[1];
+    memset(&geom, 0, sizeof(geom));
+    geom.pose.px = pose[0];
+    geom.pose.py = pose[1];
+    geom.pose.pyaw = pose[2];
+    geom.size.sl = size[0];
+    geom.size.sw = size[1];
 
     this->Publish(this->device_addr,
                   resp_queue,
@@ -404,229 +379,36 @@ SickS3000::ProcessMessage(MessageQueue * resp_queue,
                   (void*)&geom, sizeof(geom), NULL);
     return(0);
   }
-
   // Don't know how to handle this message.
   return(-1);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Main function for device thread
-void SickS3000::Main() 
+void SickS3000::Main()
 {
   for(;;)
   {
+	// Waits for the opaque driver to pass data onto this thread.
+	InQueue->Wait();
+
     // test if we are supposed to cancel
     pthread_testcancel();
-    
+
     // process any pending messages
     ProcessMessages();
-    
-    // read data into our ring buffer and then process it
-    ReadLaserData();
-    ProcessLaserData();
+
+    //usleep(1000);
+
   }
 }
 
 
-
-////////////////////////////////////////////////////////////////////////////////
-// Open the terminal
-// Returns 0 on success
-int SickS3000::OpenTerm()
-{
-  this->laser_fd = ::open(this->device_name, O_RDWR | O_SYNC , S_IRUSR | S_IWUSR );
-  if (this->laser_fd < 0)
-  {
-    PLAYER_ERROR2("unable to open serial port [%s]; [%s]",
-                  (char*) this->device_name, strerror(errno));
-    return 1;
-  }
-
-  // set the serial port speed to 9600 to match the laser
-  // later we can ramp the speed up to the SICK's 38K
-  //
-  struct termios term;
-  if( tcgetattr( this->laser_fd, &term ) < 0 )
-    RETURN_ERROR(1, "Unable to get serial port attributes");
-  
-  cfmakeraw( &term );
-  cfsetispeed( &term, B9600 );
-  cfsetospeed( &term, B9600 );
-  
-  if( tcsetattr( this->laser_fd, TCSAFLUSH, &term ) < 0 )
-    RETURN_ERROR(1, "Unable to set serial port attributes");
-
-  // Make sure queue is empty
-  //
-  tcflush(this->laser_fd, TCIOFLUSH);
-    
-  return 0;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-// Close the terminal
-// Returns 0 on success
-//
-int SickS3000::CloseTerm()
-{
-  ::close(this->laser_fd);
-  return 0;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-// Set the terminal speed
-// Valid values are 9600 and 38400
-// Returns 0 on success
-//
-int SickS3000::ChangeTermSpeed(int speed)
-{
-  struct termios term;
-
-  current_rate = speed;
-
-#ifdef HAVE_HI_SPEED_SERIAL
-  struct serial_struct serial;
-
-  // we should check and reset the AYSNC_SPD_CUST flag
-  // since if it's set and we request 38400, we're likely
-  // to get another baud rate instead (based on custom_divisor)
-  // this way even if the previous player doesn't reset the
-  // port correctly, we'll end up with the right speed we want
-  if (ioctl(this->laser_fd, TIOCGSERIAL, &serial) < 0) 
-  {
-    //RETURN_ERROR(1, "error on TIOCGSERIAL in beginning");
-    PLAYER_WARN("ioctl() failed while trying to get serial port info");
-  }
-  else
-  {
-    serial.flags &= ~ASYNC_SPD_CUST;
-    serial.custom_divisor = 0;
-    if (ioctl(this->laser_fd, TIOCSSERIAL, &serial) < 0) 
-    {
-      //RETURN_ERROR(1, "error on TIOCSSERIAL in beginning");
-      PLAYER_WARN("ioctl() failed while trying to set serial port info");
-    }
-  }
-#endif  
-
-  //printf("LASER: change TERM speed: %d\n", speed);
-
-  int term_speed;
-  switch(speed)
-  {
-    case 9600:
-		term_speed = B9600;
-		break;
-	case 19200:
-		term_speed = B19200;
-		break;
-	case 38400:
-		term_speed = B38400;
-		break;
-	default:
-		term_speed = speed;
-  }
-
-  switch(term_speed)
-  {
-    case B9600:
-    case B19200:
-    case B38400:
-      //PLAYER_MSG0(2, "terminal speed to 9600");
-      if( tcgetattr( this->laser_fd, &term ) < 0 )
-        RETURN_ERROR(1, "unable to get device attributes");
-        
-      cfmakeraw( &term );
-	  cfsetispeed( &term, term_speed );
-	  cfsetospeed( &term, term_speed );
-        
-      if( tcsetattr( this->laser_fd, TCSAFLUSH, &term ) < 0 )
-        RETURN_ERROR(1, "unable to set device attributes");
-      break;
-
-    case 500000:
-      //PLAYER_MSG0(2, "terminal speed to 500000");
-
-#ifdef HAVE_HI_SPEED_SERIAL    
-      if (ioctl(this->laser_fd, TIOCGSERIAL, &this->old_serial) < 0) {
-        RETURN_ERROR(1, "error on TIOCGSERIAL ioctl");
-      }
-    
-      serial = this->old_serial;
-    
-      serial.flags |= ASYNC_SPD_CUST;
-      serial.custom_divisor = 48; // for FTDI USB/serial converter divisor is 240/5
-    
-      if (ioctl(this->laser_fd, TIOCSSERIAL, &serial) < 0) {
-        RETURN_ERROR(1, "error on TIOCSSERIAL ioctl");
-      }
-    
-#else
-      fprintf(stderr, "sicklms200: Trying to change to 500kbps, but no support compiled in, defaulting to 38.4kbps.\n");
-#endif
-
-      // even if we are doing 500kbps, we have to set the speed to 38400...
-      // the driver will know we want 500000 instead.
-
-      if( tcgetattr( this->laser_fd, &term ) < 0 )
-        RETURN_ERROR(1, "unable to get device attributes");    
-
-      cfmakeraw( &term );
-      cfsetispeed( &term, B38400 );
-      cfsetospeed( &term, B38400 );
-    
-      if( tcsetattr( this->laser_fd, TCSAFLUSH, &term ) < 0 )
-        RETURN_ERROR(1, "unable to set device attributes");
-    
-      break;
-    default:
-      PLAYER_ERROR1("unknown speed %d", speed);
-  }
-  return 0;
-}
-
-
-  
-
-
-
-////////////////////////////////////////////////////////////////////////////////
-// Read range data from laser
-//
-int SickS3000::ReadLaserData()
-{
-  if (rx_count == rx_buffer_size)
-  {
-    PLAYER_WARN("S3000 RX buffer full\n");
-    return 0; 
-  }
-  // Read a packet from the laser
-  //
-  //int len = ReadFromLaser(&rx_buffer[rx_count], rx_buffer_size - rx_count);
-  int len = read(this->laser_fd, &rx_buffer[rx_count], rx_buffer_size - rx_count);
-  if (len == 0)
-  {
-    PLAYER_MSG0(2, "empty packet");
-    return 1;
-  }
-  if (len < 0)
-  {
-    PLAYER_ERROR2("error reading form s3000: %d %s", errno, strerror(errno));
-    return -1;
-  }
-
-  rx_count += len;
-   
-  return 0;
-}
-
-
 int SickS3000::ProcessLaserData()
 {
   while(rx_count >= 22)
   {
+
     // find our continuous data header
     unsigned int ii;
     bool found = false;
@@ -646,23 +428,29 @@ int SickS3000::ProcessLaserData()
       rx_count -= ii;
       return 0;
     }
-    
-    // get relevant bits of the header 
+
+    // get relevant bits of the header
     // size includes all data from the data block number
     // through to the end of the packet including the checksum
     unsigned short size = 2*htons(*reinterpret_cast<unsigned short *> (&rx_buffer[6]));
-    
+    //printf("size %d", size);
+    if (size > rx_buffer_size - 26)
+    {
+      PLAYER_WARN("S3000: Requested Size of data is larger than the buffer size");
+      memmove(rx_buffer, &rx_buffer[1], --rx_count);
+      return 0;
+    }
+
     // check if we have enough data yet
     if (size > rx_count - 4)
       return 0;
-      
+
     unsigned short packet_checksum = *reinterpret_cast<unsigned short *> (&rx_buffer[size+2]);
     unsigned short calc_checksum = CreateCRC(&rx_buffer[4], size-2);
     if (packet_checksum != calc_checksum)
     {
-      PLAYER_WARN("Checksum's dont match, thats bad\n");
-      memmove(rx_buffer, &rx_buffer[1], 1);
-      rx_count -= 1;
+      PLAYER_WARN1("S3000: Checksum's dont match, thats bad (data packet size %d)\n",size);
+      memmove(rx_buffer, &rx_buffer[1], --rx_count);
       continue;
     }
     else
@@ -676,26 +464,39 @@ int SickS3000::ProcessLaserData()
       {
         if (data[0] == 0xAA)
         {
-          PLAYER_WARN("We got a I/O data packet we dont know what to do with it\n");
+          PLAYER_WARN("S3000: We got a I/O data packet we dont know what to do with it\n");
         }
         else if (data[0] == 0xBB)
         {
           int data_count = (size - 22) / 2;
+          if (data_count < 0)
+          {
+            PLAYER_WARN1("S3000: bad data count (%d)\n", data_count);
+            memmove(rx_buffer, &rx_buffer[size+4], rx_count - (size+4));
+            rx_count -= (size + 4);
+            continue;
+          }
+          if (!recognisedScanner)
+            SetScannerParams(data_count); // Set up parameters based on number of results.
           data_packet.ranges_count = data_count;
+          data_packet.ranges = new float [data_count];
           for (int ii = 0; ii < data_count; ++ii)
           {
             unsigned short Distance_CM = (*reinterpret_cast<unsigned short *> (&data[4 + 2*ii]));
             Distance_CM &= 0x1fff; // remove status bits
             double distance_m = static_cast<double>(Distance_CM)/100.0;
-            data_packet.ranges[ii] = distance_m;
+            if (mirror == 1)
+            	data_packet.ranges[data_count - ii - 1] = static_cast<float> (distance_m); // Reverse order.
+            else
+            	data_packet.ranges[ii] = static_cast<float> (distance_m);
           }
-          
+
           this->Publish(this->device_addr,
-                        NULL,
                         PLAYER_MSGTYPE_DATA,
                         PLAYER_LASER_DATA_SCAN,
-                        (void*)&data_packet, sizeof(data_packet), NULL);
-          
+                        (void*)&data_packet);
+          delete [] data_packet.ranges;
+
         }
         else if (data[0] == 0xCC)
         {
@@ -707,8 +508,8 @@ int SickS3000::ProcessLaserData()
         }
       }
     }
-      
-    memmove(rx_buffer, &rx_buffer[size+4], size+4);
+
+    memmove(rx_buffer, &rx_buffer[size+4], rx_count - (size+4));
     rx_count -= (size + 4);
     continue;
   }
@@ -750,7 +551,7 @@ static const unsigned short crc_table[256] = {
   0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
   0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
 };
-           
+
 unsigned short SickS3000::CreateCRC(uint8_t *Data, ssize_t length)
 {
   unsigned short CRC_16 = 0xFFFF;
@@ -761,37 +562,5 @@ unsigned short SickS3000::CreateCRC(uint8_t *Data, ssize_t length)
   }
   return CRC_16;
 }
-           
-           
-           
-/*           
-////////////////////////////////////////////////////////////////////////////////
-// Create a CRC for the given packet
-//
-unsigned short SickS3000::CreateCRC(uint8_t* data, ssize_t len)
-{
-  uint16_t uCrc16;
-  uint8_t abData[2];
-  
-  uCrc16 = 0;
-  abData[0] = 0;
-  
-  while(len-- )
-  {
-    abData[1] = abData[0];
-    abData[0] = *data++;
-    
-    if( uCrc16 & 0x8000 )
-    {
-      uCrc16 = (uCrc16 & 0x7fff) << 1;
-      uCrc16 ^= CRC16_GEN_POL;
-    }
-    else
-    {    
-      uCrc16 <<= 1;
-    }
-    uCrc16 ^= MAKEUINT16(abData[0],abData[1]);
-  }
-  return (uCrc16); 
-}
-*/
+
+
