@@ -22,7 +22,7 @@
 // Desc: Adaptive Monte-Carlo localization
 // Author: Andrew Howard
 // Date: 6 Feb 2003
-// CVS: $Id: amcl.cc,v 1.70.2.3 2007/01/24 18:02:33 gerkey Exp $
+// CVS: $Id: amcl.cc 8003 2009-07-13 10:34:37Z thjc $
 //
 // Theory of operation:
 //  TODO
@@ -276,9 +276,7 @@ changed to match your particular configuration.
 
 /** @} */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <assert.h>
 #include <errno.h>
@@ -286,14 +284,15 @@ changed to match your particular configuration.
 #include <math.h>
 #include <stdlib.h>       // for atoi(3)
 #include <sys/types.h>
-#include <unistd.h>
-#include <sys/time.h>
+#if !defined (WIN32)
+  #include <unistd.h>
+  #include <sys/time.h>
+#endif
 
 #define PLAYER_ENABLE_TRACE 1
 #define PLAYER_ENABLE_MSG 1
 
 #include <libplayercore/playercore.h>
-#include <libplayercore/error.h>
 
 #include "amcl.h"
 // Sensors
@@ -303,6 +302,7 @@ changed to match your particular configuration.
 //#include "amcl_gps.h"
 //#include "amcl_imu.h"
 
+#define MAX_HYPS 8
 
 ////////////////////////////////////////////////////////////////////////////////
 // Create an instance of the driver
@@ -314,7 +314,7 @@ Driver* AdaptiveMCL_Init( ConfigFile* cf, int section)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Register the driver
-void AdaptiveMCL_Register(DriverTable* table)
+void amcl_Register(DriverTable* table)
 {
   table->AddDriver("amcl", AdaptiveMCL_Init);
   return;
@@ -328,7 +328,7 @@ void AdaptiveMCL_Register(DriverTable* table)
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 AdaptiveMCL::AdaptiveMCL( ConfigFile* cf, int section)
-    : Driver(cf, section)
+    : ThreadedDriver(cf, section)
 {
   int i;
   double u[3];
@@ -462,6 +462,8 @@ AdaptiveMCL::AdaptiveMCL( ConfigFile* cf, int section)
 
   // Initial hypothesis list
   this->hyp_count = 0;
+  this->hyp_alloc = 0;
+  this->hyps = NULL;
   pthread_mutex_init(&this->best_hyp_lock,NULL);
 
 #ifdef INCLUDE_RTKGUI
@@ -482,6 +484,7 @@ AdaptiveMCL::~AdaptiveMCL(void)
 
   // Delete sensor data queue
   delete[] this->q_data;
+  free(hyps);
 
   // Delete sensors
   for (i = 0; i < this->sensor_count; i++)
@@ -498,7 +501,7 @@ AdaptiveMCL::~AdaptiveMCL(void)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device (called by server thread).
-int AdaptiveMCL::Setup(void)
+int AdaptiveMCL::MainSetup(void)
 {
   PLAYER_MSG0(2, "setup");
 
@@ -518,35 +521,11 @@ int AdaptiveMCL::Setup(void)
 
 
   // Start the driver thread.
-  PLAYER_MSG0(2, "running");
-  this->StartThread();
-
+  PLAYER_MSG0(2, "AMCL running");
   return 0;
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-// Shutdown the device (called by server thread).
-int AdaptiveMCL::Shutdown(void)
-{
-  int i;
-
-  PLAYER_MSG0(2, "shutting down");
-
-  // Stop the driver thread.
-  this->StopThread();
-
-  // Stop sensors
-  for (i = 0; i < this->sensor_count; i++)
-    this->sensors[i]->Shutdown();
-
-  // Delete the particle filter
-  pf_free(this->pf);
-  this->pf = NULL;
-
-  PLAYER_MSG0(2, "shutdown done");
-  return 0;
-}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -593,19 +572,14 @@ void AdaptiveMCL::Push(AMCLSensorData *data)
 {
   int i;
 
-  this->Lock();
-
   if (this->q_len >= this->q_size)
   {
-    this->Unlock();
     PLAYER_ERROR("queue overflow");
     return;
   }
   i = (this->q_start + this->q_len++) % this->q_size;
 
   this->q_data[i] = data;
-
-  this->Unlock();
   return;
 }
 
@@ -616,16 +590,11 @@ AMCLSensorData *AdaptiveMCL::Peek(void)
 {
   int i;
 
-  this->Lock();
-
   if (this->q_len == 0)
   {
-    this->Unlock();
     return NULL;
   }
   i = this->q_start % this->q_size;
-
-  this->Unlock();
   return this->q_data[i];
 }
 
@@ -636,17 +605,13 @@ AMCLSensorData *AdaptiveMCL::Pop(void)
 {
   int i;
 
-  this->Lock();
-
   if (this->q_len == 0)
   {
-    this->Unlock();
     return NULL;
   }
   i = this->q_start++ % this->q_size;
   this->q_len--;
 
-  this->Unlock();
   return this->q_data[i];
 }
 
@@ -661,12 +626,16 @@ void AdaptiveMCL::Main(void)
   // filter has not yet been initialized
   this->pf_init = false;
 
+  this->pf_init_internal = false;
+
   // Initial hypothesis list
   this->hyp_count = 0;
 
+#if !defined (WIN32)
   // WARNING: this only works for Linux
   // Run at a lower priority
   nice(10);
+#endif
 
 #ifdef INCLUDE_RTKGUI
   pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
@@ -709,8 +678,11 @@ void AdaptiveMCL::Main(void)
 #endif
 
     // Initialize the filter if we havent already done so
-    if (!this->pf_init)
+    if (!this->pf_init && !this->pf_init_internal)
+    {
       this->InitFilter();
+      this->pf_init_internal = true;
+    }
 
     // Update the filter
     if (this->UpdateFilter())
@@ -751,6 +723,19 @@ void AdaptiveMCL::MainQuit()
   fclose(this->outfile);
 #endif
 
+  int i;
+
+  PLAYER_MSG0(2, "shutting down");
+  // Stop sensors
+  for (i = 0; i < this->sensor_count; i++)
+    this->sensors[i]->Shutdown();
+
+  // Delete the particle filter
+  pf_free(this->pf);
+  this->pf = NULL;
+
+  PLAYER_MSG0(2, "shutdown done");  
+  
   return;
 }
 
@@ -875,6 +860,9 @@ bool AdaptiveMCL::UpdateFilter(void)
           {
             data = this->Pop();
             assert(data);
+            // Catch the pose and timestamp of the discarded action data
+            //pose = ((AMCLOdomData*) data)->pose;
+            //ts = data->time;
             delete data;
           }
           // avoid a busy loop while waiting for a sensor reading to
@@ -915,15 +903,21 @@ bool AdaptiveMCL::UpdateFilter(void)
 
     // Read out the current hypotheses
     double max_weight = 0.0;
-    pf_vector_t max_weight_pose;
+    pf_vector_t max_weight_pose={{0.0,0.0,0.0}};
     this->hyp_count = 0;
-    for (i = 0; (size_t) i < sizeof(this->hyps) / sizeof(this->hyps[0]); i++)
+    //for (i = 0; (size_t) i < sizeof(this->hyps) / sizeof(this->hyps[0]); i++)
+    for (i = 0; MAX_HYPS; i++)
     {
       if (!pf_get_cluster_stats(this->pf, i, &weight, &pose_mean, &pose_cov))
         break;
 
       //pf_vector_fprintf(pose_mean, stdout, "%.3f");
 
+      if (this->hyp_count +1 > this->hyp_alloc)
+      {
+        this->hyp_alloc = this->hyp_count+1;
+        this->hyps = (amcl_hyp_t*)realloc(this->hyps, sizeof(amcl_hyp_t)*this->hyp_alloc);
+      }
       hyp = this->hyps + this->hyp_count++;
       hyp->weight = weight;
       hyp->pf_pose_mean = pose_mean;
@@ -1004,7 +998,7 @@ void AdaptiveMCL::PutDataLocalize(double time)
   amcl_hyp_t *hyp;
   pf_vector_t pose;
   pf_matrix_t pose_cov;
-  size_t datalen;
+  //size_t datalen;
   player_localize_data_t data;
 
   // Record the number of pending observations
@@ -1013,6 +1007,7 @@ void AdaptiveMCL::PutDataLocalize(double time)
 
   // Encode the hypotheses
   data.hypoths_count = this->hyp_count;
+  data.hypoths = new player_localize_hypoth_t[data.hypoths_count];
   for (i = 0; i < this->hyp_count; i++)
   {
     hyp = this->hyps + i;
@@ -1048,15 +1043,12 @@ void AdaptiveMCL::PutDataLocalize(double time)
   qsort((void*)data.hypoths,data.hypoths_count,
         sizeof(player_localize_hypoth_t),&hypoth_compare);
 
-  // Compute the length of the data packet
-  datalen = (sizeof(data) - sizeof(data.hypoths) +
-             data.hypoths_count * sizeof(data.hypoths[0]));
-
   // Push data out
-  this->Publish(this->localize_addr, NULL,
+  this->Publish(this->localize_addr,
                 PLAYER_MSGTYPE_DATA,
                 PLAYER_LOCALIZE_DATA_HYPOTHS,
-                (void*)&data,datalen,&time);
+                (void*)&data);
+  delete [] data.hypoths;
 }
 
 
@@ -1082,7 +1074,7 @@ void AdaptiveMCL::PutDataPosition(pf_vector_t delta, double time)
   data.pos.pa = pose.v[2];
 
   // Push data out
-  this->Publish(this->position_addr, NULL,
+  this->Publish(this->position_addr,
                 PLAYER_MSGTYPE_DATA,
                 PLAYER_POSITION2D_DATA_STATE,
                 (void*)&data,sizeof(data),&time);
@@ -1090,7 +1082,7 @@ void AdaptiveMCL::PutDataPosition(pf_vector_t delta, double time)
 
 // MessageHandler
 int
-AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
+AdaptiveMCL::ProcessMessage(QueuePointer & resp_queue,
                             player_msghdr * hdr,
                             void * data)
 {
@@ -1101,12 +1093,6 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
                            PLAYER_LOCALIZE_REQ_SET_POSE,
                            this->localize_addr))
   {
-    if(hdr->size != sizeof(player_localize_set_pose_t))
-    {
-      PLAYER_ERROR2("request is wrong length (%d != %d); ignoring",
-                    hdr->size, sizeof(player_localize_set_pose_t));
-      return(-1);
-    }
     setposereq = (player_localize_set_pose_t*)data;
 
     pf_vector_t pose;
@@ -1125,6 +1111,7 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
     this->pf_init_pose_mean = pose;
     this->pf_init_pose_cov = cov;
     this->pf_init = false;
+    this->pf_init_internal = false;
 
     // Send an ACK
     this->Publish(this->localize_addr, resp_queue,
@@ -1137,14 +1124,6 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
                                 PLAYER_LOCALIZE_REQ_GET_PARTICLES,
                                 this->localize_addr))
   {
-    if(hdr->size != 0)
-    {
-      PLAYER_ERROR2("request is wrong length (%d != %d); ignoring",
-                    hdr->size, 0);
-      return(PLAYER_MSGTYPE_RESP_NACK);
-    }
-
-
     pf_vector_t mean;
     double var;
     player_localize_get_particles_t resp;
@@ -1161,8 +1140,8 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
 
     set = this->pf->sets + this->pf->current_set;
 
-    resp.particles_count =
-            MIN(set->sample_count,PLAYER_LOCALIZE_PARTICLES_MAX);
+    resp.particles_count = set->sample_count;
+    resp.particles = new player_localize_particle_t [resp.particles_count];
 
     // TODO: pick representative particles
     for(i=0;i<resp.particles_count;i++)
@@ -1177,7 +1156,8 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
     this->Publish(this->localize_addr, resp_queue,
                   PLAYER_MSGTYPE_RESP_ACK,
                   PLAYER_LOCALIZE_REQ_GET_PARTICLES,
-                  (void*)&resp, sizeof(resp), NULL);
+                  (void*)&resp);
+    delete [] resp.particles;
     return(0);
   } else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ,
                                   PLAYER_POSITION2D_REQ_GET_GEOM, device_addr))
@@ -1199,13 +1179,10 @@ AdaptiveMCL::ProcessMessage(MessageQueue * resp_queue,
 }
 
 void
-AdaptiveMCL::ProcessGeom(MessageQueue* resp_queue, player_msghdr_t* hdr)
+AdaptiveMCL::ProcessGeom(QueuePointer &resp_queue, player_msghdr_t* hdr)
 {
-  player_position2d_geom_t geom;
+  player_position2d_geom_t geom={{0}};
   // just return a point so we don't get errors from playerv
-  geom.pose.px = 0;
-  geom.pose.py = 0;
-  geom.pose.pa = 0;
   geom.size.sl = 0.01;
   geom.size.sw = 0.01;
 
@@ -1333,8 +1310,11 @@ void AdaptiveMCL::DrawPoseEst()
   amcl_hyp_t *hyp;
   pf_vector_t pose;
 
-  this->Lock();
+  pthread_mutex_lock(&this->best_hyp_lock);
+  pose = this->best_hyp;
+  pthread_mutex_unlock(&this->best_hyp_lock);
 
+  /*
   max_weight = -1;
   for (i = 0; i < this->hyp_count; i++)
   {
@@ -1346,11 +1326,12 @@ void AdaptiveMCL::DrawPoseEst()
       pose = hyp->pf_pose_mean;
     }
   }
+  */
 
-  this->Unlock();
-
+  /*
   if (max_weight < 0.0)
     return;
+    */
 
   // Shift the robot figure
   rtk_fig_origin(this->robot_fig, pose.v[0], pose.v[1], pose.v[2]);
