@@ -3,7 +3,7 @@
 /**
   *  Copyright (C) 2006
   *     Videre Design
-  *  Copyright (C) 2000  
+  *  Copyright (C) 2000
   *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
   *
   *  Videre Erratic robot driver for Player
@@ -51,7 +51,7 @@ them named:
   - Returns the current battery voltage (12 V when fully charged).
 
 - @ref interface_aio
-  - Returns data from analog and digital input pins 
+  - Returns data from analog and digital input pins
 
 - @ref interface_ir
   - Returns ranges from IR sensors, assuming they're connected to the analog input pins
@@ -70,7 +70,7 @@ them named:
   - PLAYER_POSITION_GET_GEOM_REQ
   - PLAYER_POSITION_VELOCITY_MODE_REQ
 - @ref interface_ir :
-  - PLAYER_IR_POSE_REQ
+  - PLAYER_IR_REQ_POSE
 - @ref interface_sonar :
   - PLAYER_SONAR_GET_GEOM_REQ
 
@@ -147,7 +147,7 @@ driver
              "sonar:0"
              "aio:0"
              "ir:0"
-             "ptz:0" 
+             "ptz:0"
              "ptz:1" ]
 
   port "/dev/erratic"
@@ -176,12 +176,12 @@ driver
 #include <unistd.h>
 #include <math.h>
 #include <termios.h>
+#include <stdlib.h> // for abs()
+#include <sys/stat.h>
 
 #include "erratic.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 bool debug_mode = FALSE;
 
@@ -190,12 +190,12 @@ bool debug_mode = FALSE;
 /** Setting up, tearing down **/
 
 // These get the driver in where it belongs, loading
-Driver* Erratic_Init(ConfigFile* cf, int section) 
+Driver* Erratic_Init(ConfigFile* cf, int section)
 {
   return (Driver*)(new Erratic(cf,section));
 }
 
-void Erratic_Register(DriverTable* table) 
+void erratic_Register(DriverTable* table)
 {
   table->AddDriver("erratic", Erratic_Init);
 }
@@ -214,8 +214,8 @@ extern "C" {
 #endif
 
 // Constructor of the driver from configuration entry
-Erratic::Erratic(ConfigFile* cf, int section) 
-  : Driver(cf,section,true,PLAYER_MSGQUEUE_DEFAULT_MAXLEN) 
+Erratic::Erratic(ConfigFile* cf, int section)
+  : Driver(cf,section,true,PLAYER_MSGQUEUE_DEFAULT_MAXLEN)
 {
   // zero ids, so that we'll know later which interfaces were requested
   memset(&this->position_id, 0, sizeof(player_devaddr_t));
@@ -237,6 +237,7 @@ Erratic::Erratic(ConfigFile* cf, int section)
   // intialise members
   motor_packet = NULL;
   mcount = 0;
+  memset(&this->erratic_data,0,sizeof(this->erratic_data));
 
   // Do we create a robot position interface?
   if(cf->ReadDeviceAddr(&(this->position_id), section, "provides", PLAYER_POSITION2D_CODE, -1, NULL) == 0) {
@@ -253,7 +254,7 @@ Erratic::Erratic(ConfigFile* cf, int section)
       return;
     }
   }
-  
+
   // Do we create a aio interface?
   if(cf->ReadDeviceAddr(&(this->aio_id), section, "provides", PLAYER_AIO_CODE, -1, NULL) == 0) {
     if(this->AddInterface(this->aio_id) != 0) {
@@ -278,17 +279,21 @@ Erratic::Erratic(ConfigFile* cf, int section)
     }
   }
 
-  // Do we create a ptz interface?
-  if(cf->ReadDeviceAddr(&(this->ptz_id), section, "provides", PLAYER_PTZ_CODE, -1, NULL) == 0) {
-    if(this->AddInterface(this->ptz_id) != 0) {
+  // Do we create a tilt-only ptz interface?
+	// Must have a key of "tilt", e.g., "tilt:::ptz:1"
+	// This device must be listed after the default PTZ device
+  if(cf->ReadDeviceAddr(&(this->ptz2_id), section, "provides", PLAYER_PTZ_CODE, -1, "tilt") == 0) {
+    if(this->AddInterface(this->ptz2_id) != 0) {
       this->SetError(-1);
       return;
     }
   }
 
-  // Do we create the second ptz interface?
-  if(cf->ReadDeviceAddr(&(this->ptz2_id), section, "provides", PLAYER_PTZ_CODE, -1, "tilt") == 0) {
-    if(this->AddInterface(this->ptz2_id) != 0) {
+  // Do we create a standard ptz interface?
+	// NOTE: this interface has no key, it picks up the default PTZ device
+	// MUST BE LISTED FIRST of all PTZ devices
+  if(cf->ReadDeviceAddr(&(this->ptz_id), section, "provides", PLAYER_PTZ_CODE, -1, NULL) == 0) {
+    if(this->AddInterface(this->ptz_id) != 0) {
       this->SetError(-1);
       return;
     }
@@ -327,13 +332,13 @@ Erratic::Erratic(ConfigFile* cf, int section)
 
   this->read_fd = -1;
   this->write_fd = -1;
-  
+
   // Data mutexes and semaphores
   pthread_mutex_init(&send_queue_mutex, 0);
   pthread_cond_init(&send_queue_cond, 0);
   pthread_mutex_init(&motor_packet_mutex, 0);
-  
-  if (Connect()) 
+
+  if (Connect())
     {
       printf("Error connecting to Erratic robot\n");
       exit(1);
@@ -347,7 +352,7 @@ int Erratic::Setup() {
 }
 
 // Establishes connection and starts threads
-int Erratic::Connect() 
+int Erratic::Connect()
 {
   printf("  Erratic connection initializing (%s)...",this->psos_serial_port); fflush(stdout);
 
@@ -372,13 +377,13 @@ int Erratic::Connect()
       close(this->read_fd); close(this->write_fd);
       this->read_fd = -1; this->write_fd = -1;
       return(1);
-    } 
-    
+    }
+
     cfmakeraw(&read_term);
     //cfsetspeed(&read_term, bauds[currbaud]);
     cfsetispeed(&read_term, bauds[currbaud]);
     cfsetospeed(&read_term, bauds[currbaud]);
-      
+
     if(tcsetattr(this->read_fd, TCSAFLUSH, &read_term ) < 0) {
       perror("Erratic::Setup():tcsetattr(read channel):");
       close(this->read_fd); close(this->write_fd);
@@ -386,9 +391,9 @@ int Erratic::Connect()
       return(1);
     }
   }
-  
-  
-  // Empty buffers 
+
+
+  // Empty buffers
   if (tcflush(this->read_fd, TCIOFLUSH ) < 0) {
     perror("Erratic::Setup():tcflush():");
     close(this->read_fd); close(this->write_fd);
@@ -401,7 +406,7 @@ int Erratic::Connect()
     this->read_fd = -1; this->write_fd = -1;
     return(1);
   }
-  
+
   // will send config packets until a response is gotten
   int num_sync_attempts = 10;
   int num_patience = 200;
@@ -496,9 +501,9 @@ int Erratic::Connect()
   }
 
   char name[20], type[20], subtype[20];
-  sprintf(name, "%s", &received_packet.packet[5]);
-  sprintf(type, "%s", &received_packet.packet[25]);
-  sprintf(subtype, "%s", &received_packet.packet[45]);
+  snprintf(name, 20, "%s", &received_packet.packet[5]);
+  snprintf(type, 20, "%s", &received_packet.packet[25]);
+  snprintf(subtype, 20, "%s", &received_packet.packet[45]);
 
   // Open the driver, and tickle it a bit
   {
@@ -518,9 +523,11 @@ int Erratic::Connect()
   printf(" done.\n  Connected to <%s>, an %s %s\n", name, type, subtype);
 
   // Set the robot type
-	if (subtype == "Rev G")
+	if (strcmp(subtype,"Rev H")==0)
+		param_idx = 2;
+	else 	if (strcmp(subtype,"Rev G")==0)
 		param_idx = 1;
-	else if (subtype == "Rev E")
+	else 	if (strcmp(subtype,"Rev E")==0)
 		param_idx = 0;
 	else
 		param_idx = 0;
@@ -561,7 +568,7 @@ int Erratic::Connect()
         accel_packet->Build(accel_command, 4);
         this->Send(accel_packet);
     }
-    
+
     if(this->motor_max_rot_accel != 0) {
       accel_packet = new ErraticPacket();
       accel_command[0] = (command_e)set_max_rot_acc;
@@ -589,7 +596,7 @@ int Erratic::Connect()
         this->Send(accel_packet);
     }
   }
-  
+
   {
     ErraticPacket *packet;
     unsigned char payload[4];
@@ -681,7 +688,7 @@ int Erratic::Connect()
       this->Send(packet);
     }
   }
-  
+
   // TODO: figure out what the right behavior here is
 
   // zero position command buffer
@@ -698,23 +705,23 @@ int Erratic::Connect()
 
 // Called by player when the driver is supposed to disconnect
 int Erratic::Shutdown() {
-  // We don't care, we'll never disconnect
+  // kill with destructor instead of here
   return 0;
 }
 
 // Is theoretically able to disconnect, but we don't do that
 int Erratic::Disconnect() {
   printf("Shutting Erratic driver down\n");
-  
+
   this->StopThreads();
-  
+
   // If we're connected, send some kill commands before killing connections
   if (this->write_fd > -1) {
     unsigned char command[20],buffer[20];
     ErraticPacket packet;
 
     memset(buffer,0,20);
-  
+
     command[0] = (command_e)stop;
     packet.Build(command, 1);
     packet.Send(this->write_fd);
@@ -724,7 +731,7 @@ int Erratic::Disconnect() {
     packet.Build(command, 1);
     packet.Send(this->write_fd);
     usleep(ROBOT_CYCLETIME);
-  
+
     close(this->write_fd);
     this->write_fd = -1;
   }
@@ -737,7 +744,7 @@ int Erratic::Disconnect() {
     delete this->motor_packet;
     this->motor_packet = NULL;
   }
-  
+
   printf("Erratic has been shut down");
 
   return(0);
@@ -745,7 +752,6 @@ int Erratic::Disconnect() {
 
 // These call the supplied Driver::StartThread() method, but adds additional threads
 void Erratic::StartThreads() {
-  StartThread();
   pthread_create(&send_thread, 0, &SendThreadDummy, this);
   pthread_create(&receive_thread, 0, &ReceiveThreadDummy, this);
 }
@@ -754,28 +760,27 @@ void Erratic::StopThreads() {
   pthread_cancel(receive_thread);
   //TODO destroy threads?
   //TODO tickle threads to cancel point
-  StopThread();
-  
+
   // Cleanup mutexes
   pthread_mutex_trylock(&send_queue_mutex);
   pthread_mutex_unlock(&send_queue_mutex);
-  
+
   pthread_mutex_trylock(&motor_packet_mutex);
   pthread_mutex_unlock(&motor_packet_mutex);
 }
 
 // Subscription is overridden to add a subscription count of our own
 int Erratic::Subscribe(player_devaddr_t id) {
-  int setupResult;                     
-                                       
-  // do the subscription               
+  int setupResult;
+
+  // do the subscription
   if((setupResult = Driver::Subscribe(id)) == 0)
   {
     // also increment the appropriate subscription counters
 
     if(Device::MatchDeviceAddress(id, this->position_id))
       this->position_subscriptions++;
-    
+
     if(Device::MatchDeviceAddress(id, this->aio_id))
       this->aio_ir_subscriptions++;
 
@@ -790,10 +795,10 @@ int Erratic::Subscribe(player_devaddr_t id) {
 
     if(Device::MatchDeviceAddress(id, this->ptz2_id))
       this->ptz2_subscriptions++;
-  }                                    
-                                       
-  return(setupResult);                 
-}                                      
+  }
+
+  return(setupResult);
+}
 int Erratic::Unsubscribe(player_devaddr_t id) {
   int shutdownResult;
 
@@ -833,7 +838,7 @@ int Erratic::Unsubscribe(player_devaddr_t id) {
 /** Talking to the robot **/
 
 // Listens to the robot
-void Erratic::ReceiveThread() 
+void Erratic::ReceiveThread()
 {
   for (;;) {
     pthread_testcancel();
@@ -847,21 +852,19 @@ void Erratic::ReceiveThread()
       waited += 5;
       printf("Lost serial communication with Erratic (%d) - no data received for %i seconds\n", error_code, waited);
     }
-		
+
     if (waited)
       printf("Connection re-established\n");
-		
-		
+
+
     if (print_all_packets) {
       printf("Got: ");
       packet.PrintHex();
     }
-		
+
     // Process the packet
-    //Lock();
-		
     int count, maxcount;
-    switch(packet.packet[3]) 
+    switch(packet.packet[3])
       {
       case (reply_e)motor:
       case (reply_e)motor+2:
@@ -878,14 +881,32 @@ void Erratic::ReceiveThread()
 
       case (reply_e)ain:
 	// This data goes in two places, analog input and ir rangers
-	erratic_data.aio.voltages_count = packet.packet[4];
-	erratic_data.ir.voltages_count = RobotParams[this->param_idx]->NumIR;
-	erratic_data.ir.ranges_count = RobotParams[this->param_idx]->NumIR;;
+        if(erratic_data.aio.voltages_count != (unsigned int)(packet.packet[4]+4))
+        {
+          erratic_data.aio.voltages_count = packet.packet[4]+4;
+          if(erratic_data.aio.voltages)
+            delete[] erratic_data.aio.voltages;
+          erratic_data.aio.voltages = new float[erratic_data.aio.voltages_count];
+        }
+        if(erratic_data.ir.voltages_count !=
+           (unsigned int)RobotParams[this->param_idx]->NumIR)
+        {
+          erratic_data.ir.voltages_count = RobotParams[this->param_idx]->NumIR;
+          if(erratic_data.ir.voltages)
+            delete[] erratic_data.ir.voltages;
+          erratic_data.ir.voltages = new float[erratic_data.ir.voltages_count];
+        }
+	if(erratic_data.ir.ranges_count !=
+           (unsigned int)RobotParams[this->param_idx]->NumIR)
+        {
+          erratic_data.ir.ranges_count = RobotParams[this->param_idx]->NumIR;
+          if(erratic_data.ir.ranges)
+            delete[] erratic_data.ir.ranges;
+          erratic_data.ir.ranges = new float[erratic_data.ir.ranges_count];
+        }
 	unsigned int i_voltage;
-	for (i_voltage = 0; i_voltage < erratic_data.aio.voltages_count ;i_voltage++) 
+	for (i_voltage = 0; i_voltage < erratic_data.aio.voltages_count ;i_voltage++)
 	  {
-	    if (i_voltage >= PLAYER_AIO_MAX_INPUTS)
-	      continue;
 	    erratic_data.aio.voltages[i_voltage] = (packet.packet[5+i_voltage*2]
 						    + 256*packet.packet[6+i_voltage*2]) * (1.0 / 1024.0) * CPU_VOLTAGE;
 	    erratic_data.ir.voltages[i_voltage] = (packet.packet[5+i_voltage*2]
@@ -894,20 +915,17 @@ void Erratic::ReceiveThread()
 	  }
 
 	// add digital inputs, four E port bits
-	erratic_data.aio.voltages_count += 4;
-	for (int i=0; i < 4; i++) 
+	for (int i=0; i < 4; i++)
 	  {
-	    if (i_voltage >= PLAYER_AIO_MAX_INPUTS)
-	      continue;
-	    erratic_data.aio.voltages[i_voltage+i] = 
-	      (packet.packet[5+i_voltage*2] & (0x1 << i+4)) ? 1.0 : 0.0;
-	    erratic_data.ir.voltages[i] = 
-	      (packet.packet[5+i_voltage*2] & (0x1 << i+4)) ? 1.0 : 0.0;
-	    erratic_data.ir.ranges[i] = 
+	    erratic_data.aio.voltages[i_voltage+i] =
+	      (packet.packet[5+i_voltage*2] & (0x1 << (i+4))) ? 1.0 : 0.0;
+	    erratic_data.ir.voltages[i] =
+	      (packet.packet[5+i_voltage*2] & (0x1 << (i+4))) ? 1.0 : 0.0;
+	    erratic_data.ir.ranges[i] =
 	      IRFloorRange(erratic_data.ir.voltages[i]);
 	  }
 
-					
+
 
 	PublishAIn();
 	PublishIR();
@@ -917,11 +935,15 @@ void Erratic::ReceiveThread()
 	// Sonar packet
 	maxcount = RobotParams[this->param_idx]->NumSonars;
 	count = packet.packet[4];
-	erratic_data.sonar.ranges_count = maxcount;
-	for (int i = 0; i < count; i++) 
+        if(erratic_data.sonar.ranges_count != (unsigned int)maxcount)
+        {
+          erratic_data.sonar.ranges_count = maxcount;
+          if(erratic_data.sonar.ranges)
+            delete[] erratic_data.sonar.ranges;
+          erratic_data.sonar.ranges = new float[erratic_data.sonar.ranges_count];
+        }
+	for (int i = 0; i < count; i++)
 	  {
-	    if (i >= PLAYER_SONAR_MAX_SAMPLES)
-	      continue;
 	    int ch = packet.packet[5+i*3]; // channel number
 	    if (ch >= maxcount)
 	      continue;		// bad channel number
@@ -948,8 +970,6 @@ void Erratic::ReceiveThread()
 	  packet.Print();
 	}
       }
-		
-    //Unlock();
   }
 }
 
@@ -962,10 +982,10 @@ void* Erratic::ReceiveThreadDummy(void *driver) {
 void Erratic::SendThread() {
   for (;;) {
     pthread_testcancel();
-    
+
     // Get rights to the queue
     pthread_mutex_lock(&send_queue_mutex);
-    
+
     // If there is nothing, we wait for signal
     if (!send_queue.size())
       pthread_cond_wait(&send_queue_cond, &send_queue_mutex);
@@ -977,7 +997,7 @@ void Erratic::SendThread() {
       send_queue.pop();
     }
     pthread_mutex_unlock(&send_queue_mutex);
-    
+
     // Send the packet and destroy it
     if (packet) {
       if (print_all_packets) {
@@ -1008,6 +1028,7 @@ void Erratic::Send(ErraticPacket *packet) {
 }
 
 // Tells the robot to reset the odometry center
+// Just resets accumulation in this driver, not the controller
 void Erratic::ResetRawPositions() {
   ErraticPacket *pkt;
   unsigned char erraticcommand[4];
@@ -1017,8 +1038,9 @@ void Erratic::ResetRawPositions() {
 
   if(this->motor_packet) {
     pkt = new ErraticPacket();
-    this->motor_packet->rawxpos = 0;
-    this->motor_packet->rawypos = 0;
+		// don't reset raw values, else there will be a jump
+		//    this->motor_packet->rawxpos = 0;
+		//    this->motor_packet->rawypos = 0;
     this->motor_packet->xpos = 0;
     this->motor_packet->ypos = 0;
     this->motor_packet->x_offset = 0;
@@ -1043,12 +1065,12 @@ void Erratic::ToggleMotorPower(unsigned char val) {
   command[2] = val;
   command[3] = 0;
   packet->Build(command, 4);
-  
+
   Send(packet);
 }
 
 // Enable or disable analog input reporting
-void Erratic::ToggleAIn(unsigned char val) 
+void Erratic::ToggleAIn(unsigned char val)
 {
   unsigned char command[4];
   ErraticPacket *packet = new ErraticPacket();
@@ -1058,12 +1080,12 @@ void Erratic::ToggleAIn(unsigned char val)
   command[2] = val ? 1 : 0;
   command[3] = 0;
   packet->Build(command, 4);
-  
-  Send(packet);  
+
+  Send(packet);
 }
 
 // Enable or disable sonar reporting
-void Erratic::ToggleSonar(unsigned char val) 
+void Erratic::ToggleSonar(unsigned char val)
 {
   unsigned char command[4];
   ErraticPacket *packet = new ErraticPacket();
@@ -1073,18 +1095,18 @@ void Erratic::ToggleSonar(unsigned char val)
   command[2] = val ? 1 : 0;
   command[3] = 0;
   packet->Build(command, 4);
-  
-  Send(packet);  
+
+  Send(packet);
 }
 
 // This describes the IR hardware
-float Erratic::IRRangeFromVoltage(float voltage) 
+float Erratic::IRRangeFromVoltage(float voltage)
 {
   // This is for the Sharp 2Y0A02, 150 cm ranger
   return -.2475 + .1756 * voltage + .7455 / voltage - .0446 * voltage * voltage;
 }
 
-float Erratic::IRFloorRange(float value) 
+float Erratic::IRFloorRange(float value)
 {
   // floor range is 1 (floor present) or 0 (no floor)
   if (value >= 0.9)
@@ -1110,8 +1132,6 @@ void Erratic::Main() {
     //    Wait();
     usleep(10000);              // Wait() blocks too much, doesn't get subscriptions
 
-    this->Lock();
-
     // we want to reset the odometry and enable the motors if the first
     // client just subscribed to the position device, and we want to stop
     // and disable the motors if the last client unsubscribed.
@@ -1145,8 +1165,6 @@ void Erratic::Main() {
       this->ToggleSonar(0);
     last_sonar_subscriptions = this->sonar_subscriptions;
 
-    this->Unlock();
-
     // handle pending messages
     //printf( "Will look for incoming messages\n" );
     if(!this->InQueue->Empty())
@@ -1159,17 +1177,17 @@ void Erratic::Main() {
       //TODO reenable?
       //this->HandlePositionCommand(this->last_position_cmd);
     }
-    
+
     // Do some little nice printout
     if (print_status_summary) {
-      
+
     }
   }
 }
 
 // Publishes the indicated interface data
 void Erratic::PublishPosition2D() {
-  this->Publish(this->position_id, NULL,
+  this->Publish(this->position_id,
     PLAYER_MSGTYPE_DATA,
     PLAYER_POSITION2D_DATA_STATE,
     (void*)&(this->erratic_data.position),
@@ -1177,7 +1195,7 @@ void Erratic::PublishPosition2D() {
     NULL);
 }
 void Erratic::PublishPower() {
-  this->Publish(this->power_id, NULL,
+  this->Publish(this->power_id,
     PLAYER_MSGTYPE_DATA,
     PLAYER_POWER_DATA_STATE,
     (void*)&(this->erratic_data.power),
@@ -1185,7 +1203,7 @@ void Erratic::PublishPower() {
     NULL);
 }
 void Erratic::PublishAIn() {
-  this->Publish(this->aio_id, NULL,
+  this->Publish(this->aio_id,
     PLAYER_MSGTYPE_DATA,
     PLAYER_AIO_DATA_STATE,
     (void*)&(this->erratic_data.aio),
@@ -1193,7 +1211,7 @@ void Erratic::PublishAIn() {
     NULL);
 }
 void Erratic::PublishIR() {
-  this->Publish(this->ir_id, NULL,
+  this->Publish(this->ir_id,
     PLAYER_MSGTYPE_DATA,
     PLAYER_IR_DATA_RANGES,
     (void*)&(this->erratic_data.ir),
@@ -1201,7 +1219,7 @@ void Erratic::PublishIR() {
     NULL);
 }
 void Erratic::PublishSonar() {
-  this->Publish(this->sonar_id, NULL,
+  this->Publish(this->sonar_id,
     PLAYER_MSGTYPE_DATA,
     PLAYER_SONAR_DATA_RANGES,
     (void*)&(this->erratic_data.sonar),
@@ -1217,7 +1235,7 @@ void Erratic::PublishAllData() {
 }
 
 // Gets called from ProcessMessages to handle one message
-int Erratic::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void * data) {
+int Erratic::ProcessMessage(QueuePointer &resp_queue, player_msghdr * hdr, void * data) {
   // Look for configuration requests
   if(hdr->type == PLAYER_MSGTYPE_REQ)
     return(this->HandleConfig(resp_queue,hdr,data));
@@ -1228,7 +1246,7 @@ int Erratic::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void
 }
 
 // Handles one Player command with a configuration request
-int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void * data) {
+int Erratic::HandleConfig(QueuePointer &resp_queue, player_msghdr * hdr, void * data) {
 
   // check for position config requests
   if(Message::MatchMessage(hdr,PLAYER_MSGTYPE_REQ,
@@ -1243,17 +1261,17 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
     player_position2d_set_odom_req_t* set_odom_req =
       (player_position2d_set_odom_req_t*)data;
 
-    this->motor_packet->x_offset = ((int)rint(set_odom_req->pose.px*1e3))
-                                 - this->motor_packet->xpos;
+    this->motor_packet->x_offset = ((int )rint(set_odom_req->pose.px*1e3))
+                                    + this->motor_packet->xpos;
     this->motor_packet->y_offset = ((int)rint(set_odom_req->pose.py*1e3))
-                                 - this->motor_packet->ypos;
-    this->motor_packet->angle_offset = ((int)rint(RTOD(set_odom_req->pose.pa)))
-                                     - this->motor_packet->angle;
+                                    + this->motor_packet->ypos;
+    this->motor_packet->angle_offset = ((int)rint(RTOA(set_odom_req->pose.pa)))
+                                     + this->motor_packet->angle;
 
     //**************************
-    //    printf("Reset odometry: %f %f %f\n", set_odom_req->pose.px, set_odom_req->pose.py, set_odom_req->pose.pa);
-    //    printf("Reset odometry: %d %d %d\n", motor_packet->x_offset, motor_packet->y_offset, motor_packet->angle_offset);
-    
+		//    printf("Reset odometry: %f %f %f\n", set_odom_req->pose.px, set_odom_req->pose.py, set_odom_req->pose.pa);
+		//    printf("Reset odometry: %d %d %d\n", motor_packet->x_offset, motor_packet->y_offset, motor_packet->angle_offset);
+
 
     this->Publish(this->position_id, resp_queue,
                   PLAYER_MSGTYPE_RESP_ACK, PLAYER_POSITION2D_REQ_SET_ODOM);
@@ -1282,7 +1300,11 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
       return(-1);
     }
 
-    ResetRawPositions();
+		this->motor_packet->x_offset = this->motor_packet->xpos;
+		this->motor_packet->y_offset = this->motor_packet->ypos;;
+		this->motor_packet->angle_offset = this->motor_packet->angle;;
+		printf("Resetting odometry offsets: %d %d %d\n", motor_packet->xpos,
+					 motor_packet->ypos, motor_packet->angle);
 
     this->Publish(this->position_id, resp_queue,
       PLAYER_MSGTYPE_RESP_ACK, PLAYER_POSITION2D_REQ_RESET_ODOM);
@@ -1300,7 +1322,7 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
 
     geom.pose.px = -RobotParams[param_idx]->RobotAxleOffset / 1e3;
     geom.pose.py = 0.0;
-    geom.pose.pa = 0.0;
+    geom.pose.pyaw = 0.0;
 
     geom.size.sl = RobotParams[param_idx]->RobotLength / 1e3;
     geom.size.sw = RobotParams[param_idx]->RobotWidth / 1e3;
@@ -1337,19 +1359,19 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
                   PLAYER_POSITION2D_REQ_VELOCITY_MODE);
     return(0);
   }
-  
+
   // Request for getting IR locations
   else if(Message::MatchMessage(hdr,PLAYER_MSGTYPE_REQ,
-                                    PLAYER_IR_POSE,
+                                    PLAYER_IR_REQ_POSE,
                                     this->ir_id)) {
     player_ir_pose_t pose;
     pose.poses_count = RobotParams[param_idx]->NumIR;
     for (uint16_t i = 0; i < pose.poses_count ;i++)
       pose.poses[i] = RobotParams[param_idx]->IRPose[i];
-    
+
     this->Publish(this->ir_id, resp_queue,
                   PLAYER_MSGTYPE_RESP_ACK,
-                  PLAYER_IR_POSE,
+                  PLAYER_IR_REQ_POSE,
                   (void*)&pose, sizeof(pose), NULL);
     return(0);
   }
@@ -1357,13 +1379,13 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
   // two types of requests...
   else if (Message::MatchMessage(hdr,PLAYER_MSGTYPE_REQ,
                                     PLAYER_SONAR_DATA_GEOM,
-                                    this->sonar_id)) 
+                                    this->sonar_id))
     {
       player_sonar_geom_t pose;
       pose.poses_count = RobotParams[param_idx]->NumSonars;
       for (uint16_t i = 0; i < pose.poses_count ;i++)
         pose.poses[i] = RobotParams[param_idx]->sonar_pose[i];
-    
+
       this->Publish(this->sonar_id, resp_queue,
                     PLAYER_MSGTYPE_RESP_ACK,
                     PLAYER_SONAR_DATA_GEOM,
@@ -1375,13 +1397,13 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
   // two types of requests...
   else if (Message::MatchMessage(hdr,PLAYER_MSGTYPE_REQ,
                                     PLAYER_SONAR_REQ_GET_GEOM,
-                                    this->sonar_id)) 
+                                    this->sonar_id))
     {
       player_sonar_geom_t pose;
       pose.poses_count = RobotParams[param_idx]->NumSonars;
       for (uint16_t i = 0; i < pose.poses_count ;i++)
         pose.poses[i] = RobotParams[param_idx]->sonar_pose[i];
-    
+
       this->Publish(this->sonar_id, resp_queue,
                     PLAYER_MSGTYPE_RESP_ACK,
                     PLAYER_SONAR_REQ_GET_GEOM,
@@ -1400,7 +1422,7 @@ int Erratic::HandleConfig(MessageQueue* resp_queue, player_msghdr * hdr, void *
 // translational velocity target.
 // Erratic handles this natively
 //
-void 
+void
 Erratic::HandleCarCommand(player_position2d_cmd_car_t cmd)
 {
   int speedDemand, angleDemand;
@@ -1410,10 +1432,10 @@ Erratic::HandleCarCommand(player_position2d_cmd_car_t cmd)
 
   speedDemand = (int)rint(cmd.velocity * 1e3);  // convert to mm/s
   angleDemand = (int)rint(RTOD(cmd.angle)); // convert to deg heading
-  angleDemand -= this->motor_packet->angle_offset;  // check for angle offset of odometry
-  if (angleDemand > 360)        // normalize
+  angleDemand += (int)rint(ATOD(this->motor_packet->angle_offset));  // check for angle offset of odometry
+	while (angleDemand > 360)        // normalize
     angleDemand -= 360;
-  if (angleDemand < 0)
+  while (angleDemand < 0)
     angleDemand += 360;
 
   // do separate trans and rot vels
@@ -1434,7 +1456,7 @@ Erratic::HandleCarCommand(player_position2d_cmd_car_t cmd)
       motorcommand[2] = this->motor_max_speed & 0x00FF;
       motorcommand[3] = (this->motor_max_speed & 0xFF00) >> 8;
     }
-    
+
     motorpacket = new ErraticPacket();
     motorpacket->Build(motorcommand, 4);
     Send(motorpacket);
@@ -1450,7 +1472,7 @@ Erratic::HandleCarCommand(player_position2d_cmd_car_t cmd)
   absangleDemand = (unsigned short)abs(angleDemand);
   motorcommand[2] = absangleDemand & 0x00FF;
   motorcommand[3] = (absangleDemand & 0xFF00) >> 8;
-    
+
   motorpacket = new ErraticPacket();
   motorpacket->Build(motorcommand, 4);
   Send(motorpacket);
@@ -1467,7 +1489,7 @@ int getms()
 
 
 // Handles one Player command detailing a velocity
-void Erratic::HandlePositionCommand(player_position2d_cmd_vel_t position_cmd) 
+void Erratic::HandlePositionCommand(player_position2d_cmd_vel_t position_cmd)
 {
   int speedDemand, turnRateDemand;
   double leftvel, rightvel;
@@ -1602,7 +1624,7 @@ void Erratic::HandlePositionCommand(player_position2d_cmd_vel_t position_cmd)
       motorcommand[2] = this->motor_max_speed & 0x00FF;
       motorcommand[3] = (this->motor_max_speed & 0xFF00) >> 8;
     }
-    
+
     motorpacket = new ErraticPacket();
     motorpacket->Build(motorcommand, 4);
     Send(motorpacket);
@@ -1638,7 +1660,7 @@ void Erratic::HandlePositionCommand(player_position2d_cmd_vel_t position_cmd)
 // For PTZ #0:
 //    Pan is on servo 1, tilt on servo 2
 // For PTZ #1:
-//    Pan is on servo 0
+//    Tilt is on servo 0
 //
 // Commands are in degrees, positive and negative
 // Conversions are handled through the Erratic parameter structure,
@@ -1653,10 +1675,10 @@ void Erratic::HandlePositionCommand(player_position2d_cmd_vel_t position_cmd)
 #define SERVO_MIN_COUNT 1100
 
 
-void 
+void
 Erratic::HandlePtzCommand(player_ptz_cmd_t cmd, player_devaddr_t id)
 {
-  int pan, tilt, servo;
+  int pan, tilt, servo=0;
   unsigned char payload[6];
   ErraticPacket *packet;
 
@@ -1673,7 +1695,7 @@ Erratic::HandlePtzCommand(player_ptz_cmd_t cmd, player_devaddr_t id)
 		{
 			packet = new ErraticPacket();
 			payload[0] = (command_e)servo_pos;
-			payload[1] = (argtype_e)argstr;	
+			payload[1] = (argtype_e)argstr;
 			payload[2] = 3;						// 3 bytes in string
 			payload[3] = 2;						// servo #2
 			payload[4] = pan&0xff;
@@ -1710,18 +1732,23 @@ Erratic::HandlePtzCommand(player_ptz_cmd_t cmd, player_devaddr_t id)
 
 // Switchboard for robot commands, called from ProcessMessage
 // Note that these don't perform a response...
-int Erratic::HandleCommand(player_msghdr * hdr, void* data) 
+int Erratic::HandleCommand(player_msghdr * hdr, void* data)
 {
   if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, PLAYER_POSITION2D_CMD_VEL, this->position_id))
     {
       player_position2d_cmd_vel_t position_cmd = *(player_position2d_cmd_vel_t*)data;
       this->HandlePositionCommand(position_cmd);
-    } 
+    }
   else if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, PLAYER_POSITION2D_CMD_CAR, this->position_id))
     {
       player_position2d_cmd_car_t position_cmd = *(player_position2d_cmd_car_t*)data;
       this->HandleCarCommand(position_cmd);
-    } 
+    }
+  else if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, PLAYER_POSITION2D_CMD_VEL_HEAD, this->position_id))
+    {
+      player_position2d_cmd_car_t position_cmd = *(player_position2d_cmd_car_t*)data;
+      this->HandleCarCommand(position_cmd);
+    }
   else if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, PLAYER_PTZ_CMD_STATE, this->ptz_id))
     {
       player_ptz_cmd_t ptz_cmd = *(player_ptz_cmd_t*)data;
