@@ -1,6 +1,6 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2000  
+ *  Copyright (C) 2000
  *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
  *
  *
@@ -30,13 +30,13 @@
 /** @defgroup driver_khepera khepera
  * @brief K-Team Khepera mobile robot
 
-The khepera driver is used to interface to the K-Team khepera robot. 
+The khepera driver is used to interface to the K-Team khepera robot.
 
 This driver is experimental and should be treated with caution. At
-this point it supports the @ref interface_position2d and 
+this point it supports the @ref interface_position2d and
 @ref interface_ir interfaces.
 
-TODO: 
+TODO:
  - Add support for position control (currently only velocity control)
  - Add proper calibration for IR sensors
 
@@ -56,13 +56,13 @@ TODO:
 @par Supported configuration requests
 
 - The @ref interface_position2d interface supports:
-  - PLAYER_POSITION_GET_GEOM_REQ
-  - PLAYER_POSITION_MOTOR_POWER_REQ
-  - PLAYER_POSITION_VELOCITY_MODE_REQ
-  - PLAYER_POSITION_RESET_ODOM_REQ
-  - PLAYER_POSITION_SET_ODOM_REQ
+  - PLAYER_POSITION2D_REQ_GET_GEOM
+  - PLAYER_POSITION2D_REQ_MOTOR_POWER
+  - PLAYER_POSITION2D_REQ_VELOCITY_MODE
+  - PLAYER_POSITION2D_REQ_RESET_ODOM
+  - PLAYER_POSITION2D_REQ_SET_ODOM
 - The @ref interface_ir interface supports:
-  - PLAYER_IR_POSE_REQ
+  - PLAYER_IR_REQ_POSE
 
 @par Configuration file options
 
@@ -90,7 +90,7 @@ TODO:
   - Default: [10 24 90 19 17 45 25 6 0 25 -6 0 19 -17 -45 10 -24 -90 -24 -10 180 -24 10 180]
   - Poses of the IRs (mm mm deg for each one)
 
-@par Example 
+@par Example
 
 @verbatim
 driver
@@ -119,9 +119,10 @@ driver
 
 #include <assert.h>
 
-#include <khepera.h>
+#include "khepera.h"
 
 #include <libplayercore/playercore.h>
+#include <libplayerinterface/playerxdr.h>
 
 // we need to debug different things at different times
 //#define DEBUG_POS
@@ -150,20 +151,19 @@ Khepera_Init(ConfigFile *cf, int section)
 
 /* register the Khepera IR driver in the drivertable
  *
- * returns: 
+ * returns:
  */
 void
-Khepera_Register(DriverTable *table) 
+khepera_Register(DriverTable *table)
 {
   table->AddDriver("khepera", Khepera_Init);
 }
 
-int Khepera::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void * data)
+int Khepera::ProcessMessage(QueuePointer & resp_queue, player_msghdr * hdr, void * data)
 {
 	assert(hdr);
-	assert(data);
 
-	if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ, PLAYER_IR_POSE, ir_addr))
+	if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ, PLAYER_IR_REQ_POSE, ir_addr))
 	{
 		Publish(ir_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype, &geometry->ir, sizeof(geometry->ir));
 		return 0;
@@ -179,9 +179,9 @@ int Khepera::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void
 		int rightvel = transvel + rotvel;
 
 		// now we set the speed
-		if (this->motors_enabled) 
+		if (this->motors_enabled)
 			SetSpeed(leftvel,rightvel);
-		else 
+		else
 			SetSpeed(0,0);
 		return 0;
 	}
@@ -216,7 +216,7 @@ int Khepera::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void
 	return -1;
 }
 
-Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
+Khepera::Khepera(ConfigFile *cf, int section) : ThreadedDriver(cf, section)
 {
   // zero ids, so that we'll know later which interfaces were requested
   memset(&position_addr, 0, sizeof(this->position_addr));
@@ -225,23 +225,23 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
   this->position_subscriptions = this->ir_subscriptions = 0;
 
   // Do we create a robot position interface?
-  if(cf->ReadDeviceAddr(&(this->position_addr), section, "provides", 
+  if(cf->ReadDeviceAddr(&(this->position_addr), section, "provides",
                       PLAYER_POSITION2D_CODE, -1, NULL) == 0)
   {
     if(this->AddInterface(this->position_addr) != 0)
     {
-      this->SetError(-1);    
+      this->SetError(-1);
       return;
     }
   }
 
   // Do we create an ir interface?
-  if(cf->ReadDeviceAddr(&(this->ir_addr), section, "provides", 
+  if(cf->ReadDeviceAddr(&(this->ir_addr), section, "provides",
                       PLAYER_IR_CODE, -1, NULL) == 0)
   {
     if(this->AddInterface(this->ir_addr) != 0)
     {
-      this->SetError(-1);    
+      this->SetError(-1);
       return;
     }
   }
@@ -262,15 +262,12 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
   if (geometry->scale == 0)
     geometry->scale = cf->ReadFloat(section, "scale_factor", KHEPERA_DEFAULT_SCALE);
 
-  // set sub type here
-//  geometry->position.subtype = PLAYER_POSITION_GET_GEOM_REQ;
-
   geometry->encoder_res = cf->ReadFloat(section,"encoder_res", KHEPERA_DEFAULT_ENCODER_RES);
 
   // Load position config
   geometry->position.pose.px = cf->ReadTupleFloat(section,"position_pose",0,0) * geometry->scale;
   geometry->position.pose.py = cf->ReadTupleFloat(section,"position_pose",1,0) * geometry->scale;
-  geometry->position.pose.pa = cf->ReadTupleFloat(section,"position_pose",2,0) * geometry->scale;
+  geometry->position.pose.pyaw = cf->ReadTupleFloat(section,"position_pose",2,0) * geometry->scale;
 
   // load dimension of the base
   geometry->position.size.sw = cf->ReadTupleFloat(section,"position_size",0,57) * geometry->scale;
@@ -278,40 +275,41 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
 
   // load ir geometry config
   geometry->ir.poses_count = (cf->ReadInt(section,"ir_pose_count", 8));
+  geometry->ir.poses = new player_pose3d_t[geometry->ir.poses_count];
   if (geometry->ir.poses_count == 8 && cf->ReadTupleFloat(section,"ir_poses",0,-1) == -1)
   {
     // load the default ir geometry
     geometry->ir.poses[0].px = 10/1000*geometry->scale;
     geometry->ir.poses[0].py = 24/1000*geometry->scale;
-    geometry->ir.poses[0].pa = DTOR(90);
+    geometry->ir.poses[0].pyaw = DTOR(90);
 
     geometry->ir.poses[1].px = 19/1000*geometry->scale;
     geometry->ir.poses[1].py = 17/1000*geometry->scale;
-    geometry->ir.poses[1].pa = DTOR(45);
+    geometry->ir.poses[1].pyaw = DTOR(45);
 
     geometry->ir.poses[2].px = 25/1000*geometry->scale;
     geometry->ir.poses[2].py = 6/1000*geometry->scale;
-    geometry->ir.poses[2].pa = DTOR(0);
+    geometry->ir.poses[2].pyaw = DTOR(0);
 
     geometry->ir.poses[3].px = 25/1000*geometry->scale;
     geometry->ir.poses[3].py = -6/1000*geometry->scale;
-    geometry->ir.poses[3].pa = DTOR(0);
+    geometry->ir.poses[3].pyaw = DTOR(0);
 
     geometry->ir.poses[4].px = 19/1000*geometry->scale;
     geometry->ir.poses[4].py = -17/1000*geometry->scale;
-    geometry->ir.poses[4].pa = DTOR(-45);
+    geometry->ir.poses[4].pyaw = DTOR(-45);
 
     geometry->ir.poses[5].px = 10/1000*geometry->scale;
     geometry->ir.poses[5].py = -24/1000*geometry->scale;
-    geometry->ir.poses[5].pa = DTOR(-90);
+    geometry->ir.poses[5].pyaw = DTOR(-90);
 
     geometry->ir.poses[6].px = -24/1000*geometry->scale;
     geometry->ir.poses[6].py = -10/1000*geometry->scale;
-    geometry->ir.poses[6].pa = DTOR(180);
+    geometry->ir.poses[6].pyaw = DTOR(180);
 
     geometry->ir.poses[7].px = -24/1000*geometry->scale;
     geometry->ir.poses[7].py = 10/1000*geometry->scale;
-    geometry->ir.poses[7].pa = DTOR(180);
+    geometry->ir.poses[7].pyaw = DTOR(180);
   }
   else
   {
@@ -320,8 +318,8 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
     {
       geometry->ir.poses[i].px = cf->ReadTupleFloat(section,"ir_poses",3*i,0)*geometry->scale;
       geometry->ir.poses[i].py = cf->ReadTupleFloat(section,"ir_poses",3*i+1,0)*geometry->scale;
-      geometry->ir.poses[i].pa = cf->ReadTupleFloat(section,"ir_poses",3*i+2,0);
-    }				
+      geometry->ir.poses[i].pyaw = cf->ReadTupleFloat(section,"ir_poses",3*i+2,0);
+    }
   }
   // laod ir calibration from config file
   geometry->ir_calib_a = new double[geometry->ir.poses_count];
@@ -331,7 +329,6 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
     geometry->ir_calib_a[i] = cf->ReadTupleFloat(section,"ir_calib_a", i, KHEPERA_DEFAULT_IR_CALIB_A);
     geometry->ir_calib_b[i] = cf->ReadTupleFloat(section,"ir_calib_b", i, KHEPERA_DEFAULT_IR_CALIB_B);
   }
-  geometry->ir.poses_count = geometry->ir.poses_count;
 
   // zero position counters
   last_lpos = 0;
@@ -339,11 +336,19 @@ Khepera::Khepera(ConfigFile *cf, int section) : Driver(cf, section)
   last_x_f=0;
   last_y_f=0;
   last_theta = 0.0;
-  
-  
+
+
 }
 
-int 
+Khepera::~Khepera()
+{
+  delete geometry->ir.poses;
+  delete geometry->ir_calib_a;
+  delete geometry->ir_calib_b;
+  delete geometry;
+}
+
+int
 Khepera::Subscribe(player_devaddr_t addr)
 {
   int setupResult;
@@ -366,7 +371,7 @@ Khepera::Subscribe(player_devaddr_t addr)
   return(setupResult);
 }
 
-int 
+int
 Khepera::Unsubscribe(player_devaddr_t addr)
 {
   int shutdownResult;
@@ -393,10 +398,10 @@ Khepera::Unsubscribe(player_devaddr_t addr)
  *
  * returns: 0 on success
  */
-int 
-Khepera::Setup()
+int
+Khepera::MainSetup()
 {
-  // open and initialize the serial to -> Khepera  
+  // open and initialize the serial to -> Khepera
   printf("Khepera: connection initializing (%s)...", this->khepera_serial_port);
   fflush(stdout);
   Serial = new KheperaSerial(geometry->PortName,KHEPERA_BAUDRATE);
@@ -413,17 +418,14 @@ Khepera::Setup()
 
   desired_heading = 0;
 
-  /* now spawn reading thread */
-  StartThread();
   return(0);
 }
 
 
-int 
-Khepera::Shutdown()
+void
+Khepera::MainQuit()
 {
   printf("Khepera: SHUTDOWN\n");
-  StopThread();
 
   // Killing the thread seems to leave out serial
   // device in a bad state, need to fix this,
@@ -434,20 +436,19 @@ Khepera::Shutdown()
   delete Serial;
   Serial = NULL;
 
-  return(0);
 }
 
-void 
+void
 Khepera::Main()
 {
   int last_position_subscrcount=0;
 
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
 
-  while (1) 
+  while (1)
   {
-    // we want to reset the odometry and enable the motors if the first 
-    // client just subscribed to the position device, and we want to stop 
+    // we want to reset the odometry and enable the motors if the first
+    // client just subscribed to the position device, and we want to stop
     // and disable the motors if the last client unsubscribed.
     if(!last_position_subscrcount && this->position_subscriptions)
     {
@@ -456,7 +457,7 @@ Khepera::Main()
       SetSpeed(0,0);
       ResetOdometry();
 
-    } 
+    }
     else if(last_position_subscrcount && !(this->position_subscriptions))
     {
       // last sub just unsubbed
@@ -489,22 +490,27 @@ Khepera::UpdateData()
 {
   player_position2d_data_t position_data;
   player_ir_data_t ir_data;
+  ir_data.ranges_count = geometry->ir.poses_count;
+  ir_data.voltages_count = geometry->ir.poses_count;
+  ir_data.ranges = new float[ir_data.ranges_count];
+  ir_data.voltages = new float[ir_data.voltages_count];
 
   UpdatePosData(&position_data);
 
   // put position data
-  Publish(position_addr,NULL,PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, (unsigned char *) &position_data, sizeof(player_position2d_data_t),NULL);
+  Publish(position_addr,PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, (unsigned char *) &position_data, sizeof(player_position2d_data_t),NULL);
 
   UpdateIRData(&ir_data);
 
   // put ir data
-  Publish(position_addr,NULL,PLAYER_MSGTYPE_DATA, PLAYER_IR_DATA_RANGES, (unsigned char *) &ir_data, sizeof(ir_data),NULL);
+  Publish(position_addr,PLAYER_MSGTYPE_DATA, PLAYER_IR_DATA_RANGES, (unsigned char *) &ir_data, sizeof(ir_data),NULL);
+  player_ir_data_t_cleanup(&ir_data);
 }
 
 /* this will update the IR part of the client data
  * it entails reading the currently active IR sensors
  * and then changing their state to off and turning on
- * 2 new IRs.  
+ * 2 new IRs.
  *
  * returns:
  */
@@ -513,17 +519,16 @@ Khepera::UpdateIRData(player_ir_data_t * d)
 {
   ReadAllIR(d);
 
-  d->ranges_count = geometry->ir.poses_count;
-  for (unsigned int i =0; i < geometry->ir.poses_count; i++) 
+  for (unsigned int i =0; i < geometry->ir.poses_count; i++)
   {
-    d->ranges[i] = geometry->scale * geometry->ir_calib_a[i] * pow(d->voltages[i],geometry->ir_calib_b[i]);
+    d->ranges[i] = geometry->scale * geometry->ir_calib_a[i] * pow(static_cast<double> (d->voltages[i]),static_cast<double> (geometry->ir_calib_b[i]));
     d->voltages[i] = d->voltages[i];
   }
 }
 
-  
+
 /* this will update the position data.  this entails odometry, etc
- */ 
+ */
 void
 Khepera::UpdatePosData(player_position2d_data_t *d)
 {
@@ -543,7 +548,7 @@ Khepera::UpdatePosData(player_position2d_data_t *d)
 
   if (transchange == 0)
   {
-    Theta = 360 * rotchange/(2 * M_PI * r);	
+    Theta = 360 * rotchange/(2 * M_PI * r);
     dx = dy= 0;
   }
   else if (rotchange == 0)
@@ -586,7 +591,7 @@ Khepera::UpdatePosData(player_position2d_data_t *d)
 /* this will set the odometry to a given position
  * ****NOTE: assumes that the arguments are in network byte order!*****
  *
- * returns: 
+ * returns:
  */
 int
 Khepera::ResetOdometry()
@@ -603,48 +608,31 @@ Khepera::ResetOdometry()
 
   player_position2d_data_t data;
   memset(&data,0,sizeof(player_position2d_data_t));
-  Publish(position_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, &data, sizeof(data),NULL);
+  Publish(position_addr, PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, &data, sizeof(data),NULL);
 
   x=y=yaw=0;
   return 0;
 }
 
 
-/* this will read the given AD channel
- *
- * returns: the value of the AD channel
- */
-/*unsigned short
-REB::ReadAD(int channel) 
-{
-  char buf[64];
-
-  sprintf(buf, "I,%d\r", channel);
-  write_command(buf, strlen(buf), sizeof(buf));
-  
-  return atoi(&buf[2]);
-}*/
-
 /* reads all the IR values at once.  stores them
  * in the uint16_t array given as arg ir
  *
- * returns: 
+ * returns:
  */
 int
 Khepera::ReadAllIR(player_ir_data_t* d)
 {
-  int Values[PLAYER_IR_MAX_SAMPLES];
-
-  // changed these variable-size array declarations to the 
-  // bigger-than-necessary ones above, because older versions of gcc don't
-  // support variable-size arrays.
+  int * Values;
 
+  Values = new int [geometry->ir.poses_count];
   if(Serial->KheperaCommand('N',0,NULL,geometry->ir.poses_count,Values) < 0)
-    return -1;			
+    return -1;
   for (unsigned int i=0; i< geometry->ir.poses_count; ++i)
   {
     d->voltages[i] = static_cast<short> (Values[i]);
   }
+  delete [] Values;
   return 0;
 }
 
@@ -676,19 +664,6 @@ Khepera::ReadSpeed(int * left,int * right)
 	return 0;
 }
 
-/* this sets the desired position motor mn should go to
- *
- * returns:
- */
-/*void
-REB::SetPos(int mn, int pos)
-{
-  char buf[64];
-  
-  sprintf(buf,"C,%c,%d\r", '0'+mn,pos);
-
-  write_command(buf, strlen(buf), sizeof(buf));
-}*/
 
 /* this sets the position counter of motor mn to the given value
  *
@@ -719,66 +694,4 @@ Khepera::ReadPos(int * pos1, int * pos2)
 	return 0;
 }
 
-/* this will configure the position PID for motor mn
- * using paramters Kp, Ki, and Kd
- *
- * returns:
- */
-/*void
-REB::ConfigPosPID(int mn, int kp, int ki, int kd)
-{ 
-  char buf[64];
 
-  sprintf(buf, "F,%c,%d,%d,%d\r", '0'+mn, kp,ki,kd);
-  write_command(buf, strlen(buf), sizeof(buf));
-}*/
-
-/* this will configure the speed PID for motor mn
- *
- * returns:
- */
-/*void
-REB::ConfigSpeedPID(int mn, int kp, int ki, int kd)
-{
-  char buf[64];
-  
-  sprintf(buf, "A,%c,%d,%d,%d\r", '0'+mn, kp,ki,kd);
-  
-  write_command(buf, strlen(buf), sizeof(buf));
-}*/
-
-/* this will set the speed profile for motor mn
- * it takes the max velocity and acceleration
- *
- * returns:
- */
-/*void
-REB::ConfigSpeedProfile(int mn, int speed, int acc)
-{
-  char buf[64];
-  
-  sprintf(buf, "J,%c,%d,%d\r", '0'+mn, speed,acc);
-  write_command(buf, strlen(buf), sizeof(buf));
-}*/
-
-/* this will read the status of the motion controller.
- * mode is set to 1 if in position mode, 0 if velocity mode
- * error is set to the position/speed error
- *
- * returns: target status: 1 is on target, 0 is not on target
- */
-/*unsigned char
-Khepera::ReadStatus(int mn, int *mode, int *error)
-{
-  char buf[64];
-
-  sprintf(buf, "K,%c\r", '0'+mn);
-  //write_command(buf, strlen(buf), sizeof(buf));
-
-  // buf now has the form "k,target,mode,error"
-  int target;
-
-  sscanf(buf, "k,%d,%d,%d", &target, mode, error);
-
-  return (unsigned char)target;
-}*/
