@@ -1,14 +1,35 @@
+/* 
+ * Copyright (C) 1991-2000  Nomadic Technologies
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option) 
+ * any later version.
+ *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
+ * GNU General Public License for more details.
+ *  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * 02111-1307, USA.
+ */
+
 /*
  * Nclient.h
  *
  * Interface file for direct connections to the robot or for
  * connections to Nserver.
- * 
- * Copyright 1991-1998, Nomadic Technologies, Inc.
- *
  */
 
-/* $Header: /cvsroot/playerstage/code/player/server/drivers/mixed/nomad/Nclient.h,v 1.4 2004/05/14 06:14:13 rtv Exp $ */
+/*
+ * $Log: Nclient.h,v $
+ * Revision 1.1.1.1  2001/08/23 19:05:14  bananajr
+ * initial commit
+ *
+ */
 
 #ifndef _HOST_CLIENT_NCLIENT_H_
 #define _HOST_CLIENT_NCLIENT_H_
@@ -483,8 +504,8 @@ extern int Laser[2*NUM_LASER+1];
  *     ROBOT_MACHINE_NAME
  * usually don't need to be changed:
  *     CONN_TYPE
- *     NOMAD_SERIAL_PORT
- *     NOMAD_SERIAL_BAUD
+ *     SERIAL_PORT
+ *     SERIAL_BAUD
  *     ROBOT_TCP_PORT
  *
  * If an application program should run with Nclient.o and Ndirect.o
@@ -520,12 +541,12 @@ extern int CONN_TYPE;
 /* SERIAL_PORT is a string containing the filename of the serial port you
  * choose to communicate to the robot on.  The default is "/dev/ttyS0".
  */
-//extern char NOMAD_SERIAL_PORT[256];
+extern char SERIAL_PORT[40];
 
 /* SERIAL_BAUD is the baud rate to set the serial communication to.  It
  * defaults to 9600.
  */
-//extern int NOMAD_SERIAL_BAUD;
+extern int SERIAL_BAUD;
 
 /* ROBOT_TCP_PORT is the port number the robot listens on for request of
  * connection.  It defaults (and should always be set) to 65001 for the
