@@ -1,8 +1,8 @@
 /*
  *  Player - One Hell of a Robot Server
- *  Copyright (C) 2000  
+ *  Copyright (C) 2000
  *     Brian Gerkey, Kasper Stoy, Richard Vaughan, & Andrew Howard
- *                      
+ *
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 /*
- * $Id: sip.cc,v 1.20.2.1 2007/01/31 21:18:13 gerkey Exp $
+ * $Id: sip.cc 8003 2009-07-13 10:34:37Z thjc $
  *
  * part of the P2OS parser.  methods for filling and parsing server
  * information packets (SIPs)
@@ -33,11 +33,16 @@
 #include <stdlib.h> /* for abs() */
 #include <unistd.h>
 
-#include <libplayercore/error.h>
+#include <libplayercore/playercore.h>
 
 #include "sip.h"
 
-void SIP::Fill(player_p2os_data_t* data)
+// Variable with constant lifetime to store lift actuator data
+static player_actarray_actuator_t liftActuator;
+// Same thing for blobs
+static player_blobfinder_blob_t cmucamBlob;
+
+void SIP::FillStandard(player_p2os_data_t* data)
 {
   ///////////////////////////////////////////////////////////////
   // odometry
@@ -47,16 +52,16 @@ void SIP::Fill(player_p2os_data_t* data)
   data->position.pos.py = this->y_offset / 1e3;
   // now transform current position by rotation if there is one
   // and add to offset
-  if(this->angle_offset != 0) 
+  if(this->angle_offset != 0)
   {
     double rot = DTOR(this->angle_offset);    // convert rotation to radians
-    data->position.pos.px +=  ((this->xpos/1e3) * cos(rot) - 
+    data->position.pos.px +=  ((this->xpos/1e3) * cos(rot) -
                                (this->ypos/1e3) * sin(rot));
-    data->position.pos.py +=  ((this->xpos/1e3) * sin(rot) + 
+    data->position.pos.py +=  ((this->xpos/1e3) * sin(rot) +
                                (this->ypos/1e3) * cos(rot));
     data->position.pos.pa = DTOR(this->angle_offset + angle);
   }
-  else 
+  else
   {
     data->position.pos.px += this->xpos / 1e3;
     data->position.pos.py += this->ypos / 1e3;
@@ -73,31 +78,75 @@ void SIP::Fill(player_p2os_data_t* data)
   memset(&(data->compass),0,sizeof(data->compass));
   data->compass.pos.pa = DTOR(this->compass);
 
-  ///////////////////////////////////////////////////////////////
-  // gyro
-  memset(&(data->gyro),0,sizeof(data->gyro));
-  data->gyro.pos.pa = DTOR(this->gyro_rate);
-
   ///////////////////////////////////////////////////////////////
   // sonar
   data->sonar.ranges_count = PlayerRobotParams[param_idx].SonarNum;
-  for(int i=0;i<MIN(PlayerRobotParams[param_idx].SonarNum,ARRAYSIZE(sonars));i++)
+  data->sonar.ranges = new float[data->sonar.ranges_count];
+  for(int i=0;i<MIN(PlayerRobotParams[param_idx].SonarNum,sonarreadings);i++)
     data->sonar.ranges[i] = this->sonars[i] / 1e3;
 
   ///////////////////////////////////////////////////////////////
   // gripper
-  data->gripper.state = (unsigned char)(this->timer >> 8);
-  data->gripper.beams = (unsigned char)this->digin;
+  unsigned char gripState = timer >> 8;
+  if ((gripState & 0x01) && (gripState & 0x02) && !(gripState & 0x04))
+    data->gripper.state = PLAYER_GRIPPER_STATE_ERROR;
+  else
+    data->gripper.state = (gripState & 0x01) ? PLAYER_GRIPPER_STATE_OPEN :
+      ((gripState & 0x02) ? PLAYER_GRIPPER_STATE_CLOSED :
+      ((gripState & 0x04) ? PLAYER_GRIPPER_STATE_MOVING : PLAYER_GRIPPER_STATE_ERROR));
+  data->gripper.beams = (digin & 0x02) & (digin & 0x04);
+  data->gripper.stored = 0;
+
+  ///////////////////////////////////////////////////////////////
+  // lift
+  data->lift.actuators_count = 1;
+  data->lift.actuators = &liftActuator;
+  data->lift.actuators[0].speed = 0;
+  data->lift.actuators[0].acceleration = -1;
+  data->lift.actuators[0].current = -1;
+  if ((gripState & 0x10) && (gripState & 0x20) && !(gripState & 0x40))
+  {
+    // In this case, the lift is somewhere in between, so
+    // must be at an intermediate carry position. Use last commanded position.
+    data->lift.actuators[0].state = PLAYER_ACTARRAY_ACTSTATE_IDLE;
+    data->lift.actuators[0].position = lastLiftPos;
+  }
+  else if (gripState & 0x10)  // Up
+  {
+    data->lift.actuators[0].state = PLAYER_ACTARRAY_ACTSTATE_IDLE;
+    data->lift.actuators[0].position = 1.0f;
+  }
+  else if (gripState & 0x20)  // Down
+  {
+    data->lift.actuators[0].state = PLAYER_ACTARRAY_ACTSTATE_IDLE;
+    data->lift.actuators[0].position = 0.0f;
+  }
+  else if (gripState & 0x40)  // Moving
+  {
+    data->lift.actuators[0].state = PLAYER_ACTARRAY_ACTSTATE_MOVING;
+    // There is no way to know where it is for sure, so use last commanded position.
+    data->lift.actuators[0].position = lastLiftPos;
+  }
+  else    // Assume stalled
+  {
+    data->lift.actuators[0].state = PLAYER_ACTARRAY_ACTSTATE_STALLED;
+  }
 
   ///////////////////////////////////////////////////////////////
   // bumper
-  data->bumper.bumpers_count = PlayerRobotParams[param_idx].NumFrontBumpers + PlayerRobotParams[param_idx].NumRearBumpers;
+  unsigned int bump_count = PlayerRobotParams[param_idx].NumFrontBumpers + PlayerRobotParams[param_idx].NumRearBumpers;
+  if (data->bumper.bumpers_count != bump_count)
+  {
+    data->bumper.bumpers_count = bump_count;
+    delete [] data->bumper.bumpers;
+    data->bumper.bumpers = new uint8_t[bump_count];
+  }
   int j = 0;
   for(int i=PlayerRobotParams[param_idx].NumFrontBumpers-1;i>=0;i--)
-    data->bumper.bumpers[j++] = 
+    data->bumper.bumpers[j++] =
       (unsigned char)((this->frontbumpers >> i) & 0x01);
   for(int i=PlayerRobotParams[param_idx].NumRearBumpers-1;i>=0;i--)
-    data->bumper.bumpers[j++] = 
+    data->bumper.bumpers[j++] =
       (unsigned char)((this->rearbumpers >> i) & 0x01);
 
   ///////////////////////////////////////////////////////////////
@@ -110,29 +159,45 @@ void SIP::Fill(player_p2os_data_t* data)
   ///////////////////////////////////////////////////////////////
   // digital I/O
   data->dio.count = (unsigned char)8;
-  data->dio.digin = (unsigned int)this->digin;
-  
+  data->dio.bits = (unsigned int)this->digin;
+
   ///////////////////////////////////////////////////////////////
   // analog I/O
   //TODO: should do this smarter, based on which analog input is selected
   data->aio.voltages_count = (unsigned char)1;
+  if (!data->aio.voltages)
+    data->aio.voltages = new float[1];
   data->aio.voltages[0] = (this->analog / 255.0) * 5.0;
+}
 
+void SIP::FillGyro(player_p2os_data_t* data)
+{
+  ///////////////////////////////////////////////////////////////
+  // gyro
+  memset(&(data->gyro),0,sizeof(data->gyro));
+  data->gyro.pos.pa = DTOR(this->gyro_rate);
+}
+
+void SIP::FillSERAUX(player_p2os_data_t* data)
+{
   /* CMUcam blob tracking interface.  The CMUcam only supports one blob
   ** (and therefore one channel too), so everything else is zero.  All
-  ** data is storde in the blobfinder packet in Network byte order.  
+  ** data is storde in the blobfinder packet in Network byte order.
   ** Note: In CMUcam terminology, X is horizontal and Y is vertical, with
   ** (0,0) being TOP-LEFT (from the camera's perspective).  Also,
   ** since CMUcam doesn't have range information, but does have a
   ** confidence value, I'm passing it back as range. */
+  data->blobfinder.blobs = &cmucamBlob;
   memset(data->blobfinder.blobs,0,
-         sizeof(player_blobfinder_blob_t)*PLAYER_BLOBFINDER_MAX_BLOBS);
+         sizeof(player_blobfinder_blob_t));
   data->blobfinder.width = CMUCAM_IMAGE_WIDTH;
   data->blobfinder.height = CMUCAM_IMAGE_HEIGHT;
 
   if (blobarea > 1)	// With filtering, definition of track is 2 pixels
   {
     data->blobfinder.blobs_count = 1;
+    if (!data->blobfinder.blobs)
+    	data->blobfinder.blobs = new player_blobfinder_blob_t[1];
     data->blobfinder.blobs[0].color = this->blobcolor;
     data->blobfinder.blobs[0].x = this->blobmx;
     data->blobfinder.blobs[0].y = this->blobmy;
@@ -145,31 +210,50 @@ void SIP::Fill(player_p2os_data_t* data)
   }
   else
     data->blobfinder.blobs_count = 0;
+}
 
+void SIP::FillArm(player_p2os_data_t* data)
+{
+  ///////////////////////////////////////////////////////////////
   // Fill in arm data
-  memset (data->actarray.actuators, 0, sizeof (player_actarray_actuator_t) * PLAYER_ACTARRAY_NUM_ACTUATORS);
-  data->actarray.actuators_count = armNumJoints;
+  data->actArray.actuators_count = armNumJoints;
+  data->actArray.actuators = new player_actarray_actuator_t[armNumJoints];  // This will be cleaned up in P2OS::PutData
+  memset (data->actArray.actuators, 0, sizeof (player_actarray_actuator_t) * armNumJoints);
   for (int ii = 0; ii < armNumJoints; ii++)
   {
-    data->actarray.actuators[ii].position = armJointPosRads[ii];
-    data->actarray.actuators[ii].speed = 0;
+    data->actArray.actuators[ii].position = armJointPosRads[ii];
+    data->actArray.actuators[ii].speed = 0;
+    data->actArray.actuators[ii].acceleration = -1;
+    data->actArray.actuators[ii].current = -1;
     // State is complex. It can be idle, moving, or stalled (we don't have brakes so don't need to worry about the brake state).
     // Moving means have moving state from status packet
     // Idle means have not moving state from status packet and are at target position
     // Stalled means have not moving state from status packet and are not at target position
     if (armJointMoving[ii])
-      data->actarray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_MOVING;
+      data->actArray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_MOVING;
+    else if (armJointPos[ii] == armJointTargetPos[ii])
+      data->actArray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_IDLE;
     else
-    {
-      if (armJointPos[ii] == armJointTargetPos[ii])
-        data->actarray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_IDLE;
-      else
-        data->actarray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_STALLED;
-    }
+      data->actArray.actuators[ii].state = PLAYER_ACTARRAY_ACTSTATE_STALLED;
   }
+
+  ///////////////////////////////////////////////////////////////
+  // Fill in arm gripper data
+  memset (&(data->armGripper), 0, sizeof (player_gripper_data_t));
+  if (armJointMoving[5])
+    data->armGripper.state = PLAYER_GRIPPER_STATE_MOVING;
+  else if (armJointPos[5] == armJointTargetPos[5])
+    if (armJointPos[5] > 128)    // Position is between 0 and 255
+      data->armGripper.state = PLAYER_GRIPPER_STATE_OPEN;
+    else
+      data->armGripper.state = PLAYER_GRIPPER_STATE_CLOSED;
+  else
+    data->armGripper.state = PLAYER_GRIPPER_STATE_ERROR;
+  data->armGripper.beams = 0;
+  data->armGripper.stored = 0;
 }
 
-int SIP::PositionChange( unsigned short from, unsigned short to ) 
+int SIP::PositionChange( unsigned short from, unsigned short to )
 {
   int diff1, diff2;
 
@@ -183,14 +267,14 @@ int SIP::PositionChange( unsigned short from, unsigned short to )
     diff2 = 4096 - from + to;
   }
 
-  if ( abs(diff1) < abs(diff2) ) 
+  if ( abs(diff1) < abs(diff2) )
     return(diff1);
   else
     return(diff2);
 
 }
 
-void SIP::Print() 
+void SIP::Print()
 {
   int i;
 
@@ -211,28 +295,28 @@ void SIP::Print()
   printf("status: 0x%x analog: %d ", status, analog);
   printf("digin: ");
   for(i=0;i<8;i++) {
-    printf("%d", (digin >> 7-i ) & 0x01);
+    printf("%d", (digin >> (7-i) ) & 0x01);
   }
   printf(" digout: ");
   for(i=0;i<8;i++) {
-    printf("%d", (digout >> 7-i ) & 0x01);
+    printf("%d", (digout >> (7-i) ) & 0x01);
   }
   puts("");
   printf("battery: %d compass: %d sonarreadings: %d\n", battery, compass, sonarreadings);
   printf("xpos: %d ypos:%d ptu:%hu timer:%hu\n", xpos, ypos, ptu, timer);
   printf("angle: %d lvel: %d rvel: %d control: %d\n", angle, lvel, rvel, control);
-  
+
   PrintSonars();
   PrintArmInfo ();
   PrintArm ();
 }
 
-void SIP::PrintSonars() 
+void SIP::PrintSonars()
 {
   printf("Sonars: ");
   for(int i = 0; i < 16; i++){
     printf("%hu ", sonars[i]);
-  } 
+  }
   puts("");
 }
 
@@ -253,7 +337,7 @@ void SIP::PrintArmInfo ()
 		printf ("%d |\t%d\t%d\t%d\t%d\t%d\t%d\n", ii, armJoints[ii].speed, armJoints[ii].home, armJoints[ii].min, armJoints[ii].centre, armJoints[ii].max, armJoints[ii].ticksPer90);
 }
 
-void SIP::Parse( unsigned char *buffer ) 
+void SIP::ParseStandard( unsigned char *buffer )
 {
   int cnt = 0, change;
   unsigned short newxpos, newypos;
@@ -261,7 +345,7 @@ void SIP::Parse( unsigned char *buffer )
   status = buffer[cnt];
   cnt += sizeof(unsigned char);
   /*
-   * Remember P2OS uses little endian: 
+   * Remember P2OS uses little endian:
    * for a 2 byte short (called integer on P2OS)
    * byte0 is low byte, byte1 is high byte
    * The following code is host-machine endian independant
@@ -273,16 +357,16 @@ void SIP::Parse( unsigned char *buffer )
    */
   newxpos = ((buffer[cnt] | (buffer[cnt+1] << 8))
 	     & 0xEFFF) % 4096; /* 15 ls-bits */
-  
+
   if (xpos!=INT_MAX) {
-    change = (int) rint(PositionChange( rawxpos, newxpos ) * 
+    change = (int) rint(PositionChange( rawxpos, newxpos ) *
 			PlayerRobotParams[param_idx].DistConvFactor);
     if (abs(change)>100)
       PLAYER_WARN1("invalid odometry change [%d]; odometry values are tainted", change);
     else
       xpos += change;
   }
-  else 
+  else
     xpos = 0;
   rawxpos = newxpos;
   cnt += sizeof(short);
@@ -320,11 +404,11 @@ void SIP::Parse( unsigned char *buffer )
 
   battery = buffer[cnt];
   cnt += sizeof(unsigned char);
-  
+
   lwstall = buffer[cnt] & 0x01;
   rearbumpers = buffer[cnt] >> 1;
   cnt += sizeof(unsigned char);
-  
+
   rwstall = buffer[cnt] & 0x01;
   frontbumpers = buffer[cnt] >> 1;
   cnt += sizeof(unsigned char);
@@ -342,21 +426,39 @@ void SIP::Parse( unsigned char *buffer )
     compass = (buffer[cnt]-1)*2;
   cnt += sizeof(unsigned char);
 
-  sonarreadings = buffer[cnt];
-  cnt += sizeof(unsigned char);
+  unsigned char numSonars=buffer[cnt];
+  cnt+=sizeof(unsigned char);
 
-  //printf("%hu sonar readings:\n", sonarreadings);
-  for(unsigned char i = 0;i < sonarreadings;i++) {
+  if(numSonars>0)
+  {
+    //find the largest sonar index supplied
+    unsigned char maxSonars=sonarreadings;
+    for(unsigned char i=0;i<numSonars;i++)
+    {
+      unsigned char sonarIndex=buffer[cnt+i*(sizeof(unsigned char)+sizeof(unsigned short))];
+      if((sonarIndex+1)>maxSonars) maxSonars=sonarIndex+1;
+    }
+
+    //if necessary make more space in the array and preserve existing readings
+    if(maxSonars>sonarreadings)
+    {
+      unsigned short *newSonars=new unsigned short[maxSonars];
+      for(unsigned char i=0;i<sonarreadings;i++)
+        newSonars[i]=sonars[i];
+      if(sonars!=NULL) delete[] sonars;
+      sonars=newSonars;
+      sonarreadings=maxSonars;
+    }
+
+    //update the sonar readings array with the new readings
+    for(unsigned char i=0;i<numSonars;i++)
+    {
     sonars[buffer[cnt]]=   (unsigned short)
       rint((buffer[cnt+1] | (buffer[cnt+2] << 8)) *
 	   PlayerRobotParams[param_idx].RangeConvFactor);
-    //printf("%d %hu:",buffer[cnt],*((unsigned short *)&buffer[cnt+1]));
-    //     
-    //printf("%hu %hu %hu\n", buffer[cnt], buffer[cnt+1], buffer[cnt+2]);
-    //printf("index %d value %hu\n", buffer[cnt], sonars[buffer[cnt]]);
-    cnt += 3*sizeof(unsigned char);
+      cnt+=sizeof(unsigned char)+sizeof(unsigned short);
+  }
   }
-  //printf("\n");
 
   timer = (buffer[cnt] | (buffer[cnt+1] << 8));
   cnt += sizeof(short);
@@ -375,7 +477,7 @@ void SIP::Parse( unsigned char *buffer )
 
 
 /** Parse a SERAUX SIP packet.  For a CMUcam, this will have blob
- **  tracking messages in the format (all one-byte values, no spaces): 
+ **  tracking messages in the format (all one-byte values, no spaces):
  **
  **      255 M mx my x1 y1 x2 y2 pixels confidence  (10-bytes)
  **
@@ -383,7 +485,7 @@ void SIP::Parse( unsigned char *buffer )
  **
  **      255 S Rval Gval Bval Rvar Gvar Bvar    (8-bytes)
  */
-void SIP::ParseSERAUX( unsigned char *buffer ) 
+void SIP::ParseSERAUX( unsigned char *buffer )
 {
   unsigned char type = buffer[1];
   if (type != SERAUX && type != SERAUX2)
@@ -439,7 +541,7 @@ void SIP::ParseSERAUX( unsigned char *buffer )
      return;
   }
 
-  printf("ERROR: Unknown blob tracker packet type: %c\n", buffer[ix+1]); 
+  printf("ERROR: Unknown blob tracker packet type: %c\n", buffer[ix+1]);
   return;
 }
 
@@ -484,7 +586,7 @@ SIP::ParseGyro(unsigned char* buffer)
   // probably should) be done here, like filtering, calibration, conversion
   // from the gyro's arbitrary units to something meaningful, etc.
   //
-  // As a first cut, we'll just average all the rate measurements in this 
+  // As a first cut, we'll just average all the rate measurements in this
   // set, and ignore the temperatures.
   float ratesum = 0;
   int bufferpos = 3;
