@@ -63,13 +63,13 @@ them named:
 @par Supported configuration requests
 
 - @ref interface_position2d :
-  - PLAYER_POSITION_SET_ODOM_REQ
-  - PLAYER_POSITION_MOTOR_POWER_REQ
-  - PLAYER_POSITION_RESET_ODOM_REQ
-  - PLAYER_POSITION_GET_GEOM_REQ
+  - PLAYER_POSITION2D_REQ_SET_ODOM
+  - PLAYER_POSITION2D_REQ_MOTOR_POWER
+  - PLAYER_POSITION2D_REQ_RESET_ODOM
+  - PLAYER_POSITION2D_REQ_GET_GEOM
 
 - @ref interface_ir :
-  - PLAYER_IR_POSE
+  - PLAYER_IR_REQ_POSE
 
 @par Supported commands
 
@@ -101,9 +101,7 @@ driver
 */
 /** @} */
 
-#ifdef HAVE_CONFIG_H
-  #include "config.h"
-#endif
+#include "config.h"
 
 #include <unistd.h>
 #include <inttypes.h>
@@ -132,7 +130,7 @@ driver
 static void LogMe( const char* s )
 {
   FILE* fp = fopen( "plog", "a+" );
-  fprintf( fp, s );
+  fprintf( fp, "%s", s );
   fclose( fp );
 }
 
@@ -149,7 +147,7 @@ void wbr914_Register(DriverTable* table)
 }
 
 wbr914::wbr914(ConfigFile* cf, int section)
-  : Driver(cf,section,true,PLAYER_MSGQUEUE_DEFAULT_MAXLEN),
+  : ThreadedDriver(cf,section,true,PLAYER_MSGQUEUE_DEFAULT_MAXLEN),
     _tioChanged( false ),
     _stopped( true ), _motorsEnabled( false ), _lastDigOut( 0 )
 {
@@ -159,7 +157,7 @@ wbr914::wbr914(ConfigFile* cf, int section)
   // Baud rate
   _baud = 416666;
 
-  ErrorInit( 9 );
+  ErrorInit( 9, NULL );
 
   // zero ids, so that we'll know later which interfaces were
   // requested
@@ -237,66 +235,72 @@ wbr914::wbr914(ConfigFile* cf, int section)
 
   // Set up the robot geometry
 
+  memset(&_robot2d_geom, 0, sizeof(_robot2d_geom));
   // X location in meters
-  _robot2d_geom.pose.px     = 0.0;
+//   _robot2d_geom.pose.px     = 0.0;
   // Y location in meters
-  _robot2d_geom.pose.py     = 0.0;
+//   _robot2d_geom.pose.py     = 0.0;
   // yaw in rads
-  _robot2d_geom.pose.pa     = 0.0;
+//   _robot2d_geom.pose.pyaw     = 0.0;
 
   // Width in meters
   _robot2d_geom.size.sw     = 0.37;
   // Length in meters
   _robot2d_geom.size.sl     = 0.42;
+  _robot2d_geom.size.sh     = 0.3;
 
-  _robot3d_geom.pose.px     = _robot2d_geom.pose.px;
-  _robot3d_geom.pose.py     = _robot2d_geom.pose.py;
-  _robot3d_geom.pose.pz     = 0.0;
-  _robot3d_geom.pose.proll  = 0.0;
-  _robot3d_geom.pose.ppitch = 0.0;
-  _robot3d_geom.pose.pyaw   = _robot2d_geom.pose.pa;
+  _robot3d_geom.pose             = _robot2d_geom.pose;
+  _robot3d_geom.size             = _robot2d_geom.size;
 
+/*
   _robot3d_geom.size.sw     = _robot2d_geom.size.sw;
   _robot3d_geom.size.sl     = _robot2d_geom.size.sl;
-  _robot3d_geom.size.sh     = 0.3;
+  _robot3d_geom.size.sh     = 0.3;*/
 
   // Set up the IR array geometry
   _ir_geom.poses_count = NUM_IR_SENSORS;
+  _ir_geom.poses = new player_pose3d_t[_ir_geom.poses_count];
 
   _ir_geom.poses[ 0 ].px = 0.030;
   _ir_geom.poses[ 0 ].py = -0.190;
-  _ir_geom.poses[ 0 ].pa = DTOR( -90 );
+  _ir_geom.poses[ 0 ].pyaw = DTOR( -90 );
 
   _ir_geom.poses[ 1 ].px = 0.190;
   _ir_geom.poses[ 1 ].py = -0.090;
-  _ir_geom.poses[ 1 ].pa = DTOR( -30 );
+  _ir_geom.poses[ 1 ].pyaw = DTOR( -30 );
 
   _ir_geom.poses[ 2 ].px = 0.210;
   _ir_geom.poses[ 2 ].py = 0.0;
-  _ir_geom.poses[ 2 ].pa = DTOR( 0 );
+  _ir_geom.poses[ 2 ].pyaw = DTOR( 0 );
 
   _ir_geom.poses[ 3 ].px = 0.190;
   _ir_geom.poses[ 3 ].py = 0.090;
-  _ir_geom.poses[ 3 ].pa = DTOR( 30 );
+  _ir_geom.poses[ 3 ].pyaw = DTOR( 30 );
 
   _ir_geom.poses[ 4 ].px = 0.030;
   _ir_geom.poses[ 4 ].py = 0.190;
-  _ir_geom.poses[ 4 ].pa = DTOR( 90 );
+  _ir_geom.poses[ 4 ].pyaw = DTOR( 90 );
 
   // These 3 sensor have a z value of 0.35m and a pitch of 30 degrees down
   // This type of pose is not currently handled by Player so do the best we
   // can with what we have.
   _ir_geom.poses[ 5 ].px = 0.200;
   _ir_geom.poses[ 5 ].py = -0.060;
-  _ir_geom.poses[ 5 ].pa = DTOR( -60 );
+  _ir_geom.poses[ 5 ].pyaw = DTOR( -60 );
 
   _ir_geom.poses[ 6 ].px = 0.210;
   _ir_geom.poses[ 6 ].py = 0.0;
-  _ir_geom.poses[ 6 ].pa = DTOR( 0 );
+  _ir_geom.poses[ 6 ].pyaw = DTOR( 0 );
 
   _ir_geom.poses[ 7 ].px = 0.200;
   _ir_geom.poses[ 7 ].py = 0.060;
-  _ir_geom.poses[ 7 ].pa = DTOR( 60 );
+  _ir_geom.poses[ 7 ].pyaw = DTOR( 60 );
+
+  _data.ir.ranges_count = NUM_IR_SENSORS;
+  _data.ir.voltages_count = _data.ir.ranges_count;
+  _data.ir.ranges = new float [_data.ir.ranges_count];
+  _data.ir.voltages = new float [_data.ir.voltages_count];
+
 }
 
 /**
@@ -306,10 +310,14 @@ wbr914::~wbr914()
 {
   if ( _tioChanged )
     tcsetattr( this->_fd, TCSADRAIN, &_old_tio);
-  Shutdown();
+  MainQuit();
+  delete [] _ir_geom.poses;
+  delete [] _data.ir.ranges;
+  delete [] _data.ir.voltages;
+
 }
 
-int wbr914::Setup()
+int wbr914::MainSetup()
 {
   struct termios term;
   int flags;
@@ -435,7 +443,7 @@ int wbr914::Setup()
   {
     printf( "Error setting sign-magnitude mode\n" );
   }
-  
+
   /*  This might be a good time to reset the odometry values */
   if ( _debug )
     printf( "ResetRawPositions\n" );
@@ -450,7 +458,6 @@ int wbr914::Setup()
   /* now spawn reading thread */
   if ( _debug )
     printf( "Starting Thread...\n" );
-  StartThread();
   return(0);
 }
 
@@ -481,13 +488,10 @@ int wbr914::InitRobot()
 }
 
 
-int wbr914::Shutdown()
+void wbr914::MainQuit()
 {
   if( this->_fd == -1 )
-    return(0);
-
-  // Stop any more processing
-  StopThread();
+    return;
 
   // Stop the robot
   StopRobot();
@@ -500,8 +504,6 @@ int wbr914::Shutdown()
   close( fd );
 
   puts( "914 has been shut down" );
-
-  return(0);
 }
 
 int wbr914::Subscribe( player_devaddr_t id )
@@ -573,7 +575,7 @@ void wbr914::PublishData(void)
   if ( position_subscriptions )
   {
     // put odometry data
-    this->Publish(this->position_id, NULL,
+    this->Publish(this->position_id,
 		  PLAYER_MSGTYPE_DATA,
 		  PLAYER_POSITION2D_DATA_STATE,
 		  (void*)&(this->_data.position),
@@ -584,7 +586,7 @@ void wbr914::PublishData(void)
   if ( ir_subscriptions )
   {
     // put ir data
-    this->Publish(this->ir_id, NULL,
+    this->Publish(this->ir_id,
 		  PLAYER_MSGTYPE_DATA,
 		  PLAYER_IR_DATA_RANGES,
 		  (void*)&(_data.ir),
@@ -595,7 +597,7 @@ void wbr914::PublishData(void)
   if ( aio_subscriptions )
   {
     // put Analog Input data
-    this->Publish(this->aio_id, NULL,
+    this->Publish(this->aio_id,
 		  PLAYER_MSGTYPE_DATA,
 		  PLAYER_AIO_DATA_STATE,
 		  (void*)&(_data.aio),
@@ -606,7 +608,7 @@ void wbr914::PublishData(void)
   if ( dio_subscriptions )
   {
     // put Digital Input data
-    this->Publish(this->dio_id, NULL,
+    this->Publish(this->dio_id,
 		  PLAYER_MSGTYPE_DATA,
 		  PLAYER_DIO_DATA_VALUES,
 		  (void*)&(_data.dio),
@@ -643,8 +645,6 @@ void wbr914::Main()
     }
     last_position_subscrcount = this->position_subscriptions;
 
-    this->Unlock();
-
     // handle pending messages
     if(!this->InQueue->Empty())
     {
@@ -671,7 +671,7 @@ void wbr914::Main()
   }
 }
 
-int wbr914::ProcessMessage(MessageQueue * resp_queue,
+int wbr914::ProcessMessage(QueuePointer &resp_queue,
 			   player_msghdr * hdr,
 			   void * data)
 {
@@ -685,7 +685,7 @@ int wbr914::ProcessMessage(MessageQueue * resp_queue,
   return(-1);
 }
 
-int wbr914::HandleConfig(MessageQueue* resp_queue,
+int wbr914::HandleConfig(QueuePointer &resp_queue,
 			 player_msghdr * hdr,
 			 void * data)
 {
@@ -766,7 +766,7 @@ int wbr914::HandleConfig(MessageQueue* resp_queue,
     return(0);
   }
   else if(Message::MatchMessage(hdr,PLAYER_MSGTYPE_REQ,
-                                PLAYER_IR_POSE,
+                                PLAYER_IR_REQ_POSE,
                                 ir_id))
   {
     /* Return the ir pose info */
@@ -805,7 +805,7 @@ int wbr914::HandleCommand(player_msghdr * hdr, void* data)
 				  PLAYER_DIO_CMD_VALUES,
 				  this->dio_id))
   {
-    HandleDigitalOutCommand( (player_dio_cmd_t*)data );
+    HandleDigitalOutCommand( (player_dio_data_t*)data );
     return(0);
   }
 
@@ -861,7 +861,7 @@ void wbr914::HandleVelocityCommand(player_position2d_cmd_vel_t* velcmd)
   }
 }
 
-void wbr914::HandleDigitalOutCommand( player_dio_cmd_t* doutCmd )
+void wbr914::HandleDigitalOutCommand( player_dio_data_t* doutCmd )
 {
   SetDigitalData( doutCmd );
 }
@@ -873,7 +873,7 @@ void wbr914::GetAllData( void )
   {
     GetPositionData( &_data.position );
   }
-  
+
   if ( ir_subscriptions )
   {
     GetIRData( &_data.ir );
@@ -923,8 +923,8 @@ void wbr914::GetPositionData( player_position2d_data_t* d )
     _yaw -= TWOPI;
 
   // calc current x and y position
-  _x += ( transchange * cos( _yaw )); 
-  _y += ( transchange * sin( _yaw )); 
+  _x += ( transchange * cos( _yaw ));
+  _y += ( transchange * sin( _yaw ));
 
 
   // add code to read in the speed data
@@ -962,10 +962,7 @@ void wbr914::GetIRData(player_ir_data_t * d)
   //  float v80 = 0.25;
   //  float deltaV = 2.25;
   //  float v10 = v80+deltaV;
-  //  float mmPerVolt = (800.0-100.0)/(v80-v10); 
-
-  d->voltages_count = NUM_IR_SENSORS;
-  d->ranges_count = d->voltages_count;
+  //  float mmPerVolt = (800.0-100.0)/(v80-v10);
 
   for (uint32_t i=0; i < d->ranges_count; i++)
   {
@@ -1043,7 +1040,7 @@ void wbr914::GetDigitalData(player_dio_data_t * d)
 
   // Byte flip the data to make the Input from the
   // optional I/O board show up in the upper byte.
-  d->digin = (uint32_t)( (din>>8) | (din<<8));
+  d->bits = (uint32_t)( (din>>8) | (din<<8));
 }
 
 /*
@@ -1052,10 +1049,10 @@ void wbr914::GetDigitalData(player_dio_data_t * d)
   We cannot reliably detect whether there is an I/O
   board attached to the M3 so blindly set the data.
  */
-void wbr914::SetDigitalData( player_dio_cmd_t * d )
+void wbr914::SetDigitalData( player_dio_data_t * d )
 {
   // We only have 16 bits of Dig out, so strip extra bits
-  uint16_t data = d->digout & 0xFFFF;
+  uint16_t data = d->bits & 0xFFFF;
 
   // Different number of digital bits being requested to
   // be set than we must actually set in the hardware.
@@ -1114,7 +1111,7 @@ const char* wbr914::GetPMDErrorString( int rc )
     return errorStrings[ rc ];
   }
 
-  sprintf( bogusRC, "Unknown error %d", rc );
+  snprintf( bogusRC,sizeof(bogusRC), "Unknown error %d", rc );
   return bogusRC;
 }
 
@@ -1136,7 +1133,7 @@ int wbr914::ResetRawPositions()
   /*
   player_position2d_data_t data;
   memset(&data,0,sizeof(player_position2d_data_t));
-  Publish( position_id, NULL, PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, &data, sizeof(data),NULL);
+  Publish( position_id, PLAYER_MSGTYPE_DATA, PLAYER_POSITION2D_DATA_STATE, &data, sizeof(data),NULL);
   */
   _x   = 0;
   _y   = 0;
@@ -1324,8 +1321,8 @@ int wbr914::WriteBuf(unsigned char* s, size_t len)
       return -1;
     }
   }
-    
-  printf( "Write timeout; wrote %d bytes, tried to write %d\n",
+
+  printf( "Write timeout; wrote %ld bytes, tried to write %ld\n",
 	  numwritten, len );
 
   return numwritten;
@@ -1353,8 +1350,8 @@ int wbr914::WriteBuf(unsigned char* s, size_t len)
   return numwritten;
 }
 
-int wbr914::sendCmdCom( unsigned char address, unsigned char c, 
-			int cmd_num, unsigned char* arg, 
+int wbr914::sendCmdCom( unsigned char address, unsigned char c,
+			int cmd_num, unsigned char* arg,
 			int ret_num, unsigned char * ret )
 {
   assert( cmd_num<=4 );
@@ -1362,7 +1359,7 @@ int wbr914::sendCmdCom( unsigned char address, unsigned char c,
   unsigned char cmd[8];
   //bool retry = true;
   unsigned char savecs;
-  
+
   cmd[0] = address;
   cmd[1] = 0;          // checksum. to be overwritten
   cmd[2] = 0x00;
@@ -1406,7 +1403,7 @@ int wbr914::sendCmdCom( unsigned char address, unsigned char c,
     //    if ( _fd_blocking == false )
       usleep( DELAY_US );
 
-    int rc; 
+    int rc;
     if( (rc = ReadBuf( ret, ret_num )) < 0 )
     {
       //      printf( "failed to read response\n" );
@@ -1419,13 +1416,13 @@ int wbr914::sendCmdCom( unsigned char address, unsigned char c,
   return result;
 }
 
-int wbr914::sendCmd0( unsigned char address, unsigned char c, 
+int wbr914::sendCmd0( unsigned char address, unsigned char c,
 		      int ret_num, unsigned char * ret )
 {
   return sendCmdCom( address, c, 0, NULL, ret_num, ret );
 }
 
-int wbr914::sendCmd16( unsigned char address, unsigned char c, 
+int wbr914::sendCmd16( unsigned char address, unsigned char c,
 		       int16_t arg, int ret_num, unsigned char * ret )
 {
 
@@ -1438,7 +1435,7 @@ int wbr914::sendCmd16( unsigned char address, unsigned char c,
   return sendCmdCom( address, c, 2, args, ret_num, ret );
 }
 
-int wbr914::sendCmd32( unsigned char address, unsigned char c, 
+int wbr914::sendCmd32( unsigned char address, unsigned char c,
 		       int32_t arg, int ret_num, unsigned char * ret )
 {
   unsigned char args[4];
@@ -1683,15 +1680,15 @@ void wbr914::SetAccelerationProfile()
 }
 
 void wbr914::Stop( int StopMode ) {
-        
+
   unsigned char ret[8];
-  
+
   if ( _debug )
     printf( "Stop\n" );
 
   /* Start with motor 0*/
   _stopped = true;
-  
+
   if( StopMode == FULL_STOP )
   {
     if (sendCmd16( LEFT_MOTOR, RESETEVENTSTATUS, 0x0000, 2, ret )<0 )
