@@ -1,3 +1,25 @@
+/*
+ *  Player - One Hell of a Robot Server
+ *  Copyright (C) 2007
+ *     Brian Gerkey
+ *
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+
 /** @ingroup drivers */
 /** @{ */
 /** @defgroup driver_nd nd
@@ -85,7 +107,7 @@ underlying "output" @ref interface_position2d device.
     progress toward the goal orientation before giving up.
 
 - translate_stuck_time (float)
-  - Default: 2.0 seconds         
+  - Default: 2.0 seconds
   - How long the robot is allowed to translate without making sufficient
     progress toward the goal position before giving up.
 
@@ -108,7 +130,7 @@ underlying "output" @ref interface_position2d device.
     validity of pose estimates.
 
 - laser_buffer (integer)
-  - Default: 10          
+  - Default: 10
   - How many recent laser scans to consider in the local navigation.
 
 
@@ -155,7 +177,9 @@ driver
 */
 /** @} */
 #include <math.h>
-#include <unistd.h>
+#if !defined (WIN32)
+  #include <unistd.h>
+#endif
 #include <stdlib.h>
 #include <float.h>
 
@@ -166,9 +190,11 @@ driver
   #define SIGN(x) (((x) == 0) ? 0 : (((x) > 0) ? 1 : -1))
 #endif
 
-extern PlayerTime *GlobalTime;
+#if defined (WIN32)
+  #define hypot _hypot
+#endif
 
-class ND : public Driver 
+class ND : public ThreadedDriver 
 {
   public:
     // Constructor
@@ -178,12 +204,12 @@ class ND : public Driver
     virtual ~ND();
 
     // Setup/shutdown routines.
-    virtual int Setup();
-    virtual int Shutdown();
+    virtual int MainSetup();
+    virtual void MainQuit();
 
-    // Process incoming messages from clients 
-    virtual int ProcessMessage(MessageQueue* resp_queue, 
-                               player_msghdr * hdr, 
+    // Process incoming messages from clients
+    virtual int ProcessMessage(QueuePointer &resp_queue,
+                               player_msghdr * hdr,
                                void * data);
     // Main function for device thread.
     virtual void Main();
@@ -191,10 +217,10 @@ class ND : public Driver
   private:
     bool active_goal;
     int dir;
-    player_pose_t goal;
-    player_pose_t last_odom_pose;
-    player_pose_t odom_pose;
-    player_pose_t odom_vel;
+    player_pose2d_t goal;
+    player_pose2d_t last_odom_pose;
+    player_pose2d_t odom_pose;
+    player_pose2d_t odom_vel;
     bool odom_stall;
     int current_dir;
     TParametersND NDparametros;
@@ -227,7 +253,7 @@ class ND : public Driver
     double avoid_dist;
     player_position2d_geom_t robot_geom;
     double safety_dist;
-    
+
     double Threshold(double v, double v_min, double v_max);
     void SetDirection(int dir);
     int SetupOdom();
@@ -248,7 +274,7 @@ class ND : public Driver
 
     // Computes the signed minimum difference between the two angles.
     double angle_diff(double a, double b);
-    
+
     // Odometry device info
     Device *odom;
     player_devaddr_t odom_addr;
@@ -260,38 +286,38 @@ class ND : public Driver
     // Laser device info
     Device *laser;
     player_devaddr_t laser_addr;
-    player_pose_t laser_pose;
+    player_pose3d_t laser_pose;
     int laser_buffer;
 
     // Sonar device info
     Device *sonar;
     player_devaddr_t sonar_addr;
     int num_sonars;
-    player_pose_t sonar_poses[PLAYER_SONAR_MAX_SAMPLES];
+    player_pose3d_t * sonar_poses;
     // indices of known bad sonars
-    int bad_sonars[PLAYER_SONAR_MAX_SAMPLES];
+    int * bad_sonars;
     int bad_sonar_count;
     int sonar_buffer;
 };
 
 // Initialization function
-Driver* 
-ND_Init(ConfigFile* cf, int section) 
+Driver*
+ND_Init(ConfigFile* cf, int section)
 {
   return ((Driver*) (new ND( cf, section)));
-} 
+}
 
 // a driver registration function
-void ND_Register(DriverTable* table)
+void nd_Register(DriverTable* table)
 {
   table->AddDriver("nd",  ND_Init);
   return;
-} 
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 ND::ND( ConfigFile* cf, int section)
-  : Driver(cf, section, false, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_POSITION2D_CODE)
+  : ThreadedDriver(cf, section, false, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_POSITION2D_CODE)
 {
   this->dist_eps = cf->ReadTupleLength(section, "goal_tol", 0, 0.5);
   this->ang_eps = cf->ReadTupleAngle(section, "goal_tol", 1, DTOR(10.0));
@@ -305,20 +331,20 @@ ND::ND( ConfigFile* cf, int section)
   this->safety_dist = cf->ReadLength(section, "safety_dist", 0.0);
   this->rotate_stuck_time = cf->ReadFloat(section, "rotate_stuck_time", 2.0);
 
-  this->translate_stuck_time = cf->ReadFloat(section, "translate_stuck_time", 
+  this->translate_stuck_time = cf->ReadFloat(section, "translate_stuck_time",
                                              2.0);
-  this->translate_stuck_dist = cf->ReadLength(section, "translate_stuck_dist", 
+  this->translate_stuck_dist = cf->ReadLength(section, "translate_stuck_dist",
                                               0.25);
-  this->translate_stuck_angle = cf->ReadAngle(section, "translate_stuck_angle", 
+  this->translate_stuck_angle = cf->ReadAngle(section, "translate_stuck_angle",
                                                DTOR(20.0));
-  this->wait_on_stall = 
+  this->wait_on_stall =
           cf->ReadInt(section, "wait_on_stall", 0) ?  true : false;
 
   this->odom = NULL;
   if (cf->ReadDeviceAddr(&this->odom_addr, section, "requires",
                          PLAYER_POSITION2D_CODE, -1, "output") != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
 
@@ -326,7 +352,7 @@ ND::ND( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->localize_addr, section, "requires",
                          PLAYER_POSITION2D_CODE, -1, "input") != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
 
@@ -345,11 +371,12 @@ ND::ND( ConfigFile* cf, int section)
                      PLAYER_SONAR_CODE, -1, NULL);
   if(this->sonar_addr.interf)
   {
-    if((this->bad_sonar_count = 
+    if((this->bad_sonar_count =
         cf->GetTupleCount(section, "sonar_bad_transducers")))
     {
+      this->bad_sonars = new int[bad_sonar_count];
       for(int i=0;i<this->bad_sonar_count;i++)
-        this->bad_sonars[i] = cf->ReadTupleInt(section, 
+        this->bad_sonars[i] = cf->ReadTupleInt(section,
                                                "sonar_bad_transducers",
                                                i, -1);
     }
@@ -362,26 +389,28 @@ ND::ND( ConfigFile* cf, int section)
     this->SetError(-1);
     return;
   }
-  
+  this->dir = 1;
+
   return;
 }
 
 
-ND::~ND() 
+ND::~ND()
 {
+  delete [] bad_sonars;
   return;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device (called by server thread).
-int ND::Setup() 
+int ND::MainSetup() 
 {
   // Initialise the underlying position device.
   if (this->SetupOdom() != 0)
     return -1;
-  
+
   this->active_goal = false;
-  
+
   // Initialise the laser.
   if (this->laser_addr.interf && this->SetupLaser() != 0)
     return -1;
@@ -391,25 +420,19 @@ int ND::Setup()
   this->obstacles.longitud = 0;
   this->stall = false;
   this->turning_in_place = false;
-  this->last_odom_pose.px = 
-          this->last_odom_pose.py = 
+  this->last_odom_pose.px =
+          this->last_odom_pose.py =
           this->last_odom_pose.pa = FLT_MAX;
 
   this->waiting = false;
 
-  // Start the driver thread.
-  this->StartThread();
-
   return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the device (called by server thread).
-int ND::Shutdown() 
+void ND::MainQuit() 
 {
-  // Stop the driver thread.
-  this->StopThread();
-
   // Stop the laser
   if(this->laser)
     this->ShutdownLaser();
@@ -420,13 +443,11 @@ int ND::Shutdown()
 
   // Stop the odom device.
   this->ShutdownOdom();
-
-  return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the underlying odom device.
-int ND::SetupOdom() 
+int ND::SetupOdom()
 {
   // Setup the output position device
   if(!(this->odom = deviceTable->GetDevice(this->odom_addr)))
@@ -473,13 +494,13 @@ int ND::SetupOdom()
          this->robot_geom.size.sw,
          this->robot_geom.pose.px,
          this->robot_geom.pose.py,
-         RTOD(this->robot_geom.pose.pa));
-         
+         RTOD(this->robot_geom.pose.pyaw));
+
 
   delete msg;
 
-  memset(&this->odom_pose, 0, sizeof(player_pose_t));
-  memset(&this->odom_vel, 0, sizeof(player_pose_t));
+  memset(&this->odom_pose, 0, sizeof(this->odom_pose));
+  memset(&this->odom_vel, 0, sizeof(this->odom_vel));
   this->odom_stall = false;
 
   return 0;
@@ -487,7 +508,7 @@ int ND::SetupOdom()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the underlying odom device.
-int ND::ShutdownOdom() 
+int ND::ShutdownOdom()
 {
   // Stop the robot before unsubscribing
   this->PutPositionCmd(0.0,0.0);
@@ -498,7 +519,7 @@ int ND::ShutdownOdom()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the laser
-int ND::SetupLaser() 
+int ND::SetupLaser()
 {
   if(!(this->laser = deviceTable->GetDevice(this->laser_addr)))
   {
@@ -515,7 +536,7 @@ int ND::SetupLaser()
   Message* msg;
 
   // Get the laser pose
-  if(!(msg = this->laser->Request(this->InQueue, 
+  if(!(msg = this->laser->Request(this->InQueue,
                                   PLAYER_MSGTYPE_REQ,
                                   PLAYER_LASER_REQ_GET_GEOM,
                                   NULL, 0, NULL,false)))
@@ -523,7 +544,7 @@ int ND::SetupLaser()
     PLAYER_ERROR("failed to get laser geometry");
     return(-1);
   }
-  
+
   // Store the laser pose
   cfg = (player_laser_geom_t*)msg->GetPayload();
   this->laser_pose = cfg->pose;
@@ -540,7 +561,7 @@ int ND::SetupLaser()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the sonar
-int ND::SetupSonar() 
+int ND::SetupSonar()
 {
   if(!(this->sonar = deviceTable->GetDevice(this->sonar_addr)))
   {
@@ -557,7 +578,7 @@ int ND::SetupSonar()
   Message* msg;
 
   // Get the sonar poses
-  if(!(msg = this->sonar->Request(this->InQueue, 
+  if(!(msg = this->sonar->Request(this->InQueue,
                                   PLAYER_MSGTYPE_REQ,
                                   PLAYER_SONAR_REQ_GET_GEOM,
                                   NULL, 0, NULL,false)))
@@ -565,10 +586,11 @@ int ND::SetupSonar()
     PLAYER_ERROR("failed to get sonar geometry");
     return(-1);
   }
-  
+
   // Store the sonar poses
   cfg = (player_sonar_geom_t*)msg->GetPayload();
   this->num_sonars = cfg->poses_count;
+  this->sonar_poses = new player_pose3d_t[num_sonars];
   for(int i=0;i<this->num_sonars;i++)
   {
     this->sonar_poses[i] = cfg->poses[i];
@@ -587,7 +609,7 @@ int ND::SetupSonar()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shut down the laser
-int ND::ShutdownLaser() 
+int ND::ShutdownLaser()
 {
   this->laser->Unsubscribe(this->InQueue);
   free(this->laser_obstacles);
@@ -596,9 +618,10 @@ int ND::ShutdownLaser()
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shut down the sonar
-int ND::ShutdownSonar() 
+int ND::ShutdownSonar()
 {
   this->sonar->Unsubscribe(this->InQueue);
+  delete [] sonar_poses;
   free(this->sonar_obstacles);
   return 0;
 }
@@ -653,13 +676,13 @@ ND::ProcessInputOdom(player_msghdr_t* hdr, player_position2d_data_t* data)
     this->waiting = false;
   }
 
-  // this->stall indicates that we're stuck (either ND threw an emergency 
+  // this->stall indicates that we're stuck (either ND threw an emergency
   // stop or it was failing to make progress).  Set the stall flag to let
   // whoever's listening that we've given up.
   if(this->stall)
     newdata.stall = 1;
 
-  this->Publish(NULL, &newhdr, &newdata);
+  this->Publish(&newhdr, &newdata);
 }
 
 void
@@ -705,18 +728,18 @@ ND::ProcessLaser(player_msghdr_t* hdr, player_laser_data_t* scan)
     y = r * sin(b);
 
     // convert to the robot's frame
-    rx = (this->laser_pose.px + 
-          x * cos(this->laser_pose.pa) -
-          y * sin(this->laser_pose.pa));
-    ry = (this->laser_pose.py + 
-          x * sin(this->laser_pose.pa) +
-          y * cos(this->laser_pose.pa));
+    rx = (this->laser_pose.px +
+          x * cos(this->laser_pose.pyaw) -
+          y * sin(this->laser_pose.pyaw));
+    ry = (this->laser_pose.py +
+          x * sin(this->laser_pose.pyaw) +
+          y * cos(this->laser_pose.pyaw));
 
     // convert to the odometric frame and add to the obstacle list
-    this->laser_obstacles[idx].punto[i].x = (this->odom_pose.px + 
+    this->laser_obstacles[idx].punto[i].x = static_cast<float> (this->odom_pose.px +
                                              rx * cos(this->odom_pose.pa) -
                                              ry * sin(this->odom_pose.pa));
-    this->laser_obstacles[idx].punto[i].y = (this->odom_pose.py + 
+    this->laser_obstacles[idx].punto[i].y = static_cast<float> (this->odom_pose.py +
                                              rx * sin(this->odom_pose.pa) +
                                              ry * cos(this->odom_pose.pa));
   }
@@ -766,18 +789,18 @@ ND::ProcessSonar(player_msghdr_t* hdr, player_sonar_data_t* scan)
     y = 0.0;
 
     // convert to the robot's frame
-    rx = (this->sonar_poses[i].px + 
-          x * cos(this->sonar_poses[i].pa) -
-          y * sin(this->sonar_poses[i].pa));
-    ry = (this->sonar_poses[i].py + 
-          x * sin(this->sonar_poses[i].pa) +
-          y * cos(this->sonar_poses[i].pa));
+    rx = (this->sonar_poses[i].px +
+          x * cos(this->sonar_poses[i].pyaw) -
+          y * sin(this->sonar_poses[i].pyaw));
+    ry = (this->sonar_poses[i].py +
+          x * sin(this->sonar_poses[i].pyaw) +
+          y * cos(this->sonar_poses[i].pyaw));
 
     // convert to the odometric frame and add to the obstacle list
-    this->sonar_obstacles[idx].punto[count].x = (this->odom_pose.px + 
+    this->sonar_obstacles[idx].punto[count].x = static_cast<float> (this->odom_pose.px +
                                                  rx * cos(this->odom_pose.pa) -
                                                  ry * sin(this->odom_pose.pa));
-    this->sonar_obstacles[idx].punto[count].y = (this->odom_pose.py + 
+    this->sonar_obstacles[idx].punto[count].y = static_cast<float> (this->odom_pose.py +
                                                  rx * sin(this->odom_pose.pa) +
                                                  ry * cos(this->odom_pose.pa));
     count++;
@@ -785,7 +808,7 @@ ND::ProcessSonar(player_msghdr_t* hdr, player_sonar_data_t* scan)
   this->sonar_obstacles[idx].longitud = count;
 }
 
-void 
+void
 ND::ProcessCommand(player_msghdr_t* hdr, player_position2d_cmd_vel_t* cmd)
 {
   if(!cmd->state)
@@ -793,8 +816,8 @@ ND::ProcessCommand(player_msghdr_t* hdr, player_position2d_cmd_vel_t* cmd)
     this->PutPositionCmd(0.0,0.0);
     this->active_goal = false;
   }
- 
-  else 
+
+  else
   {
     PLAYER_MSG2(2, "Stopped by velocity command (%.3f %.3f)",
                 cmd->vel.px, RTOD(cmd->vel.pa));
@@ -807,12 +830,12 @@ ND::ProcessCommand(player_msghdr_t* hdr, player_position2d_cmd_vel_t* cmd)
       this->dir = 1;
 }
 
-void 
+void
 ND::ProcessCommand(player_msghdr_t* hdr, player_position2d_cmd_pos_t* cmd)
 {
   PLAYER_MSG3(2, "New goal: (%.3f %.3f %.3f)",
-              cmd->pos.px, 
-              cmd->pos.py, 
+              cmd->pos.px,
+              cmd->pos.py,
               RTOD(cmd->pos.pa));
   // position control;  cache the goal and we'll process it in the main
   // loop.  also cache vel.px, which tells us whether to go forward or
@@ -834,14 +857,14 @@ ND::SetDirection(int dir)
 
   if(dir > 0)
   {
-    this->NDparametros.front = this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist; // Distance to the front
-    this->NDparametros.back = this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist;  // Distance to the back
+    this->NDparametros.front = static_cast<float> (this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist); // Distance to the front
+    this->NDparametros.back = static_cast<float> (this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist);  // Distance to the back
     InicializarND(&this->NDparametros);
   }
   else
   {
-    this->NDparametros.front = this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist; // Distance to the front
-    this->NDparametros.back = this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist;  // Distance to the back
+    this->NDparametros.front = static_cast<float> (this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist); // Distance to the front
+    this->NDparametros.back = static_cast<float> (this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist);  // Distance to the back
     InicializarND(&this->NDparametros);
   }
   this->current_dir = dir;
@@ -849,59 +872,59 @@ ND::SetDirection(int dir)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Process an incoming message
-int ND::ProcessMessage(MessageQueue* resp_queue, 
-                       player_msghdr * hdr, 
+int ND::ProcessMessage(QueuePointer &resp_queue,
+                       player_msghdr * hdr,
                        void * data)
 {
   // Is it new odometry data?
-  if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, 
-                           PLAYER_POSITION2D_DATA_STATE, 
+  if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA,
+                           PLAYER_POSITION2D_DATA_STATE,
                            this->odom_addr))
   {
     this->ProcessOutputOdom(hdr, (player_position2d_data_t*)data);
 
     // In case the input and output are the same device
-    if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, 
-                             PLAYER_POSITION2D_DATA_STATE, 
+    if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA,
+                             PLAYER_POSITION2D_DATA_STATE,
                              this->localize_addr))
       this->ProcessInputOdom(hdr, (player_position2d_data_t*)data);
 
     return(0);
   }
   // Is it new localization data?
-  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, 
-                           PLAYER_POSITION2D_DATA_STATE, 
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA,
+                           PLAYER_POSITION2D_DATA_STATE,
                            this->localize_addr))
   {
     this->ProcessInputOdom(hdr, (player_position2d_data_t*)data);
     return(0);
   }
   // Is it a new laser scan?
-  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, 
-                                PLAYER_LASER_DATA_SCAN, 
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA,
+                                PLAYER_LASER_DATA_SCAN,
                                 this->laser_addr))
   {
     this->ProcessLaser(hdr, (player_laser_data_t*)data);
     return(0);
   }
   // Is it a new sonar scan?
-  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA, 
-                                PLAYER_SONAR_DATA_RANGES, 
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_DATA,
+                                PLAYER_SONAR_DATA_RANGES,
                                 this->sonar_addr))
   {
     this->ProcessSonar(hdr, (player_sonar_data_t*)data);
     return(0);
   }
   // Is it a new goal?
- else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, 
+ else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
 			       PLAYER_POSITION2D_CMD_POS,
                                 this->device_addr))
   {
     this->ProcessCommand(hdr, (player_position2d_cmd_pos_t*)data);
     return 0;
   }
-  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, 
-                                PLAYER_POSITION2D_CMD_VEL, 
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
+                                PLAYER_POSITION2D_CMD_VEL,
                                 this->device_addr))
   {
     this->ProcessCommand(hdr, (player_position2d_cmd_vel_t*)data);
@@ -917,8 +940,8 @@ int ND::ProcessMessage(MessageQueue* resp_queue,
     if(!(msg = this->odom->Request(this->InQueue,
                                    hdr->type,
                                    hdr->subtype,
-                                   (void*)data, 
-                                   hdr->size, 
+                                   (void*)data,
+                                   hdr->size,
                                    &hdr->timestamp)))
     {
       PLAYER_WARN1("failed to forward config request with subtype: %d\n",
@@ -969,30 +992,30 @@ ND::Main()
   // Fill in the ND's parameter structure
 
   // Rectangular geometry
-  this->NDparametros.geometryRect = 1;		      
+  this->NDparametros.geometryRect = 1;
   // Distance to the front
-  this->NDparametros.front = this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist; 
+  this->NDparametros.front = static_cast<float> (this->robot_geom.size.sl/2.0 + this->robot_geom.pose.px + this->safety_dist);
   // Distance to the back
-  this->NDparametros.back = this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist;  
-  // Distance to the left side 
-  this->NDparametros.left = this->robot_geom.size.sw/2.0 + this->safety_dist;  
+  this->NDparametros.back = static_cast<float> (this->robot_geom.size.sl/2.0 - this->robot_geom.pose.px + this->safety_dist);
+  // Distance to the left side
+  this->NDparametros.left = static_cast<float> (this->robot_geom.size.sw/2.0 + this->safety_dist);
 
-  this->NDparametros.R = 0.2F;			// Not used 
+  this->NDparametros.R = 0.2F;			// Not used
 
   this->NDparametros.holonomic = 0;		// Non holonomic vehicle
 
-  this->NDparametros.vlmax = this->vx_max;
-  this->NDparametros.vamax = this->va_max;
+  this->NDparametros.vlmax = static_cast<float> (this->vx_max);
+  this->NDparametros.vamax = static_cast<float> (this->va_max);
 
   this->NDparametros.almax = 0.75F;		// More or less it will be like this
   this->NDparametros.aamax = 0.75F;
 
-  this->NDparametros.dsmax = this->avoid_dist; // Security distance
+  this->NDparametros.dsmax = static_cast<float> (this->avoid_dist); // Security distance
   this->NDparametros.dsmin = this->NDparametros.dsmax/4.0F;
   //NDparametros.enlarge = NDparametros.dsmin/2.0F;
   this->NDparametros.enlarge = this->NDparametros.dsmin*0.2F;
-  
-  this->NDparametros.discontinuity = this->NDparametros.left;  // Discontinuity (check whether it fits)
+
+  this->NDparametros.discontinuity = static_cast<float> (2.0 * this->NDparametros.left);  // Discontinuity (check whether it fits)
 
   this->NDparametros.T = 0.1F;  // Sample rate of the SICK
 
@@ -1020,9 +1043,9 @@ ND::Main()
       continue;
 
     // The robot's current odometric pose
-    pose.SR1.posicion.x = this->odom_pose.px;
-    pose.SR1.posicion.y = this->odom_pose.py;
-    pose.SR1.orientacion = this->odom_pose.pa;
+    pose.SR1.posicion.x = static_cast<float> (this->odom_pose.px);
+    pose.SR1.posicion.y = static_cast<float> (this->odom_pose.py);
+    pose.SR1.orientacion = static_cast<float> (this->odom_pose.pa);
 
     // Merge the (possibly buffered) laser and sonar obstacle lists
     this->obstacles.longitud = 0;
@@ -1064,15 +1087,15 @@ ND::Main()
         // To make the robot turn (safely) to the goal orientation, we'll
         // give it a fake goal that is in the right direction, and just
         // ignore the translational velocity.
-        goal.x = this->odom_pose.px + 10.0 * cos(this->goal.pa);
-        goal.y = this->odom_pose.py + 10.0 * sin(this->goal.pa);
+        goal.x = static_cast<float> (this->odom_pose.px + 10.0 * cos(this->goal.pa));
+        goal.y = static_cast<float> (this->odom_pose.py + 10.0 * sin(this->goal.pa));
 
         // In case we went backward to get here, reverse direction so that we
         // can attain the goal heading
         this->SetDirection(1);
 
         cmd_vel = IterarND(goal,
-                           this->dist_eps,
+                           static_cast<float> (this->dist_eps),
                            &pose,
                            &this->obstacles,
                            NULL);
@@ -1110,7 +1133,7 @@ ND::Main()
           else
           {
             // no; have we run out of time?
-            double t; 
+            double t;
             GlobalTime->GetTimeDouble(&t);
             if((t - this->rotate_start_time) > this->rotate_stuck_time)
             {
@@ -1140,7 +1163,7 @@ ND::Main()
         else
         {
           // Has it been long enough?
-          double t; 
+          double t;
           GlobalTime->GetTimeDouble(&t);
           if((t - this->translate_start_time) > this->translate_stuck_time)
           {
@@ -1153,15 +1176,15 @@ ND::Main()
         }
 
         // The current odometric goal
-        goal.x = this->goal.px;
-        goal.y = this->goal.py;
+        goal.x = static_cast<float> (this->goal.px);
+        goal.y = static_cast<float> (this->goal.py);
 
         // Were we asked to go backward?
         if(this->dir < 0)
         {
-          // Trick the ND by telling it that the robot is pointing the 
+          // Trick the ND by telling it that the robot is pointing the
           // opposite direction
-          pose.SR1.orientacion = NORMALIZE(pose.SR1.orientacion + M_PI);
+          pose.SR1.orientacion = static_cast<float> (NORMALIZE(pose.SR1.orientacion + M_PI));
           // Also reverse the robot's geometry (it may be asymmetric
           // front-to-back)
           this->SetDirection(-1);
@@ -1170,7 +1193,7 @@ ND::Main()
           this->SetDirection(1);
 
         cmd_vel = IterarND(goal,
-                           this->dist_eps,
+                           static_cast<float> (this->dist_eps),
                            &pose,
                            &this->obstacles,
                            NULL);
@@ -1222,7 +1245,7 @@ ND::Main()
 double
 ND::angle_diff(double a, double b)
 {
-  double d1, d2; 
+  double d1, d2;
   a = NORMALIZE(a);
   b = NORMALIZE(b);
   d1 = a-b;
