@@ -1,24 +1,50 @@
+/*
+ *  Player - One Hell of a Robot Server
+ *  Copyright (C) 2003
+ *     Chris Jones, Brian Gerkey, Alex Brooks, Richard Vaughan
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
 #include <assert.h>
 #include <math.h>
 #include <fstream>
 #include <stdlib.h>
 #include <stdio.h>
 #include <vector>
-#include <unistd.h>
+#if !defined (WIN32)
+  #include <unistd.h>
+#endif
 #include <errno.h>
 #include <time.h>
 
 #include <libplayercore/playercore.h>
 #include "vfh_algorithm.h"
 
-extern PlayerTime *GlobalTime;
-
 /** @ingroup drivers */
 /** @{ */
 /** @defgroup driver_vfh vfh
  * @brief Vector Field Histogram local navigation algorithm
 
-@note This driver may take several seconds to start up, especially on slower machines.  You may want to set the 'alwayson' option for vfh to '1' in your configuration file in order to front-load this delay.  Otherwise, your client may experience a timeout in trying to subscribe to this device.
+@note This driver may take several seconds to start up, especially on
+slower machines.  You may want to set the 'alwayson' option for vfh to
+'1' in your configuration file in order to front-load this delay.
+Otherwise, your client may experience a timeout in trying to subscribe
+to this device.
 
 The vfh driver implements the Vector Field Histogram Plus local
 navigation method by Ulrich and Borenstein.  VFH+ provides real-time
@@ -54,6 +80,9 @@ or sector_angle.
   underlying @ref interface_position2d device.  All data and commands
   are in the odometric frame of the underlying device.
 
+- @ref interface_planner : similar to above, but uses the planer
+  interface which allows extras such as "done" when arrived.
+
 @par Requires
 
 - @ref interface_position2d : the underlying robot that will be
@@ -158,12 +187,15 @@ or sector_angle.
       to escape.
   - escape_time (float)
     - Default: 0.0
-    - If non-zero, the time (in seconds) for which an escape attempt will be 
+    - If non-zero, the time (in seconds) for which an escape attempt will be
       made.
-  - escape_max_turnspeed (angle / sec)
+  - escape_max_turnrate (angle / sec)
     - Default: 0.0
-    - If non-zero, the maximum angular velocity that will be used when 
+    - If non-zero, the maximum angular velocity that will be used when
       trying to escape.
+  - synchronous (int)
+    - default: 0
+    -  If zero (the default), VFH runs in its own thread. If non-zero, VFH runs in the main Player thread, which will make the server less responsive, but prevent nasty asynchronous behaviour under high CPU load. This is probably only useful when running demanding simulations.
 
 @par Example
 @verbatim
@@ -190,13 +222,13 @@ driver
 )
 @endverbatim
 
-@author Chris Jones, Brian Gerkey, Alex Brooks
+@author Chris Jones, Brian Gerkey, Alex Brooks, Richard Vaughan
 
 */
 
 /** @} */
 
-class VFH_Class : public Driver
+class VFH_Class : public ThreadedDriver
 {
   public:
     // Constructor
@@ -210,16 +242,23 @@ class VFH_Class : public Driver
     virtual int Shutdown();
 
     // Process incoming messages from clients
-    virtual int ProcessMessage(MessageQueue* resp_queue,
+    virtual int ProcessMessage(QueuePointer & resp_queue,
                                player_msghdr * hdr,
                                void * data);
     // Main function for device thread.
     virtual void Main();
 
+    // Update method called in Player's main thread
+    virtual void Update();
+
   private:
     bool active_goal;
     bool turninginplace;
 
+    // most of the work is done in this method. It is either called
+    // from ::Main() or ::Update(), depending on ::synchronous_mode.
+    void DoOneUpdate();
+
     // Set up the odometry device.
     int SetupOdom();
     int ShutdownOdom();
@@ -254,9 +293,19 @@ class VFH_Class : public Driver
     // and return values are in degrees.
     double angle_diff(double a, double b);
 
+    // Devices we provide
+    player_devaddr_t position_id;
+    player_devaddr_t planner_id;
+    bool planner;
+
+    player_planner_data_t planner_data;
+
+
+    // Devices we require
     // Odometry device info
     Device *odom;
     player_devaddr_t odom_addr;
+
     double dist_eps;
     double ang_eps;
 
@@ -280,11 +329,12 @@ class VFH_Class : public Driver
     Device *sonar;
     player_devaddr_t sonar_addr;
     int num_sonars;
-    player_pose_t sonar_poses[PLAYER_SONAR_MAX_SAMPLES];
+    player_pose3d_t * sonar_poses;
 
     // Laser range and bearing values
     int laser_count;
-    double laser_ranges[PLAYER_LASER_MAX_SAMPLES][2];
+    //double (*laser_ranges)[2];
+    double laser_ranges[361][2];
 
     // Control velocity
     double con_vel[3];
@@ -293,6 +343,31 @@ class VFH_Class : public Driver
     double reset_odom_x, reset_odom_y, reset_odom_t;
     int32_t goal_x, goal_y, goal_t;
     int cmd_state, cmd_type;
+
+
+  // iff true, run in the main Player thread instead of a dedicated
+  // thread. Useful under heavy CPU load, for example when using a
+  // simulator with lots of robots - rtv
+  bool synchronous_mode;
+
+  /* the following vars used to be local to the Main() method. I moved
+     them here when implementing the synchronous mode - rtv */
+
+  float dist;
+  double angdiff;
+  struct timeval startescape, curr;
+  bool escaping;
+  double timediff; int
+  escape_turnrate_deg;
+
+  // bookkeeping to implement hysteresis when rotating at the goal
+  int rotatedir;
+
+  // bookkeeping to implement smarter escape policy
+  int escapedir;
+
+  /* end of moved vars - rtv*/
+
 };
 
 // Initialization function
@@ -303,7 +378,7 @@ VFH_Init(ConfigFile* cf, int section)
 }
 
 // a driver registration function
-void VFH_Register(DriverTable* table)
+void vfh_Register(DriverTable* table)
 {
   table->AddDriver("vfh",  VFH_Init);
   return;
@@ -328,12 +403,14 @@ int VFH_Class::Setup()
   if (this->sonar_addr.interf && this->SetupSonar() != 0)
     return -1;
 
-  // FIXME
-  // Allocate and intialize
-  vfh_Algorithm->Init();
+  // initialize some navigation state
+  rotatedir = 1;
+  escapedir = 1;
+  escaping = 0;
 
   // Start the driver thread.
-  this->StartThread();
+  if( ! synchronous_mode )
+    this->StartThread();
 
   return 0;
 }
@@ -343,7 +420,8 @@ int VFH_Class::Setup()
 int VFH_Class::Shutdown()
 {
   // Stop the driver thread.
-  this->StopThread();
+  if( ! this->synchronous_mode )
+    this->StopThread();
 
   // Stop the laser
   if(this->laser)
@@ -359,6 +437,17 @@ int VFH_Class::Shutdown()
   return 0;
 }
 
+
+/////////////////////////////////////////////////////////////////////////////////
+// Update the driver in the main Player thread
+void VFH_Class::Update()
+{
+  if( synchronous_mode )
+    this->DoOneUpdate();
+  // otherwise, a dedicated thread is running: see VFH_Class::Main()
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the underlying odom device.
 int VFH_Class::SetupOdom()
@@ -390,12 +479,12 @@ int VFH_Class::SetupOdom()
   player_position2d_geom_t* geom = (player_position2d_geom_t*)msg->GetPayload();
 
   // take the bigger of the two dimensions and halve to get a radius
-  float robot_radius = MAX(geom->size.sl,geom->size.sw);
+  float robot_radius = static_cast<float> (MAX(geom->size.sl,geom->size.sw));
   robot_radius /= 2.0;
 
   delete msg;
 
-  vfh_Algorithm->SetRobotRadius( robot_radius * 1e3 );
+  vfh_Algorithm->SetRobotRadius( robot_radius * 1000 );
 
   this->odom_pose[0] = this->odom_pose[1] = this->odom_pose[2] = 0.0;
   this->odom_vel[0] = this->odom_vel[1] = this->odom_vel[2] = 0.0;
@@ -435,11 +524,7 @@ int VFH_Class::SetupLaser()
   }
 
   this->laser_count = 0;
-  for(int i=0;i<PLAYER_LASER_MAX_SAMPLES;i++)
-  {
-    this->laser_ranges[i][0] = 0;
-    this->laser_ranges[i][1] = 0;
-  }
+  //this->laser_ranges = NULL;
   return 0;
 }
 
@@ -474,6 +559,7 @@ int VFH_Class::SetupSonar()
   // Store the sonar poses
   cfg = (player_sonar_geom_t*)msg->GetPayload();
   this->num_sonars = cfg->poses_count;
+  this->sonar_poses = new player_pose3d_t[num_sonars];
   for(int i=0;i<this->num_sonars;i++)
   {
     this->sonar_poses[i] = cfg->poses[i];
@@ -482,11 +568,7 @@ int VFH_Class::SetupSonar()
   delete msg;
 
   this->laser_count = 0;
-  for(int i=0;i<PLAYER_LASER_MAX_SAMPLES;i++)
-  {
-    this->laser_ranges[i][0] = 0;
-    this->laser_ranges[i][1] = 0;
-  }
+  //this->laser_ranges = NULL;
   return 0;
 }
 
@@ -496,6 +578,8 @@ int VFH_Class::SetupSonar()
 int VFH_Class::ShutdownLaser()
 {
   this->laser->Unsubscribe(this->InQueue);
+  //delete [] laser_ranges;
+  //laser_ranges = NULL;
   return 0;
 }
 
@@ -504,6 +588,10 @@ int VFH_Class::ShutdownLaser()
 int VFH_Class::ShutdownSonar()
 {
   this->sonar->Unsubscribe(this->InQueue);
+  //delete [] laser_ranges;
+  //laser_ranges = NULL;
+  delete [] sonar_poses;
+  sonar_poses = NULL;
   return 0;
 }
 
@@ -525,8 +613,21 @@ VFH_Class::ProcessOdom(player_msghdr_t* hdr, player_position2d_data_t &data)
 
   // Also change this info out for use by others
   player_msghdr_t newhdr = *hdr;
-  newhdr.addr = this->device_addr;
-  this->Publish(NULL, &newhdr, (void*)&data);
+  newhdr.addr = this->position_id;
+  this->Publish(&newhdr, (void*)&data);
+
+ if(this->planner)
+ {
+   this->planner_data.pos.px = data.pos.px;
+   this->planner_data.pos.py = data.pos.py;
+   this->planner_data.pos.pa = data.pos.pa;
+
+   this->Publish(this->planner_id,
+                 PLAYER_MSGTYPE_DATA,
+                 PLAYER_PLANNER_DATA_STATE,
+                 (void*)&this->planner_data,sizeof(this->planner_data), NULL);
+ }
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -540,11 +641,11 @@ VFH_Class::ProcessLaser(player_laser_data_t &data)
   b = RTOD(data.min_angle);
   db = RTOD(data.resolution);
 
-  this->laser_count = 181;
-  assert(this->laser_count <
-         (int)sizeof(this->laser_ranges) / (int)sizeof(this->laser_ranges[0]));
+  this->laser_count = 361;
+  //if (!laser_ranges)
+	  //this->laser_ranges = new double[laser_count][2];
 
-  for(i = 0; i < PLAYER_LASER_MAX_SAMPLES; i++)
+  for(i = 0; i < laser_count; i++)
     this->laser_ranges[i][0] = -1;
 
   // vfh seems to be very oriented around 180 degree scans so interpolate to get 180 degrees
@@ -552,14 +653,16 @@ VFH_Class::ProcessLaser(player_laser_data_t &data)
   for(i = 0; i < 181; i++)
   {
   	unsigned int index = (int)rint(i/db);
-  	assert(index >= 0 && index < data.ranges_count);
+  	//assert(index >= 0 && index < data.ranges_count);
+  	if(index < 0 || index >= data.ranges_count)
+          continue;
     this->laser_ranges[i*2][0] = data.ranges[index] * 1e3;
 //    this->laser_ranges[i*2][1] = index;
 //    b += db;
   }
 
   r = 1000000.0;
-  for (i = 0; i < PLAYER_LASER_MAX_SAMPLES; i++)
+  for (i = 0; i < laser_count; i++)
   {
     if (this->laser_ranges[i][0] != -1) {
       r = this->laser_ranges[i][0];
@@ -581,17 +684,17 @@ VFH_Class::ProcessSonar(player_sonar_data_t &data)
   float sonarDistToCenter = 0.0;
 
   this->laser_count = count;
-  assert(this->laser_count <
-         (int)sizeof(this->laser_ranges) / (int)sizeof(this->laser_ranges[0]));
+  //if (!laser_ranges)
+	  //this->laser_ranges = new double[laser_count][2];
 
-  for(i = 0; i < PLAYER_LASER_MAX_SAMPLES; i++)
+  for(i = 0; i < laser_count; i++)
     this->laser_ranges[i][0] = -1;
 
   //b += 90.0;
   for(i = 0; i < (int)data.ranges_count; i++)
   {
-    for(b = RTOD(this->sonar_poses[i].pa) + 90.0 - cone_width/2.0;
-        b < RTOD(this->sonar_poses[i].pa) + 90.0 + cone_width/2.0;
+    for(b = RTOD(this->sonar_poses[i].pyaw) + 90.0 - cone_width/2.0;
+        b < RTOD(this->sonar_poses[i].pyaw) + 90.0 + cone_width/2.0;
         b+=0.5)
     {
       if((b < 0) || (rint(b*2) >= count))
@@ -602,14 +705,14 @@ VFH_Class::ProcessSonar(player_sonar_data_t &data)
       // laser ranges, we must make the sonar ranges appear like laser ranges. To do this, we take
       // into account the offset of a sonar's geometry from the center. Simply add the distance from
       // the center of the robot to a sonar to the sonar's range reading.
-      sonarDistToCenter = sqrt(pow(this->sonar_poses[i].px,2) + pow(this->sonar_poses[i].py,2));
+      sonarDistToCenter = static_cast<float> (sqrt(pow(this->sonar_poses[i].px,2) + pow(this->sonar_poses[i].py,2)));
       this->laser_ranges[(int)rint(b * 2)][0] = (sonarDistToCenter + data.ranges[i]) * 1e3;
       this->laser_ranges[(int)rint(b * 2)][1] = b;
     }
   }
 
   r = 1000000.0;
-  for (i = 0; i < PLAYER_LASER_MAX_SAMPLES; i++)
+  for (i = 0; i < laser_count; i++)
   {
     if (this->laser_ranges[i][0] != -1) {
       r = this->laser_ranges[i][0];
@@ -666,7 +769,7 @@ VFH_Class::PutCommand( int cmd_speed, int cmd_turnrate )
 
 ////////////////////////////////////////////////////////////////////////////////
 // Process an incoming message
-int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
+int VFH_Class::ProcessMessage(QueuePointer & resp_queue,
                               player_msghdr * hdr,
                               void * data)
 {
@@ -693,9 +796,29 @@ int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
     ProcessSonar(*reinterpret_cast<player_sonar_data_t *> (data));
     return 0;
   }
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
+                                PLAYER_PLANNER_CMD_GOAL,
+                                this->planner_id))
+  {
+    // Message on the planner interface
+    // Emulate a message on the position2d interface
+
+    player_position2d_cmd_pos_t cmd_pos;
+    player_planner_cmd_t *cmd_planner = (player_planner_cmd_t *) data;
+
+    memset(&cmd_pos, 0, sizeof(cmd_pos));
+    cmd_pos.pos.px = cmd_planner->goal.px;
+    cmd_pos.pos.py = cmd_planner->goal.py;
+    cmd_pos.pos.pa = cmd_planner->goal.pa;
+    cmd_pos.state = 1;
+
+    /* Process position2d command */
+    ProcessCommand(hdr, cmd_pos);
+    return 0;
+  }
   else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
                                 PLAYER_POSITION2D_CMD_POS,
-                                this->device_addr))
+                                this->position_id))
   {
     assert(hdr->size == sizeof(player_position2d_cmd_pos_t));
     ProcessCommand(hdr, *reinterpret_cast<player_position2d_cmd_pos_t *> (data));
@@ -703,7 +826,7 @@ int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
   }
   else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
                                 PLAYER_POSITION2D_CMD_VEL,
-                                this->device_addr))
+                                this->position_id))
   {
     assert(hdr->size == sizeof(player_position2d_cmd_vel_t));
     // make a copy of the header and change the address
@@ -714,8 +837,8 @@ int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
     this->active_goal = false;
 
     return 0;
-  }  
-  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ, -1, this->device_addr))
+  }
+  else if(Message::MatchMessage(hdr, PLAYER_MSGTYPE_REQ, -1, this->position_id))
   {
     // Pass the request on to the underlying position device and wait for
     // the reply.
@@ -736,7 +859,7 @@ int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
     player_msghdr_t* rephdr = msg->GetHeader();
     void* repdata = msg->GetPayload();
     // Copy in our address and forward the response
-    rephdr->addr = this->device_addr;
+    rephdr->addr = this->position_id;
     this->Publish(resp_queue, rephdr, repdata);
     delete msg;
     return(0);
@@ -751,37 +874,35 @@ int VFH_Class::ProcessMessage(MessageQueue* resp_queue,
 // Main function for device thread
 void VFH_Class::Main()
 {
-  float dist;
-  double angdiff;
-  struct timeval startescape, curr;
-  bool escaping = false;
-  double timediff;
-  int escape_turnrate_deg;
+  // loop over the main VFH activity. If we were in synchronous mode,
+  // this would happen in VFH_Class::Update(), in the main Player
+  // thread instead.
+  while( true )
+    {
+      // Wait till we get new data
+      this->Wait();
+      // Test if we are supposed to cancel this thread.
+      pthread_testcancel();
+      this->DoOneUpdate();
+    }
+}
 
-  // bookkeeping to implement hysteresis when rotating at the goal
-  int rotatedir = 1;
+////////////////////////////////////////////////////////////////////////////////
+//
+void VFH_Class::DoOneUpdate()
+{
+  if( this->InQueue->Empty() )
+    return;
 
-  // bookkeeping to implement smarter escape policy
-  int escapedir;
-
-  escapedir = 1;
-  while (true)
-  {
-    // Wait till we get new data
-    this->Wait();
-
-    // Test if we are supposed to cancel this thread.
-    pthread_testcancel();
-
-    // Process any pending requests.
+  // Process any pending requests.
     this->ProcessMessages();
 
     if(!this->active_goal)
-      continue;
+      return;//continue;
 
     // Figure how far, in distance and orientation, we are from the goal
-    dist = sqrt(pow((goal_x - this->odom_pose[0]),2) +
-                pow((goal_y - this->odom_pose[1]),2));
+    dist = static_cast<float> (sqrt(pow((goal_x - this->odom_pose[0]),2) +
+                pow((goal_y - this->odom_pose[1]),2)));
     angdiff = this->angle_diff((double)this->goal_t,this->odom_pose[2]);
 
     // If we're currently escaping after a stall, check whether we've done
@@ -836,14 +957,15 @@ void VFH_Class::Main()
       this->speed = this->turnrate = 0;
       PutCommand( this->speed, this->turnrate );
       this->turninginplace = false;
+      this->planner_data.done = 1;
     }
     // CASE 3: The robot is too far from the goal position, so invoke VFH to
     //         get there.
     else if (dist > (this->dist_eps * 1e3))
     {
-      float Desired_Angle = (90 + atan2((goal_y - this->odom_pose[1]),
-                                        (goal_x - this->odom_pose[0]))
-                             * 180 / M_PI - this->odom_pose[2]);
+      float Desired_Angle = static_cast<float> ((90 + atan2((goal_y - this->odom_pose[1]),
+                                                (goal_x - this->odom_pose[0]))
+                                                * 180 / M_PI - this->odom_pose[2]));
 
       while (Desired_Angle > 360.0)
         Desired_Angle -= 360.0;
@@ -854,18 +976,18 @@ void VFH_Class::Main()
                                  (int)(this->odom_vel[0]),
                                  Desired_Angle,
                                  dist,
-                                 this->dist_eps * 1e3,
+                                 static_cast<float> (this->dist_eps * 1000),
                                  this->speed,
                                  this->turnrate );
-      
-      // HACK: if we're within twice the distance threshold, 
+
+      // HACK: if we're within twice the distance threshold,
       // and still going fast, slow down.
 
       if((dist < (this->dist_eps * 1e3 * 2.0)) &&
          (this->speed > (vfh_Algorithm->GetCurrentMaxSpeed() / 2.0)))
       {
 //        int foo = this->speed;
-        this->speed = 
+        this->speed =
                 (int)rint(vfh_Algorithm->GetCurrentMaxSpeed() / 2.0);
         //printf("slowing down from %d to %d\n",
 	//     foo, this->speed);
@@ -909,7 +1031,7 @@ void VFH_Class::Main()
 
       this->PutCommand( this->speed, this->turnrate );
     }
-  }
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -933,14 +1055,27 @@ VFH_Class::ProcessCommand(player_msghdr_t* hdr, player_position2d_cmd_pos_t &cmd
     this->goal_x = x;
     this->goal_y = y;
     this->goal_t = t;
+
+    if(this->planner)
+    {
+       this->planner_data.goal.px = cmd.pos.px;
+       this->planner_data.goal.py = cmd.pos.py;
+       this->planner_data.goal.pa = cmd.pos.pa;
+       this->planner_data.done = 0;
+
+       this->planner_data.valid = 1;
+            /* Not necessarily. But VFH will try anything once */
+
+       this->planner_data.waypoint_idx = -1; /* Not supported */
+       this->planner_data.waypoints_count = -1; /* Not supported */
+    }
   }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor
 VFH_Class::VFH_Class( ConfigFile* cf, int section)
-  : Driver(cf, section, true,
-           PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_POSITION2D_CODE)
+  : ThreadedDriver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN)
 {
   double cell_size;
   int window_diameter;
@@ -966,6 +1101,9 @@ VFH_Class::VFH_Class( ConfigFile* cf, int section)
   this->speed = 0;
   this->turnrate = 0;
 
+  // read the synchronous flag from the cfg file: defaults to not synchronous
+  this->synchronous_mode = cf->ReadInt(section, "synchronous", 0 ) != 0 ? true : false;
+
   cell_size = cf->ReadLength(section, "cell_size", 0.1) * 1e3;
   window_diameter = cf->ReadInt(section, "window_diameter", 61);
   sector_angle = cf->ReadInt(section, "sector_angle", 5);
@@ -1033,6 +1171,40 @@ VFH_Class::VFH_Class( ConfigFile* cf, int section)
                                           obs_cutoff_1ms,
                                           weight_desired_dir,
                                           weight_current_dir);
+
+  // Devices we provide
+  memset(&this->planner_id, 0, sizeof(player_devaddr_t));
+  memset(&this->planner_data, 0, sizeof(player_planner_data_t));
+  planner = false;
+  if (cf->ReadDeviceAddr(&(this->planner_id), section, "provides",
+                        PLAYER_PLANNER_CODE, -1, NULL) == 0)
+  {
+    planner = true;
+    if (this->AddInterface(this->planner_id) != 0)
+    {
+      this->SetError(-1);
+      return;
+    }
+
+    /* Initialize planner data */
+    this->planner_data.done = 1;
+    this->planner_data.valid = 0;
+    this->planner_data.waypoint_idx = -1; /* Not supported */
+    this->planner_data.waypoints_count = -1; /* Not supported */
+  }
+
+  memset(&this->position_id, 0, sizeof(player_devaddr_t));
+  if (cf->ReadDeviceAddr(&(this->position_id), section, "provides",
+                        PLAYER_POSITION2D_CODE, -1, NULL) == 0)
+  {
+    if (this->AddInterface(this->position_id) != 0)
+    {
+      this->SetError(-1);
+      return;
+    }
+  }
+
+
   this->odom = NULL;
   if (cf->ReadDeviceAddr(&this->odom_addr, section, "requires",
                          PLAYER_POSITION2D_CODE, -1, NULL) != 0)
@@ -1061,6 +1233,13 @@ VFH_Class::VFH_Class( ConfigFile* cf, int section)
   // Laser settings
   //TODO this->laser_max_samples = cf->ReadInt(section, "laser_max_samples", 10);
 
+  // FIXME
+  // Allocate and intialize
+  vfh_Algorithm->Init();
+
+
+
+
   return;
 }
 
@@ -1069,7 +1248,8 @@ VFH_Class::VFH_Class( ConfigFile* cf, int section)
 // VFH Code
 
 
-VFH_Class::~VFH_Class() {
+VFH_Class::~VFH_Class()
+{
   delete this->vfh_Algorithm;
 
   return;
