@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: amtecpowercube.cc,v 1.17.2.2 2006/09/25 15:54:58 gerkey Exp $
+ * $Id: amtecpowercube.cc 7208 2008-12-19 02:10:56Z gbiggs $
  */
 
 /** @ingroup drivers */
@@ -36,7 +36,7 @@ the newer CAN-based units.  Please submit a patch to support the CAN
 protocol.
 
 The amtecpowercube driver supports both position and velocity control,
-via the PLAYER_PTZ_CONTROL_MODE_REQ request.  For constant swiveling,
+via the PLAYER_PTZ_REQ_CONTROL_MODE request.  For constant swiveling,
 the PowerCube works better under velocity control.
 
 Note that this driver is relatively new and not thoroughly tested.
@@ -55,7 +55,7 @@ Note that this driver is relatively new and not thoroughly tested.
 
 @par Configuration requests
 
-- PLAYER_PTZ_CONTROL_MODE_REQ
+- PLAYER_PTZ_REQ_CONTROL_MODE
 
 @par Configuration file options
 
@@ -68,7 +68,7 @@ Note that this driver is relatively new and not thoroughly tested.
     commanding it
 - speed (angle)
   - Default: 40 deg/sec
-  - Maximum pan/tilt speed 
+  - Maximum pan/tilt speed
 
 @par Example
 
@@ -87,9 +87,7 @@ driver
 
 /** @} */
 
-#ifdef HAVE_CONFIG_H
-  #include "config.h"
-#endif
+#include "config.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -106,7 +104,6 @@ driver
 #include <libplayercore/playercore.h>
 #include <replace/replace.h>
 
-
 #define AMTEC_DEFAULT_PORT "/dev/ttyS0"
 
 #define AMTEC_SLEEP_TIME_USEC 20000
@@ -154,7 +151,7 @@ driver
 #define AMTEC_STATE_HOME_OK   0x02
 #define AMTEC_STATE_HALTED    0x04
 
-class AmtecPowerCube:public Driver 
+class AmtecPowerCube:public ThreadedDriver 
 {
   private:
     // this function will be run in a separate thread
@@ -213,10 +210,10 @@ class AmtecPowerCube:public Driver
     AmtecPowerCube( ConfigFile* cf, int section);
 
     // MessageHandler
-    int ProcessMessage(MessageQueue* resp_queue, player_msghdr * hdr, void * data);
+    int ProcessMessage(QueuePointer &resp_queue, player_msghdr * hdr, void * data);
 
-    virtual int Setup();
-    virtual int Shutdown();
+    virtual int MainSetup();
+    virtual void MainQuit();
 };
 
 // initialization function
@@ -226,14 +223,14 @@ Driver* AmtecPowerCube_Init( ConfigFile* cf, int section)
 }
 
 // a driver registration function
-void 
-AmtecPowerCube_Register(DriverTable* table)
+void
+amtecpowercube_Register(DriverTable* table)
 {
   table->AddDriver("amtecpowercube",  AmtecPowerCube_Init);
 }
 
-AmtecPowerCube::AmtecPowerCube( ConfigFile* cf, int section) 
-        : Driver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_PTZ_CODE)
+AmtecPowerCube::AmtecPowerCube( ConfigFile* cf, int section)
+        : ThreadedDriver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_PTZ_CODE)
 {
   fd = -1;
 /*  player_ptz_data_t data;
@@ -247,11 +244,11 @@ AmtecPowerCube::AmtecPowerCube( ConfigFile* cf, int section)
 
   this->serial_port = cf->ReadString(section, "port", AMTEC_DEFAULT_PORT);
   this->return_to_home = cf->ReadInt(section, "home", 0);
-  this->speed = (int)rint(RTOD(cf->ReadAngle(section, "speed", 
+  this->speed = (int)rint(RTOD(cf->ReadAngle(section, "speed",
                                              AMTEC_DEFAULT_SPEED_DEG_PER_SEC)));
 }
 
-int 
+int
 AmtecPowerCube::Reset()
 {
   unsigned char buf[AMTEC_MAX_CMDSIZE];
@@ -281,7 +278,7 @@ AmtecPowerCube::Reset()
   return(0);
 }
 
-int 
+int
 AmtecPowerCube::Home()
 {
   unsigned char buf[AMTEC_MAX_CMDSIZE];
@@ -336,7 +333,7 @@ AmtecPowerCube::Home()
   return(0);
 }
 
-int 
+int
 AmtecPowerCube::Halt()
 {
   unsigned char buf[AMTEC_MAX_CMDSIZE];
@@ -366,8 +363,8 @@ AmtecPowerCube::Halt()
   return(0);
 }
 
-int 
-AmtecPowerCube::Setup()
+int
+AmtecPowerCube::MainSetup()
 {
   struct termios term;
   short pan,tilt;
@@ -387,8 +384,8 @@ AmtecPowerCube::Setup()
   {
     PLAYER_ERROR1("open() failed: %s", strerror(errno));
     return(-1);
-  }  
- 
+  }
+
   if(tcflush(fd, TCIFLUSH ) < 0 )
   {
     PLAYER_ERROR1("tcflush() failed: %s", strerror(errno));
@@ -403,11 +400,11 @@ AmtecPowerCube::Setup()
     fd = -1;
     return(-1);
   }
-  
+
   cfmakeraw(&term);
   cfsetispeed(&term, B38400);
   cfsetospeed(&term, B38400);
-  
+
   if(tcsetattr(fd, TCSAFLUSH, &term) < 0 )
   {
     PLAYER_ERROR1("tcsetattr() failed: %s", strerror(errno));
@@ -421,7 +418,7 @@ AmtecPowerCube::Setup()
   if(GetPanTiltPos(&pan,&tilt))
   {
     printf("Couldn't connect to Amtec PowerCube most likely because the unit\n"
-                    "is not connected or is connected not to %s\n", 
+                    "is not connected or is connected not to %s\n",
                     serial_port);
     close(fd);
     fd = -1;
@@ -465,19 +462,12 @@ AmtecPowerCube::Setup()
     return(-1);
   }
 
-  // start the thread to talk with the camera
-  StartThread();
-
   return(0);
 }
 
-int
-AmtecPowerCube::Shutdown()
+void
+AmtecPowerCube::MainQuit()
 {
-  if(fd == -1)
-    return(0);
-
-  StopThread();
 
   // stop the unit
   if(Halt())
@@ -491,7 +481,6 @@ AmtecPowerCube::Shutdown()
     PLAYER_ERROR1("close() failed:%s",strerror(errno));
   fd = -1;
   puts("Amtec PowerCube has been shutdown");
-  return(0);
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -525,7 +514,7 @@ AmtecPowerCube::Uint16ToBytes(unsigned char *bytes, unsigned short s)
   memcpy(bytes, (void*)&s, 2);
 }
 
-int 
+int
 AmtecPowerCube::SendCommand(int id, unsigned char* cmd, size_t len)
 {
   size_t i;
@@ -685,7 +674,7 @@ AmtecPowerCube::AwaitAnswer(unsigned char* buf, size_t len)
     }
     else
     {
-      if(buf[0]==AMTEC_STX) 
+      if(buf[0]==AMTEC_STX)
         return(AwaitETX(buf,len));
       else
         continue;
@@ -700,27 +689,27 @@ AmtecPowerCube::ConvertBuffer(unsigned char* buf, size_t len)
 
   actual_len = len;
 
-  for (i=0;i<len;i++) 
+  for (i=0;i<len;i++)
   {
-    if(buf[i]==AMTEC_DLE) 
+    if(buf[i]==AMTEC_DLE)
     {
-      switch(buf[i+1]) 
+      switch(buf[i+1])
       {
         case 0x82:
           buf[i] = 0x02;
-          for(j=i+2;j<len;j++) 
+          for(j=i+2;j<len;j++)
             buf[j-1] = buf[j];
           actual_len--;
           break;
         case 0x83:
           buf[i] = 0x03;
-          for(j=i+2;j<len;j++) 
+          for(j=i+2;j<len;j++)
             buf[j-1] = buf[j];
           actual_len--;
           break;
         case 0x90:
           buf[i] = 0x10;
-          for(j=i+2;j<len;j++) 
+          for(j=i+2;j<len;j++)
             buf[j-1] = buf[j];
           actual_len--;
           break;
@@ -832,7 +821,7 @@ AmtecPowerCube::GetPanTiltPos(short* pan, short* tilt)
   }
   // reverse pan angle, to increase ccw, then normalize
   *pan = -(short)RTOD(NORMALIZE(tmp));
-  
+
   // get the tilt
   if(GetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_ACT_POS, &tmp) < 0)
   {
@@ -857,7 +846,7 @@ AmtecPowerCube::GetPanTiltVel(short* panspeed, short* tiltspeed)
   }
   // reverse pan angle, to increase ccw, then normalize
   *panspeed = -(short)RTOD(NORMALIZE(tmp));
-  
+
   // get the tilt
   if(GetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_ACT_VEL, &tmp) < 0)
   {
@@ -876,7 +865,7 @@ AmtecPowerCube::SetLimits()
   // counts up clockwise, rather than ccw.
   if(this->minpan != INT_MAX)
   {
-    if(SetFloatParam(AMTEC_MODULE_PAN, AMTEC_PARAM_MAX_POS, 
+    if(SetFloatParam(AMTEC_MODULE_PAN, AMTEC_PARAM_MAX_POS,
                      NORMALIZE(DTOR(-this->minpan))) < 0)
     {
       PLAYER_ERROR("SetFloatParam() failed");
@@ -885,7 +874,7 @@ AmtecPowerCube::SetLimits()
   }
   if(this->maxpan != INT_MAX)
   {
-    if(SetFloatParam(AMTEC_MODULE_PAN, AMTEC_PARAM_MIN_POS, 
+    if(SetFloatParam(AMTEC_MODULE_PAN, AMTEC_PARAM_MIN_POS,
                      NORMALIZE(DTOR(-this->maxpan))) < 0)
     {
       PLAYER_ERROR("SetFloatParam() failed");
@@ -894,7 +883,7 @@ AmtecPowerCube::SetLimits()
   }
   if(this->mintilt != INT_MAX)
   {
-    if(SetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_MIN_POS, 
+    if(SetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_MIN_POS,
                      NORMALIZE(DTOR(this->mintilt))) < 0)
     {
       PLAYER_ERROR("SetFloatParam() failed");
@@ -903,7 +892,7 @@ AmtecPowerCube::SetLimits()
   }
   if(this->maxtilt != INT_MAX)
   {
-    if(SetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_MAX_POS, 
+    if(SetFloatParam(AMTEC_MODULE_TILT, AMTEC_PARAM_MAX_POS,
                      NORMALIZE(DTOR(this->maxtilt))) < 0)
     {
       PLAYER_ERROR("SetFloatParam() failed");
@@ -922,7 +911,7 @@ AmtecPowerCube::SetPanPos(short oldpan, short pan)
   unsigned short time;
 
   newpan = DTOR(pan);
-  time = (unsigned short)rint(((double)abs(pan - oldpan) / 
+  time = (unsigned short)rint(((double)abs(pan - oldpan) /
                                (double)this->speed) * 1e3);
 
   cmd[0] = AMTEC_CMD_SET_MOTION;
@@ -945,7 +934,7 @@ AmtecPowerCube::SetTiltPos(short oldtilt, short tilt)
   unsigned short time;
 
   newtilt = DTOR(tilt);
-  time = (unsigned short)rint(((double)abs(tilt - oldtilt) / 
+  time = (unsigned short)rint(((double)abs(tilt - oldtilt) /
                                (double)this->speed) * 1e3);
 
   cmd[0] = AMTEC_CMD_SET_MOTION;
@@ -999,10 +988,9 @@ AmtecPowerCube::SetTiltVel(short tiltspeed)
 }
 
 
-int AmtecPowerCube::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void * data)
+int AmtecPowerCube::ProcessMessage(QueuePointer &resp_queue, player_msghdr * hdr, void * data)
 {
   assert(hdr);
-  assert(data);
 
 	if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, PLAYER_PTZ_CMD_STATE, device_addr))
 	{
@@ -1011,7 +999,7 @@ int AmtecPowerCube::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hd
       short newpanspeed, newtiltspeed;
 	  assert(hdr->size == sizeof(player_ptz_cmd_t));
 	  player_ptz_cmd_t & command = *reinterpret_cast<player_ptz_cmd_t *> (data);
-  
+
       if(this->controlmode == PLAYER_PTZ_POSITION_CONTROL)
       {
         // reverse pan angle, to increase ccw
@@ -1100,11 +1088,11 @@ int AmtecPowerCube::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hd
         return 0;
       }
 	}
-    
+
     return -1;
 }
- 
-void 
+
+void
 AmtecPowerCube::Main()
 {
   player_ptz_data_t data;
@@ -1152,7 +1140,7 @@ AmtecPowerCube::Main()
     data.panspeed = (currpanspeed);
     data.tiltspeed = (currtiltspeed);
 
-    Publish(device_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, (unsigned char*)&data, sizeof(player_ptz_data_t),NULL);
+    Publish(device_addr, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, (unsigned char*)&data, sizeof(player_ptz_data_t),NULL);
 
     // get the module state (for debugging and warning)
     unsigned int state;
@@ -1165,11 +1153,11 @@ AmtecPowerCube::Main()
     if(state & AMTEC_STATE_ERROR)
     {
       PLAYER_ERROR1("the Amtec unit has encountered an error and will need\n"
-                    "    to be reset; bailing.   Current module state: 0x%x", 
+                    "    to be reset; bailing.   Current module state: 0x%x",
                     state);
       pthread_exit(NULL);
     }
-    
+
     usleep(AMTEC_SLEEP_TIME_USEC);
   }
 }
