@@ -1,6 +1,78 @@
-#ifdef HAVE_CONFIG_H
-  #include "config.h"
-#endif
+/*
+ *  Player - One Hell of a Robot Server
+ *  Copyright (C) 2000  Brian Gerkey   &  Kasper Stoy
+ *                      gerkey@usc.edu    kaspers@robotics.usc.edu
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
+/** @ingroup drivers */
+/** @{ */
+/** @defgroup driver_canonvcc4 canonvcc4
+ * @brief Canon VC-C4 pan-tilt-zoom cameras
+
+The canonvcc4 driver provides control of a Canon VC-C4 (and possibly similar)
+pan-tilt-zoom camera unit.
+
+@todo Provide more detailed documentation
+
+@par Compile-time dependencies
+
+- none
+
+@par Provides
+
+- @ref interface_ptz
+
+@par Requires
+
+- None
+
+@par Configuration requests
+
+- PLAYER_PTZ_REQ_GENERIC
+
+@par Configuration file options
+
+- port (string)
+  - Default: "/dev/ttyS1"
+  - The serial port to be used.
+
+- fov (angle tuple)
+  - Default: [3.0 30.0]
+  - The minimum and maximum fields of view (in degrees), which will depend on
+    the lens(es) you are using.
+
+@par Example
+
+@verbatim
+driver
+(
+  name "canonvcc4"
+  provides ["ptz:0"]
+  port "/dev/ttyS1"
+  fov [3 30]
+)
+@endverbatim
+
+@author ??
+
+*/
+/** @} */
+#include "config.h"
 
 #include <fcntl.h>
 #include <stdio.h>
@@ -18,6 +90,14 @@
 #include <libplayercore/playercore.h>
 #include <replace/replace.h>
 
+#ifndef CRTSCTS
+#ifdef IHFLOW
+#ifdef OHFLOW
+#define CRTSCTS ((IHFLOW) | (OHFLOW))
+#endif
+#endif
+#endif
+
 #define CAM_ERROR_NONE 0x30
 #define CAM_ERROR_BUSY 0x31
 #define CAM_ERROR_PARAM 0x35
@@ -46,7 +126,7 @@
 static unsigned int error_code;
 
 
-class canonvcc4:public Driver 
+class canonvcc4:public ThreadedDriver
 {
  private:
   bool ptz_fd_blocking;
@@ -68,7 +148,7 @@ class canonvcc4:public Driver
   virtual int SendAbsZoom(int zoom);
   virtual int GetAbsZoom(int* zoom);
   virtual int GetAbsPanTilt(int* pan, int* tilt);
-  virtual void PrintPacket(char* str, unsigned char* cmd, int len);
+  virtual void PrintPacket(const char* str, unsigned char* cmd, int len);
 
 
  public:
@@ -87,16 +167,16 @@ class canonvcc4:public Driver
   void ProcessCommand(player_ptz_cmd_t & command);
   int pandemand , tiltdemand , zoomdemand ;
 
-  // MessageHandler  
-int ProcessMessage(MessageQueue* resp_queue, player_msghdr * hdr, void * data);
+  // MessageHandler
+int ProcessMessage(QueuePointer &resp_queue, player_msghdr * hdr, void * data);
   // int ProcessMessage(ClientData * client, player_msghdr * hdr, uint8_t * data, uint8_t * resp_data, int * resp_len);
 
-  virtual int Setup();
-  virtual int Shutdown();
+  virtual int MainSetup();
+  virtual void MainQuit();
 };
 
 /************************************************************************/
-Driver* 
+Driver*
 canonvcc4_Init(ConfigFile* cf, int section)
 {
   return((Driver*)(new canonvcc4(cf, section)));
@@ -105,7 +185,7 @@ canonvcc4_Init(ConfigFile* cf, int section)
 /************************************************************************/
 
 // a driver registration function
-void 
+void
 canonvcc4_Register(DriverTable* table)
 {
   table->AddDriver("canonvcc4", canonvcc4_Init);
@@ -113,11 +193,11 @@ canonvcc4_Register(DriverTable* table)
 
 /************************************************************************/
 
-//canonvcc4::canonvcc4(ConfigFile* cf, int section) 
+//canonvcc4::canonvcc4(ConfigFile* cf, int section)
 // : Driver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_PTZ_CODE, PLAYER_OPEN_MODE)
 
 canonvcc4::canonvcc4( ConfigFile* cf, int section)
-: Driver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_PTZ_CODE)
+: ThreadedDriver(cf, section, true, PLAYER_MSGQUEUE_DEFAULT_MAXLEN, PLAYER_PTZ_CODE)
 {
   pandemand = 0; tiltdemand = 0; zoomdemand = 0;
   ptz_fd = -1;
@@ -128,8 +208,8 @@ canonvcc4::canonvcc4( ConfigFile* cf, int section)
 }
 
 /************************************************************************/
-int 
-canonvcc4::Setup()
+int
+canonvcc4::MainSetup()
 {
   int pan,tilt;
   int flags;
@@ -139,14 +219,14 @@ canonvcc4::Setup()
   fflush(stdout);
 
   // open it.  non-blocking at first, in case there's no ptz unit.
-  if((ptz_fd = 
+  if((ptz_fd =
       open(ptz_serial_port,
 	   O_RDWR | O_SYNC | O_NONBLOCK, S_IRUSR | S_IWUSR )) < 0 )
   {
     perror("canonvcc4::Setup():open():");
     return(-1);
-  }  
-  
+  }
+
   if(tcflush(ptz_fd, TCIFLUSH ) < 0 )
   {
     perror("canonvcc4::Setup():tcflush():");
@@ -164,14 +244,14 @@ canonvcc4::Setup()
 
   //  ptz_fd_blocking = false;
   err = setPower(0);
-  fprintf(stderr, "\nPowering off/on the camera!!!!!!!!!!!!!!\n");  
-  err = setPower(1);  
+  fprintf(stderr, "\nPowering off/on the camera!!!!!!!!!!!!!!\n");
+  err = setPower(1);
   while (error_code == CAM_ERROR_BUSY)
     {
       fprintf(stdout, "power on busy: %x\n", error_code);
       err = setPower(1);
     }
-  if ((err != 0) && 
+  if ((err != 0) &&
       (error_code != CAM_ERROR_NONE) && (error_code != CAM_ERROR_MODE))
     {
       printf("Could not set power on: %x\n", error_code);
@@ -197,7 +277,7 @@ canonvcc4::Setup()
   if (err)
     {
       printf("Couldn't connect to PTZ device most likely because the camera\n"
-	     "is not connected or is connected not to %s; %x\n", 
+	     "is not connected or is connected not to %s; %x\n",
 	     ptz_serial_port, error_code);
       setPower(0);
       close(ptz_fd);
@@ -218,7 +298,7 @@ canonvcc4::Setup()
       close(ptz_fd);
       return -1;
     }
-  
+
   /* ok, we got data, so now set NONBLOCK, and continue */
   if ((flags = fcntl(ptz_fd, F_GETFL)) < 0)
     {
@@ -237,7 +317,7 @@ canonvcc4::Setup()
     }
   //  ptz_fd_blocking = true;
   puts("Done.");
-  
+
   // zero the command and data buffers
   player_ptz_data_t data;
   player_ptz_cmd_t cmd;
@@ -245,12 +325,10 @@ canonvcc4::Setup()
   data.pan = data.tilt = data.zoom = 0;
   cmd.pan = cmd.tilt = cmd.zoom = 0;
 
-  StartThread();
-  
   return(0);
 }
 /************************************************************************/
-int 
+int
 canonvcc4::configurePort()
 {
   struct termios tio;
@@ -265,7 +343,7 @@ canonvcc4::configurePort()
   cfmakeraw(&tio);
   cfsetospeed(&tio, B9600);
   cfsetispeed(&tio, B9600);
-  if (tcsetattr(ptz_fd, TCSAFLUSH, &tio) < 0) 
+  if (tcsetattr(ptz_fd, TCSAFLUSH, &tio) < 0)
     {
       fprintf(stderr, "Could not config serial port.\n");
       return -1;
@@ -273,22 +351,21 @@ canonvcc4::configurePort()
   tcgetattr(ptz_fd, &tio);
 
   /* check for hardware flow control */
-  
+
   //tio.c_cflag |= CRTSCTS;
-  
+
   tio.c_cflag &= ~CRTSCTS;
   tcsetattr(ptz_fd, TCSAFLUSH, &tio);
   return 0;
 }
 
 /************************************************************************/
-int 
-canonvcc4::Shutdown()
+void
+canonvcc4::MainQuit()
 {
   if(ptz_fd == -1)
-    return(0);
-  
-  StopThread();
+    return;
+
   usleep(PTZ_SLEEP_TIME_USEC);
   SendAbsPanTilt(0,0);
   usleep(PTZ_SLEEP_TIME_USEC);
@@ -298,7 +375,6 @@ canonvcc4::Shutdown()
     perror("canonvcc4::Shutdown():close():");
   ptz_fd = -1;
   puts("PTZ camera has been shutdown");
-  return(0);
 }
 
 /************************************************************************/
@@ -309,7 +385,7 @@ canonvcc4::SendCommand(unsigned char *str, int len)
 
   if(len > MAX_PTZ_COMMAND_LENGTH)
   {
-    fprintf(stderr, 
+    fprintf(stderr,
 	    "CANNONvcc4::SendCommand(): message is too large (%d bytes)\n",
 	    len);
     return(-1);
@@ -335,7 +411,7 @@ int canonvcc4::SendRequest(unsigned char* str, int len, unsigned char* reply, ui
 
   if(len > MAX_PTZ_REQUEST_LENGTH)
   {
-    fprintf(stderr, 
+    fprintf(stderr,
 	    "canonvcc4::SendRequest(): message is too large (%d bytes)\n",
 	    len);
     return(-1);
@@ -353,7 +429,7 @@ int canonvcc4::SendRequest(unsigned char* str, int len, unsigned char* reply, ui
 
 /************************************************************************/
 void
-canonvcc4::PrintPacket(char* str, unsigned char* cmd, int len)
+canonvcc4::PrintPacket(const char* str, unsigned char* cmd, int len)
 {
   for(int i=0;i<len;i++)
     printf(" %.2x", cmd[i]);
@@ -371,22 +447,22 @@ canonvcc4::SendAbsPanTilt(int pan, int tilt)
   int ppan, ttilt;
 
   ppan = pan; ttilt = tilt;
-  if (abs(pan) > PTZ_PAN_MAX) 
+  if (abs(pan) > PTZ_PAN_MAX)
     {
       if(pan < -PTZ_PAN_MAX)
 	ppan = (int)-PTZ_PAN_MAX;
-      else 
+      else
 	if(pan > PTZ_PAN_MAX)
 	  ppan = (int)PTZ_PAN_MAX;
       //puts("Camera pan angle thresholded");
     }
-  if (tilt > PTZ_TILT_MAX) 
+  if (tilt > PTZ_TILT_MAX)
     ttilt = (int)PTZ_TILT_MAX;
-  else 
+  else
     if(tilt < PTZ_TILT_MIN)
       ttilt = (int)PTZ_TILT_MIN;
   //puts("Camera pan angle thresholded");
-  
+
   //puts("Camera tilt angle thresholded");
 
   convpan = (int)floor(ppan/.1125) + 0x8000;
@@ -400,14 +476,14 @@ canonvcc4::SendAbsPanTilt(int pan, int tilt)
   command[4] = 0x62;
   // pan position
 
-  sprintf((char *)buf, "%X", convpan);
+  snprintf((char *)buf,sizeof(buf), "%X", convpan);
 
-  command[5] = buf[0]; 
+  command[5] = buf[0];
   command[6] = buf[1];
   command[7] = buf[2];
   command[8] = buf[3];
   // tilt position
-  sprintf((char *)buf, "%X", convtilt);
+  snprintf((char *)buf,sizeof(buf), "%X", convtilt);
   command[9]  = buf[0];
   command[10] = buf[1];
   command[11] = buf[2];
@@ -418,7 +494,7 @@ canonvcc4::SendAbsPanTilt(int pan, int tilt)
   return(ReceiveCommandAnswer());
 }
 /************************************************************************/
-int 
+int
 canonvcc4::setDefaultTiltRange()
 {
   unsigned char command[MAX_PTZ_COMMAND_LENGTH];
@@ -433,15 +509,15 @@ canonvcc4::setDefaultTiltRange()
   command[5] = 0x31;
 
   mintilt = (int)(floor(PTZ_TILT_MIN/.1125) + 0x8000);
-  sprintf((char *)buf, "%X", mintilt);
+  snprintf((char *)buf,sizeof(buf), "%X", mintilt);
 
-  command[6] = buf[0]; 
+  command[6] = buf[0];
   command[7] = buf[1];
   command[8] = buf[2];
   command[9] = buf[3];
   // tilt position
   maxtilt = (int)(floor(PTZ_TILT_MAX/.1125) + 0x8000);
-  sprintf((char *)buf, "%X", maxtilt);
+  snprintf((char *)buf,sizeof(buf), "%X", maxtilt);
 
   command[10] = buf[0];
   command[11] = buf[1];
@@ -452,7 +528,7 @@ canonvcc4::setDefaultTiltRange()
   SendCommand(command, 15);
   //  PrintPacket( "setDefaultRange: ", command, 15);
   return(ReceiveCommandAnswer());
-  
+
 }
 
 /************************************************************************/
@@ -527,7 +603,7 @@ canonvcc4::GetAbsZoom(int* zoom)
   unsigned char buf[4];
   unsigned int u_zoom;
   int i;
-  
+
   command[0] = 0xFF;
   command[1] = 0x30;
   command[2] = 0x30;
@@ -559,7 +635,7 @@ canonvcc4::GetAbsZoom(int* zoom)
   u_zoom = 0;
   for (i = 0; i < 4; i++)
     u_zoom += buf[i] * (unsigned int) pow(16.0, (double)(3 - i));
-  *zoom = u_zoom;  
+  *zoom = u_zoom;
   return(0);
 }
 
@@ -571,11 +647,11 @@ canonvcc4::SendAbsZoom(int zoom)
   unsigned char buf[5];
   int i;
 
-  if(zoom < 0) 
+  if(zoom < 0)
     zoom = 0;
   //puts("Camera zoom thresholded");
-  
-  else 
+
+  else
     if(zoom > MAX_ZOOM)
       zoom = MAX_ZOOM;
 
@@ -585,17 +661,17 @@ canonvcc4::SendAbsZoom(int zoom)
   command[3] = 0x00;
   command[4] = 0xB3;
 
-  sprintf((char *)buf, "%4X", zoom);
+  snprintf((char *)buf,sizeof(buf), "%4X", zoom);
 
   for (i=0;i<3;i++)
     if (buf[i] == ' ')
       buf[i] = '0';
 
   // zoom position
-  command[5] = buf[0]; 
-  command[6] = buf[1];  
-  command[7] = buf[2];  
-  command[8] = buf[3];  
+  command[5] = buf[0];
+  command[6] = buf[1];
+  command[7] = buf[2];
+  command[8] = buf[3];
   command[9] = 0xEF;
 
   if (SendCommand(command, 10))
@@ -613,13 +689,13 @@ canonvcc4::SendAbsZoom(int zoom)
 
   poll_fd.fd = ptz_fd;
   poll_fd.events = POLLIN;
-  
+
   if (poll(&poll_fd, 1, 1000) <= 0)
     return -1;
   len = read(ptz_fd, reply, size);
   if (len < 0)
     return -1;
-  
+
   return len;
 }
 
@@ -655,7 +731,7 @@ canonvcc4::ReceiveCommandAnswer()
   for(num = 1; num <= MAX_PTZ_REQUEST_LENGTH; num++)
     {
       err = read_ptz(&byte, 1);
-      if (err == 0) 
+      if (err == 0)
 	continue;
       if (err < 0)
 	{
@@ -716,7 +792,7 @@ canonvcc4::ReceiveRequestAnswer(unsigned char *data)
     {
       // if we don't get any bytes, or if we've just exceeded the limit
       // then return null
-      err = read_ptz(&byte, 1); 
+      err = read_ptz(&byte, 1);
       if (byte == (unsigned char)0xFE)
 	{
 	  reply[0] = byte;
@@ -732,7 +808,7 @@ canonvcc4::ReceiveRequestAnswer(unsigned char *data)
       err = read_ptz(&byte, 1);
       if (err == 0)
 	continue;
-      if (err < 0) 
+      if (err < 0)
 	{
 	  // there are no more bytes, so check the last byte for the footer
 	  if (reply[len - 1] !=  (unsigned char)0xEF)
@@ -757,8 +833,8 @@ canonvcc4::ReceiveRequestAnswer(unsigned char *data)
       fputs("Arvcc4::packetHandler: Incorrect number of bytes in response packet.", stderr);
       return -1;
     }
-  
-  if (reply[0] !=  (unsigned char)0xFE || 
+
+  if (reply[0] !=  (unsigned char)0xFE ||
       reply[len - 1] != (unsigned char)0xEF)
     {
       fputs("canonvcc4::receiveRequestArvcc4: Bad header or footer character in response packet.", stderr);
@@ -821,58 +897,57 @@ canonvcc4::setPower(int on)
 /************************************************************************/
 
 // this function will be run in a separate thread
-void 
+void
 canonvcc4::Main()
 {
   player_ptz_data_t data;
   int pan, tilt, zoom;
 
-  while(1) 
+  while(1)
     {
       pthread_testcancel();
       ProcessMessages();
       /* get current state */
       if(GetAbsPanTilt(&pan,&tilt) < 0)
 	{
-	  fputs("canonvcc4:Main():GetAbsPanTilt() errored. bailing.\n", 
+	  fputs("canonvcc4:Main():GetAbsPanTilt() errored. bailing.\n",
 		stderr);
 	  pthread_exit(NULL);
 	}
       /* get current state */
 
-   
+
       if(GetAbsZoom(&zoom) < 0)
 	{
 	  fputs("canonvcc4:Main():GetAbsZoom() errored. bailing.\n", stderr);
 	  pthread_exit(NULL);
 	}
-      
+
       // Do the necessary coordinate conversions.  Camera's natural pan
       // coordinates increase clockwise; we want them the other way, so
       // we negate pan here.  Zoom values are converted from arbitrary
       // units to a field of view (in degrees).
       pan = -pan;
 
-      //zoom = this->maxfov + (zoom * (this->maxfov - this->minfov)) / 1960; 
+      //zoom = this->maxfov + (zoom * (this->maxfov - this->minfov)) / 1960;
 
       data.pan = DTOR((unsigned short)pan);
       data.tilt = DTOR((unsigned short)tilt);
-      data.zoom = (unsigned short) (zoom - 3056)/ZOOM_CONV_FACTOR; 
+      data.zoom = (unsigned short) (zoom - 3056)/ZOOM_CONV_FACTOR;
       pthread_testcancel();
-      
-      Publish(device_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, &data,sizeof(player_ptz_data_t),NULL);
+
+      Publish(device_addr, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, &data,sizeof(player_ptz_data_t),NULL);
 
       usleep(PTZ_SLEEP_TIME_USEC);
     }
 }
 
-int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, void * data)
+int canonvcc4::ProcessMessage(QueuePointer &resp_queue, player_msghdr * hdr, void * data)
 {
   assert(hdr);
-  assert(data);
 
-  if (Message::MatchMessage(hdr, 
-			    PLAYER_MSGTYPE_REQ, 
+  if (Message::MatchMessage(hdr,
+			    PLAYER_MSGTYPE_REQ,
 			    PLAYER_PTZ_REQ_GENERIC, device_addr))
   {
     assert(hdr->size == sizeof(player_ptz_req_generic_t));
@@ -880,15 +955,15 @@ int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, vo
     player_ptz_req_generic_t *cfg = (player_ptz_req_generic_t *)data;
 
     // check whether command or inquiry...
-    if (cfg->config[0] == 0x01) 
+    if (cfg->config[0] == 0x01)
     {
-      if (SendCommand((uint8_t *)cfg->config, cfg->config_count) < 0) 
+      if (SendCommand((uint8_t *)cfg->config, cfg->config_count) < 0)
         Publish(device_addr, resp_queue, PLAYER_MSGTYPE_RESP_NACK, hdr->subtype);
       else
         Publish(device_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
       return 0;
-    } 
-    else 
+    }
+    else
     {
       // this is an inquiry, so we have to send data back
       cfg->config_count = SendRequest((uint8_t*)cfg->config, cfg->config_count, (uint8_t*)cfg->config);
@@ -896,16 +971,16 @@ int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, vo
     }
   }
 
-  if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD, 
+  if (Message::MatchMessage(hdr, PLAYER_MSGTYPE_CMD,
 			    PLAYER_PTZ_CMD_STATE, device_addr))
     {
-      
+
       short zoomdemand=0;
       bool newpantilt=true, newzoom=true;
-      
+
       assert (hdr->size == sizeof (player_ptz_cmd_t));
       player_ptz_cmd_t command = *reinterpret_cast<player_ptz_cmd_t *> (data);
-      
+
       if(pandemand != (int)rint(RTOD(command.pan)))
 	{
 	  pandemand = (int)rint(RTOD(command.pan));
@@ -916,17 +991,17 @@ int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, vo
 	  tiltdemand = (int)rint(RTOD(command.tilt));
 	  newpantilt = true;
 	}
-      
+
       if(zoomdemand != command.zoom)
 	{
 	  zoomdemand = (int)command.zoom; //(int)rint(RTOD(command.zoom));
 	  newzoom = true;
 	}
-      ////////////////////////////////////////////////////////////   
-      //zoomdemand = (1960 * (this->maxfov - zoomdemand)) / 
+      ////////////////////////////////////////////////////////////
+      //zoomdemand = (1960 * (this->maxfov - zoomdemand)) /
       //	(this->maxfov - this->minfov);
       ////////////////////////////////////////////////////////////
-      
+
       if(newzoom)
 	{
 	  if(SendAbsZoom(zoomdemand))
@@ -935,7 +1010,7 @@ int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, vo
 	      pthread_exit(NULL);
 	    }
 	}
-      
+
       if(newpantilt)
 	{
 	  pandemand = -pandemand;
@@ -945,10 +1020,10 @@ int canonvcc4::ProcessMessage(MessageQueue * resp_queue, player_msghdr * hdr, vo
 	      pthread_exit(NULL);
 	    }
 	  //  }
-	} 
+	}
       return 0;
     }
-  
+
   return -1;
 }
 
