@@ -30,8 +30,8 @@
 /** @defgroup driver_mica2 mica2
  * @brief Crossbow Mica2/Mica2Dot mote sensor node
 
-The mica2 driver controls the Crossbow Mica2/Mica2DOT mote sensor node. The 
-MTS310 and MTS510 boards are supported, as well as the M1-mini RFID reader 
+The mica2 driver controls the Crossbow Mica2/Mica2DOT mote sensor node. The
+MTS310 and MTS510 boards are supported, as well as the M1-mini RFID reader
 board.
 
 @par Compile-time dependencies
@@ -89,8 +89,8 @@ board.
   - Default: 0.
   - Filter the base node (ID 0) in case there is another application != TOSBase
     installed on it.
-  
-@par Example 
+
+@par Example
 
 @verbatim
 driver
@@ -129,92 +129,107 @@ driver
 #include <sys/ioctl.h>
 #include <math.h>
 #include <vector>
+#include <iostream>
+
+//For nanosleep:
+#include <time.h>
+#include <sys/time.h>
 
 // Includes needed for player
 #include <libplayercore/playercore.h>
 
 // The Mica2 device class.
-class Mica2 : public Driver
+class Mica2 : public ThreadedDriver
 {
     public:
     	// Constructor
-		Mica2 (ConfigFile* cf, int section);
+	Mica2 (ConfigFile* cf, int section);
 
-		// Destructor
-		~Mica2 ();
+	// Destructor
+	~Mica2 ();
 
-		// Implementations of virtual functions
-		virtual int Setup       ();
-		virtual int Shutdown    ();
-		virtual int Subscribe   (player_devaddr_t id);
-		virtual int Unsubscribe (player_devaddr_t id);
+	// Implementations of virtual functions
+	virtual int MainSetup       ();
+	virtual void MainQuit    ();
+	virtual int Subscribe   (player_devaddr_t id);
+	virtual int Unsubscribe (player_devaddr_t id);
 
-		// This method will be invoked on each incoming message
-		virtual int ProcessMessage (MessageQueue* resp_queue, 
+	// This method will be invoked on each incoming message
+	virtual int ProcessMessage (QueuePointer & resp_queue,
 				    player_msghdr * hdr,
 				    void * data);
     private:
 
-		// Main function for device thread.
-		virtual void Main ();
-		void RefreshData  ();
+	// Main function for device thread.
+	virtual void Main ();
+	void RefreshData  ();
 
-		// Port file descriptor
-		int                fd;
+        //!Time between samples (in mS)
+        float rfidsamplingrate;
+        //!Alarm time (mS)
+        float alarmtime;
+        //Need two timers: one for calculating the sleep time to keep a desired framerate.
+        // The other for measuring the real elapsed time. (And maybe give an alarm)
+        struct timeval tv_realtime_start;
+        struct timeval tv_realtime_end;
+        float real_elapsed;
+        bool send_rfidcmd;
 
-		// Does the user want RAW or converted values?
-		int                raw_or_converted;
+	// Port file descriptor
+	int                fd;
 
-		// Is the base node awake or sleeping?
-		int                base_node_status;
+	// Does the user want RAW or converted values?
+	int                raw_or_converted;
 
-		// Interfaces that we might be using
-		// WSN interface
-		player_devaddr_t   wsn_addr;
-		player_wsn_cmd_t   wsn_cmd;
-		bool               provideWSN;
-		int                wsn_subscriptions;
+	// Is the base node awake or sleeping?
+	int                base_node_status;
 
-		// RFID interface
-		player_devaddr_t   rfid_addr;
-		player_rfid_cmd_t  rfid_cmd;
-		bool               provideRFID;
-		int                rfid_subscriptions;
+	// Interfaces that we might be using
+	// WSN interface
+	player_devaddr_t   wsn_addr;
+	player_wsn_cmd_t   wsn_cmd;
+	bool               provideWSN;
+	int                wsn_subscriptions;
 
-		const char*        port_name;
-		int                port_speed;
+	// RFID interface
+	player_devaddr_t   rfid_addr;
+	player_rfid_data_t  rfid_cmd;
+	bool               provideRFID;
+	int                rfid_subscriptions;
 
-		// Filter base node from readings ?
-		int                filterbasenode;
+	const char*        port_name;
+	int                port_speed;
 
-		// Calibration values
-		int                nodes_count;
-		NCV                ncv;
-		
-		// Calibration values
-		int                calibration_values[6];
-		// Calibration node ID
-		int                calibration_node_id;
-		
-		int ReadSerial   (unsigned char *buffer);
-		int WriteSerial  (unsigned char *buffer, int length, unsigned char ack);
-		NodeCalibrationValues FindNodeValues (unsigned int nodeID);
-		int DecodeSerial (unsigned char *buffer, int length);
+	// Filter base node from readings ?
+	int                filterbasenode;
 
-		int BuildXCommandHeader (unsigned char* buffer, int command, 
-				 int node_id, int group_id, 
-				 int device, int state, 
+	// Calibration values
+	int                nodes_count;
+	NCV                ncv;
+
+	// Calibration values
+	int                calibration_values[6];
+	// Calibration node ID
+	int                calibration_node_id;
+
+	int ReadSerial   (unsigned char *buffer);
+	int WriteSerial  (unsigned char *buffer, int length, unsigned char ack);
+	NodeCalibrationValues FindNodeValues (unsigned int nodeID);
+	int DecodeSerial (unsigned char *buffer, int length);
+	int BuildXCommandHeader (unsigned char* buffer, int command,
+				 int node_id, int group_id,
+				 int device, int state,
 				 int rate);
-		int BuildRFIDHeader (unsigned char command, unsigned char* buf, 
-							 unsigned short len, int node_id, int group_id);
-		int  calcByte (int crc, int b);
-		char getDigit (char c);
-		void calcCRC  (unsigned char *packet, int length);
-		
-		void ChangeNodeState (int node_id, int group_id, unsigned char state, 
+	int BuildRFIDHeader (unsigned char command, unsigned char* buf,
+			     unsigned short len, int node_id, int group_id);
+	int  calcByte (int crc, int b);
+	char getDigit (char c);
+	void calcCRC  (unsigned char *packet, int length);
+
+	void ChangeNodeState (int node_id, int group_id, unsigned char state,
 			      int device, int enable, double rate);
-		float ConvertAccel (unsigned short raw_accel, int neg_1g, int pos_1g,
-		              int converted);
+	float ConvertAccel (unsigned short raw_accel, int neg_1g, int pos_1g,
+	            	    int converted);
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -227,18 +242,24 @@ Driver* Mica2_Init (ConfigFile* cf, int section)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-//Registers the driver in the driver table. Called from the 
+//Registers the driver in the driver table. Called from the
 // player_driver_init function that the loader looks for
-void Mica2_Register (DriverTable* table)
+void mica2_Register (DriverTable* table)
 {
     table->AddDriver ("mica2", Mica2_Init);
 }
 
+//This function returns the difference in mS between two timeval structures
+inline float timediffms(struct timeval start, struct timeval end) {
+    return(end.tv_sec*1000.0 + end.tv_usec/1000.0 - (start.tv_sec*1000.0 + start.tv_usec/1000.0));
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor.  Retrieve options from the configuration file and do any
 // pre-Setup() setup.
 Mica2::Mica2 (ConfigFile* cf, int section)
-	: Driver (cf, section)
+	: ThreadedDriver (cf, section)
 {
     int i = 0;
     int j = 0;
@@ -253,11 +274,11 @@ Mica2::Mica2 (ConfigFile* cf, int section)
     port_name   = cf->ReadString (section, "port", DEFAULT_MICA2_PORT);
     port_speed  = cf->ReadInt (section, "speed", DEFAULT_MICA2_RATE);
     nodes_count = cf->ReadInt (section, "nodes", 0);
-	
+
     for (i = 0; i < nodes_count; i++)
     {
     	char node_nr[7];
-    	sprintf (node_nr, "node%d", (i+1));
+    	snprintf (node_nr, sizeof(node_nr), "node%d", (i+1));
 	NodeCalibrationValues n;
 	n.node_id  = cf->ReadTupleInt (section, node_nr, 0, 0);
 	n.group_id = cf->ReadTupleInt (section, node_nr, 1, 0);
@@ -271,6 +292,8 @@ Mica2::Mica2 (ConfigFile* cf, int section)
 
     // Filter base node from readings ?
     filterbasenode   = cf->ReadInt (section, "filterbasenode", 0);
+    rfidsamplingrate = cf->ReadInt (section, "rfidsamplingrate", 500);
+
 
     // Do we create a WSN interface?
     if (cf->ReadDeviceAddr (&wsn_addr, section, "provides",
@@ -310,55 +333,56 @@ Mica2::~Mica2()
 // Subscribe
 int Mica2::Subscribe (player_devaddr_t id)
 {
-	int setupResult;
-	
-	// do the subscription
-	if ((setupResult = Driver::Subscribe (id)) == 0)
-	{
-		// also increment the appropriate subscription counter
-		if (Device::MatchDeviceAddress (id, this->wsn_addr))
-			this->wsn_subscriptions++;
-		else if (Device::MatchDeviceAddress (id, this->rfid_addr))
-			this->rfid_subscriptions++;
-	}
-	return setupResult;
+    int setupResult;
+
+    // do the subscription
+    if ((setupResult = Driver::Subscribe (id)) == 0)
+    {
+	// also increment the appropriate subscription counter
+	if (Device::MatchDeviceAddress (id, this->wsn_addr))
+	    this->wsn_subscriptions++;
+	else if (Device::MatchDeviceAddress (id, this->rfid_addr))
+	    this->rfid_subscriptions++;
+    }
+    return setupResult;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Unsubscribe
 int Mica2::Unsubscribe (player_devaddr_t id)
 {
-	int (shutdownResult);
-	
-	// do the unsubscription
-	if ((shutdownResult = Driver::Unsubscribe (id)) == 0)
+    int (shutdownResult);
+
+    // do the unsubscription
+    if ((shutdownResult = Driver::Unsubscribe (id)) == 0)
+    {
+	// also increment the appropriate subscription counter
+	if (Device::MatchDeviceAddress (id, this->wsn_addr))
 	{
-		// also increment the appropriate subscription counter
-		if (Device::MatchDeviceAddress (id, this->wsn_addr))
-		{
-			this->wsn_subscriptions--;
-			assert(this->wsn_subscriptions >= 0);
-		}
-		else if (Device::MatchDeviceAddress (id, this->rfid_addr))
-		{
-			this->rfid_subscriptions--;
-			assert(this->rfid_subscriptions >= 0);
-		}
+	    this->wsn_subscriptions--;
+	    assert(this->wsn_subscriptions >= 0);
 	}
-	return (shutdownResult);
+	else if (Device::MatchDeviceAddress (id, this->rfid_addr))
+	{
+	    this->rfid_subscriptions--;
+	    assert(this->rfid_subscriptions >= 0);
+	}
+    }
+    return (shutdownResult);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Set up the device.  Return 0 if things go well, and -1 otherwise.
-int Mica2::Setup ()
+int Mica2::MainSetup ()
 {
+    real_elapsed=0;
     // Open serial port
-    fd = open (port_name, O_RDWR | O_NOCTTY);
+    fd = open (port_name, O_RDWR | O_NOCTTY | O_NONBLOCK);
     if (fd < 0)
     {
-		PLAYER_ERROR2 ("> Connecting to MIB510 on [%s]; [%s]...[failed!]",
-		      (char*) port_name, strerror (errno));
-		return (-1);
+	PLAYER_ERROR2 ("> Connecting to MIB510 on [%s]; [%s]...[failed!]",
+	      (char*) port_name, strerror (errno));
+	return (-1);
     }
     PLAYER_MSG0 (1, "> Connecting to MIB510... [done]");
 
@@ -372,21 +396,21 @@ int Mica2::Setup ()
 
     switch (port_speed)
     {
-		case 19200:
-		{
-			port_speed = B19200;           // mica2dot
-			break;
-		}
-		case 57600:
-		{
-			port_speed = B57600;           // mica2
-			break;
-		}
-		default:
-		{
-			port_speed = B57600;
-			break;
-		}
+	case 19200:
+	{
+	    port_speed = B19200;           // mica2dot
+	    break;
+	}
+	case 57600:
+	{
+	    port_speed = B57600;           // mica2
+	    break;
+	}
+	default:
+	{
+	    port_speed = B57600;
+	    break;
+	}
     }
     // Set the baudrate to the given portSpeed
     cfsetispeed (&options, port_speed);
@@ -395,151 +419,134 @@ int Mica2::Setup ()
     // Activate the settings for the port
     if (tcsetattr (fd, TCSANOW, &options) < 0)
     {
-		PLAYER_ERROR (">> Unable to set serial port attributes !");
-		return (-1);
+	PLAYER_ERROR (">> Unable to set serial port attributes !");
+	return (-1);
     }
 
     // Make sure queues are empty before we begin
     tcflush (fd, TCIFLUSH);
 
-    // Start the device thread
-    StartThread ();
-
     return (0);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Shutdown the device
-int Mica2::Shutdown ()
+void Mica2::MainQuit ()
 {
-    // Stop the driver thread
-    StopThread ();
-	
     // Close the serial port
     close (fd);
-	
+
     PLAYER_MSG0 (1, "> Mica2 driver shutting down... [done]");
-    return (0);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // Main function for device thread
-void Mica2::Main () 
+void Mica2::Main ()
 {
-    timespec sleepTime = {0, 0};
+    gettimeofday( &tv_realtime_start, NULL );  // NULL -> don't want timezone information
 
     // The main loop; interact with the device here
     while (true)
     {
-		// test if we are supposed to cancel
-		pthread_testcancel ();
+	// test if we are supposed to cancel
+	pthread_testcancel ();
 
-		// Process any pending messages
-		ProcessMessages ();
+	// Process any pending messages
+	ProcessMessages ();
 
-		// Interact with the device, and push out the resulting data.
-		if (base_node_status != 0) // if the base node is asleep, no serial
-	        	            	   // data can be read
-		    	RefreshData ();
-
-		nanosleep (&sleepTime, NULL);
+	// Interact with the device, and push out the resulting data.
+	if (base_node_status != 0) // if the base node is asleep, no serial
+        	            	   // data can be read
+    	    RefreshData ();
     }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // ProcessMessage function
-int Mica2::ProcessMessage (MessageQueue* resp_queue, 
+int Mica2::ProcessMessage (QueuePointer & resp_queue,
 			   player_msghdr * hdr,
 			   void * data)
-{	
-    assert (hdr);
-    assert (data);
-	
-    if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_CMD, 
+{
+    if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_CMD,
 	PLAYER_WSN_CMD_DEVSTATE, wsn_addr))
     {
     	// Actuate various devices on the node
-		player_wsn_cmd_t *command = (player_wsn_cmd_t*)data;
+	player_wsn_cmd_t *command = (player_wsn_cmd_t*)data;
 
-		ChangeNodeState (command->node_id, command->group_id, 
+	ChangeNodeState (command->node_id, command->group_id,
 			 2, command->device, command->enable, -1);
 
-		return 0;
+	return 0;
     }
-    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ, 
+    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ,
 	 PLAYER_WSN_REQ_POWER, wsn_addr))
     {
-		// Put the node in sleep mode (0) or wake it up (1)
-		player_wsn_power_config *powerconfig = 
+	// Put the node in sleep mode (0) or wake it up (1)
+	player_wsn_power_config *powerconfig =
 			(player_wsn_power_config*)data;
-		
-		// Only allow 0/1 values here
-		if ((powerconfig->value != 0) && (powerconfig->value != 1))
-		{
-			Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_NACK, 
+
+	// Only allow 0/1 values here
+	if ((powerconfig->value != 0) && (powerconfig->value != 1))
+	{
+	    Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_NACK,
 		     hdr->subtype);
-			return -1;
-		}
-			
-		ChangeNodeState (powerconfig->node_id, powerconfig->group_id, 
+	    return -1;
+	}
+
+	ChangeNodeState (powerconfig->node_id, powerconfig->group_id,
 			 powerconfig->value, -1, -1, -1);
-		
-		Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
-		return 0;
+
+	Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
+	return 0;
     }
-    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ, 
+    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ,
 	 PLAYER_WSN_REQ_DATATYPE, wsn_addr))
     {
-		// Change the data type to RAW or converted metric units
-		player_wsn_datatype_config *datatype = 
+	// Change the data type to RAW or converted metric units
+	player_wsn_datatype_config *datatype =
 			(player_wsn_datatype_config*)data;
 
-		unsigned int val = datatype->value;
-		if ((val >= 0) && (val < 3))
-		{
-			raw_or_converted = val;
-			Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK,
-                	 hdr->subtype);
-		}
-		else
-			Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_NACK,
-                	 hdr->subtype);
-		return 0;
+	unsigned int val = datatype->value;
+	if ((val >= 0) && (val < 3))
+	{
+	    raw_or_converted = val;
+	    Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
+	}
+	else
+	    Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_NACK, hdr->subtype);
+	return 0;
     }
-    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ, 
+    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ,
 	 PLAYER_WSN_REQ_DATAFREQ, wsn_addr))
     {
-		// Change the data frequency rate
-		player_wsn_datafreq_config *datafreq = 
+	// Change the data frequency rate
+	player_wsn_datafreq_config *datafreq =
 			(player_wsn_datafreq_config*)data;
-		ChangeNodeState (datafreq->node_id, datafreq->group_id, 
+	ChangeNodeState (datafreq->node_id, datafreq->group_id,
 			 3, -1, -1, datafreq->frequency);
 
-		Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, 
-		 hdr->subtype);
-		return 0;
+	Publish (wsn_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
+	return 0;
     }
-    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ, 
+    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ,
 	 PLAYER_RFID_REQ_READTAG, rfid_addr))
     {
-		// to be implemented
-		Publish (rfid_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, 
-		 hdr->subtype);
-		return 0;
+	// to be implemented
+	Publish (rfid_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
+	return 0;
     }
-    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ, 
+    else if (Message::MatchMessage (hdr, PLAYER_MSGTYPE_REQ,
 	 PLAYER_RFID_REQ_WRITETAG, rfid_addr))
     {
-		// to be implemented
-		Publish (rfid_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, 
-		 hdr->subtype);
-		return 0;
+	// to be implemented
+	Publish (rfid_addr, resp_queue, PLAYER_MSGTYPE_RESP_ACK, hdr->subtype);
+	return 0;
     }
     else
     {
-		return -1;
+	return -1;
     }
-	
+
     return 0;
 }
 
@@ -551,10 +558,10 @@ int Mica2::calcByte (int crc, int b)
 
     for (int i = 0; i < 8; i++)
     {
-		if ((crc & 0x8000) == 0x8000)
-			crc = crc << 1 ^ 0x1021;
-		else
-			crc = crc << 1;
+	if ((crc & 0x8000) == 0x8000)
+	    crc = crc << 1 ^ 0x1021;
+	else
+	    crc = crc << 1;
     }
 
     return (crc & 0xFFFF);
@@ -567,7 +574,7 @@ void Mica2::calcCRC (unsigned char *packet, int length)
     int crc   = 0;
     int index = 1;
     int count = length - 3;
-	
+
     while (count > 0) {
     	crc = calcByte (crc, packet[index++]);
 	count--;
@@ -590,9 +597,9 @@ char Mica2::getDigit (char c)
 }
 ////////////////////////////////////////////////////////////////////////////////
 // BuildXCommandHeader function
-int Mica2::BuildXCommandHeader (unsigned char* buffer, int command, 
-				int node_id, int group_id, 
-				int device, int state, 
+int Mica2::BuildXCommandHeader (unsigned char* buffer, int command,
+				int node_id, int group_id,
+				int device, int state,
 				int rate)
 {
     // TinyOS header
@@ -608,14 +615,14 @@ int Mica2::BuildXCommandHeader (unsigned char* buffer, int command,
     msg->inst[0].cmd    = (unsigned short)command;
     if (device != -1)
     {
-		msg->inst[0].param.actuate.device = (unsigned short)device;
-		msg->inst[0].param.actuate.state  = (unsigned short)state;
-		return sizeof (XCommandMsg);
+	msg->inst[0].param.actuate.device = (unsigned short)device;
+	msg->inst[0].param.actuate.state  = (unsigned short)state;
+	return sizeof (XCommandMsg);
     }
     if (rate != -1)
     {
-		msg->inst[0].param.new_rate = (unsigned int)rate;
-		return sizeof (XCommandMsg);
+	msg->inst[0].param.new_rate = (unsigned int)rate;
+	return sizeof (XCommandMsg);
     }
     msg->inst[0].param.new_rate = 0xCCCCCCCC;  // Fill unused in known way
     return sizeof (XCommandMsg);
@@ -650,46 +657,46 @@ int Mica2::BuildRFIDHeader (unsigned char command, unsigned char* buffer,
 
     switch (command)
     {
-		case 0:			// SELECT_TAG (0x14)
-		{
-			cmd.flags[0]   = '0';
-			cmd.flags[1]   = '0';
-			cmd.request[0] = '1';
-			cmd.request[1] = '4';
-		        cmd_len = 6;
-			break;
-		}
-		case 1:			// READ_TAG (0x24)
-		{
-			cmd.flags[0]   = '4';
-			cmd.flags[1]   = '0';
-			cmd.request[0] = '2';
-			cmd.request[1] = '4';
-		        cmd_len = 26;
-			memcpy (cmd.type, &buffer[0], len);
-			break;
-		}
-		case 2:			// WRITE_TAG (0x44)
-		{
-			cmd.flags[0]   = '4';
-			cmd.flags[1]   = '0';
-			cmd.request[0] = '4';
-			cmd.request[1] = '4';
-	    		cmd_len = 34;
-			memcpy (cmd.type, &buffer[0], len);
-			break;
-		}
-		// Only allow already implemented commands
-		default:
-			return -1;
+	case 0:			// SELECT_TAG (0x14)
+	{
+	    cmd.flags[0]   = '0';
+	    cmd.flags[1]   = '0';
+	    cmd.request[0] = '1';
+	    cmd.request[1] = '4';
+	    cmd_len = 6;
+	    break;
 	}
+	case 1:			// READ_TAG (0x24)
+	{
+	    cmd.flags[0]   = '4';
+	    cmd.flags[1]   = '0';
+	    cmd.request[0] = '2';
+	    cmd.request[1] = '4';
+	    cmd_len = 26;
+	    memcpy (cmd.type, &buffer[0], len);
+	    break;
+	}
+	case 2:			// WRITE_TAG (0x44)
+	{
+	    cmd.flags[0]   = '4';
+	    cmd.flags[1]   = '0';
+	    cmd.request[0] = '4';
+	    cmd.request[1] = '4';
+	    cmd_len = 34;
+	    memcpy (cmd.type, &buffer[0], len);
+	    break;
+	}
+	// Only allow already implemented commands
+	default:
+	    return -1;
+    }
 
 	// Split the packet into several chunks for Read/Write tag commands
     if (cmd_len > msg_len)
     {
         ptotal = cmd_len / msg_len;
         if (cmd_len % msg_len)
-            ptotal++; 
+            ptotal++;
     }
 
     // TOS packet struct
@@ -710,7 +717,7 @@ int Mica2::BuildRFIDHeader (unsigned char command, unsigned char* buffer,
 
         msg.tos.length = datalen + 6;		// PAYLOAD_DATA_INDEX
         msg.pi = pi;
-		
+
 	ptr = ((unsigned char*)&cmd) + cmd_i;
 
         for (i = 0; i < datalen; i++)
@@ -741,62 +748,62 @@ int Mica2::BuildRFIDHeader (unsigned char command, unsigned char* buffer,
 
 ////////////////////////////////////////////////////////////////////////////////
 // ChangeNodeState function
-void Mica2::ChangeNodeState (int node_id, int group_id, unsigned char state, 
+void Mica2::ChangeNodeState (int node_id, int group_id, unsigned char state,
 			     int device, int enable, double rate)
 {
     unsigned char buffer[255];
     int index = 0;
-	
+
     // Assign defaults to {node,group}_id
     if ((group_id == -1) || (group_id == 0))
     	group_id = 0xFF;
     if (node_id == -1)
 	node_id = 0xFFFF;
-		
+
     int node_sleep = 0;
     if (rate != -1)
 	node_sleep =  static_cast<int> (1000 / rate);
 
     // Start constructing the TinyOS packet
-	// (serial start byte and ACK are handled in the WriteSerial method) 
+	// (serial start byte and ACK are handled in the WriteSerial method)
     buffer[index++] = 0xFF;                // Broadcast sequence number
-	
+
     switch (state)
     {
     	case 0:                            // sleep (XCOMMAND_SLEEP)
 	{
-		index += BuildXCommandHeader 
-			(buffer+index, 0x11, node_id, group_id, -1, -1, -1);
-		if (node_id == 0)             // base node?
-			base_node_status = 0;
-		
-		break;
+	    index += BuildXCommandHeader
+		(buffer+index, 0x11, node_id, group_id, -1, -1, -1);
+	    if (node_id == 0)             // base node?
+		base_node_status = 0;
+
+	    break;
 	}
 	case 1:                            // wake up (XCOMMAND_WAKEUP)
 	{
-		index += BuildXCommandHeader 
-			(buffer+index, 0x12, node_id, group_id, -1, -1, -1);
-		if (node_id == 0)             // base node?
-			base_node_status = 1;
-		break;
+	    index += BuildXCommandHeader
+		(buffer+index, 0x12, node_id, group_id, -1, -1, -1);
+	    if (node_id == 0)             // base node?
+		base_node_status = 1;
+	    break;
 	}
 	case 2:                            // actuate (XCOMMAND_ACTUATE)
 	{
-		index += BuildXCommandHeader 
-			(buffer+index, 0x40, node_id, group_id, device, enable, -1);
-		break;
+	    index += BuildXCommandHeader
+		(buffer+index, 0x40, node_id, group_id, device, enable, -1);
+	    break;
 	}
 	case 3:                            // set_rate (XCOMMAND_SET_RATE)
 	{
-		index += BuildXCommandHeader 
-			(buffer+index, 0x20, node_id, group_id, -1, -1, node_sleep);
-		break;
+	    index += BuildXCommandHeader
+		(buffer+index, 0x20, node_id, group_id, -1, -1, node_sleep);
+	    break;
 	}
     }
-	
+
     index += 2;
     calcCRC (buffer, index);               // calculate and add CRC
-	
+
     WriteSerial (buffer, index, 0x41);		// Write with ACK
 }
 
@@ -808,12 +815,24 @@ void Mica2::RefreshData ()
     unsigned char buffer[255];
     // Get the time at which we started reading
     // This will be a pretty good estimate of when the phenomena occured
-    struct timeval time;
-    GlobalTime->GetTime (&time);
-	
+
+    //find out the real elapsed time
+    gettimeofday( &tv_realtime_end, NULL );
+    //calculate the time in mS
+    real_elapsed=timediffms(tv_realtime_start,tv_realtime_end)+real_elapsed;
+    //restart the timer
+    gettimeofday( &tv_realtime_start, NULL );
+    //check if the time was too long
+    if (real_elapsed > rfidsamplingrate) {
+      send_rfidcmd=true;
+      real_elapsed=0;
+    }
+
     // In case the RFID interface is enabled, send a "select_tag" command first
-    if ((provideRFID) && (this->rfid_subscriptions > 0))
+    if (((provideRFID) && (this->rfid_subscriptions > 0)) && send_rfidcmd) {
+        send_rfidcmd=false;
         BuildRFIDHeader (0, NULL, 0, 0xFFFF, 1);
+    }
 
 	// Reading from UART
     length = ReadSerial (buffer);
@@ -836,20 +855,26 @@ int Mica2::ReadSerial (unsigned char *buffer)
     int err, i = 0;
 
     buffer[i] = 0x7e;          // serial start byte
-    while (1) {
-    	err = read (fd, &c, 1);
-		if (err < 0)
-		{
-			PLAYER_ERROR (">> Error reading from serial port !");
-			return (-1);
-		}
-		if (err == 1)
-		{
-			if (++i > 255) return i;
-			buffer[i] = c;
-			if (c == 0x7e) return i;
-		}
+    int no_read=0;
+
+    while (no_read<100) {
+	err = read (fd, &c, 1);
+	if (err < 0)
+	{
+            if (errno!=EAGAIN) {
+  	      PLAYER_ERROR (">> Error reading from serial port !");
+	      return (-1);
+            } else { no_read++;}
+	}
+	if (err == 1)
+	{
+	    no_read=0;
+	    if (++i > 255) return i;
+		buffer[i] = c;
+	    if (c == 0x7e) return i;
+	}
     }
+    return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -862,21 +887,21 @@ int Mica2::WriteSerial (unsigned char *buffer, int length, unsigned char ack)
     c = 0x7e;					// serial start byte
     write (fd, &c , 1);
     c = ack;					// P_PACKET_ACK or P_PACKET_NOACK
-    write (fd, &c , 1); 
+    write (fd, &c , 1);
 
     c = buffer[0];
     while (1)
     {
-		if (i>= length)
-			return length;
-		c = buffer[i++];
-		err = write (fd, &c, 1);
+	if (i>= length)
+	    return length;
+	c = buffer[i++];
+	err = write (fd, &c, 1);
 
-		if (err < 0)
-		{
-			PLAYER_ERROR (">> Error writing to serial port !");
-			return (-1);
-		}
+	if (err < 0)
+	{
+	    PLAYER_ERROR (">> Error writing to serial port !");
+	    return (-1);
+	}
     }
     c = 0x7e;					// serial SYNC_BYTE
     write (fd, &c , 1);
@@ -889,17 +914,14 @@ int Mica2::WriteSerial (unsigned char *buffer, int length, unsigned char ack)
 NodeCalibrationValues Mica2::FindNodeValues (unsigned int nodeID)
 {
     NodeCalibrationValues n;
-	
-    unsigned int i = 0;
-	
-    for (i = 0; i < ncv.size (); i++)
+
+    for (NCV::iterator it = ncv.begin(); it != ncv.end(); it++)
     {
-    	n = ncv.at (i);
-	
-	if (n.node_id == nodeID)
-		break;
+	n = *it;
+        if (n.node_id == nodeID)
+            break;
     }
-	
+
     return n;
 }
 
@@ -910,6 +932,8 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
     NodeCalibrationValues node_values;
     player_wsn_data_t  wsn_data;
     player_rfid_data_t rfid_data;
+    rfid_data.tags = new player_rfid_tag_t[1];
+    rfid_data.tags[0].guid = new char[8];
     bool rfidPacket = FALSE;
     bool wsnPacket  = FALSE;
     int  i = 0, o = 2;    // index and offset
@@ -917,33 +941,30 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
     if ((this->wsn_subscriptions < 1) && (this->rfid_subscriptions < 1))
 	return -1;
 
-    // Zero data
-    memset (&wsn_data,  0, sizeof (player_wsn_data_t));
-    memset (&rfid_data, 0, sizeof (player_rfid_data_t));
-
     while (i < length)
     {
-	if (buffer[o] == 0x7d) {          // handle escape characters
-		buffer[i++] = buffer[++o] ^ 0x20;
-		++o;
+	if (buffer[o] == 0x7d)           // handle escape characters
+	{
+	    buffer[i++] = buffer[++o] ^ 0x20;
+	    ++o;
 	}
 	else
 	{
-		buffer[i++] = buffer[o++];
+	    buffer[i++] = buffer[o++];
 	}
     }
-	
+
     switch (buffer[2])
     {
 	case 0x03:
-	{       // a HEALTH packet
+	{   // a HEALTH packet
 	    // Health offset to data payload
 	    //SensorPacket *packet = (SensorPacket *)(buffer + 5);
 	    //HealthData *data = (HealthData *)packet;
 	    break;
 	}
 	case 0x33:
-	{       // a MULTIHOP packet
+	{   // a MULTIHOP packet
 	    if (wsn_subscriptions < 1)
 		break;
 	    // Multihop offset to data payload
@@ -958,32 +979,32 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
 			MTS510Data *data = (MTS510Data *)packet->data;
 			wsn_data.node_type      = packet->board_id;
 			wsn_data.node_id        = packet->node_id;
-			
+
 			wsn_data.node_parent_id = packet->parent;
-				
+
 			wsn_data.data_packet.light       = data->light;
-			int sound = (data->sound[0] + data->sound[1] + 
-				data->sound[2] + data->sound[3] + 
+			int sound = (data->sound[0] + data->sound[1] +
+				data->sound[2] + data->sound[3] +
 				data->sound[4]) / 5;
 			wsn_data.data_packet.mic         = sound;
-						
+
 			if (raw_or_converted != 0)
 			{
-				node_values = FindNodeValues (packet->node_id);
-						
-				wsn_data.data_packet.accel_x = ConvertAccel 
-					(data->accelX, 
-					node_values.c_values[0], 
+			    node_values = FindNodeValues (packet->node_id);
+
+			    wsn_data.data_packet.accel_x = ConvertAccel
+					(data->accelX,
+					node_values.c_values[0],
 					node_values.c_values[1], raw_or_converted);
-				wsn_data.data_packet.accel_y = ConvertAccel 
+			    wsn_data.data_packet.accel_y = ConvertAccel
 					(data->accelY,
 					node_values.c_values[2],
 					node_values.c_values[3], raw_or_converted);
-			} 
+			}
 			else
 			{
-				wsn_data.data_packet.accel_x     = data->accelX;
-				wsn_data.data_packet.accel_y     = data->accelY;
+			    wsn_data.data_packet.accel_x     = data->accelX;
+			    wsn_data.data_packet.accel_y     = data->accelY;
 			}
 			wsn_data.data_packet.accel_z     = -1;
 			wsn_data.data_packet.magn_x      = -1;
@@ -1003,55 +1024,54 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
 			wsn_data.node_type      = packet->board_id;
 			wsn_data.node_id        = packet->node_id;
 			wsn_data.node_parent_id = packet->parent;
-	
+
 			wsn_data.data_packet.mic         = data->mic;
-	
+
 			if (raw_or_converted != 0)
 			{
-				node_values = FindNodeValues (packet->node_id);
-				
-				wsn_data.data_packet.accel_x = ConvertAccel 
+			    node_values = FindNodeValues (packet->node_id);
+
+			    wsn_data.data_packet.accel_x = ConvertAccel
 					(data->accelX,
 					node_values.c_values[0],
 					node_values.c_values[1], raw_or_converted);
-				wsn_data.data_packet.accel_y = ConvertAccel 
+			    wsn_data.data_packet.accel_y = ConvertAccel
 					(data->accelY,
 					node_values.c_values[2],
 					node_values.c_values[3], raw_or_converted);
-				// Convert battery to Volts
-				wsn_data.data_packet.battery     =
+			    // Convert battery to Volts
+			    wsn_data.data_packet.battery     =
 					(1252352 / (float)data->vref) / 1000;
-						
-				// Convert temperature to degrees Celsius
-				float thermistor = (float)data->thermistor;
-				unsigned short rthr = (unsigned short)
+
+			    // Convert temperature to degrees Celsius
+			    float thermistor = (float)data->thermistor;
+			    unsigned short rthr = (unsigned short)
 					(10000 * (1023 - thermistor) / thermistor);
-							
-				wsn_data.data_packet.temperature = 
+
+			    wsn_data.data_packet.temperature =
 					(1 / (0.001307050f + 0.000214381f *
-					log (rthr) + 0.000000093f *
-					pow (log (rthr),3))) - 273.15;
-							
-				// Convert the magnetometer data to Gauss
-				wsn_data.data_packet.magn_x      = 
+					log (static_cast<double> (rthr)) + 0.000000093f *
+					pow (log (static_cast<double> (rthr)),3))) - 273.15;
+
+			    // Convert the magnetometer data to Gauss
+			    wsn_data.data_packet.magn_x      =
 					(data->magX / (1.023*2.262*3.2)) / 1000;
-				wsn_data.data_packet.magn_y      = 
+			    wsn_data.data_packet.magn_y      =
 					(data->magY / (1.023*2.262*3.2)) / 1000;
-							
-				// Convert the light to mV
-				wsn_data.data_packet.light       = (data->light *
+
+			    // Convert the light to mV
+			    wsn_data.data_packet.light       = (data->light *
 					wsn_data.data_packet.battery / 1023);
-			} 
+			}
 			else
 			{
-				wsn_data.data_packet.accel_x     = data->accelX;
-				wsn_data.data_packet.accel_y     = data->accelY;
-				wsn_data.data_packet.battery     = data->vref;
-				wsn_data.data_packet.temperature = 
-					data->thermistor;
-				wsn_data.data_packet.magn_x      = data->magX;
-				wsn_data.data_packet.magn_y      = data->magY;
-				wsn_data.data_packet.light       = data->light;
+			    wsn_data.data_packet.accel_x     = data->accelX;
+			    wsn_data.data_packet.accel_y     = data->accelY;
+			    wsn_data.data_packet.battery     = data->vref;
+			    wsn_data.data_packet.temperature = data->thermistor;
+			    wsn_data.data_packet.magn_x      = data->magX;
+			    wsn_data.data_packet.magn_y      = data->magY;
+			    wsn_data.data_packet.light       = data->light;
 			}
 			wsn_data.data_packet.accel_z     = -1;
 			wsn_data.data_packet.magn_z      = -1;
@@ -1062,77 +1082,77 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
 	    break;
 	}
 	case 0x51:
-	{                       // a RFID packet
-		if (rfid_subscriptions < 1)
-			break;
-		
-		rfidPacket = TRUE;
-			
-		player_rfid_tag_t RFIDtag;
-		memset (&RFIDtag, 0, sizeof (RFIDtag));
-			
-		RFIDMsg *rmsg = (RFIDMsg *)buffer;
-		int dataoffset;
-			
-		// Get tag information if first packet
-		if ((rmsg->ptotal == 1) && (rmsg->pi == 0))
-		{
-		    unsigned char response_code = getDigit (rmsg->data[0]);
-		    response_code <<= 4;
-		    response_code &= 0xF0;
-		    response_code |= getDigit (rmsg->data[1]);
-				
-		    if (response_code == 0x14)		// SELECT TAG pass
+	{   // a RFID packet
+	    if (rfid_subscriptions < 1)
+	        break;
+
+	    rfidPacket = TRUE;
+
+	    RFIDMsg *rmsg = (RFIDMsg *)buffer;
+	    int dataoffset;
+
+	    // Get tag information if first packet
+	    if ((rmsg->ptotal == 1) && (rmsg->pi == 0))
+	    {
+	        unsigned char response_code = getDigit (rmsg->data[0]);
+	        response_code <<= 4;
+	        response_code &= 0xF0;
+	        response_code |= getDigit (rmsg->data[1]);
+                rfid_data.tags_count = 0;
+                rfid_data.tags[0].guid_count=0;
+                rfid_data.tags[0].type=0;
+
+	        if (response_code == 0x14)		// SELECT TAG pass
+	        {
+		    unsigned char tag_type = getDigit (rmsg->data[2]);
+		    tag_type <<= 4;
+		    tag_type &= 0xF0;
+		    tag_type |= getDigit (rmsg->data[3]);
+                    rfid_data.tags_count = 1;
+                    rfid_data.tags[0].type = tag_type;
+		    dataoffset = 4;
+
+                    rfid_data.tags[0].guid_count = 8;
+
+		    int x = 0, cc = 0;
+		    int xlength = 23 - (29 - buffer[4]);
+		    for (x = dataoffset; x < xlength; x += 2)
 		    {
-			unsigned char tag_type = getDigit (rmsg->data[2]);
-			tag_type <<= 4;
-			tag_type &= 0xF0;
-			tag_type |= getDigit (rmsg->data[3]);
-			RFIDtag.type = tag_type;
-			dataoffset = 4;
-					
-			RFIDtag.guid_count = 8;
-		
-			int x = 0, cc = 0;
-			int xlength = 23 - (29 - buffer[4]);
-			for (x = dataoffset; x < xlength; x += 2)
+		        if ((((rmsg->data[x]   > 0x2F) && (rmsg->data[x]   < 0x3A)) ||     // if p[i] is a digit
+			    ((rmsg->data[x]    > 0x40) && (rmsg->data[x]   < 0x47)) ) &&   // if p[i] is a capital lette
+			    (((rmsg->data[x+1] > 0x2F) && (rmsg->data[x+1] < 0x3A)) ||     // if p[i+1] is a digit
+			    ((rmsg->data[x+1]  > 0x40) && (rmsg->data[x+1] < 0x47)) ))     // if p[i+1] is a capital let
 			{
-				if ((((rmsg->data[x]   > 0x2F) && (rmsg->data[x]   < 0x3A)) ||     // if p[i] is a digit
-					((rmsg->data[x]    > 0x40) && (rmsg->data[x]   < 0x47)) ) &&   // if p[i] is a capital lette
-					(((rmsg->data[x+1] > 0x2F) && (rmsg->data[x+1] < 0x3A)) ||     // if p[i+1] is a digit
-					((rmsg->data[x+1]  > 0x40) && (rmsg->data[x+1] < 0x47)) ))     // if p[i+1] is a capital let
-				{
-					char str[3];
-					sprintf (str, "%c%c", rmsg->data[x], rmsg->data[x+1]);
-					sscanf (str, "%x", (unsigned int*)&RFIDtag.guid[cc]);
-					cc++;
-				}
+			    char str[3];
+			    snprintf (str, sizeof(str), "%c%c", rmsg->data[x], rmsg->data[x+1]);
+                            sscanf (str, "%x", (unsigned int*)&rfid_data.tags[0].guid[cc]);
+			    cc++;
 			}
-		
-			rfid_data.tags_count = 1;
-			rfid_data.tags[0] = RFIDtag;
 		    }
 		}
-    		break;
+	    }
+    	    break;
 	}
 	default:
 	{
-		// we only handle RFID, HEALTH and MULTIHOP package types for now
-		break;
+	    // we only handle RFID, HEALTH and MULTIHOP package types for now
+	    break;
 	}
     }
-	
+
     if ((wsn_data.node_id == 0) && (filterbasenode == 1) && (!rfidPacket))
         return -1;
 
     if ((provideRFID) && (rfidPacket))
 	// Write the RFID data
-	Publish (rfid_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_RFID_DATA,
+	Publish (rfid_addr, PLAYER_MSGTYPE_DATA, PLAYER_RFID_DATA_TAGS,
 		&rfid_data, sizeof (player_rfid_data_t), NULL);
+    delete [] rfid_data.tags[0].guid;
+    delete [] rfid_data.tags;
 
     if ((provideWSN) && (wsnPacket))
 	// Write the WSN data
-	Publish (wsn_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_WSN_DATA,
+	Publish (wsn_addr, PLAYER_MSGTYPE_DATA, PLAYER_WSN_DATA_STATE,
 		&wsn_data, sizeof (player_wsn_data_t), NULL);
 
     return 0;
@@ -1140,14 +1160,14 @@ int Mica2::DecodeSerial (unsigned char *buffer, int length)
 
 ////////////////////////////////////////////////////////////////////////////////
 // ConvertAccel function - convert RAW accel. data to metric units (m/s^2)
-float Mica2::ConvertAccel (unsigned short raw_accel, 
+float Mica2::ConvertAccel (unsigned short raw_accel,
                            int neg_1g, int pos_1g, int converted)
 {
     if (neg_1g == 0)
 	neg_1g = 450;
     if (pos_1g == 0)
 	pos_1g = 550;
-	
+
     float sensitivity  = (pos_1g - neg_1g) / 2.0f;
     float offset       = (pos_1g + neg_1g) / 2.0f;
     float acceleration = (raw_accel - offset) / sensitivity;
