@@ -21,7 +21,7 @@
  * Desc: PlayerCam
  * Author: Brad Kratochvil
  * Date: 20050902
- * CVS: $Id: playercam.c 7807 2009-06-07 07:44:40Z thjc $
+ * CVS: $Id: playercam.c 8799 2010-06-28 04:12:42Z jpgr87 $
  *************************************************************************/
 
 /** @ingroup utils */
@@ -78,9 +78,15 @@ value at that place will be written to standard out.
 
 #include "config.h"
 
-#include <stdint.h>
+#if defined WIN32
+  #include <replace.h>
+  #define strcasecmp _stricmp
+#else
+  #include <stdint.h>
+  #include <unistd.h>
+#endif
+
 #include <assert.h>
-#include <unistd.h>
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
