@@ -1,10 +1,32 @@
+/*
+ *  Player - One Hell of a Robot Server
+ *  Copyright (C) 2002
+ *     Brian Gerkey, Richard Vaughan
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
 /*
  * playerjoy.cc - control position devices with a joystick
  *
  * Author: Richard Vaughan
  * Created: 25 July 2002
  *
- * $Id: playerjoy.cc,v 1.35.2.1 2006/09/21 03:51:36 gerkey Exp $
+ * $Id: playerjoy.cc 8128 2009-07-24 17:19:09Z thjc $
  */
 
 /** @ingroup utils */
@@ -38,6 +60,7 @@ Where options can be:
 - -n   : dont send commands or enable motors (debugging)
 - -k   : use keyboard control
 - -p   : print out speeds on the console
+- -a   : send car like commands (velocity and steering angle)
 - -udp : use UDP instead of TCP (deprecated, currently disabled)
 - -speed     : maximum linear speed (m/sec)
 - -turnspeed : maximum angular speed (deg/sec)
@@ -63,9 +86,7 @@ Details of keyboard control are printed out on the console.
 
 /** @} */
 
-#if HAVE_CONFIG_H
-  #include <config.h>
-#endif
+#include <config.h>
 
 #if HAVE_STRINGS_H
   #include <strings.h>
@@ -80,6 +101,7 @@ Details of keyboard control are printed out on the console.
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h> /* for strcpy() */
@@ -161,6 +183,9 @@ bool print_speeds = false;
 // use the keyboard instead of the joystick?
 bool use_keyboard = false;
 
+// send carlike commands
+bool use_car = false;
+
 // create a gripper proxy and keys to control it?
 bool use_gripper = false;
 GripperProxy *gp = NULL;
@@ -186,7 +211,7 @@ private:
   struct timeval lastcommand;
 
 public:
-  Client(char* host, int port ); // constructor
+  Client(const char* host, int port ); // constructor
 
   void Read( void ); // get data from Player
   void Update( struct controller* cont ); // send commands to Player
@@ -252,7 +277,12 @@ joystick_handler(void* arg)
   for(;;)
   {
     // get the next event from the joystick
-    read (jfd, &event, sizeof(struct js_event));
+    int ret = read (jfd, &event, sizeof(struct js_event));
+    if (ret < 0)
+    {
+    	fprintf(stderr,"Failed to read from joystick, trying again");
+    	continue;
+    }
 
     if ((event.type & ~JS_EVENT_INIT) == JS_EVENT_BUTTON) {
       if (event.value)
@@ -429,19 +459,19 @@ keyboard_handler(void* arg)
         break;
     case KEYCODE_R:
       // open gripper
-      if( gp ) gp->SetGrip(GRIPopen);
+      if( gp ) gp->Open();
       break;
     case KEYCODE_V:
       // close gripper
-      if( gp ) gp->SetGrip(GRIPclose);
+      if( gp ) gp->Close();
       break;
     case KEYCODE_B:
       // open gripper
-      if( gp ) gp->SetGrip(LIFTdown);
+      if( lp ) lp->MoveTo(0, 1.0f);
       break;
     case KEYCODE_T:
       // open gripper
-      if( gp ) gp->SetGrip(LIFTup);
+      if( lp ) lp->MoveTo(0, 0.0f);
       break;
     default:
       speed = 0;
@@ -458,7 +488,7 @@ keyboard_handler(void* arg)
 }
 #endif
 
-Client::Client(char* host, int port )
+Client::Client(const char* host, int port )
 {
   printf( "Connecting to Player at %s:%d - ", host, port );
   fflush( stdout );
@@ -540,10 +570,12 @@ void Client::Update( struct controller* cont )
         printf("%5.3f %5.3f\n", cont->speed, RTOD(cont->turnrate));
 
       // send the speed commands
-      if(!threed)
-        pp->SetSpeed(cont->speed, cont->turnrate);
-      else
+      if(threed)
         pp3->SetSpeed(cont->speed, cont->turnrate);
+      if(use_car)
+        pp->SetCarlike(cont->speed, cont->turnrate);
+      else
+        pp->SetSpeed(cont->speed, cont->turnrate);
     }
     else
       printf("%5.3f %5.3f\n", cont->speed, RTOD(cont->turnrate));
@@ -561,10 +593,12 @@ void Client::Update( struct controller* cont )
       {
         if(print_speeds)
           printf("%5.3f %5.3f\n", cont->speed, RTOD(cont->turnrate));
-        if(!threed)
-          pp->SetSpeed(0,0);
+        if(threed)
+          pp3->SetSpeed(cont->speed, cont->turnrate);
+        if(use_car)
+          pp->SetCarlike(cont->speed, cont->turnrate);
         else
-          pp3->SetSpeed(0,0);
+          pp->SetSpeed(cont->speed, cont->turnrate);
       }
       else
         printf("%5.3f %5.3f\n", cont->speed, cont->turnrate);
@@ -582,7 +616,7 @@ int main(int argc, char** argv)
   for( int i=1; i<argc; i++ )
   {
     // if we find a colon in the arg, it's a player address
-    if( char* colon = index( argv[i], ':'  ) )
+    if( char* colon = strchr( argv[i], ':'  ) )
     {
       // replace the colon with a terminator
       *colon = 0;
@@ -615,6 +649,8 @@ int main(int argc, char** argv)
       use_keyboard = true;
     else if( strcmp( argv[i], "-g" ) == 0 )
       use_gripper = true;
+    else if( strcmp( argv[i], "-a" ) == 0 )
+      use_car = true;
     else if( strcmp( argv[i], "-speed" ) == 0 )
       {
   if(i++ < argc)
