@@ -21,12 +21,15 @@
  * Desc: Read program options from command line and config file.
  * Author: Andrew Howard
  * Date: 28 Mar 2002
- * CVS: $Id: opt.h 639 2002-05-17 00:40:35Z inspectorg $
+ * CVS: $Id: opt.h 8799 2010-06-28 04:12:42Z jpgr87 $
  **************************************************************************/
 
 #ifndef OPT_H
 #define OPT_H
 
+#if defined WIN32
+  #define snprintf _snprintf
+#endif
 
 // Structure describing a single option.
 // This structure is opaque.
