@@ -21,7 +21,7 @@
  * Desc: Device registry.
  * Author: Andrew Howard
  * Date: 16 Aug 2002
- * CVS: $Id: registry.c 4325 2008-01-09 11:45:37Z thjc $
+ * CVS: $Id: registry.c 8799 2010-06-28 04:12:42Z jpgr87 $
  *************************************************************************/
 
 #include "playerv.h"
@@ -193,18 +193,16 @@ void create_proxy(device_t *device, opt_t *opt, mainwnd_t *mainwnd, playerc_clie
       break;
 
 
-#if 0
+
     case PLAYER_LOCALIZE_CODE:
       device->proxy = localize_create(mainwnd, opt, client,
-                                 device->index, device->drivername, device->subscribe);
+         device->addr.index,
+         device->drivername,
+         device->subscribe);
       device->fndestroy = (fndestroy_t) localize_destroy;
       device->fnupdate = (fnupdate_t) localize_update;
       break;
 
-
-
-#endif
-
     default:
       device->proxy = NULL;
       device->fndestroy = NULL;
