@@ -21,9 +21,11 @@
   Desc: Laser-odometry (incremental SLAM) package
   Author: Andrew Howard
   Date: 19 Nov 2004
-  CVS: $Id: lodo.cpp,v 1.1.2.2 2006/06/07 16:12:57 gerkey Exp $
+  CVS: $Id: lodo.cpp 7165 2008-11-21 17:03:54Z gerkey $
 */
 
+#include "pmapconfig.h"
+
 #include <assert.h>
 #include <math.h>
 #include <float.h>
@@ -33,12 +35,16 @@
 
 #include <gsl/gsl_errno.h>
 
-#ifdef ENABLE_GL
-#include <GL/glut.h>
-#endif
-
 #include "lodo.h"
 
+#ifdef GLUT_FOUND
+#ifdef __APPLE__ 
+  #include <glut.h>
+#else
+  #include <GL/glut.h>
+#endif
+#endif
+
 
 // GSL-styled callback function for minimization
 static double lodo_correct_func(double offset, lodo_t *self);
@@ -162,6 +168,7 @@ pose2_t lodo_add_scan(lodo_t *self, pose2_t odom_pose, int num_ranges, double *r
   // Initialize the new scan
   scan.opose = odom_pose;
   assert(num_ranges == self->num_ranges);
+  assert(num_ranges <= LODO_MAX_RANGES);
   memcpy(scan.ranges, ranges, num_ranges * sizeof(ranges[0]));
 
   if (self->scan_count == 0)
@@ -391,6 +398,7 @@ void lodo_project_map_free(lodo_t *self, lodo_scan_t *scan_m,
       // Transform into data polar frame
       q.x = sqrt(p.x * p.x + p.y * p.y);
       q.y = atan2(p.y, p.x);
+      q.z = 0;
       free->polar = q;
 
       // Transform into polar grid coordinates
@@ -672,7 +680,7 @@ void lodo_print_err(lodo_t *self)
 // Draw a scan
 void lodo_draw_scan(lodo_t *self, lodo_scan_t *scan)
 {
-#ifdef ENABLE_GL
+#ifdef GLUT_FOUND
   int i;
   vector2_t p;
 
@@ -709,7 +717,7 @@ void lodo_draw_scan(lodo_t *self, lodo_scan_t *scan)
 // Draw current map (hits)
 void lodo_draw_map(lodo_t *self)
 {
-#ifdef ENABLE_GL
+#ifdef GLUT_FOUND
   int i;
   vector3_t p;
 
