@@ -20,7 +20,7 @@
  * Desc: Simple PF mapper
  * Author: Andrew Howard
  * Date: 24 Nov 2004
- * CVS: $Id: pmap_test.cpp,v 1.1.2.3 2007/01/24 18:25:11 gerkey Exp $
+ * CVS: $Id: pmap_test.cpp 7165 2008-11-21 17:03:54Z gerkey $
  */
 
 /** @defgroup pmap_test PF mapping utility (pmap_test)
@@ -144,6 +144,7 @@ $ pmap_test -g --num_samples 500 --grid_width 50 --grid_height 25 <logfile>
 @endverbatim
 
 */
+#include "pmapconfig.h"
 
 #include <assert.h>
 #include <math.h>
@@ -155,7 +156,13 @@ $ pmap_test -g --num_samples 500 --grid_width 50 --grid_height 25 <logfile>
 #include <unistd.h>
 #include <sys/time.h>
 
-#include <GL/glut.h>
+#ifdef GLUT_FOUND
+#ifdef __APPLE__ 
+  #include <glut.h>
+#else
+  #include <GL/glut.h>
+#endif
+#endif
 
 #include "logfile.h"
 #include "lodo.h"
@@ -172,7 +179,7 @@ static double opt_skip = 0.0;
 static int opt_position_index = 0;
 static int opt_laser_index = 0;
 static double opt_scale = 0.025;
-static double opt_range_max = 8.0;
+static double opt_range_max = -1.0;
 static double opt_range_res = 0.10;
 static pose2_t opt_robot_pose;
 static pose2_t opt_laser_pose;
@@ -184,6 +191,14 @@ static double opt_grid_height = 48.0;
 static double opt_grid_scale = 0.10;
 static int opt_fine_max = 100;
 
+// configuration data, read from logfile
+static int laser_range_count;
+static double laser_range_max;
+static double laser_angle_min;
+static double laser_angle_max;
+static double laser_angle_step;
+
+
 // GUI settings
 static int win;
 static int gui_pause = 0;
@@ -213,15 +228,60 @@ static int fine_count = 0;
 // Local functions
 static void process();
 static void save();
+static void process_init();
 static int process_coarse();
 static void save_coarse();
 static int process_fine();
 static void save_fine();
 
+// Shows help
+void show_help ()
+{
+  fprintf (stdout,
+	   "The pmap_test utility demonstrates the basic functionality of\n"
+	   "the pmap library and serves a handy mapping utility in its own\n"
+	   "right.  Given a Player logfile containing odometry and laser\n"
+	   "data, pmap_test will produce an occupancy grid map of the\n"
+	   "environment.\n\n"
+	   "Basic usage is as follows:\n"
+	   "\t$ pmap_test [options] <logfilename>\n\n"
+	   "where the supported options are:\n"
+"   -g                                 disable the GUI (run in console mode\n"
+"                                      only).\n"
+"   --range_max                        maximum effective range for the laser\n"
+"                                      (m).\n"
+"   --position_index                   index of odometry device in logfile.\n"
+"   --laser_index                      index of laser device in logfile.\n"
+"   --num_samples                      number of samples in particle filter.\n"
+"   --resample_interval                number of scans between resampling\n"
+"                                      steps.\n"
+"   --resample_sigma                   width of resampling gaussian.\n"
+"   --num_cycles                       number of optimization cycles in the\n"
+"                                      fine phase.\n"
+"   --robot_x, --robot_y, --robot_rot  initial robot pose.\n"
+"   --grid_width, --grid_height        grid dimensions (m).\n"
+"   --grid_scale                       grid scale (m/cell).\n"
+"   --laser_x, --laser_rot             laser position offset.\n"
+"   --robot_hostname                   the name of the robot to verify in\n"
+"                                      the log.\n"
+"   --skip                             amount of time to skip between log\n"
+"                                      entries.\n"
+"   --range_res                        Resolution of the laser (only used in\n"
+"                                      lodo, not lodo2 which is currently\n"
+"                                      used.\n"
+"   --action_model_xx,\n"
+"   --action_model_rx,\n"
+"   --action_model_rr                  believe factors in the change of the\n"
+"                                      robot's pose.\n"
+);
+  
+  exit (EXIT_SUCCESS);
+}
 
 // Key callback
 void win_key(unsigned char key, int x, int y)
 {
+#ifdef GLUT_FOUND
   // Show the samples
   if (key == 'T' || key == 't')
   {
@@ -274,7 +334,7 @@ void win_key(unsigned char key, int x, int y)
   {
     save();
   }
-  
+#endif  
   return;
 }
 
@@ -283,6 +343,8 @@ void win_key(unsigned char key, int x, int y)
 // Mouse callback
 void win_mouse(int button, int state, int x, int y)
 {
+#ifdef GLUT_FOUND
+
   if (state == GLUT_DOWN)
   {
     mouse_offset.x = 2 * x * opt_scale;
@@ -294,7 +356,7 @@ void win_mouse(int button, int state, int x, int y)
     viewport_offset.y -= 2 * y * opt_scale - mouse_offset.y;
     glutPostRedisplay();
   }
-
+#endif
   return;
 }
 
@@ -302,10 +364,12 @@ void win_mouse(int button, int state, int x, int y)
 // Handle window reshape events
 void win_reshape(int width, int height)
 {
+#ifdef GLUT_FOUND
+
   glViewport(0, 0, width, height);
   viewport_width = width;
   viewport_height = height;
-
+#endif
   return;
 }
 
@@ -313,6 +377,8 @@ void win_reshape(int width, int height)
 // Redraw the window
 void win_redraw()
 {
+#ifdef GLUT_FOUND
+
   double left, right, top, bot;
 
   glClearColor(0.7, 0.7, 0.7, 1);
@@ -367,7 +433,7 @@ void win_redraw()
   }
 
   glutSwapBuffers();
-    
+#endif    
   return;
 }
 
@@ -378,7 +444,9 @@ void win_idle()
   if (!gui_pause)
   {
     process();
+	#ifdef GLUT_FOUND
     glutPostRedisplay();      
+	#endif
   }
   else
     usleep(100000);
@@ -390,6 +458,8 @@ void win_idle()
 // Run the GUI
 int win_run(int *argc, char **argv)
 {
+#ifdef GLUT_FOUND
+
   glutInit(argc, argv);
     
   // Create a window
@@ -408,9 +478,45 @@ int win_run(int *argc, char **argv)
   glutIdleFunc(win_idle);
 
   glutMainLoop();
+#endif
   return 0;
 }
 
+// recover configuration data from logfile
+void process_init()
+{ 
+  while (1)
+  {
+    int logresult = logfile_read(logfile); 
+    if (logresult < 0)
+    {
+      fprintf(stderr, "\n");
+      return;
+    }
+    else if(logresult > 0)
+      continue;
+    
+    // wait for first laser scan
+    if (strcmp(logfile->interface, "laser") == 0 &&
+        logfile->index == opt_laser_index)
+    {
+      // save data
+      laser_range_count = logfile->laser_range_count;
+      laser_range_max = logfile->laser_range_max;
+      laser_angle_min = logfile->laser_angle_min;
+      laser_angle_max = logfile->laser_angle_max;
+      laser_angle_step = logfile->laser_angle_step;
+    
+      fprintf(stderr, "range_count = %d\n", laser_range_count);
+      fprintf(stderr, "range_max = %f\n", laser_range_max);
+      fprintf(stderr, "angle_min = %f\n", laser_angle_min);
+      fprintf(stderr, "angle_max = %f\n", laser_angle_max);
+      fprintf(stderr, "angle_step = %f\n", laser_angle_step);
+    
+      return;
+    }
+  }
+}
 
 // Coarse map generation (process log file)
 int process_coarse()
@@ -452,13 +558,6 @@ int process_coarse()
     if (strcmp(logfile->interface, "laser") == 0 &&
         logfile->index == opt_laser_index)
     {
-      if (logfile->laser_range_count != 181)
-      {
-        fprintf(stderr, "incorrect range count; expecting %d, got %d",
-                lodo->num_ranges, logfile->laser_range_count);
-        exit(-1);
-      }
-
       // Update the local pose estimate (corrected odometry)
       lodo_pose = lodo_add_scan(lodo, odom_pose,
                                 logfile->laser_range_count, logfile->laser_ranges);
@@ -698,7 +797,7 @@ int main(int argc, char **argv)
 {
   struct timeval tv_a, tv_b;
   int opt;
-  const char *opts = "g";      
+  const char *opts = "gh";
   const struct option lopts[] =
     {
       {"nogui", 0, 0, 'g'},
@@ -720,6 +819,7 @@ int main(int argc, char **argv)
       {"resample_interval", 1, 0, 120},
       {"resample_sigma", 1, 0, 121},
       {"num_cycles", 1, 0, 122},
+      {"help", 0, 0, 'h'},
       {0, 0, 0, 0}
     };
   
@@ -728,6 +828,8 @@ int main(int argc, char **argv)
   {
     if (opt == 'g')
       opt_gui = 0;
+    else if (opt == 'h')
+      show_help();
     else if (opt == 1)
       opt_max_scans = atoi(optarg);
     else if (opt == 2)
@@ -769,7 +871,8 @@ int main(int argc, char **argv)
   // Get filename to process
   if (optind < 0 || optind >= argc)
   {
-    fprintf(stderr, "usage: pmap_test [options] <logfile>\n");
+    //fprintf(stderr, "usage: pmap_test [options] <logfile>\n");
+    show_help ();
     exit(-1);
   }
   opt_filename = argv[optind];
@@ -781,9 +884,19 @@ int main(int argc, char **argv)
   logfile = logfile_alloc(opt_filename);
   if (!logfile)
     return -1;
+    
+  //aquire laser configuration data
+  process_init();
+  
+  if (opt_range_max > 0)
+    laser_range_max = opt_range_max;  // override maximum laser range if specified in command line
+  
+  // restart logreader
+  logfile_free(logfile);
+  logfile = logfile_alloc(opt_filename);
 
   // Create lodo handle
-  lodo = lodo_alloc(181, opt_range_max, opt_range_res, -M_PI / 2, M_PI / 180);
+  lodo = lodo_alloc(laser_range_count, laser_range_max, opt_range_res, laser_angle_min, laser_angle_step);
   if (!lodo)
     return -1;
 
@@ -791,8 +904,8 @@ int main(int argc, char **argv)
   lodo->laser_pose = opt_laser_pose;
 
   // Create pmap handle
-  pmap = pmap_alloc(181, opt_range_max, -M_PI / 2, M_PI / 180,
-                    opt_num_samples, opt_grid_width, opt_grid_height, opt_grid_scale);
+  pmap = pmap_alloc(laser_range_count, laser_range_max, laser_angle_min, laser_angle_step,
+                      opt_num_samples, opt_grid_width, opt_grid_height, opt_grid_scale);  
   if (!pmap)
     return -1;
 
@@ -804,14 +917,14 @@ int main(int argc, char **argv)
   pmap_set_pose(pmap, opt_robot_pose);
 
   // Create rmap handle
-  rmap = rmap_alloc(181, opt_range_max, -M_PI / 2, M_PI / 180,
-                    opt_grid_width, opt_grid_height);
+  rmap = rmap_alloc(laser_range_count, laser_range_max, laser_angle_min, laser_angle_step,
+                      opt_grid_width, opt_grid_height);
   if (!rmap)
     return -1;
 
   // Create omap handle
-  omap = omap_alloc(181, opt_range_max, -M_PI / 2, M_PI / 180,
-                    opt_grid_width, opt_grid_height, opt_grid_scale);
+  omap = omap_alloc(laser_range_count, laser_range_max, laser_angle_min, laser_angle_step,
+                      opt_grid_width, opt_grid_height, opt_grid_scale); 
   if (!rmap)
     return -1;
 
